@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   5/5/00
 *
-* $Revision: 1.22 $
+* $Revision: 1.23 $
 *
 * File Description: 
 *
@@ -80,6 +80,12 @@ NLM_EXTERN CONN GiRevHistOpenConnection (
   Int4Ptr uids
 );
 
+NLM_EXTERN CONN GiAccVerOpenConnection (
+  Int4 uid,
+  Int4 num,
+  Int4Ptr uids
+);
+
 NLM_EXTERN CONN AccnRevHistOpenConnection (
   CharPtr accn
 );
@@ -104,6 +110,10 @@ NLM_EXTERN CharPtr GiRevHistWaitForReply (
   CONN conn
 );
 
+NLM_EXTERN CharPtr GiAccVerWaitForReply (
+  CONN conn
+);
+
 NLM_EXTERN Int4 AccnRevHistWaitForReply (
   CONN conn
 );
@@ -137,6 +147,12 @@ NLM_EXTERN CharPtr GiRevHistSynchronousQuery (
   Int4Ptr uids
 );
 
+NLM_EXTERN CharPtr GiAccVerSynchronousQuery (
+  Int4 uid,
+  Int4 num,
+  Int4Ptr uids
+);
+
 NLM_EXTERN Int4 AccnRevHistSynchronousQuery (
   CharPtr accn
 );
@@ -217,6 +233,24 @@ NLM_EXTERN CharPtr GiRevHistReadReply (
   EIO_Status status
 );
 
+NLM_EXTERN Boolean GiAccVerAsynchronousQuery (
+  Int4 uid,
+  Int4 num,
+  Int4Ptr uids,
+  QUEUE* q,
+  QueryResultProc resultproc,
+  VoidPtr userdata
+);
+
+NLM_EXTERN Int4 GiAccVerCheckQueue (
+  QUEUE* q
+);
+
+NLM_EXTERN CharPtr GiAccVerReadReply (
+  CONN conn,
+  EIO_Status status
+);
+
 NLM_EXTERN Boolean AccnRevHistAsynchronousQuery (
   CharPtr accn,
   QUEUE* q,
