@@ -1,4 +1,4 @@
-/* $Id: blast_lookup.c,v 1.25 2004/05/19 14:52:02 camacho Exp $
+/* $Id: blast_lookup.c,v 1.31 2004/09/21 13:47:41 dondosha Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,35 +37,57 @@
 #include "blast_inline.h"
 
 static char const rcsid[] = 
-    "$Id: blast_lookup.c,v 1.25 2004/05/19 14:52:02 camacho Exp $";
+    "$Id: blast_lookup.c,v 1.31 2004/09/21 13:47:41 dondosha Exp $";
 
-static void AddWordHits( LookupTable *lookup,
-			 Int4** matrix,
-			 Uint1* word,
-			 Int4 offset,
-                         Int4 query_bias);
+typedef struct NeighborInfo {
+    BlastLookupTable *lookup;
+    Uint1 *query_word;
+    Uint1 *subject_word;
+    Int4 alphabet_size;
+    Int4 wordsize;
+    Int4 **matrix;
+    Int4 *row_max;
+    Int4 *offset_list;
+    Int4 threshold;
+    Int4 query_bias;
+} NeighborInfo;
 
-static void AddPSSMWordHits( LookupTable *lookup,
+static void AddWordHits( BlastLookupTable *lookup,
 			 Int4** matrix,
-			 Int4 offset,
-                         Int4 query_bias);
+			 Uint1* query,
+			 Int4* offset_list,
+			 Int4 query_bias,
+                         Int4 *row_max);
+
+static void _AddWordHits(NeighborInfo *info, 
+                         Int4 score, 
+                         Int4 current_pos);
+
+static void AddPSSMWordHits( BlastLookupTable *lookup,
+			 Int4** matrix,
+			 Int4 query_bias,
+                         Int4 *row_max);
+
+static void _AddPSSMWordHits(NeighborInfo *info, 
+                         Int4 score, 
+                         Int4 current_pos);
+
 
 Int4 BlastAaLookupNew(const LookupTableOptions* opt,
-		      LookupTable* * lut)
+		      BlastLookupTable* * lut)
 {
   return LookupTableNew(opt, lut, TRUE);
 }
 
 Int4 RPSLookupTableNew(const RPSInfo *info,
-		      RPSLookupTable* * lut)
+		      BlastRPSLookupTable* * lut)
 {
    Int4 i;
    RPSLookupFileHeader *lookup_header;
    RPSProfileHeader *profile_header;
-   RPSLookupTable* lookup = *lut = 
-      (RPSLookupTable*) calloc(1, sizeof(RPSLookupTable));
+   BlastRPSLookupTable* lookup = *lut = 
+      (BlastRPSLookupTable*) calloc(1, sizeof(BlastRPSLookupTable));
    Int4* pssm_start;
-   Int4 num_profiles;
    Int4 num_pssm_rows;
    Int4 longest_chain;
 
@@ -79,7 +101,7 @@ Int4 RPSLookupTableNew(const RPSInfo *info,
 
    lookup->rps_aux_info = (RPSAuxInfo *)(&info->aux_info);
    lookup->wordsize = BLAST_WORDSIZE_PROT;
-   lookup->alphabet_size = PSI_ALPHABET_SIZE;
+   lookup->alphabet_size = BLASTAA_SIZE;
    lookup->charsize = ilog2(lookup->alphabet_size) + 1;
    lookup->backbone_size = 1 << (lookup->wordsize * lookup->charsize);
    lookup->mask = lookup->backbone_size - 1;
@@ -113,10 +135,10 @@ Int4 RPSLookupTableNew(const RPSInfo *info,
       return -2;
 
    lookup->rps_seq_offsets = profile_header->start_offsets;
-   num_profiles = profile_header->num_profiles;
-   num_pssm_rows = lookup->rps_seq_offsets[num_profiles];
+   lookup->num_profiles = profile_header->num_profiles;
+   num_pssm_rows = lookup->rps_seq_offsets[lookup->num_profiles];
    lookup->rps_pssm = (Int4 **)malloc((num_pssm_rows+1) * sizeof(Int4 *));
-   pssm_start = profile_header->start_offsets + num_profiles + 1;
+   pssm_start = profile_header->start_offsets + lookup->num_profiles + 1;
 
    for (i = 0; i < num_pssm_rows + 1; i++) {
       lookup->rps_pssm[i] = pssm_start;
@@ -127,11 +149,11 @@ Int4 RPSLookupTableNew(const RPSInfo *info,
 }
 
 Int4 LookupTableNew(const LookupTableOptions* opt,
-		      LookupTable* * lut,
+		      BlastLookupTable* * lut,
 		      Boolean is_protein)
 {
-   LookupTable* lookup = *lut = 
-      (LookupTable*) calloc(1, sizeof(LookupTable));
+   BlastLookupTable* lookup = *lut = 
+      (BlastLookupTable*) calloc(1, sizeof(BlastLookupTable));
 
    ASSERT(lookup != NULL);
 
@@ -168,12 +190,11 @@ Int4 LookupTableNew(const LookupTableOptions* opt,
   ASSERT(lookup->thin_backbone != NULL);
 
   lookup->use_pssm = opt->use_pssm;
-  lookup->neighbors=NULL;
   lookup->overflow=NULL;
   return 0;
 }
 
-Int4 BlastAaLookupAddWordHit(LookupTable* lookup, /* in/out: the lookup table */
+Int4 BlastAaLookupAddWordHit(BlastLookupTable* lookup,
                              Uint1* w,
 			     Int4 query_offset)
 {
@@ -225,7 +246,7 @@ Int4 BlastAaLookupAddWordHit(LookupTable* lookup, /* in/out: the lookup table */
   return 0;
 }
 
-Int4 _BlastAaLookupFinalize(LookupTable* lookup)
+Int4 _BlastAaLookupFinalize(BlastLookupTable* lookup)
 {
   Int4 i;
   Int4 overflow_cells_needed=0;
@@ -342,7 +363,7 @@ Int4 BlastAaScanSubject(const LookupTableWrap* lookup_wrap,
   Uint1* s_last=NULL;
   Int4 numhits = 0; /* number of hits found for a given subject offset */
   Int4 totalhits = 0; /* cumulative number of hits found */
-  LookupTable* lookup = lookup_wrap->lut;
+  BlastLookupTable* lookup = lookup_wrap->lut;
 
   s_first = subject->sequence + *offset;
   s_last  = subject->sequence + subject->length - lookup->wordsize; 
@@ -420,7 +441,7 @@ Int4 BlastRPSScanSubject(const LookupTableWrap* lookup_wrap,
   Uint1* s_last=NULL;
   Int4 numhits = 0; /* number of hits found for a given subject offset */
   Int4 totalhits = 0; /* cumulative number of hits found */
-  RPSLookupTable* lookup = (RPSLookupTable *)lookup_wrap->lut;
+  BlastRPSLookupTable* lookup = (BlastRPSLookupTable *)lookup_wrap->lut;
   RPSBackboneCell *cell;
 
   s_first = sequence->sequence + *offset;
@@ -498,311 +519,379 @@ Int4 BlastRPSScanSubject(const LookupTableWrap* lookup_wrap,
 }
 
 
-Int4 BlastAaLookupIndexQueries(LookupTable* lookup,
+Int4 BlastAaLookupIndexQuery(BlastLookupTable* lookup,
 			       Int4 ** matrix,
 			       BLAST_SequenceBlk* query,
-			       ListNode* locations,
-			       Int4 num_queries)
+			       BlastSeqLoc* locations)
 {
-  Int4 i;
 
-  /* create neighbor array */
-
-  if (lookup->threshold>0)
-    {
-      MakeAllWordSequence(lookup);
-    }
-
-  /* index queries */
-  for(i=0;i<num_queries;i++)
-    {
-      _BlastAaLookupIndexQuery(lookup, matrix, 
-                               (lookup->use_pssm == TRUE) ? NULL : &(query[i]), 
-                               &(locations[i]), 0);
-    }
-
-  /* free neighbor array*/
-  if (lookup->neighbors != NULL)
-    sfree(lookup->neighbors);
-
-  return 0;
+return _BlastAaLookupIndexQuery(lookup,
+                               matrix, 
+                               (lookup->use_pssm == TRUE) ? NULL : query, 
+                               locations, 0);
 }
 
-Int4 _BlastAaLookupIndexQuery(LookupTable* lookup,
+Int4 _BlastAaLookupIndexQuery(BlastLookupTable* lookup,
 			      Int4 ** matrix,
 			      BLAST_SequenceBlk* query,
-			      ListNode* location,
+			      BlastSeqLoc* location,
                               Int4 query_bias)
 {
-  ListNode* loc;
-  Int4 from, to;
-  Int4 w;
+    if (lookup->use_pssm)
+        AddPSSMNeighboringWords(lookup, matrix, 
+                                query_bias, location);
+    else
+        AddNeighboringWords(lookup, matrix, query,
+                            query_bias, location);
+  return 0;
+}
+
+Int4 AddNeighboringWords(BlastLookupTable* lookup, Int4 ** matrix, BLAST_SequenceBlk* query, Int4 query_bias, BlastSeqLoc* location)
+{
+  Int4 offset;
+  Int4 i, j;
+  Int4 **exact_backbone;
+  Int4 **old_backbone;
+  BlastSeqLoc* loc;
+  Int4 *row_max;
+
+  /* Determine the maximum possible score for
+     each row of the score matrix */
+
+  row_max = (Int4 *)malloc(lookup->alphabet_size * sizeof(Int4));
+  ASSERT(row_max != NULL);
+
+  for (i = 0; i < lookup->alphabet_size; i++) {
+      row_max[i] = matrix[i][0];
+      for (j = 1; j < lookup->alphabet_size; j++)
+          row_max[i] = MAX(row_max[i], matrix[i][j]);
+  }
+
+  /* Swap out the existing thin backbone for an empty
+     backbone */
+
+  old_backbone = lookup->thin_backbone;
+  lookup->thin_backbone = (Int4 **)calloc(lookup->backbone_size, 
+                                           sizeof(Int4 *));
+
+  /* find all the exact matches, grouping together all
+     offsets of identical query words. The query bias
+     is not used here, since the next stage will need real 
+     offsets into the query sequence */
 
   for(loc=location; loc; loc=loc->next)
-    {
-      from = ((SSeqRange*) loc->ptr)->left;
-      to = ((SSeqRange*) loc->ptr)->right - lookup->wordsize + 1;
-
-      for(w=from;w<=to;w++)
-	{
-	  AddNeighboringWords(lookup,
-			      matrix,
-			      query,
-			      w,
-                              query_bias);
-	}      
-    }
-  return 0;
-}
-
-Int4 MakeAllWordSequence(LookupTable* lookup)
-{
-  Int4 k,n;
-  Int4 i;
-  Int4 len;
-  k=lookup->alphabet_size;
-  n=lookup->wordsize;
-
-  /* need k^n bytes for the de Bruijn sequence and n-1 to unwrap it. */ 
-
-  len = iexp(k,n) + (n-1);
-  
-  lookup->neighbors_length = len;
-
-  lookup->neighbors = (Uint1*) malloc( len );
-  ASSERT(lookup->neighbors != NULL);
-
-  /* generate the de Bruijn sequence */
-
-  debruijn(n,k,lookup->neighbors,NULL);
-
-  /* unwrap it */
-
-  for(i=0;i<(n-1);i++)
-    lookup->neighbors[len-n+1+i] = lookup->neighbors[i];
-  
-  return 0;
-}
-
-Int4 AddNeighboringWords(LookupTable* lookup, Int4 ** matrix, BLAST_SequenceBlk* query, Int4 offset, Int4 query_bias)
-{
-  
-  if (lookup->use_pssm)
   {
-      AddPSSMWordHits(lookup, matrix, offset, query_bias);
+      Int4 from = loc->ssr->left;
+      Int4 to = loc->ssr->right - lookup->wordsize + 1;
+      for (offset = from; offset <= to; offset++) 
+      {
+          Uint1* w = query->sequence + offset;
+          BlastAaLookupAddWordHit(lookup, w, offset);
+          lookup->exact_matches++;
+      }
+  }
+
+  /* return the original thin backbone */
+
+  exact_backbone = lookup->thin_backbone;
+  lookup->thin_backbone = old_backbone;
+
+  /* walk though the list of exact matches
+     previously computed. Find neighboring words
+     for entire lists at a time */
+
+  for (i = 0; i < lookup->backbone_size; i++)
+  {
+      if (exact_backbone[i] != NULL)
+      {
+          AddWordHits(lookup, matrix, query->sequence,
+                      exact_backbone[i], query_bias, row_max);
+          sfree(exact_backbone[i]);
+      }
+  }
+
+  sfree(exact_backbone);
+  sfree(row_max);
+  return 0;
+}
+
+static void AddWordHits(BlastLookupTable* lookup, Int4** matrix, 
+			Uint1* query, Int4 *offset_list, 
+                        Int4 query_bias, Int4 *row_max)
+{
+  Uint1* w;
+  Uint1 s[32];
+  Int4 score;
+  Int4 i;
+  NeighborInfo info;
+
+  /* All of the offsets in the list refer to the
+     same query word. Thus, neighboring words only
+     have to be found for the first offset in the
+     list (since all other offsets would have the 
+     same neighbors) */
+
+  w = query + offset_list[2];
+
+  /* Compute the self-score of this word */
+
+  score = matrix[w[0]][w[0]];
+  for (i = 1; i < lookup->wordsize; i++)
+      score += matrix[w[i]][w[i]];
+
+  /* If the self-score is above the threshold, then the
+     neighboring computation will automatically add the
+     word to the lookup table. Otherwise, either the score
+     is too low or neighboring is not done at all, so that
+     all of these exact matches must be explicitly added
+     to the lookup table */
+
+  if (lookup->threshold == 0 || score < lookup->threshold)
+  {
+      for (i = 0; i < offset_list[1]; i++)
+          BlastAaLookupAddWordHit(lookup, w,
+                                  query_bias + offset_list[i+2]);
   }
   else
   {
-    Uint1* w = NULL;
-    ASSERT(query != NULL);
-    w = query->sequence + offset;
-  
-    if (lookup->threshold == 0)
-    {
-      BlastAaLookupAddWordHit(lookup, w, query_bias + offset);
-      lookup->exact_matches++;
-    }
-    else
-    {
-      AddWordHits(lookup, matrix, w, offset, query_bias);
-    }
+      lookup->neighbor_matches -= offset_list[1];
   }
+
+  /* check if neighboring words need to be found */
+
+  if (lookup->threshold == 0)
+      return;
+
+  /* Set up the structure of information to be used
+     during the recursion */
+
+  info.lookup = lookup;
+  info.query_word = w;
+  info.subject_word = s;
+  info.alphabet_size = lookup->alphabet_size;
+  info.wordsize = lookup->wordsize;
+  info.matrix = matrix;
+  info.row_max = row_max;
+  info.offset_list = offset_list;
+  info.threshold = lookup->threshold;
+  info.query_bias = query_bias;
+
+  /* compute the largest possible score that any neighboring
+     word can have; this maximum will gradually be replaced 
+     by exact scores as subject words are built up */
+
+  score = row_max[w[0]];
+  for (i = 1; i < lookup->wordsize; i++)
+      score += row_max[w[i]];
+
+  _AddWordHits(&info, score, 0);
+}
+
+static void _AddWordHits(NeighborInfo *info, Int4 score, Int4 current_pos)
+{
+    Int4 alphabet_size = info->alphabet_size;
+    Int4 threshold = info->threshold;
+    Uint1 *query_word = info->query_word;
+    Uint1 *subject_word = info->subject_word;
+    Int4 *row;
+    Int4 i;
+
+    /* remove the maximum score of letters that
+       align with the query letter at position 
+       'current_pos'. Later code will align the
+       entire alphabet with this letter, and compute
+       the exact score each time. Also point to the 
+       row of the score matrix corresponding to the
+       query letter at current_pos */
+
+    score -= info->row_max[query_word[current_pos]];
+    row = info->matrix[query_word[current_pos]];
+
+    if (current_pos == info->wordsize - 1) {
+
+        /* The recursion has bottomed out, and we can
+           produce complete subject words. Pass the
+           entire alphabet through the last position
+           in the subject word, then save the list of
+           query offsets in all positions corresponding
+           to subject words that yield a high enough score */
+
+        Int4 *offset_list = info->offset_list;
+        Int4 query_bias = info->query_bias;
+        BlastLookupTable *lookup = info->lookup;
+        Int4 j;
+
+        for (i = 0; i < alphabet_size; i++) {
+            if (score + row[i] >= threshold) {
+                subject_word[current_pos] = i;
+                for (j = 0; j < offset_list[1]; j++) {
+                    BlastAaLookupAddWordHit(lookup, subject_word,
+                                            query_bias + offset_list[j+2]);
+                }
+                lookup->neighbor_matches += offset_list[1];
+            }
+        }
+        return;
+    }
+
+    /* Otherwise, pass the entire alphabet through position
+       current_pos of the subject word, and recurse on all
+       words that could possibly exceed the threshold later */
+
+    for (i = 0; i < alphabet_size; i++) {
+        if (score + row[i] >= threshold) {
+            subject_word[current_pos] = i;
+            _AddWordHits(info, score + row[i], current_pos + 1);
+        }
+    }
+}
+
+Int4 AddPSSMNeighboringWords(BlastLookupTable* lookup, Int4 ** matrix, Int4 query_bias, BlastSeqLoc *location)
+{
+  Int4 offset;
+  Int4 i, j;
+  BlastSeqLoc* loc;
+  Int4 *row_max;
+  Int4 wordsize = lookup->wordsize;
+
+  /* for PSSMs, we only have to track the maximum
+     score of 'wordsize' matrix columns */
+
+  row_max = (Int4 *)malloc(lookup->wordsize * sizeof(Int4));
+  ASSERT(row_max != NULL);
+
+  for(loc=location; loc; loc=loc->next)
+  {
+      Int4 from = loc->ssr->left;
+      Int4 to = loc->ssr->right - wordsize + 1;
+      Int4 **row = matrix + from;
+
+      /* prepare to start another run of adjacent query
+         words. Find the maximum possible score for the
+         first wordsize-1 rows of the PSSM */
+
+      if (to >= from)
+      {
+          for (i = 0; i < wordsize - 1; i++) 
+          {
+              row_max[i] = row[i][0];
+              for (j = 1; j < lookup->alphabet_size; j++)
+                  row_max[i] = MAX(row_max[i], row[i][j]);
+          }
+      }
+
+      for (offset = from; offset <= to; offset++, row++) 
+      {
+          /* find the maximum score of the next PSSM row */
+
+          row_max[wordsize - 1] = row[wordsize - 1][0];
+          for (i = 1; i < lookup->alphabet_size; i++)
+              row_max[wordsize - 1] = MAX(row_max[wordsize - 1], 
+                                          row[wordsize - 1][i]);
+
+          /* find all neighboring words */
+
+          AddPSSMWordHits(lookup, row, offset + query_bias, row_max);
+
+          /* shift the list of maximum scores over by one,
+             to make room for the next maximum in the next
+             loop iteration */
+
+          for (i = 0; i < wordsize - 1; i++)
+              row_max[i] = row_max[i+1];
+      }
+  }
+
+  sfree(row_max);
   return 0;
 }
 
-static void AddWordHits(LookupTable* lookup, Int4** matrix, 
-			Uint1* word, Int4 offset, Int4 query_bias)
+static void AddPSSMWordHits(BlastLookupTable* lookup, Int4** matrix, 
+			    Int4 offset, Int4 *row_max)
 {
-  Uint1* s = lookup->neighbors;
-  Uint1* s_end=s + lookup->neighbors_length - lookup->wordsize + 1;
-  Uint1* w = word;
-  Uint1 different;
+  Uint1 s[32];
   Int4 score;
-  Int4 threshold = lookup->threshold;
   Int4 i;
-  Int4* p0;
-  Int4* p1;
-  Int4* p2;
-  Int4 corrected_offset = offset + query_bias;
+  NeighborInfo info;
 
-  /* For each group of 'wordsize' bytes starting at 'w', 
-   * add the group to the lookup table at each offset 's' if
-   * either
-   *    - w matches s, or 
-   *    - the score derived from aligning w and s is high enough
-   */
+  /* Set up the structure of information to be used
+     during the recursion */
 
-  switch (lookup->wordsize)
-    {
-      case 1:
-	p0 = matrix[w[0]];
+  info.lookup = lookup;
+  info.query_word = NULL;
+  info.subject_word = s;
+  info.alphabet_size = lookup->alphabet_size;
+  info.wordsize = lookup->wordsize;
+  info.matrix = matrix;
+  info.row_max = row_max;
+  info.offset_list = NULL;
+  info.threshold = lookup->threshold;
+  info.query_bias = offset;
 
-        while (s < s_end)
-          {
-            if ( (s[0] == w[0]) || (p0[s[0]] >= threshold) )
-	    {
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-		if (s[0] == w[0])
-		    lookup->exact_matches++;
-		else
-		    lookup->neighbor_matches++;
-	    }
-            s++;
-  	  }
+  /* compute the largest possible score that any neighboring
+     word can have; this maximum will gradually be replaced 
+     by exact scores as subject words are built up */
 
-        break;
+  score = row_max[0];
+  for (i = 1; i < lookup->wordsize; i++)
+      score += row_max[i];
 
-      case 2:
-	p0 = matrix[w[0]];
-	p1 = matrix[w[1]];
-
-        while (s < s_end)
-          {
-            score = p0[s[0]] + p1[s[1]];
-	    different = (w[0] ^ s[0]) | (w[1] ^ s[1]);
-  
-            if ( !different || (score >= threshold) )
-	    {
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-		if (!different)
-		    lookup->exact_matches++;
-		else
-		    lookup->neighbor_matches++;
-	    }
-            s++;
-	  }
-
-        break;
-
-      case 3:
-	p0 = matrix[w[0]];
-	p1 = matrix[w[1]];
-	p2 = matrix[w[2]];
-
-        while (s < s_end)
-          {
-            score = p0[s[0]] + p1[s[1]] + p2[s[2]];
-	    different = (w[0] ^ s[0]) | (w[1] ^ s[1]) | (w[2] ^ s[2]);
-  
-            if ( !different || (score >= threshold) )
-	    {
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-		if (!different)
-		    lookup->exact_matches++;
-		else
-		    lookup->neighbor_matches++;
-	    }
-            s++;
-  	  }
-
-        break;
-
-      default:
-        while (s < s_end)
-          {
-            score = 0;
-            different = 0;
-            for (i = 0; i < lookup->wordsize; i++)
-              {
-                score += matrix[w[i]][s[i]];
-                different |= w[i] ^ s[i];
-              }
-  
-            if ( !different || (score >= threshold) )
-	    {
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-		if (!different)
-		    lookup->exact_matches++;
-		else
-		    lookup->neighbor_matches++;
-	    }
-            s++;
-  	  }
-
-        break;
-    }
+  _AddPSSMWordHits(&info, score, 0);
 }
 
-
-static void AddPSSMWordHits(LookupTable* lookup, Int4** matrix, 
-                            Int4 offset, Int4 query_bias)
+static void _AddPSSMWordHits(NeighborInfo *info, Int4 score, Int4 current_pos)
 {
-  Uint1* s = lookup->neighbors;
-  Uint1* s_end=s + lookup->neighbors_length - lookup->wordsize;
-  Int4 score;
-  Int4 threshold = lookup->threshold;
-  Int4 i;
-  Int4* p0;
-  Int4* p1;
-  Int4* p2;
-  Int4 corrected_offset = offset + query_bias;
+    Int4 alphabet_size = info->alphabet_size;
+    Int4 threshold = info->threshold;
+    Uint1 *subject_word = info->subject_word;
+    Int4 *row;
+    Int4 i;
 
-  /* Equivalent to AddWordHits(), except that the word score
-   * is derived from a Position-Specific Scoring Matrix (PSSM) 
-   */
+    /* remove the maximum score of letters that
+       align with the query letter at position 
+       'current_pos'. Later code will align the
+       entire alphabet with this letter, and compute
+       the exact score each time. Also point to the 
+       row of the score matrix corresponding to the
+       query letter at current_pos */
 
-  switch (lookup->wordsize)
-    {
-      case 1:
-	p0 = matrix[offset];
+    score -= info->row_max[current_pos];
+    row = info->matrix[current_pos];
 
-        while (s < s_end)
-          {
-            if (p0[s[0]] >= threshold)
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-            s++;
-  	  }
+    if (current_pos == info->wordsize - 1) {
 
-        break;
+        /* The recursion has bottomed out, and we can
+           produce complete subject words. Pass the
+           entire alphabet through the last position
+           in the subject word, then save the query offset
+           in all lookup table positions corresponding
+           to subject words that yield a high enough score */
 
-      case 2:
-	p0 = matrix[offset];
-	p1 = matrix[offset+1];
+        Int4 offset = info->query_bias;
+        BlastLookupTable *lookup = info->lookup;
 
-        while (s < s_end)
-          {
-            score = p0[s[0]] + p1[s[1]];
-  
-            if (score >= threshold)
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-            s++;
-  	  }
+        for (i = 0; i < alphabet_size; i++) {
+            if (score + row[i] >= threshold) {
+                subject_word[current_pos] = i;
+                BlastAaLookupAddWordHit(lookup, subject_word, offset);
+                lookup->neighbor_matches++;
+            }
+        }
+        return;
+    }
 
-        break;
+    /* Otherwise, pass the entire alphabet through position
+       current_pos of the subject word, and recurse on all
+       words that could possibly exceed the threshold later */
 
-      case 3:
-	p0 = matrix[offset];
-	p1 = matrix[offset + 1];
-	p2 = matrix[offset + 2];
-
-        while (s < s_end)
-          {
-            score = p0[s[0]] + p1[s[1]] + p2[s[2]];
-  
-            if (score >= threshold)
-                BlastAaLookupAddWordHit(lookup,s,corrected_offset);
-            s++;
-  	  }
-
-        break;
-
-      default:
-        while (s < s_end)
-          {
-            score = 0;
-            for(i=0;i<lookup->wordsize;i++)
-                score += matrix[offset + i][s[i]];
-  
-            if (score >= threshold)
-                BlastAaLookupAddWordHit(lookup,s,offset);
-            s++;
-  	  }
-
-        break;
+    for (i = 0; i < alphabet_size; i++) {
+        if (score + row[i] >= threshold) {
+            subject_word[current_pos] = i;
+            _AddPSSMWordHits(info, score + row[i], current_pos + 1);
+        }
     }
 }
 
-
 /******************************************************
  *
  * Nucleotide BLAST specific functions and definitions
@@ -818,7 +907,7 @@ Int4 BlastNaScanSubject_AG(const LookupTableWrap* lookup_wrap,
 			   Int4 max_hits,  
        Int4* end_offset)
 {
-   LookupTable* lookup = (LookupTable*) lookup_wrap->lut;
+   BlastLookupTable* lookup = (BlastLookupTable*) lookup_wrap->lut;
    Uint1* s;
    Uint1* abs_start;
    Int4  index=0, s_off;
@@ -928,7 +1017,7 @@ Int4 BlastNaScanSubject(const LookupTableWrap* lookup_wrap,
    Uint1* s;
    Uint1* abs_start,* s_end;
    Int4  index=0, s_off;
-   LookupTable* lookup = (LookupTable*) lookup_wrap->lut;
+   BlastLookupTable* lookup = (BlastLookupTable*) lookup_wrap->lut;
    Int4* lookup_pos;
    Int4 num_hits;
    Int4 q_off;
@@ -986,7 +1075,7 @@ Int4 BlastNaScanSubject(const LookupTableWrap* lookup_wrap,
    return total_hits;
 }
 
-LookupTable* LookupTableDestruct(LookupTable* lookup)
+BlastLookupTable* LookupTableDestruct(BlastLookupTable* lookup)
 {
    sfree(lookup->thick_backbone);
    sfree(lookup->overflow);
@@ -995,7 +1084,7 @@ LookupTable* LookupTableDestruct(LookupTable* lookup)
    return NULL;
 }
 
-RPSLookupTable* RPSLookupTableDestruct(RPSLookupTable* lookup)
+BlastRPSLookupTable* RPSLookupTableDestruct(BlastRPSLookupTable* lookup)
 {
    /* The following will only free memory that was 
       allocated by RPSLookupTableNew. */
@@ -1010,7 +1099,7 @@ RPSLookupTable* RPSLookupTableDestruct(RPSLookupTable* lookup)
  * @param w Pointer to the start of a word [in]
  * @param query_offset Offset into the query sequence where this word ends [in]
  */
-static Int4 BlastNaLookupAddWordHit(LookupTable* lookup, Uint1* w,
+static Int4 BlastNaLookupAddWordHit(BlastLookupTable* lookup, Uint1* w,
                                     Int4 query_offset)
 {
   Int4 index=0;
@@ -1063,17 +1152,17 @@ static Int4 BlastNaLookupAddWordHit(LookupTable* lookup, Uint1* w,
 }
 
 /* See description in blast_lookup.h */
-Int4 BlastNaLookupIndexQuery(LookupTable* lookup, BLAST_SequenceBlk* query,
-			ListNode* location)
+Int4 BlastNaLookupIndexQuery(BlastLookupTable* lookup, BLAST_SequenceBlk* query,
+			BlastSeqLoc* location)
 {
-  ListNode* loc;
+  BlastSeqLoc* loc;
   Int4 from, to;
   Int4 offset;
   Uint1* sequence;
 
   for(loc=location; loc; loc=loc->next) {
-     from = ((SSeqRange*) loc->ptr)->left;
-     to = ((SSeqRange*) loc->ptr)->right + 1;
+     from = loc->ssr->left;
+     to = loc->ssr->right + 1;
      
      sequence = query->sequence + from;
      /* Last offset is such that full word fits in the sequence */
