@@ -5,6 +5,12 @@
 
 #include <aliparse.h>
 
+#ifdef OS_UNIX_DARWIN
+#define NLM_GETC fgetc
+#else
+#define NLM_GETC getc
+#endif
+
 /* Defined constants */
 
 #define ALI_DATA_NUCLEOTIDE  1      /* Values for dataType   */
@@ -23,7 +29,8 @@ Int2        IsValidIdChar (Char idChar);
 Boolean     IsValidId (CharPtr idStr);
 CharPtr     ReadAlignFileLine (FILE PNTR        alignFilePtr,
 			       ErrInfoPtr PNTR  errorListPtr,
-			       AliConfigInfoPtr configPtr);
+			       AliConfigInfoPtr configPtr,
+			       Boolean    PNTR  isEOF);
 
 
 static Boolean          s_MightBeCorruptSequence (Int4             seqCharCount,
@@ -48,7 +55,7 @@ static OtherLineInfoPtr s_ParseOtherLine (CharPtr lineStr);
 
 Boolean IsNucleotideChar (Char ch)
 {
-  if (StringChr("abcdghkmnrstvwxyABCDGHKMNRSTVWXY",ch) != NULL)
+  if (StringChr("abcdghkmnrstuvwxyABCDGHKMNRSTUVWXY",ch) != NULL)
     return TRUE;
   else
     return FALSE;
@@ -79,8 +86,11 @@ Int2 Ali_SeqLineGetType(CharPtr seqLine,
 {
   Int4    position;
   Int4    nuclCount;
+  Int4    miscCount;
   FloatLo percentNucl;
+  FloatLo percentMisc;
   Char    commonNucls[20];
+  Char    miscChars[5];
 
   /* Is it definitely a protein sequence? */
   /* The following chars are only in      */
@@ -98,30 +108,50 @@ Int2 Ali_SeqLineGetType(CharPtr seqLine,
       (StringChr (seqLine, 'p')) ||
       (StringChr (seqLine, 'Q')) ||
       (StringChr (seqLine, 'q')) ||
+      /*
       (StringChr (seqLine, 'U')) ||
       (StringChr (seqLine, 'u')) ||
+      */
       (StringChr (seqLine, 'Z')) ||
       (StringChr (seqLine, 'z')) ||
       (StringChr (seqLine, '*')))
     return ALI_PROTEIN;
 
-  /* Is it definitely a Nucleotide sequence? */
-  /* See if we have a high percentage of the */
-  /* most common nucleotide chars.           */
-
+  /* All others are technically ambiguous, but */
+  /* if we have a high enough percentage of    */
+  /* common nucleotides, then it is probably a */
+  /* nucleotide sequence.                      */
+  
   nuclCount = 0;
+  miscCount = 0;
+  sprintf (commonNucls, "ATCGNXatcgnx");
+
   if (configPtr->unalignedChar != NULL)
-    sprintf (commonNucls, "ATCGNXatcgnx-%s%s%s ", configPtr->gapChar,
+    sprintf (miscChars, "-%s%s%s ", configPtr->gapChar,
 	     configPtr->missingChar, configPtr->unalignedChar);
   else
-    sprintf (commonNucls, "ATCGNXatcgnx-%s%s ", configPtr->gapChar,
+    sprintf (miscChars, "-%s%s ", configPtr->gapChar,
 	     configPtr->missingChar);
 
-  for (position = 0; seqLine[position] != '\0'; position++)
+  for (position = 0; seqLine[position] != '\0'; position++) {
     if (StringChr (commonNucls, seqLine[position]) != NULL)
       nuclCount++;
+    else if (StringChr (miscChars, seqLine[position]) != NULL)
+      miscCount++;
+  }
 
-  percentNucl = (FloatLo) nuclCount / (FloatLo) StringLen (seqLine);
+  /* If we have a high percentage of misc chars then */
+  /* we don't have enough data to make a decision.   */
+
+  percentMisc = ((FloatLo) miscCount) / ((FloatLo) StringLen (seqLine));
+  if ((percentMisc * 100) > 80)
+    return ALI_AMBIGUOUS;
+
+  /* Else, if a high percentage are common nucleotide */
+  /* characters then it is a nucleotide line.         */
+
+  percentNucl = ((FloatLo) nuclCount + (FloatLo) miscCount) / 
+                (FloatLo) StringLen (seqLine);
 
   if ((percentNucl * 100) > configPtr->nuclLineMaxThreshold)
     return ALI_NUCLEOTIDE;
@@ -343,7 +373,8 @@ void Ali_ChangeRowToOther (ValNodePtr rowPtr)
 
 CharPtr ReadAlignFileLine (FILE PNTR        alignFilePtr,
 			   ErrInfoPtr PNTR  errorListPtr,
-			   AliConfigInfoPtr configPtr)
+			   AliConfigInfoPtr configPtr,
+			   Boolean    PNTR  isEOF)
      
 {
   CharPtr lineStr = NULL;
@@ -352,7 +383,7 @@ CharPtr ReadAlignFileLine (FILE PNTR        alignFilePtr,
   Int4    segmentLen = 0;
   Int4    segmentCount = 1;
   Boolean done = FALSE;
-  Char    ch;
+  Char    ch = 0;
 
   /* Allocate memory for the line.  More */
   /* can be added later as necessary.    */
@@ -366,22 +397,25 @@ CharPtr ReadAlignFileLine (FILE PNTR        alignFilePtr,
 
   /* Read in the characters one at a time */
 
-  while (!done && !feof(alignFilePtr))
+  while (!done && !(ch == EOF))
     {
 
       /* Process the current character */
 
-      ch = (Char) getc(alignFilePtr);
+      ch = (Char) NLM_GETC (alignFilePtr);
 
       if (ch == '\n')
 	{
 	  done = TRUE;
-	  ch = (Char) getc(alignFilePtr);
-	  if (ch != '\r')
+	  ch = (Char) NLM_GETC (alignFilePtr);
+	  if (ch != '\r') {
 	    ungetc (ch, alignFilePtr);
+	  
+	  }
 	}
-      else if (ch == '\r')
-	done = TRUE;
+      else if (ch == '\r') {
+        done = TRUE;
+      }
       else
 	{
 	  lineStr[totalLen] = ch;
@@ -415,7 +449,11 @@ CharPtr ReadAlignFileLine (FILE PNTR        alignFilePtr,
 
   /* Return successfully */
 
+  if (EOF == ch)
+    *isEOF = TRUE;
+
   lineStr[totalLen] = '\0';
+
   return lineStr;
 }
 
@@ -496,7 +534,8 @@ static DefLineInfoPtr s_ParseDefLine (CharPtr lineStr,
 	    }
 	  else
 	    {
-	      errPtr = Ali_AddError (errorListPtr, ERR_INVALID_DEFLINE, lineStr);
+	      errPtr = Ali_AddError (errorListPtr, ERR_INVALID_DEFLINE,
+				     lineStr, (Int4) ch);
 	      errPtr->rowNum = rowNum;
 	      MemFree(defStr);
 	      MemFree(idStr);
@@ -1024,6 +1063,17 @@ static Boolean s_ProcessOtherLine (AliConfigInfoPtr configPtr,
       configPtr->missingChar = (CharPtr) MemNew (2);
       sprintf (configPtr->missingChar, "%c", tmpStr[0]);
       MemFree (tmpStr);
+
+      /* If the new missing char conflicts with the */
+      /* gap or unaligned char, then blank them out */
+      /* to give the new one precedence.            */
+
+      if (StringICmp (configPtr->missingChar, configPtr->gapChar) == 0)
+	StringCpy (configPtr->gapChar, "");
+
+      if (StringICmp (configPtr->missingChar, configPtr->unalignedChar) == 0)
+	StringCpy (configPtr->unalignedChar, "");
+	
     }
   
   /* Check for definition of gap character */
@@ -1036,6 +1086,17 @@ static Boolean s_ProcessOtherLine (AliConfigInfoPtr configPtr,
       configPtr->gapChar = (CharPtr) MemNew (2);
       sprintf (configPtr->gapChar, "%c", tmpStr[0]);
       MemFree (tmpStr);
+
+      /* If the new gap char conflicts with the missing */
+      /* or unaligned char, then blank them out to give */
+      /* the new one precedence.                        */
+
+      if (StringICmp (configPtr->gapChar, configPtr->missingChar) == 0)
+	StringCpy (configPtr->missingChar, "");
+
+      if (StringICmp (configPtr->gapChar, configPtr->unalignedChar) == 0)
+	StringCpy (configPtr->unalignedChar, "");
+	
     }
   
   /* Check for definition of unaligned character */
@@ -1048,6 +1109,17 @@ static Boolean s_ProcessOtherLine (AliConfigInfoPtr configPtr,
       configPtr->unalignedChar = (CharPtr) MemNew (2);
       sprintf (configPtr->unalignedChar, "%c", tmpStr[0]);
       MemFree (tmpStr);
+
+      /* If the new unaligned char conflicts with the */
+      /* gap or missing char, then blank them out to  */
+      /* give the new one precedence.                 */
+
+      if (StringICmp (configPtr->unalignedChar, configPtr->gapChar) == 0)
+	StringCpy (configPtr->gapChar, "");
+
+      if (StringICmp (configPtr->unalignedChar, configPtr->missingChar) == 0)
+	StringCpy (configPtr->missingChar, "");
+	
     }
   
   /* Return successfully */
@@ -1071,6 +1143,7 @@ ValNodePtr Ali_ReadLines (FILE PNTR        alignFilePtr,
   ValNodePtr       newRow;
   Boolean          first = TRUE;
   SeqLineInfoPtr   seqLine;
+  SeqLineInfoPtr   reEvalSeqPtr;
   DefLineInfoPtr   defLine;
   OtherLineInfoPtr otherLine;
   Boolean          nextRowMustBeSeq;
@@ -1078,18 +1151,22 @@ ValNodePtr Ali_ReadLines (FILE PNTR        alignFilePtr,
   Boolean          lastRowWasOther = FALSE;
   Int4             rowNum;
   ErrInfoPtr       errPtr;
+  Boolean          isEOF;
 
   nextRowMustBeSeq = FALSE;
   rowNum = 0;
+  isEOF = FALSE;
 
-  while (!feof(alignFilePtr))
+  while (FALSE == isEOF)
     {
 
       /* Process the line according to its content ... */
 
-      lineStr = ReadAlignFileLine(alignFilePtr, errorListPtr, configPtr);
-      if (lineStr == NULL)
+      lineStr = ReadAlignFileLine(alignFilePtr, errorListPtr,
+				  configPtr, &isEOF);
+      if (lineStr == NULL) {
 	return NULL;
+      }
 
       rowNum++;
 
@@ -1166,8 +1243,13 @@ ValNodePtr Ali_ReadLines (FILE PNTR        alignFilePtr,
 	      /* an ID, or another sequence.              */
 	      
 	      if (lastRowWasOther == TRUE)
-		Ali_ChangeRowToOther(newRow);
-	      
+		{
+		  reEvalSeqPtr = SeqLineReEval (seqLine);
+		  if (NULL == reEvalSeqPtr)
+		    Ali_ChangeRowToOther(newRow);
+		  else
+		    newRow->data.ptrvalue = reEvalSeqPtr;
+		}
 	    }
 	  else  /* A 'maybe' sequence that we're not using */
 	    {
