@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 6/30/98
 *
-* $Revision: 6.37 $
+* $Revision: 6.48 $
 *
 * File Description:  Reengineered and optimized exploration functions
 *                      to be used for future code
@@ -70,7 +70,7 @@ extern "C" {
 
 typedef struct seqmgrbioseqcontext {
   Uint2         entityID;
-  Uint2         itemID;
+  Uint4         itemID;
   BioseqPtr     bsp;
   SeqEntryPtr   sep;
   BioseqSetPtr  bssp;
@@ -78,12 +78,12 @@ typedef struct seqmgrbioseqcontext {
   Pointer       userdata;
                           /* the following fields are for internal use only */
   Pointer       omdp;
-  Uint2         index;
+  Uint4         index;
 } SeqMgrBioseqContext, PNTR SeqMgrBioseqContextPtr;
 
 typedef struct seqmgrsegmentcontext {
   Uint2         entityID;
-  Uint2         itemID;
+  Uint4         itemID;
   SeqLocPtr     slp;
   BioseqPtr     parent;
   Int4          cumOffset;
@@ -93,12 +93,12 @@ typedef struct seqmgrsegmentcontext {
   Pointer       userdata;
                           /* the following fields are for internal use only */
   Pointer       omdp;
-  Uint2         index;
+  Uint4         index;
 } SeqMgrSegmentContext, PNTR SeqMgrSegmentContextPtr;
 
 typedef struct seqmgrdesccontext {
   Uint2         entityID;
-  Uint2         itemID;
+  Uint4         itemID;
   ValNodePtr    sdp;
   SeqEntryPtr   sep;
   Uint2         level;
@@ -106,12 +106,12 @@ typedef struct seqmgrdesccontext {
   Pointer       userdata;
                           /* the following fields are for internal use only */
   Pointer       omdp;
-  Uint2         index;
+  Uint4         index;
 } SeqMgrDescContext, PNTR SeqMgrDescContextPtr;
 
 typedef struct seqmgrfeatcontext {
   Uint2         entityID;
-  Uint2         itemID;
+  Uint4         itemID;
   SeqFeatPtr    sfp;
   SeqAnnotPtr   sap;
   BioseqPtr     bsp;
@@ -130,7 +130,7 @@ typedef struct seqmgrfeatcontext {
   Pointer       userdata;
                           /* the following fields are for internal use only */
   Pointer       omdp;
-  Uint2         index;
+  Uint4         index;
 } SeqMgrFeatContext, PNTR SeqMgrFeatContextPtr;
 
 /*****************************************************************************
@@ -156,6 +156,8 @@ NLM_EXTERN Uint2 LIBCALL SeqMgrIndexFeatures (
 *     bioseq to get the best protein feature or encoding CDS
 *   SeqMgrGetRNAgivenProduct takes an mRNA (cDNA) bioseq and gets the encoding
 *     mRNA feature on the genomic bioseq
+*   SeqMgrGetPROTgivenProduct takes an protein bioseq and gets the encoding
+*     processed protein feature on a precursor protein bioseq
 *
 *****************************************************************************/
 
@@ -178,6 +180,11 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetRNAgivenProduct (
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetPROTgivenProduct (
+  BioseqPtr bsp,
+  SeqMgrFeatContext PNTR context
+);
+
 /*****************************************************************************
 *
 *   To find the best gene feature, first call SeqMgrGetGeneXref, and if it is not
@@ -201,6 +208,11 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetOverlappingGene (
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetOverlappingOperon (
+  SeqLocPtr slp,
+  SeqMgrFeatContext PNTR context
+);
+
 /*****************************************************************************
 *
 *   SeqMgrGetOverlappingXXX returns the overlapping mRNA/CDS/publication/biosource
@@ -215,6 +227,11 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetOverlappingmRNA (
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetLocationSupersetmRNA (
+  SeqLocPtr slp,
+  SeqMgrFeatContext PNTR context
+);
+
 NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetOverlappingCDS (
   SeqLocPtr slp,
   SeqMgrFeatContext PNTR context
@@ -308,7 +325,7 @@ typedef Boolean (LIBCALLBACK *SeqMgrFeatExploreProc) (
 *
 *****************************************************************************/
 
-NLM_EXTERN Int2 LIBCALL SeqMgrExploreBioseqs (
+NLM_EXTERN Int4 LIBCALL SeqMgrExploreBioseqs (
   Uint2 entityID,
   Pointer ptr,
   Pointer userdata,
@@ -318,20 +335,20 @@ NLM_EXTERN Int2 LIBCALL SeqMgrExploreBioseqs (
   Boolean parts
 );
 
-NLM_EXTERN Int2 LIBCALL SeqMgrExploreSegments (
+NLM_EXTERN Int4 LIBCALL SeqMgrExploreSegments (
   BioseqPtr bsp,
   Pointer userdata,
   SeqMgrSegmentExploreProc userfunc
 );
 
-NLM_EXTERN Int2 LIBCALL SeqMgrExploreDescriptors (
+NLM_EXTERN Int4 LIBCALL SeqMgrExploreDescriptors (
   BioseqPtr bsp,
   Pointer userdata,
   SeqMgrDescExploreProc userfunc,
   BoolPtr seqDescFilter
 );
 
-NLM_EXTERN Int2 LIBCALL SeqMgrExploreFeatures (
+NLM_EXTERN Int4 LIBCALL SeqMgrExploreFeatures (
   BioseqPtr bsp,
   Pointer userdata,
   SeqMgrFeatExploreProc userfunc,
@@ -366,6 +383,19 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetNextFeatureByLabel (
   SeqMgrFeatContext PNTR context
 );
 
+/*****************************************************************************
+*
+*   SeqMgrGetNextGeneByLocusTag works like SeqMgrGetNextFeatureByLabel, except
+*     that it returns only genes ordered by locus_tag
+*
+*****************************************************************************/
+
+NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetNextGeneByLocusTag (
+  BioseqPtr bsp,
+  SeqFeatPtr curr,
+  SeqMgrFeatContext PNTR context
+);
+
 /*****************************************************************************
 *
 *   SeqMgrGetDesiredDescriptor and SeqMgrGetDesiredFeature return a descriptor
@@ -380,8 +410,8 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetNextFeatureByLabel (
 NLM_EXTERN ValNodePtr LIBCALL SeqMgrGetDesiredDescriptor (
   Uint2 entityID,
   BioseqPtr bsp,
-  Uint2 itemID,
-  Uint2 index,
+  Uint4 itemID,
+  Uint4 index,
   ValNodePtr sdp,
   SeqMgrDescContext PNTR context
 );
@@ -389,8 +419,8 @@ NLM_EXTERN ValNodePtr LIBCALL SeqMgrGetDesiredDescriptor (
 NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetDesiredFeature (
   Uint2 entityID,
   BioseqPtr bsp,
-  Uint2 itemID,
-  Uint2 index,
+  Uint4 itemID,
+  Uint4 index,
   SeqFeatPtr sfp,
   SeqMgrFeatContext PNTR context
 );
@@ -463,13 +493,20 @@ NLM_EXTERN Boolean LIBCALL SeqMgrGetBioseqContext (
 *     contained within the feature range, or merely that it be overlapped by the
 *     feature, and returns the position in the index
 *   SeqMgrGetFeatureInIndex gets an arbitrary feature indexed by the array
+*   SeqMgrGetAllOverlappingFeatures returns all features that overlap with the
+*     indicated overlap specificity
+*   TestFeatOverlap checks to see if feature A is in feature B with the
+*     indicated overlap specificity, -1 return value on failure
 *
 *****************************************************************************/
 
-#define SIMPLE_OVERLAP   0
-#define CONTAINED_WITHIN 1
-#define CHECK_INTERVALS  2
-#define INTERVAL_OVERLAP 3
+#define SIMPLE_OVERLAP   0 /* any overlap of extremes */
+#define CONTAINED_WITHIN 1 /* contained within extremes */
+#define LOCATION_SUBSET  2 /* SeqLocAinB must be satisfied, no boundary checking */
+#define CHECK_INTERVALS  3 /* SeqLocAinB plus internal exon-intron boundaries must match */
+#define INTERVAL_OVERLAP 4 /* at least one pair of intervals must overlap */
+#define COMMON_INTERVAL  5 /* at least one pair of intervals must match */
+#define RANGE_MATCH      6 /* the left and right ends must match exactly */
 
 NLM_EXTERN VoidPtr LIBCALL SeqMgrBuildFeatureIndex (
   BioseqPtr bsp,
@@ -492,10 +529,26 @@ NLM_EXTERN SeqFeatPtr LIBCALL SeqMgrGetFeatureInIndex (
   BioseqPtr bsp,
   VoidPtr featarray,
   Int4 numfeats,
-  Uint2 index,
+  Uint4 index,
   SeqMgrFeatContext PNTR context
 );
 
+NLM_EXTERN Int2 LIBCALL SeqMgrGetAllOverlappingFeatures (
+  SeqLocPtr slp,
+  Uint2 subtype,
+  VoidPtr featarray,
+  Int4 numfeats,
+  Int2 overlapType,
+  Pointer userdata,
+  SeqMgrFeatExploreProc userfunc
+);
+
+NLM_EXTERN Int4 TestFeatOverlap (
+  SeqFeatPtr sfpA,
+  SeqFeatPtr sfpB,
+  Int2 overlapType
+);
+
 /* the following functions are not frequently called by applications */
 
 /*****************************************************************************
@@ -530,7 +583,25 @@ NLM_EXTERN Boolean LIBCALL SeqMgrClearFeatureIndexes (
 NLM_EXTERN Uint2 LIBCALL SeqMgrIndexFeaturesEx (
   Uint2 entityID,
   Pointer ptr,
-  Boolean flip
+  Boolean flip,
+  Boolean dorevfeats
+);
+
+/*****************************************************************************
+*
+*   If indexed with dorevfeats TRUE, SeqMgrExploreFeaturesRev presents features
+*     in reverse order, used for asn2gb master style on far genomic contigs that
+*     point to the minus strand of a component
+*
+*****************************************************************************/
+
+NLM_EXTERN Int4 LIBCALL SeqMgrExploreFeaturesRev (
+  BioseqPtr bsp,
+  Pointer userdata,
+  SeqMgrFeatExploreProc userfunc,
+  SeqLocPtr locationFilter,
+  BoolPtr seqFeatFilter,
+  BoolPtr featDefFilter
 );
 
 
