@@ -44,6 +44,27 @@
 * RCS Modification History:
 * -------------------------
 * $Log: findrepl.c,v $
+* Revision 6.12  2004/04/01 13:43:05  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.11  2003/11/21 17:58:46  bollin
+* remove tax ref and common name when changing taxonomy name via ASN Find/Replace
+*
+* Revision 6.10  2003/07/31 20:54:54  kans
+* FindReplaceString does not need do_replace argument
+*
+* Revision 6.9  2003/07/31 18:18:03  kans
+* added FindReplaceString
+*
+* Revision 6.8  2003/05/11 21:12:50  kans
+* FindReplAligns loops through StdSegPtr chain, also does ssp->ids within
+*
+* Revision 6.7  2002/06/11 14:41:20  kans
+* added support for locus_tag
+*
+* Revision 6.6  2002/03/05 21:11:04  kans
+* do not replace ifp->key
+*
 * Revision 6.5  2001/12/12 17:38:38  kans
 * added new subsource qualifiers, four now have empty name
 *
@@ -289,7 +310,7 @@ static Boolean FindReplString (
   /* Make a guess of how big a working buffer we'll */
   /* need based on a worst case scenario.           */
   /*                                                */
-  /*   A = Max occurances of find string =          */
+  /*   A = Max occurrences of find string =         */
   /*               searchLen / findLen              */
   /*                                                */
   /*   B = Size increase for each replacement =     */
@@ -891,6 +912,30 @@ static void FindReplPubdesc (
   }
 }
 
+static void RemoveTaxRef (OrgRefPtr orp)
+{
+  ValNodePtr      vnp, next;
+  ValNodePtr PNTR prev;
+  DbtagPtr        dbt;
+
+  vnp = orp->db;
+  if (vnp == NULL) return;
+  prev = (ValNodePtr PNTR) &(orp->db);
+  while (vnp != NULL) {
+    next = vnp->next;
+    dbt = (DbtagPtr) vnp->data.ptrvalue;
+    if (dbt != NULL && StringICmp ((CharPtr) dbt->db, "taxon") == 0) {
+      *prev = vnp->next;
+      vnp->next = NULL;
+      DbtagFree (dbt);
+      ValNodeFree (vnp);
+    } else {
+      prev = (ValNodePtr PNTR) &(vnp->next);
+    }
+    vnp = next;
+  }
+}
+
 static void FindReplBioSource (
   BioSourcePtr biop,
   OrgRefPtr orp,
@@ -901,6 +946,7 @@ static void FindReplBioSource (
   OrgModPtr      omp;
   OrgNamePtr     onp;
   SubSourcePtr   ssp;
+  CharPtr        old_taxname;
 
   if (biop != NULL) {
     orp = biop->org;
@@ -915,7 +961,14 @@ static void FindReplBioSource (
     }
   }
   if (orp != NULL) {
+    old_taxname = StringSave (orp->taxname);
     FindReplString (&(orp->taxname), fsp);
+    if (StringCmp (old_taxname, orp->taxname) != 0)
+    {
+      RemoveTaxRef (orp);
+      orp->common = MemFree (orp->common);
+    }
+    MemFree (old_taxname);
     FindReplString (&(orp->common), fsp);
     FindReplStringList (orp->mod, fsp);
     FindReplStringList (orp->syn, fsp);
@@ -1208,9 +1261,13 @@ static void FindReplAligns (
       }
       break;
     case SAS_STD :
-      ssp = (StdSegPtr) sap->segs;
-      for (slp = ssp->loc; slp != NULL; slp = slp->next) {
-        VisitSeqIdsInSeqLoc (slp, userdata, FindReplSeqId);
+      for (ssp = (StdSegPtr) sap->segs; ssp != NULL; ssp = ssp->next) {
+        for (sip = ssp->ids; sip != NULL; sip = sip->next) {
+          FindReplSeqId (sip, userdata);
+        }
+        for (slp = ssp->loc; slp != NULL; slp = slp->next) {
+          VisitSeqIdsInSeqLoc (slp, userdata, FindReplSeqId);
+        }
       }
       break;
     case SAS_DISC :
@@ -1325,6 +1382,7 @@ static void FindReplFeats (
       FindReplString (&(grp->allele), fsp);
       FindReplString (&(grp->desc), fsp);
       FindReplString (&(grp->maploc), fsp);
+      FindReplString (&(grp->locus_tag), fsp);
       FindReplStringList (grp->syn, fsp);
       FindReplDbxrefs (grp->db, fsp);
       break;
@@ -1363,7 +1421,7 @@ static void FindReplFeats (
       break;
     case SEQFEAT_IMP :
       ifp = (ImpFeatPtr) sfp->data.value.ptrvalue;
-      FindReplString (&(ifp->key), fsp);
+      /* FindReplString (&(ifp->key), fsp); */
       FindReplString (&(ifp->loc), fsp);
       FindReplString (&(ifp->descr), fsp);
       break;
@@ -1692,3 +1750,56 @@ NLM_EXTERN void FindReplaceInEntity (
   }
 }
 
+/*=======================================================================*/
+/*                                                                       */
+/*  FindReplaceString() - find/replace just one string.                  */
+/*                                                                       */
+/*=======================================================================*/
+
+NLM_EXTERN void FindReplaceString (
+  CharPtr PNTR strp,
+  CharPtr find_string,
+  CharPtr replace_string,
+  Boolean case_counts,
+  Boolean whole_word
+)
+
+{
+  int         ch;
+  FindStruct  fs;
+  int         j;
+
+  if (strp == NULL || StringHasNoText (find_string)) return;
+
+  MemSet ((Pointer) &fs, 0, sizeof (FindStruct));
+
+  fs.entityID = 0;
+  fs.find_string = find_string;
+  fs.replace_string = replace_string;
+  fs.case_counts = case_counts;
+  fs.whole_word = whole_word;
+  fs.do_replace = TRUE;
+  fs.select_item = FALSE;
+  fs.send_update = FALSE;
+
+  fs.did_find = FALSE;
+  fs.did_replace = FALSE;
+  fs.dirty = FALSE;
+
+  /* build Boyer-Moore displacement array in advance */
+
+  fs.subLen = StringLen (find_string);
+
+  for (ch = 0; ch < 256; ch++) {
+    fs.d [ch] = fs.subLen;
+  }
+  for (j = 0; j < (int) (fs.subLen - 1); j++) {
+    ch = (int) (case_counts ? find_string [j] : TO_UPPER (find_string [j]));
+    if (ch >= 0 && ch <= 255) {
+      fs.d [ch] = fs.subLen - j - 1;
+    }
+  }
+
+  FindReplString (strp, &fs);
+}
+
