@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 9/94
 *
-* $Revision: 6.31 $
+* $Revision: 6.37 $
 *
 * File Description:  Manager for Bioseqs and BioseqSets
 *
@@ -40,6 +40,24 @@
 *
 *
 * $Log: objmgr.h,v $
+* Revision 6.37  2004/04/01 13:43:05  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.36  2002/07/30 14:41:45  kans
+* removed omdp->rearranged
+*
+* Revision 6.35  2002/07/29 21:30:17  kans
+* added rearranged flag to omdp
+*
+* Revision 6.34  2002/07/01 14:29:03  kans
+* changed totobj, currobj to Uint4
+*
+* Revision 6.33  2002/05/31 21:53:27  yaschenk
+* changing lookup by EntityID to array[][]
+*
+* Revision 6.32  2001/12/26 20:54:14  kans
+* added defines for OM_OPT_RECORD_SUPPRESSED and OM_OPT_RECORD_DEAD
+*
 * Revision 6.31  2001/11/19 15:26:19  kans
 * added ObjMgrDeleteAllInRecord, still need to bail in ObjMgrDelete if bulkIndexFree, then call from BioseqFree and BioseqSetFree
 *
@@ -357,7 +375,7 @@ typedef Int2 (LIBCALLBACK *OMMessageFunc) PROTO((OMMsgStructPtr message));
 
 	/**** defines for message function return values ******/
 
-#define OM_MSG_RET_ERROR   0	  /* an error occured handling this message */
+#define OM_MSG_RET_ERROR   0	  /* an error occurred handling this message */
 #define OM_MSG_RET_OK      1      /* message handled ok */
 #define OM_MSG_RET_DEL     2      /* due to message, remove this handler */
 #define OM_MSG_RET_DONE    3      /* handled, do not send message further */
@@ -625,9 +643,9 @@ typedef struct objmng {        /* functions for data management */
 
 	OMDataPtr ncbidata;    /* data needed by NCBI functions */
 	ObjMgrDataPtr PNTR datalist; /* sorted by pointer */
-	Uint2 totobj,                 /* number of elements in ncbidata */
-		 currobj,                /* number of elements in ncbidata occupied */
-		HighestEntityID;
+	Uint4 totobj,                 /* number of elements in ncbidata */
+		 currobj;                /* number of elements in ncbidata occupied */
+	Uint2 HighestEntityID;
 	ObjMgrDataPtr clipboard;
 	SelStructPtr sel;               /* for currently selected data item(s) */
 
@@ -642,6 +660,8 @@ typedef struct objmng {        /* functions for data management */
 	Boolean autoclean;           /* if TRUE, reap and free cache if currobj > maxobj */
 	Uint2 maxobj;                /* number of objects that tiggers autocleaning */
 
+	ObjMgrDataPtr PNTR PNTR	entityID_index; /**** two-level array index on Entity ID *****/
+
 } ObjMgr, PNTR ObjMgrPtr;
 
 #define DEFAULT_MAXTEMP 5   /* default number of temp entries to keep */
@@ -699,8 +719,11 @@ NLM_EXTERN Boolean LIBCALL ObjMgrUnlock PROTO((void));
 *      to that entity.
 *
 *****************************************************************************/
-#define OM_OPT_FREE_IF_NO_VIEW 1  /* free entity if no more UserData
-                                            structures are attached to it */
+#define OM_OPT_FREE_IF_NO_VIEW   1  /* free entity if no more UserData
+                                       structures are attached to it */
+#define OM_OPT_RECORD_SUPPRESSED 2  /* removed from distribution, still
+                                       available for retrieval */
+#define OM_OPT_RECORD_DEAD       4
 
 /*****************************************************************************
 *
@@ -1087,7 +1110,7 @@ NLM_EXTERN Boolean LIBCALL ObjMgrFreeClipBoard PROTO((void));
 *   	Frees all cached objects of type and subtypes of type
 *   		based on ObjMgrMatch()
 *   	if type == 0, frees all cached objects
-*   	returns TRUE if no errors occured
+*   	returns TRUE if no errors occurred
 *
 *       if ObjMgrSetHold() has been called but not cleared, nothing will
 *         be removed from cache, since it is held, but this function will
@@ -1305,6 +1328,9 @@ NLM_EXTERN void LIBCALL ObjMgrResetAll PROTO((void));
 NLM_EXTERN void LIBCALL ObjMgrReportFunc (CharPtr filename);
 
 
+NLM_EXTERN void LIBCALL ObjMgrAddIndexOnEntityID PROTO((ObjMgrPtr omp,Uint2 entityID,ObjMgrDataPtr omdp));
+NLM_EXTERN void LIBCALL ObjMgrDeleteIndexOnEntityID PROTO((ObjMgrPtr omp,Uint2 entityID));
+NLM_EXTERN ObjMgrDataPtr LIBCALL ObjMgrLookupIndexOnEntityID PROTO((ObjMgrPtr omp,Uint2 entityID));
 
 #ifdef __cplusplus
 }
