@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 7/13/91
 *
-* $Revision: 6.30 $
+* $Revision: 6.44 $
 *
 * File Description:  Ports onto Bioseqs
 *
@@ -40,6 +40,48 @@
 *
 *
 * $Log: seqport.h,v $
+* Revision 6.44  2004/07/16 19:37:37  kans
+* SeqPortStream and FastaStream functions return Int4, negative count if any fetch failures
+*
+* Revision 6.43  2004/05/12 18:55:33  kans
+* StreamCache takes SeqLocPtr as well as BioseqPtr optional arguments, slp version is equivalent of SeqPortNewByLoc
+*
+* Revision 6.42  2004/04/27 20:09:26  kans
+* StreamCacheGetResidue returns Uint1 because Char might be signed, preventing IS_residue from working
+*
+* Revision 6.41  2004/04/27 18:15:12  kans
+* added StreamCache functions that provide buffered request-driven access to sequence via SeqPortStream
+*
+* Revision 6.40  2004/04/14 12:39:01  kans
+* SeqPortStreamLoc is public function, SeqPortStreamRaw directly uncompresses byte store, avoids any SeqPort calls - still need more efficient way to reverse complement without a big buffer
+*
+* Revision 6.39  2004/04/08 20:19:21  kans
+* SeqPortStreamInt is external
+*
+* Revision 6.38  2004/03/15 19:54:54  kans
+* SeqPortStream takes expandable bit flags parameter
+*
+* Revision 6.37  2004/02/25 19:07:45  kans
+* ProteinFromCdRegionExEx and TransTableTranslateCdRegionEx return alternative start flag
+*
+* Revision 6.36  2003/11/18 17:08:46  kans
+* added MapNa4ByteTo4BitString, use in seqport read and get char
+*
+* Revision 6.35  2003/11/17 22:44:31  kans
+* added MapNa2ByteTo4BitString in preparation for faster SeqPortRead from 2na to 4na
+*
+* Revision 6.34  2003/11/05 21:17:22  bollin
+* added new option for Retranslate Coding Regions to handle stop codons at end of complete CDS during retranslate while ignoring stop codons
+*
+* Revision 6.33  2002/11/11 18:02:40  kans
+* added SeqPortStream to efficiently stream through a sequence
+*
+* Revision 6.32  2002/07/08 15:08:59  kans
+* made ReadCodingRegionBases extern
+*
+* Revision 6.31  2002/05/13 21:41:32  kans
+* added ConvertNsToGaps
+*
 * Revision 6.30  2001/02/18 20:58:52  kans
 * added GetSequenceByBsp
 *
@@ -314,6 +356,84 @@ NLM_EXTERN Uint1 GetGapCode PROTO((Uint1 seqcode));
 NLM_EXTERN Boolean LIBCALL SeqPortSetUpFields PROTO((SeqPortPtr spp, Int4 start, Int4 stop, Uint1 strand, Uint1 newcode));
 NLM_EXTERN Boolean LIBCALL SeqPortSetUpAlphabet PROTO((SeqPortPtr spp, Uint1 curr_code, Uint1 newcode));
 
+/*******************************************************************************
+*	
+*   SeqPortStream (bsp, flags, userdata, proc)
+*   SeqPortStreamInt (bsp, start, stop, strand, flags, userdata, proc)
+*   SeqPortStreamLoc (slp, flags, userdata, proc)
+*       Efficient functions to stream through sequence
+*
+********************************************************************************/
+
+typedef void (LIBCALLBACK *SeqPortStreamProc) (
+  CharPtr sequence,
+  Pointer userdata
+);
+
+typedef unsigned long StreamFlgType;
+
+#define STREAM_EXPAND_GAPS  1
+
+NLM_EXTERN Int4 SeqPortStream (
+  BioseqPtr bsp,
+  StreamFlgType flags,
+  Pointer userdata,
+  SeqPortStreamProc proc
+);
+
+NLM_EXTERN Int4 SeqPortStreamInt (
+  BioseqPtr bsp,
+  Int4 start,
+  Int4 stop,
+  Uint1 strand,
+  StreamFlgType flags,
+  Pointer userdata,
+  SeqPortStreamProc proc
+);
+
+NLM_EXTERN Int4 SeqPortStreamLoc (
+  SeqLocPtr slp,
+  StreamFlgType flags,
+  Pointer userdata,
+  SeqPortStreamProc proc
+);
+
+/*******************************************************************************
+*	
+*   StreamCacheSetup (bsp, slp, flags, scp)
+*   StreamCacheGetResidue (scp)
+*   StreamCacheSetPosition (scp, pos)
+*       SeqPort functional replacement implemented on top of SeqPortStreams
+*
+********************************************************************************/
+
+typedef struct streamcache {
+  BioseqPtr     bsp;
+  SeqLocPtr     slp;
+  Char          buf [4004];
+  Int2          ctr;
+  Int2          total;
+  Int4          offset;
+  Int4          length;
+  StreamFlgType flags;
+} StreamCache, PNTR StreamCachePtr;
+
+NLM_EXTERN Boolean StreamCacheSetup (
+  BioseqPtr bsp,
+  SeqLocPtr slp,
+  StreamFlgType flags,
+  StreamCache PNTR scp
+);
+
+NLM_EXTERN Uint1 StreamCacheGetResidue (
+  StreamCache PNTR scp
+);
+
+NLM_EXTERN Boolean StreamCacheSetPosition (
+  StreamCache PNTR scp,
+  Int4 pos
+);
+
 /*
 the following functions are for quick alphabet expansion, and require buffers
 allocated with 4-byte or 2-byte alignment, because they cast 2 or 4 bytes into
@@ -323,6 +443,9 @@ Uint2 or Uint4 for fast integer copying.
 NLM_EXTERN Uint4Ptr LIBCALL MapNa2ByteToIUPACString PROTO((Uint1Ptr bytep, Uint4Ptr buf, Int4 total));
 NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteToIUPACString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
 NLM_EXTERN Uint2Ptr LIBCALL MapNa2ByteToNa4String PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
+NLM_EXTERN Uint4Ptr LIBCALL MapNa2ByteTo4BitString PROTO((Uint1Ptr bytep, Uint4Ptr buf, Int4 total));
+NLM_EXTERN Uint2Ptr LIBCALL MapNa4ByteTo4BitString PROTO((Uint1Ptr bytep, Uint2Ptr buf, Int4 total));
+
 
 /*****************************************************************************
 *
@@ -399,6 +522,8 @@ NLM_EXTERN Uint4 BioseqHash PROTO((BioseqPtr bsp));
 *****************************************************************************/
 NLM_EXTERN ByteStorePtr ProteinFromCdRegion PROTO(( SeqFeatPtr sfp, Boolean include_stop));
 NLM_EXTERN ByteStorePtr ProteinFromCdRegionEx PROTO((SeqFeatPtr sfp, Boolean include_stop, Boolean remove_trailingX));
+NLM_EXTERN ByteStorePtr ProteinFromCdRegionExEx PROTO((SeqFeatPtr sfp, Boolean include_stop, Boolean remove_trailingX, BoolPtr altStartP));
+NLM_EXTERN ByteStorePtr ProteinFromCdRegionExWithTrailingCodonHandling PROTO((SeqFeatPtr sfp, Boolean include_stop, Boolean remove_trailingX, Boolean no_stop_at_end_of_complete_cds));
 
 /*****************************************************************************
 *
@@ -641,7 +766,17 @@ NLM_EXTERN ByteStorePtr TransTableTranslateCdRegion (
   TransTablePtr  PNTR tblptr,
   SeqFeatPtr cds,
   Boolean include_stop,
-  Boolean remove_trailingX
+  Boolean remove_trailingX,
+  Boolean no_stop_at_end_of_complete_cds
+);
+
+NLM_EXTERN ByteStorePtr TransTableTranslateCdRegionEx (
+  TransTablePtr  PNTR tblptr,
+  SeqFeatPtr cds,
+  Boolean include_stop,
+  Boolean remove_trailingX,
+  Boolean no_stop_at_end_of_complete_cds,
+  BoolPtr altStartP
 );
 
 NLM_EXTERN ByteStorePtr TransTableTranslateSeqLoc (
@@ -653,6 +788,15 @@ NLM_EXTERN ByteStorePtr TransTableTranslateSeqLoc (
   Boolean remove_trailingX
 );
 
+/* returns string of bases to translate */
+
+NLM_EXTERN CharPtr ReadCodingRegionBases (
+  SeqLocPtr location,
+  Int4 len,
+  Uint1 frame,
+  Int4Ptr totalP
+);
+
 /* allow reuse of translation tables by saving as AppProperty, avoids unnecessary initializations */
 
 NLM_EXTERN TransTablePtr PersistentTransTableByGenCode (
@@ -757,6 +901,13 @@ NLM_EXTERN BytePtr GetScoresbySeqId (
   Int4Ptr bsplength
 );
 
+/* ConvertNsToGaps assumes string of Ns means a gap of known length */
+
+NLM_EXTERN void ConvertNsToGaps (
+  BioseqPtr bsp,
+  Pointer userdata
+);
+
 /**************************************************************
 *
 *  Returns a protein molecular weight for a SeqLoc
