@@ -31,7 +31,7 @@
 *   
 * Version Creation Date: 11/3/93
 *
-* $Revision: 6.39 $
+* $Revision: 6.46 $
 *
 * File Description: Utilities for creating ASN.1 submissions
 *
@@ -42,6 +42,29 @@
 *
 *
 * $Log: subutil.h,v $
+* Revision 6.46  2003/10/21 18:16:05  bazhin
+* Replaced C++ comments with C ones.
+*
+* Revision 6.45  2003/10/16 17:16:33  mjohnson
+*
+* Added ORG_* and IS_ORG_* defines for origins. Use these constants
+* and macros instead of small integers.
+*
+* Revision 6.44  2003/07/11 18:22:45  kans
+* AddSourceToRefGeneTrackUserObject
+*
+* Revision 6.43  2002/07/09 16:17:35  kans
+* AddAccessionToTpaAssemblyUserObject takes from and to parameters
+*
+* Revision 6.42  2002/06/24 14:47:22  kans
+* added ORGMOD defines for gb_acronym, gb_anamorph, gb_synonym
+*
+* Revision 6.41  2002/06/04 14:40:30  kans
+* added AddPntToSeqFeat
+*
+* Revision 6.40  2002/03/27 19:26:31  kans
+* AddToGeneOntologyUserObject takes string for GO id to keep leading zeroes
+*
 * Revision 6.39  2001/12/12 16:01:47  kans
 * added MOLECULE_TYPE_TRANSCRIBED_RNA, SUBSRC_transgenic, SUBSRC_environmental_sample, and SUBSRC_isolation_source
 *
@@ -930,10 +953,33 @@ NLM_EXTERN Boolean AddSubSourceToEntry (
 #define ORGMOD_anamorph 29
 #define ORGMOD_teleomorph 30
 #define ORGMOD_breed 31
+#define ORGMOD_gb_acronym 32
+#define ORGMOD_gb_anamorph 33
+#define ORGMOD_gb_synonym 34
 #define ORGMOD_old_lineage 253
 #define ORGMOD_old_name 254
 #define ORGMOD_other 255 
 
+/* Defines for BioSrc.origin
+ */
+#define ORG_UNKNOWN 0
+#define ORG_NATURAL 1
+#define ORG_NATMUT 2
+#define ORG_MUT 3
+#define ORG_ARTIFICIAL 4
+#define ORG_SYNTHETIC 5
+#define ORG_OTHER 255
+#define ORG_DEFAULT ORG_UNKNOWN
+
+#define IS_ORG_UNKNOWN(S) ((S).origin == ORG_UNKNOWN)
+#define IS_ORG_NATURAL(S) ((S).origin == ORG_NATURAL)
+#define IS_ORG_NATMUT(S) ((S).origin == ORG_NATMUT)
+#define IS_ORG_MUT(S) ((S).origin == ORG_MUT)
+#define IS_ORG_ARTIFICIAL(S) ((S).origin == ORG_ARTIFICIAL)
+#define IS_ORG_SYNTHETIC(S) ((S).origin == ORG_SYNTHETIC)
+#define IS_ORG_OTHER(S) ((S).origin == ORG_OTHER)
+
+
 /*********************************************
 *  OrgMod defines subclasses of organism names
 *    (also see SubSource above for subclasses of source material)
@@ -1339,6 +1385,13 @@ NLM_EXTERN Boolean AddPointToFeature (
 	Boolean is_after_location ,
 	Boolean is_before_location );
 
+NLM_EXTERN Boolean AddPntToSeqFeat (
+	SeqFeatPtr sfp,
+	Int4 point,
+	BioseqPtr bsp,
+	Int2 fuzz,
+	Int2 strand);
+
 /*************************************************************************
 *
 *   Having made a generalized feature, now add type specific info to it.
@@ -1535,6 +1588,7 @@ NLM_EXTERN Boolean AddPhrapGraphToSeqLit (
 NLM_EXTERN UserObjectPtr CreateRefGeneTrackUserObject (void);
 NLM_EXTERN void AddStatusToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr status);
 NLM_EXTERN void AddCuratorToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr collaborator);
+NLM_EXTERN void AddSourceToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr genomicSource);
 NLM_EXTERN void AddAccessionToRefGeneTrackUserObject (UserObjectPtr uop, CharPtr field,
                                                       CharPtr accn, Int4 gi,
                                                       Boolean sequenceChange,
@@ -1562,7 +1616,7 @@ NLM_EXTERN void AddToGeneOntologyUserObject (
   UserObjectPtr uop,
   CharPtr type,
   CharPtr text,
-  Int4 goid,
+  CharPtr goid,
   Int4 pmid,
   CharPtr evidence
 );
@@ -1590,7 +1644,9 @@ NLM_EXTERN UserObjectPtr CreateTpaAssemblyUserObject (
 );
 NLM_EXTERN void AddAccessionToTpaAssemblyUserObject (
   UserObjectPtr uop,
-  CharPtr accn
+  CharPtr accn,
+  Int4 from,
+  Int4 to
 );
 
 
