@@ -171,7 +171,7 @@ static DatePtr GetPubDate (ValNodePtr pub)
 							if (cjp && cjp->imp) {
 								dp = cjp->imp->date;
 							}
-						} else if (cap->from == 2) {
+						} else if (cap->from == 2 || cap->from == 3) {
 							cbp = (CitBookPtr) (cap->fromptr);
 							if (cbp && cbp->imp) {
 								dp = cbp->imp->date;
@@ -188,6 +188,7 @@ static DatePtr GetPubDate (ValNodePtr pub)
 				case (PUB_Muid):
 				case (PUB_Man):
 				case (PUB_Proc):
+				case (PUB_PMid):
 				default:
 					break;
 			}
@@ -237,6 +238,7 @@ static DatePtr GetPubDate (ValNodePtr pub)
 			case (PUB_Muid):
 			case (PUB_Man):
 			case (PUB_Proc):
+			case (PUB_PMid):
 			default:
 				break;
 		}
@@ -583,10 +585,10 @@ static Int2 GetPubScore (ValNodePtr pub)
 Sirotkin's FlatRefBest.  Here 0 (not set) is given a score of 0, 1
 (gen) a score of 6, 2 (sub) a score of 11 etc. The higher the score
 the better the pub. */
-	static Int2 scores[12] = 
-	{0, 6, 11, 7, 1, 8, 4, 3, 5, 9, 2, 10};
+	static Int2 scores[14] = 
+	{0, 6, 11, 7, 1, 8, 4, 3, 5, 9, 2, 10, 12, 1};
 
-	if (pub == NULL)
+	if (pub == NULL || pub->choice > 13)
 		return 0;
 
 	return scores[pub->choice];
@@ -758,12 +760,64 @@ NLM_EXTERN void FreePubStruct(PubStructPtr psp)
 	return;
 }
 
+static Boolean GetPMFromPub(ValNodePtr v, Int4Ptr m, Int4Ptr p)
+{
+
+	*m = 0;
+	*p = 0;
+	switch (v->choice) {
+		case PUB_Muid:
+		  	*m = v->data.intvalue;
+		  	return TRUE;
+		case PUB_PMid:
+		  	*p = v->data.intvalue;
+		  	return TRUE;
+		case PUB_Equiv:
+			for (v = v->data.ptrvalue; v; v = v->next) {
+				  if (v->choice == PUB_Muid) { 
+					  *m = v->data.intvalue;
+				  }
+				  if (v->choice == PUB_PMid) { 
+					  *p = v->data.intvalue;
+				  }
+			}
+		  	break;
+	}
+	return (*m != 0 || *p != 0);
+}
+
+static Boolean CitGenTitlesMatch (ValNodePtr pub1, ValNodePtr pub2)
+
+{
+  CitGenPtr  cgp1, cgp2;
+
+  if (pub1->choice == PUB_Gen) {
+    cgp1 = (CitGenPtr) pub1->data.ptrvalue;
+    if (cgp1->serial_number != -1 && pub1->next != NULL) {
+      pub1 = pub1->next;
+    }
+  }
+  if (pub2->choice == PUB_Gen) {
+    cgp2 = (CitGenPtr) pub2->data.ptrvalue;
+    if (cgp2->serial_number != -1 && pub2->next != NULL) {
+      pub2 = pub2->next;
+    }
+  }
+
+  if (pub1->choice != PUB_Gen || pub2->choice != PUB_Gen) return TRUE;
+  cgp1 = (CitGenPtr) pub1->data.ptrvalue;
+  cgp2 = (CitGenPtr) pub2->data.ptrvalue;
+  if (cgp1->title == NULL || cgp2->title == NULL) return TRUE;
+  if (StringCmp (cgp1->title, cgp2->title) != 0) return FALSE;
+  return TRUE;
+}
+
 NLM_EXTERN Int2 PubLabelMatch (ValNodePtr pub1, ValNodePtr pub2)
 {
 	ValNodePtr m1, m2;
 	CitGenPtr g1, g2;
 	size_t len;
-	Int2 res;
+	Int4 am, bm, ap, bp;
 	
 	if ((m1 = MinimizePub(pub1)) == NULL) {
 		return -1;
@@ -772,30 +826,25 @@ NLM_EXTERN Int2 PubLabelMatch (ValNodePtr pub1, ValNodePtr pub2)
 		PubFree(m1);
 		return -1;
 	}
-	if (m1->choice == PUB_Muid) {
-		if (m2->choice != PUB_Muid) {
-			PubFree(m2);
-			ValNodeFree(m1);
+	if (GetPMFromPub(m1, &am, &ap) &&  GetPMFromPub(m2, &bm, &bp)) {
+		PubFree(m2);
+		PubFree(m1);
+		if (ap != 0 && ap == bp) {
+			return 0;
+		} else if (am != 0 && am == bm) {
+			return 0;
+		} else {
+			/* ERROR */
 			return -1;
 		}
-		if (m1->data.intvalue == m2->data.intvalue) {
-			res = 0;
-			if (m1->data.intvalue == 0) {
-				if (PubMatch(pub1, pub2) != 0) {
-					res = -1;
-				}
-			}
-			ValNodeFree(m1);
-			ValNodeFree(m2);
-			return res;
-		}
-		ValNodeFree(m1);
-		ValNodeFree(m2);
-		return -1;
-	} else if (m1->choice == PUB_Gen) {
+	}
+	if (m1->choice == PUB_Gen) {
 		if (m2->choice != PUB_Gen) {
 			PubFree(m1);
-			ValNodeFree(m2);
+			if (m2->choice == PUB_Equiv)
+				PubFree(m2);
+			else
+				ValNodeFree(m2);
 			return -1;
 		}
 		if ((g1 = m1->data.ptrvalue) == NULL) {
@@ -822,14 +871,16 @@ NLM_EXTERN Int2 PubLabelMatch (ValNodePtr pub1, ValNodePtr pub2)
 		  g1->cit [len - 1] = '\0';
 		}
 		len = MIN (StringLen (g1->cit), StringLen (g2->cit));
-		if (StringNCmp(g1->cit, g2->cit, len) == 0) {
-			PubFree(m1);
-			PubFree(m2);
-			return 0;
+		if (StringNICmp(g1->cit, g2->cit, len) == 0) {
+			if (CitGenTitlesMatch (pub1, pub2)) {
+				PubFree(m1);
+				PubFree(m2);
+				return 0;
+			}
 		}
 	}
-			PubFree(m1);
-			PubFree(m2);
+	PubFree(m1);
+	PubFree(m2);
 	return -1;
 }
 
@@ -840,10 +891,10 @@ static CharPtr  space_save(CharPtr str)
 	if (str == NULL) {
 		return NULL;
 	}
-	for (; isspace(*str) || *str == ','; str++);
+	for (; isspace(*str) || *str == ','; str++) continue;
 	for (s = str; *s != '\0'; s++) {
 		if (*s == '\n') {
-			for (ss = s+1; isspace(*ss); ss++);
+			for (ss = s+1; isspace(*ss); ss++) continue;
 			*s = ' ';
 			strcpy(s+1, ss);
 		}
@@ -862,7 +913,7 @@ static CharPtr  space_save(CharPtr str)
 
 NLM_EXTERN ValNodePtr MinimizePub(ValNodePtr pub)
 {
-	ValNodePtr  v, min_pub = NULL;
+	ValNodePtr  v, min_pub = NULL, epub, apub;
 	Char buffer[BUFLEN+1];
 	CitGenPtr	cit_gen, gen;
 	
@@ -872,26 +923,52 @@ NLM_EXTERN ValNodePtr MinimizePub(ValNodePtr pub)
 	if (pub->choice == PUB_Equiv) {
 		for (v = pub->data.ptrvalue; v != NULL; v= v->next) {
 			if (v->choice == PUB_Muid) {
-				min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
+				if (min_pub == NULL) {
+					min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
 											(AsnWriteFunc) PubAsnWrite);
-				return min_pub;
+				} else {
+					epub = ValNodeNew(NULL);
+					epub->choice = PUB_Equiv;
+					apub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
+											(AsnWriteFunc) PubAsnWrite);
+					min_pub->next = apub;
+					epub->data.ptrvalue = min_pub;
+					return epub;
+				}
+			}
+			if (v->choice == PUB_PMid) {
+				if (min_pub == NULL) {
+					min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
+											(AsnWriteFunc) PubAsnWrite);
+				} else {
+					epub = ValNodeNew(NULL);
+					epub->choice = PUB_Equiv;
+					apub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
+											(AsnWriteFunc) PubAsnWrite);
+					min_pub->next = apub;
+					epub->data.ptrvalue = min_pub;
+					return epub;
+				}
 			}
 		}
 		v = pub->data.ptrvalue;
 	} else {
 		v = pub;
 	}
-	if (v->choice == PUB_Muid) {
-		min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
-			(AsnWriteFunc) PubAsnWrite);
+	if (min_pub != NULL) {
 			return min_pub;
 	}
 	if (v->choice == PUB_Gen) {
 		gen = v->data.ptrvalue;
-		if (gen->serial_number != -1) {
+		if (gen->serial_number != -1 && v->next != NULL) {
 			v = v->next;
 		}		
 	}
+	if (v->choice == PUB_Muid || v->choice == PUB_PMid) {
+		min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
+											(AsnWriteFunc) PubAsnWrite);
+		return min_pub;
+	}
 	if ((PubLabelUnique(v, buffer, BUFLEN, OM_LABEL_CONTENT, TRUE)) == 0) {
 		min_pub = AsnIoMemCopy(v, (AsnReadFunc) PubAsnRead,
 											(AsnWriteFunc) PubAsnWrite);
@@ -917,7 +994,7 @@ NLM_EXTERN void FindCit (SeqEntryPtr sep, Pointer data, Int4 index, Int2 indent)
 	SeqAnnotPtr		sap, ap;
 	SeqFeatPtr		head, sfp;
 	ValNodePtr PNTR	vnpp, descr, tmp;
-	ImpFeatPtr		ifp;
+	ImpFeatPtr		/*UNUSED*/ifp;
 	SeqLocPtr		slp;
 	
 	vnpp = (ValNodePtr PNTR) data;
@@ -971,8 +1048,8 @@ NLM_EXTERN void ChangeCitQual (SeqEntryPtr sep, Pointer data, Int4 index, Int2 i
 	BioseqSetPtr	bssp;
 	SeqAnnotPtr		sap, ap;
 	SeqFeatPtr		sfp, head;
-	ValNodePtr PNTR	vnpp, descr;
-	ValNodePtr		vnp, pubset = NULL, pub_min, pub;
+	ValNodePtr PNTR	vnpp, /*UNUSED*/descr;
+	ValNodePtr		vnp, pubset = NULL, pub_min, /*UNUSED*/pub;
 	PubStructPtr	psp;
 	Int2			snum;
 	CharPtr			qval, s;
@@ -1176,6 +1253,14 @@ anyway by the line above.*/
 *	descrip, cit, and feat.
 *
 ***************************************************************************/
+/*______________________________________________________________________
+**
+**	This code is not currently used.
+**	I do not remove this piece of code, just comment it out.
+**	-- Dmitri Lukyanov
+*/
+#if 0
+
 static int LIBCALLBACK CompareStartForHeap (VoidPtr vp1, VoidPtr vp2)
 
 {
@@ -1210,10 +1295,14 @@ static int LIBCALLBACK CompareStartForHeap (VoidPtr vp1, VoidPtr vp2)
 
 }
 
+#endif
+/*______________________________________________________________________
+*/
+
 void VnpHeapSort (ValNodePtr PNTR vnp, int (LIBCALLBACK *compar )PROTO ((Nlm_VoidPtr, Nlm_VoidPtr )))	
 
 {
-	Int2 index, total;
+	Int4 index, total;
 	ValNodePtr vnp1;
 	ValNodePtr PNTR temp;
 
@@ -1474,12 +1563,36 @@ static void StripSerialNumber (SeqEntryPtr sep, Pointer data, Int4 index, Int2 i
 	} 	
 }
 
+static void CheckForSwissProtID (SeqEntryPtr sep, Pointer mydata, Int4 index, Int2 indent)
+
+{
+  BioseqPtr  bsp;
+  SeqIdPtr   sip;
+  BoolPtr    stripSerial;
+
+  if (sep == NULL) return;
+  if (IS_Bioseq (sep)) {
+    bsp = (BioseqPtr) sep->data.ptrvalue;
+    if (bsp == NULL) return;
+    stripSerial = (BoolPtr) mydata;
+    if (stripSerial == NULL) return;
+    for (sip = bsp->id; sip != NULL; sip = sip->next) {
+      if (sip->choice == SEQID_SWISSPROT) {
+        *stripSerial = FALSE;
+      }
+    }
+  }
+}
+
 NLM_EXTERN void EntryStripSerialNumber (SeqEntryPtr sep)
 {
-	if (sep == NULL)
-		return;
-	SeqEntryExplore(sep, NULL, StripSerialNumber);
-	return;
+	Boolean  stripSerial = TRUE;
+
+	if (sep == NULL) return;
+	SeqEntryExplore (sep, (Pointer) &stripSerial, CheckForSwissProtID);
+	if (stripSerial) {
+		SeqEntryExplore(sep, NULL, StripSerialNumber);
+	}
 }
 
 NLM_EXTERN ValNodePtr remove_node(ValNodePtr head, ValNodePtr x)
