@@ -1,4 +1,4 @@
-/* $Id: cddposutil.c,v 1.15 2001/12/31 13:47:06 bauer Exp $
+/* $Id: cddposutil.c,v 1.18 2004/03/31 17:58:51 papadopo Exp $
 *===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,7 +29,7 @@
 *
 * Initial Version Creation Date: 12/21/1999
 *
-* $Revision: 1.15 $
+* $Revision: 1.18 $
 *
 * File Description: CDD utilities involving position-specific scoring 
 *                   matrices (PSSMs)
@@ -37,6 +37,15 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: cddposutil.c,v $
+* Revision 1.18  2004/03/31 17:58:51  papadopo
+* Mike Gertz' changes for length adjustment calculations
+*
+* Revision 1.17  2004/01/22 15:43:57  bauer
+* call BlastCalculateEffectiveLengths in CddSetUpSearchInternalByLoc
+*
+* Revision 1.16  2003/05/29 13:17:56  thiessen
+* fix memory leak in CddposFreeMemory()
+*
 * Revision 1.15  2001/12/31 13:47:06  bauer
 * made block_width a local variable in CddSetUpSearchInternalByLoc to deal with changes in blast data structures
 *
@@ -271,6 +280,7 @@ void LIBCALL CddposFreeMemory(posSearchItems * posSearch) {
   MemFree(posSearch->posRowSigma);
   MemFree(posSearch->posInformation);
   MemFree(posSearch->posUseSequences);
+  MemFree(posSearch->posGaplessColumnWeights);
 }
 
 
@@ -1703,40 +1713,33 @@ static Int2 LIBCALL CddSetUpSearchInternalByLoc
 	*/
 	if (retval)
 		return retval;
+        if( 0 != BlastComputeLengthAdjustment(1/min_query_length,
+                                         search->sbp->
+                                         kbp[search->first_context]->logK,
+                                         1/search->sbp->
+                                         kbp[search->first_context]->H,
+                                         0.0, 
+                                         length,
+                                         search->dblen, search->dbseq_num,
+                                         &length_adjustment) ) {
+            ErrPostEx(SEV_WARNING, 0, 0,
+                      "BlastComputeLengthAdjustment failed!");
+        }
+        search->length_adjustment = length_adjustment;
+        search->dblen_eff =
+            search->dblen - search->dbseq_num*search->length_adjustment;
 
+        effective_query_length    = length - length_adjustment;
+        for (index=search->first_context; index<=search->last_context; index++)
+        {
+            search->context[index].query->effective_length =
+                effective_query_length;
+        }
 
-	if (search->pbp->gapped_calculation &&
-		StringCmp(search->prog_name, "blastn") != 0)
-		min_query_length = 1/(search->sbp->kbp_gap_std[search->first_context]->K);
-	else
-		min_query_length = 1/(search->sbp->kbp[search->first_context]->K);
-
-	last_length_adjustment = 0;
-	for (index=0; index<5; index++)
-	{
-		length_adjustment = ((search->sbp->kbp[search->first_context]->logK)+log((Nlm_FloatHi)(length-last_length_adjustment)*(Nlm_FloatHi)(MAX(1, (search->dblen)-(search->dbseq_num*last_length_adjustment)))))/(search->sbp->kbp[search->first_context]->H);
-		if (length_adjustment >= length-min_query_length)
-		{
-			length_adjustment = length-min_query_length;
-			break;
-		}
-	
-		if (ABS(last_length_adjustment-length_adjustment) <= 1)
-			break;
-		last_length_adjustment = length_adjustment;
-	}
-	search->length_adjustment = MAX(length_adjustment, 0);
-
-	search->dblen_eff = MAX(1, search->dblen - search->dbseq_num*search->length_adjustment);
-	effective_query_length = MAX(length - search->length_adjustment, min_query_length);
-	
-	for (index=search->first_context; index<=search->last_context; index++)
-	{
-		search->context[index].query->effective_length = effective_query_length;
-	}
-
-	if (search->searchsp_eff == 0)
-		search->searchsp_eff = ((Nlm_FloatHi) search->dblen_eff)*((Nlm_FloatHi) effective_query_length);
+        if (search->searchsp_eff == 0)
+            search->searchsp_eff =
+                ((Nlm_FloatHi) search->dblen_eff)*
+                ((Nlm_FloatHi) effective_query_length);
 
 	/* The default is that cutoff_s was not set and is zero. */
 	if (options->cutoff_s == 0)
