@@ -23,7 +23,7 @@
  *
  * ===========================================================================
  *
- * RCS $Id: accentr.c,v 6.2 1998/02/23 17:59:38 kans Exp $
+ * RCS $Id: accentr.c,v 6.9 2001/09/04 23:28:55 juran Exp $
  *
  * Author:  Ostell
  *
@@ -47,31 +47,52 @@
  * 05-19-95 Schuler     Added rcs Log directive for automatic insertion of
  *                      modification comments.
  *
- * Revision $Log: accentr.c,v $
- * Revision Revision 6.2  1998/02/23 17:59:38  kans
- * Revision mesh and taxonomy hierarchy modes both call NetEntHierarchyGet(term,db,fld)
- * Revision
- * Revision Revision 6.1  1997/09/19 13:16:19  kans
- * Revision removed or ifdef unused variables
- * Revision
- * Revision Revision 6.0  1997/08/25 18:12:26  madden
- * Revision Revision changed to 6.0
- * Revision
- * Revision Revision 5.38  1997/07/28 13:30:35  ostell
- * Revision Moved GetUniGeneIDForSeqId() to seqmgr.c
- * Revision
- * Revision Revision 5.37  1997/07/09 19:25:57  kuzio
- * Revision kludge for corn; and tidied NCBICG
- * Revision
- * Revision Revision 5.36  1997/06/26 21:55:10  vakatov
- * Revision [PC] DLL'd "ncbicdr.lib", "ncbiacc.lib", "ncbinacc.lib" and "ncbicacc.lib"
- * Revision
- * Revision Revision 5.35  1997/05/22 20:35:44  epstein
- * Revision add error strings when fetching Medline and Sequence entries, to identify which UID encountered the problem
- * Revision
- * Revision Revision 5.34  1997/04/18 16:01:08  shavirin
- * Revision Removed annoying compiler messages "Statement not reached"
- * Revision
+ * $Log: accentr.c,v $
+ * Revision 6.9  2001/09/04 23:28:55  juran
+ * Always assume network access in Mac OS.
+ *
+ * Revision 6.8  2001/08/16 19:40:33  kans
+ * reverted to not assume network - UNIX still does cdrom access compile - and cleaned up junk in log instructions
+ *
+ * Revision 6.7  2001/08/15 20:57:07  juran
+ * Always assume network access.
+ *
+ * Revision 6.6  2000/01/12 20:17:12  vakatov
+ * Get rid of the LIBCALL specifier at EntrezSeqEntryGet()
+ *
+ * Revision 6.5  1999/01/06 14:18:35  grisha
+ * add defines to switch ID0/ID1 usage
+ *
+ * Revision 6.4  1998/12/08 20:38:47  kans
+ * EntrezGIForSeqIdFunc aborts on local, other, general, gi, notset before connecting to Entrez network server
+ *
+ * Revision 6.3  1998/06/12 19:19:06  kans
+ * fixed unix compiler warnings
+ *
+ * Revision 6.2  1998/02/23 17:59:38  kans
+ * mesh and taxonomy hierarchy modes both call NetEntHierarchyGet(term,db,fld)
+ *
+ * Revision 6.1  1997/09/19 13:16:19  kans
+ * removed or ifdef unused variables
+ *
+ * Revision 6.0  1997/08/25 18:12:26  madden
+ * changed to 6.0
+ *
+ * Revision 5.38  1997/07/28 13:30:35  ostell
+ * Moved GetUniGeneIDForSeqId() to seqmgr.c
+ *
+ * Revision 5.37  1997/07/09 19:25:57  kuzio
+ * kludge for corn; and tidied NCBICG
+ *
+ * Revision 5.36  1997/06/26 21:55:10  vakatov
+ * [PC] DLL'd "ncbicdr.lib", "ncbiacc.lib", "ncbinacc.lib" and "ncbicacc.lib"
+ *
+ * Revision 5.35  1997/05/22 20:35:44  epstein
+ * add error strings when fetching Medline and Sequence entries, to identify which UID encountered the problem
+ *
+ * Revision 5.34  1997/04/18 16:01:08  shavirin
+ * Removed annoying compiler messages "Statement not reached"
+ *
  * Revision 5.33  1997/03/17  16:26:23  brandon
  * added PMEntrezDetailedInfo
  *
@@ -268,27 +289,37 @@
  * ==========================================================================
  */
 
-#define REVISION_STR "$Revision: 6.2 $"
+#define REVISION_STR "$Revision: 6.9 $"
 
 #include <accentr.h>
 #include <seqmgr.h>
 #include <sequtil.h>
 
 #include <cdconfig.h>
+/* Must not assume network - unused cdromlib stuff is still tangled in */
+/* Okay to assume network in Mac OS. */
+#ifdef __MACOS__
+#define _NETENT_
+#endif
 #ifdef _NETENT_
 #include <netentr.h>      /* support network access */
 #else
 #ifdef _MBENTREZ_
 #include <mbentrez.h>
 #define USE_ID0 1
+#define USE_IDxARCH 1
 #endif
 #ifdef _PMENTREZ_
 #include <pmentrez.h>
-#define USE_ID0 1
+#define USE_ID1 1
+#define USE_IDxARCH 1
 #endif
 #ifdef USE_ID0
 #include <id0arch.h>
 #endif
+#ifdef USE_ID1
+#include <id1arch.h>
+#endif
 #ifndef _CDENTREZ_
 #include <cdentrez.h>      /* support cdrom access */
 #endif
@@ -304,6 +335,21 @@
 
 #define HIERARCHY_CACHE_SIZE 200
 
+#ifdef USE_IDxARCH
+#ifdef USE_ID0
+#define IDxArchInit ID0ArchInit
+#define IDxArchFini ID0ArchFini
+#define IDxArchSeqEntryGet ID0ArchSeqEntryGet
+#define IDxArchGIGet ID0ArchGIGet
+#endif
+#ifdef USE_ID1
+#define IDxArchInit ID1ArchInit
+#define IDxArchFini ID1ArchFini
+#define IDxArchSeqEntryGet ID1ArchSeqEntryGet
+#define IDxArchGIGet ID1ArchGIGet
+#endif
+#endif /* USE_IDxARCH */
+
 #ifdef _NETENTREZ_
 Uint1 AdaptiveDocSumMatrix[MAX_CHUNK_IN_ADAPTIVE_MATRIX+1][MAX_SECS_IN_ADAPTIVE_MATRIX+1] = {
   8,  6,  3,  3,
@@ -333,7 +379,7 @@ static CharPtr        CombinedInfo = NULL;
 static DocType        lastTermType;
 static Int2           lastBooleanMediaType;
 static Int2           entrezInitCount = 0;
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
 static Boolean        idConnected = FALSE;
 static Boolean        idCantConnect= FALSE;
 static void DisconnID(void);
@@ -490,7 +536,7 @@ NLM_EXTERN void LIBCALL EntrezFini (void)
     if (--entrezInitCount > 0)
         return;
 
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
     DisconnID ();
 #endif
 #ifdef _CDENTREZ_
@@ -624,7 +670,7 @@ NLM_EXTERN Int4 LIBCALL EntrezGetMaxLinks (void)
 
 {
     Int4 maxlinks = INT4_MAX;
-    Int4 links;
+    Int4 links = 0;
     Boolean cdSelected = FALSE;
 
     if (! DoSelectDataSource(ENTR_LINKS_CHAN, "INFO", NULL))
@@ -1177,13 +1223,13 @@ NLM_EXTERN ByteStorePtr LIBCALL EntrezTLEvalX (ValNodePtr elst)
 }
 
 
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
 static Boolean
 ConnectID(void)
 {
     if (!idConnected && !idCantConnect)
     {
-        if (ID0ArchInit())
+        if (IDxArchInit())
         {
             idConnected = TRUE;
         } else {
@@ -1198,11 +1244,11 @@ static void
 DisconnID(void)
 {
     if (idConnected)
-        ID0ArchFini();
+        IDxArchFini();
     idConnected = FALSE;
     idCantConnect = FALSE;
 }
-#endif /* USE_ID0 */
+#endif /* USE_IDxARCH */
 
 /*****************************************************************************
 *
@@ -1237,7 +1283,7 @@ NLM_EXTERN DocSumPtr LIBCALL EntrezDocSum (DocType type, DocUid uid)
 #endif
             if ((dsp = CdDocSum (type, uid)) != NULL)
                 return dsp;
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
             if (type != TYP_ST && type != TYP_CH && ConnectID())
             {
                 SeqEntryPtr sep;
@@ -1245,7 +1291,7 @@ NLM_EXTERN DocSumPtr LIBCALL EntrezDocSum (DocType type, DocUid uid)
                 AsnIoPtr aip;
                 DocSumPtr dsp = NULL;
 
-                if ((sep = ID0ArchSeqEntryGet(uid, NULL, 0)) != NULL)
+                if ((sep = IDxArchSeqEntryGet(uid, NULL, 0)) != NULL)
                 {
                     TmpNam(fname);
                     aip = AsnIoOpen(fname, "w");
@@ -1259,7 +1305,7 @@ NLM_EXTERN DocSumPtr LIBCALL EntrezDocSum (DocType type, DocUid uid)
                     return dsp;
                 }
             }
-#endif /* USE_ID0 */
+#endif /* USE_IDxARCH */
             break;
 #endif
 #ifdef _NETENTREZ_
@@ -1528,7 +1574,6 @@ NLM_EXTERN Int2 LIBCALL EntrezTermListByTerm (DocType type, DocField field, Char
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetTermListByTerm(type, field, term, numterms, proc, first_page);
-        break;
 #endif
     default:
         return 0;
@@ -1693,11 +1738,11 @@ NLM_EXTERN Int2 LIBCALL EntrezLinkUidList (LinkSetPtr PNTR result, DocType type,
     Int2 obtained_ids = 0;
     Int4Ptr local_uids;
     Int4Ptr best_obtained_uids;
-    Int2 best_obtained_ids;
+    Int2 best_obtained_ids = 0;
     LinkSetPtr best_lsp = NULL;
     LinkSetPtr lsp;
     Int2 i;
-    Int2 missing;
+    Int2 missing = 0;
     Int2 fewest_missing = numuid + 1;
     Boolean badErr = FALSE;
 
@@ -2238,7 +2283,7 @@ NLM_EXTERN Int2 LIBCALL EntrezSeqEntryListGet (SeqEntryPtr PNTR result, Int2 num
             else
                 obtained_ids += CdEntSeqEntryListGet (res, numuid, local_uids,
                                                       retcode, TRUE);
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
             if (retcode != -1 && obtained_ids < numuid && ConnectID())
             {
                 Int2 i;
@@ -2251,7 +2296,7 @@ NLM_EXTERN Int2 LIBCALL EntrezSeqEntryListGet (SeqEntryPtr PNTR result, Int2 num
                 {
                     if (res[i] == NULL)
                     {
-                        if ((res[i] = ID0ArchSeqEntryGet(ABS(local_uids[i]),
+                        if ((res[i] = IDxArchSeqEntryGet(ABS(local_uids[i]),
 							 NULL, retcode))
                              != NULL)
                         {
@@ -2261,7 +2306,7 @@ NLM_EXTERN Int2 LIBCALL EntrezSeqEntryListGet (SeqEntryPtr PNTR result, Int2 num
                     }
                 }
             }
-#endif /* USE_ID0 */
+#endif /* USE_IDxARCH */
         break;
 #endif
 #ifdef _NETENTREZ_
@@ -2322,7 +2367,7 @@ NLM_EXTERN Int2 LIBCALL EntrezSeqEntryListGet (SeqEntryPtr PNTR result, Int2 num
 *       get one SeqEntry
 *
 *****************************************************************************/
-NLM_EXTERN SeqEntryPtr LIBCALL EntrezSeqEntryGet (Int4 uid, Int2 retcode)
+NLM_EXTERN SeqEntryPtr EntrezSeqEntryGet (Int4 uid, Int2 retcode)
 
 {
     SeqEntryPtr sep = NULL;
@@ -2378,7 +2423,7 @@ NLM_EXTERN Int4 LIBCALL EntrezFindSeqId (SeqIdPtr sip)
     case MEDIUM_CD:
     case MEDIUM_DISK:
         uid = CdEntrezFindSeqId(sip);
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
         if (uid == 0 && ConnectID())
         {
  	    TextSeqIdPtr tsip;
@@ -2421,7 +2466,7 @@ NLM_EXTERN Int4 LIBCALL EntrezFindSeqId (SeqIdPtr sip)
             default:
                 break;
             }
-            uid = ID0ArchGIGet(sip);
+            uid = IDxArchGIGet(sip);
         }
 #endif    
         return uid;
@@ -2646,13 +2691,13 @@ NLM_EXTERN Int2 LIBCALL EntrezSeqSumListGet (DocSumPtr PNTR result, Int2 numuid,
 
         for (unmatched = 0, i = 0; i < numuid; i++)
         {
-#ifdef USE_ID0
+#ifdef USE_IDxARCH
             if (res[i] == NULL) /* missing this one, try again */
 	    {
                 if ((res[i] = EntrezDocSum (type, local_uids [i])) != NULL)
                     obtained_ids++;
 	    }
-#endif /* USE_ID0 */
+#endif /* USE_IDxARCH */
             if (res[i] != NULL) /* found this one */
             {
                 if (first_time)
@@ -2716,90 +2761,6 @@ FreeHierarchyList(void)
     hierHead = ValNodeFree(hierHead);
 }
 
-/* get the parent of the specified term in the hierarchy */
-static CharPtr
-GetHierarchyParent(CharPtr term, DocType db, DocField fld)
-{
-    ValNodePtr v;
-    ValNodePtr prev;
-    ValNodePtr prevprev;
-    CachedParentPtr q;
-    ValNodePtr elst;
-    ValNodePtr node;
-    LinkSetPtr lsp;
-    Int2 i;
-    CharPtr p;
-    Int2 count;
-    CharPtr retval;
-
-    for (v = hierHead, count = 0, prev = NULL, prevprev = NULL; v != NULL;
-         prevprev = prev, prev = v, v = v->next, count++)
-    {
-        q = (CachedParentPtr) v->data.ptrvalue;
-        if (q != NULL && q->db == db && q->fld == fld &&
-            StrCmp(q->term, term) == 0)
-        {
-            /* impose LRU queuing discipline before returning */
-            if (prev != NULL)
-            {
-                prev->next = v->next;
-                v->next = hierHead;
-                hierHead = v;
-            }
-            return q->parent;
-        }
-    }
-
-    if ((elst = EntrezTLNew(db)) == NULL)
-        return NULL;
-    EntrezTLAddTerm(elst, term, db, fld, FALSE);
-    lsp = EntrezTLEval(elst);
-    EntrezTLFree(elst);
-    if (lsp == NULL)
-        return NULL;
-    if ((p = (CharPtr) lsp->uids) == NULL || lsp->num <= 0)
-    {
-        LinkSetFree(lsp);
-        return NULL;
-    }
-
-    /* perform byte-swapping as necessary to handle portability between */
-    /* different platforms; the data is in big-endian format */
-    for (i = 0; i < lsp->num; i++)
-    {
-        lsp->uids[i] = SwapUint4(lsp->uids[i]);
-    }
-
-
-    /* insert as first element in list */
-    node = ValNodeNew(NULL);
-    node->next = hierHead;
-    hierHead = node;
-
-    q = (CachedParentPtr) MemNew(sizeof(*q));
-    node->data.ptrvalue = (VoidPtr) q;
-    q->parent = *p == NULLB ? NULL : StringSave(p);
-    LinkSetFree(lsp);
-    q->term = StringSave(term);
-    q->db = db;
-    q->fld = fld;
-
-    retval = q->parent;
-
-    if (count >= HIERARCHY_CACHE_SIZE && prevprev != NULL && prev != NULL)
-    { /* cache too full, remove the last entry, which is the least recently used  */
-        q = (CachedParentPtr) prev->data.ptrvalue;
-        MemFree(q->term);
-        MemFree(q->parent);
-        MemFree(q);
-        MemFree(prev);
-        prevprev->next = NULL;
-    }
-
-    return retval;
-}
-
-
 static EntrezHierarchyPtr DoEntrezHierarchyGet(CharPtr term, DocType db, DocField fld)
 {
   switch (GetCurMediaType())
@@ -2864,11 +2825,9 @@ NLM_EXTERN Boolean LIBCALL EntrezCanNeighborText (void)
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntCanNeighborText();
-        break;
 #endif
     default:
         return FALSE;
-        break;
     }
 }
 
@@ -2892,11 +2851,9 @@ NLM_EXTERN LinkSetPtr LIBCALL EntrezDoNeighborText (EntrezNeighborTextPtr entp)
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntDoNeighborText(entp);
-        break;
 #endif
     default:
         return NULL;
-        break;
     }
 }
 
@@ -2955,11 +2912,9 @@ NLM_EXTERN Boolean LIBCALL EntrezExpandedMedlineFeatures (void)
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntExpandedMedlineFeatures();
-        break;
 #endif
     default:
         return FALSE;
-        break;
     }
 }
 
@@ -3283,6 +3238,24 @@ static Int2 LIBCALLBACK EntrezGIForSeqIdFunc (Pointer data)
 		  Boolean result;
 
         ompcp = (OMProcControlPtr)data;
+		if (ompcp == NULL)
+                return OM_MSG_RET_ERROR;
+
+        sip = (SeqIdPtr)(ompcp->input_data);
+        if (sip == NULL)
+            return OM_MSG_RET_ERROR;
+
+		switch (sip->choice) {
+			case SEQID_NOT_SET:
+			case SEQID_LOCAL:
+			case SEQID_OTHER:
+			case SEQID_GENERAL:
+			case SEQID_GI:
+				return OM_MSG_RET_ERROR;
+			default:
+				break;
+		}
+
         ompp = ompcp->proc;
         efsp = (EntrezFetchStructPtr)(ompp->procdata);
 
@@ -3301,13 +3274,6 @@ static Int2 LIBCALLBACK EntrezGIForSeqIdFunc (Pointer data)
         if (efsp->EntrezBioseqFetchState != EBFS_READY)
                 return OM_MSG_RET_ERROR;
 
-        sip = (SeqIdPtr)(ompcp->input_data);
-        if (sip == NULL)
-                return OM_MSG_RET_ERROR;
-
-		  if (sip->choice == SEQID_GI)
-					 return OM_MSG_RET_ERROR;
-
 		  gi = EntrezFindSeqId(sip);
         if (! gi) return OM_MSG_RET_OK;
 
@@ -3338,11 +3304,9 @@ NLM_EXTERN Boolean LIBCALL EntrezCanBlast(void)
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntCanBlast();
-        break;
 #endif
     default:
         return FALSE;
-        break;
     }
 }
 
@@ -3362,7 +3326,6 @@ NLM_EXTERN LinkSetPtr LIBCALL EntrezBlastBioseq(BioseqPtr bsp, DocType db, CharP
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntBlastBioseq(bsp, db, program, database, options, usemonitor);
-        break;
 #endif
     default:
         return NULL;
@@ -3390,11 +3353,9 @@ NLM_EXTERN Int4 LIBCALL EntrezClusterAnalysis(DocUidPtr uids, Int4 numuids, DocF
 #ifdef _NETENTREZ_
     case MEDIUM_NETWORK:
         return NetEntClusterAnalysis(uids, numuids, fld, minCluster, maxCluster, maxTerms, terms, termTotals);
-        break;
 #endif
     default:
         return -1;
-        break;
     }
 }
 
