@@ -1,7 +1,7 @@
-#ifndef NCBI_ANSI_EXT__H
-#define NCBI_ANSI_EXT__H
+#ifndef CONNECT___NCBI_ANSI_EXT__H
+#define CONNECT___NCBI_ANSI_EXT__H
 
-/*  $Id: ncbi_ansi_ext.h,v 6.8 2000/11/07 21:19:35 vakatov Exp $
+/*  $Id: ncbi_ansi_ext.h,v 6.12 2004/03/12 23:25:37 gorelenk Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,10 +29,109 @@
  * Author:  Anton Lavrentiev
  *
  * File Description:
- *    Non-ANSI, yet widely used functions
+ *   Non-ANSI, yet widely used functions
  *
+ */
+#include <connect/connect_export.h>
+#include "ncbi_config.h"
+#include <stddef.h>
+#include <string.h>
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
+#ifndef HAVE_STRDUP
+
+#  ifdef strdup
+#    undef strdup
+#  endif
+#  define strdup      NCBI_strdup
+
+/* Create a copy of string "str".
+ * Return an identical malloc'ed string, which must be explicitly freed 
+ * by free() when no longer needed.
+ */
+extern char* strdup(const char* str);
+
+#endif /*HAVE_STRDUP*/
+
+
+#ifndef HAVE_STRCASECMP
+
+#  ifdef strcasecmp
+#    undef strcasecmp
+#    undef strncasecmp
+#  endif
+#  define strcasecmp  NCBI_strcasecmp
+#  define strncasecmp NCBI_strncasecmp
+
+/* Compare "s1" and "s2", ignoring case.
+ * Return less than, equal to or greater than zero if
+ * "s1" is lexicographically less than, equal to or greater than "s2".
+ */
+extern NCBI_XCONNECT_EXPORT int strcasecmp(const char* s1, const char* s2);
+
+/* Compare not more than "n" characters of "s1" and "s2", ignoring case.
+ * Return less than, equal to or greater than zero if
+ * "s1" is lexicographically less than, equal to or greater than "s2".
+ */
+extern NCBI_XCONNECT_EXPORT int strncasecmp(const char* s1, const char* s2, size_t n);
+
+#endif/*HAVE_STRCASECMP*/
+
+
+#ifdef strupr
+#  undef strupr
+#  undef strlwr
+#endif
+#define strupr NCBI_strupr
+#define strlwr NCBI_strlwr
+
+/* Convert a string to uppercase, then return pointer to
+ * the altered string. Because the conversion is made in place, the
+ * returned pointer is the same as the passed one.
+ */
+extern NCBI_XCONNECT_EXPORT char* strupr(char* s);
+
+/* Convert a string to lowercase, then return pointer to
+ * the altered string. Because the conversion is made in place, the
+ * returned pointer is the same as the passed one.
+ */
+extern NCBI_XCONNECT_EXPORT char* strlwr(char* s);
+
+
+/* Copy not more than "n" characters from string "s2" into "s1"
+ * and return the result, which is always null-terminated.
+ * NOTE: The difference of this function from standard strncpy() is in
+ * that the result is always null-terminated and that the function
+ * does not pad "s1" with null bytes should "s2" be shorter than "n".
+ */
+extern NCBI_XCONNECT_EXPORT char* strncpy0(char* s1, const char* s2, size_t n);
+
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
+
+/*
  * --------------------------------------------------------------------------
  * $Log: ncbi_ansi_ext.h,v $
+ * Revision 6.12  2004/03/12 23:25:37  gorelenk
+ * Added export prefixes.
+ *
+ * Revision 6.11  2002/10/28 15:41:25  lavr
+ * Header made private (moved from include/connect to here)
+ *
+ * Revision 6.10  2002/09/24 15:01:17  lavr
+ * File description indented uniformly
+ *
+ * Revision 6.9  2002/09/19 17:59:40  lavr
+ * Header file guard macro changed; log moved to the end
+ *
  * Revision 6.8  2000/11/07 21:19:35  vakatov
  * Compilation warning fixed;  plus, some code beautification...
  *
@@ -61,68 +160,4 @@
  * ==========================================================================
  */
 
-#include <stddef.h>
-#include <string.h>
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
-#ifdef strdup
-#  undef strdup
-#endif
-#define strdup      NCBI_strdup
-
-/* Create a copy of string "str".
- * Return an identical malloc'ed string, which must be explicitly freed 
- * by free() when no longer needed.
- */
-extern char* strdup(const char* str);
-
-
-#ifdef strcasecmp
-#  undef strcasecmp
-#  undef strncasecmp
-#endif
-#define strcasecmp  NCBI_strcasecmp
-#define strncasecmp NCBI_strncasecmp
-
-/* Compare "s1" and "s2", ignoring case.
- * Return less than, equal to or greater than zero if
- * "s1" is lexicographically less than, equal to or greater than "s2".
- */
-extern int strcasecmp(const char* s1, const char* s2);
-
-/* Compare not more than "n" characters of "s1" and "s2", ignoring case.
- * Return less than, equal to or greater than zero if
- * "s1" is lexicographically less than, equal to or greater than "s2".
- */
-extern int strncasecmp(const char* s1, const char* s2, size_t n);
-
-
-#ifdef strupr
-#  undef strupr
-#  undef strlwr
-#endif
-#define strupr NCBI_strupr
-#define strlwr NCBI_strlwr
-
-/* Convert a string to uppercase, then return pointer to
- * the altered string. Because the conversion is made in place, the
- * returned pointer is the same as the passed one.
- */
-extern char* strupr(char* s);
-
-/* Convert a string to lowercase, then return pointer to
- * the altered string. Because the conversion is made in place, the
- * returned pointer is the same as the passed one.
- */
-extern char* strlwr(char* s);
-
-
-#ifdef __cplusplus
-} /* extern "C" */
-#endif
-
-#endif /* NCBI_ANSI_EXT__H */
+#endif /* CONNECT___NCBI_ANSI_EXT__H */
