@@ -1,7 +1,7 @@
-#ifndef NCBI_CORE__H
-#define NCBI_CORE__H
+#ifndef CONNECT___NCBI_CORE__H
+#define CONNECT___NCBI_CORE__H
 
-/*  $Id: ncbi_core.h,v 6.13 2001/09/27 22:29:35 vakatov Exp $
+/*  $Id: ncbi_core.h,v 6.25 2004/03/12 23:26:22 gorelenk Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
  *********************************
  * I/O status and direction:
  *    enum:       EIO_ReadMethod
+ *    enum:       EIO_WriteMethod
  *    enum:       EIO_Status,  verbal: IO_StatusStr()
  *    enum:       EIO_Event
  *
@@ -59,58 +60,16 @@
  *    methods:    REG_Create(),  REG_Reset(),  REG_AddRef(),  REG_Delete(),
  *                REG_Get(),  REG_Set()
  *
- *
- * ---------------------------------------------------------------------------
- * $Log: ncbi_core.h,v $
- * Revision 6.13  2001/09/27 22:29:35  vakatov
- * Always define "NDEBUG" if "_DEBUG" is not defined (mostly for the C Toolkit)
- *
- * Revision 6.12  2001/08/09 16:22:51  lavr
- * Remove last (unneeded) parameter from LOG_Reset()
- *
- * Revision 6.11  2001/06/19 20:16:18  lavr
- * Added #include <connect/ncbi_types.h>
- *
- * Revision 6.10  2001/06/19 19:08:28  lavr
- * STimeout and ESwitch moved to <connect/ncbi_types.h>
- *
- * Revision 6.9  2001/05/17 18:10:22  vakatov
- * Moved the logging macros from <ncbi_core.h> to <ncbi_util.h>.
- * Logging::  always call the logger if severity is eLOG_Fatal.
- *
- * Revision 6.8  2001/03/02 20:06:54  lavr
- * Typos fixed
- *
- * Revision 6.7  2001/01/23 23:07:30  lavr
- * A make-up change
- *
- * Revision 6.6  2001/01/11 16:41:18  lavr
- * Registry Get/Set methods got the 'user_data' argument, forgotten earlier
- *
- * Revision 6.5  2000/10/18 20:29:41  vakatov
- * REG_Get::  pass in the default value (rather than '\0')
- *
- * Revision 6.4  2000/06/23 19:34:41  vakatov
- * Added means to log binary data
- *
- * Revision 6.3  2000/04/07 19:55:14  vakatov
- * Standard indentation
- *
- * Revision 6.2  2000/03/24 23:12:03  vakatov
- * Starting the development quasi-branch to implement CONN API.
- * All development is performed in the NCBI C++ tree only, while
- * the NCBI C tree still contains "frozen" (see the last revision) code.
- *
- * Revision 6.1  2000/02/23 22:30:40  vakatov
- * Initial revision
- *
- * ===========================================================================
  */
 
 #include <connect/ncbi_types.h>
 
 
 /* Run-time debugging */
+#if defined(verify)
+#undef verify
+#endif
+
 #if !defined(NDEBUG)  &&  !defined(_DEBUG)
 #  define NDEBUG
 #endif
@@ -118,10 +77,21 @@
 #if defined(NDEBUG)
 #  define verify(expr)  (void)(expr)
 #else
+/* The following 2 headers are actually only required for Codewarrior
+ * on Mac to prototype printf() and abort() respectively :-/
+ */
+#  include <stdio.h>
+#  include <stdlib.h>
 #  define verify(expr)  assert(expr)
 #endif
 
 
+/** @addtogroup UtilityFunc
+ *
+ * @{
+ */
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -135,39 +105,55 @@ extern "C" {
 /* I/O read method
  */
 typedef enum {
-    eIO_Plain,  /* read the presently available data only */
-    eIO_Peek,   /* eREAD_Plain, but dont discard the data from input queue */
-    eIO_Persist /* try to read exactly "size" bytes;  wait for enough data */
+    eIO_ReadPlain, /* read presently available data only                     */
+    eIO_ReadPeek,  /* eIO_ReadPeek but dont discard the data from input queue*/
+    eIO_ReadPersist, /* try to read exactly "n" bytes; wait for enough data  */
+    /* deprecated */
+    eIO_Plain   = eIO_ReadPlain,
+    eIO_Peek    = eIO_ReadPeek,
+    eIO_Persist = eIO_ReadPersist
 } EIO_ReadMethod;
 
 
-/* I/O event (or direction)
+/* I/O write method
  */
 typedef enum {
-    eIO_Open,
-    eIO_Read,
-    eIO_Write,
-    eIO_ReadWrite,
-    eIO_Close
+    eIO_WritePlain,
+    eIO_WritePersist
+} EIO_WriteMethod;
+
+
+/* I/O event (or direction)
+ * Note: Internally, these constants are used as bit-values, and thus should
+ *       not be changed in this header. However, user code should not rely
+ *       on the values of these constants.
+ */
+typedef enum {
+    eIO_Open      = 0x0, /* also serves as no-event indicator in SOCK_Poll() */
+    eIO_Read      = 0x1,
+    eIO_Write     = 0x2,
+    eIO_ReadWrite = 0x3, /* eIO_Read | eIO_Write                             */
+    eIO_Close     = 0x4  /* also serves as error indicator in SOCK_Poll()    */
 } EIO_Event;
 
 
 /* I/O status
  */
 typedef enum {
-    eIO_Success = 0,  /* everything is fine, no errors occurred         */
-    eIO_Timeout,      /* timeout expired before the data could be i/o'd */
-    eIO_Closed,       /* peer has closed the connection                 */
-    eIO_InvalidArg,   /* bad argument value(s)                          */
-    eIO_NotSupported, /* the requested operation is not supported       */
+    eIO_Success = 0,  /* everything is fine, no errors occurred              */
+    eIO_Timeout,      /* timeout expired before the data could be i/o'd      */
+    eIO_Closed,       /* peer has closed the connection                      */
+    eIO_Interrupt,    /* signal received while the operation was in progress */
+    eIO_InvalidArg,   /* bad argument value(s)                               */
+    eIO_NotSupported, /* the requested operation is not supported            */
 
-    eIO_Unknown       /* unknown (most probably -- fatal) error         */
+    eIO_Unknown       /* unknown (most probably -- fatal) error              */
 } EIO_Status;
 
 
 /* Return verbal description of the I/O status
  */
-extern const char* IO_StatusStr(EIO_Status status);
+extern NCBI_XCONNECT_EXPORT const char* IO_StatusStr(EIO_Status status);
 
 
 
@@ -214,7 +200,7 @@ typedef void (*FMT_LOCK_Cleanup)
 
 /* Create new MT locking object (with reference counter := 1)
  */
-extern MT_LOCK MT_LOCK_Create
+extern NCBI_XCONNECT_EXPORT MT_LOCK MT_LOCK_Create
 (void*            user_data, /* to call "handler" and "cleanup" with */
  FMT_LOCK_Handler handler,   /* locking function */
  FMT_LOCK_Cleanup cleanup    /* cleanup function */
@@ -223,7 +209,7 @@ extern MT_LOCK MT_LOCK_Create
 
 /* Increment ref.counter by 1,  then return "lk"
  */
-extern MT_LOCK MT_LOCK_AddRef(MT_LOCK lk);
+extern NCBI_XCONNECT_EXPORT MT_LOCK MT_LOCK_AddRef(MT_LOCK lk);
 
 
 /* Decrement ref.counter by 1.
@@ -231,7 +217,7 @@ extern MT_LOCK MT_LOCK_AddRef(MT_LOCK lk);
  * destroy the handle, call "lk->cleanup(lk->user_data)", and return NULL.
  * Otherwise (if ref.counter is still > 0), return "lk".
  */
-extern MT_LOCK MT_LOCK_Delete(MT_LOCK lk);
+extern NCBI_XCONNECT_EXPORT MT_LOCK MT_LOCK_Delete(MT_LOCK lk);
 
 
 /* Call "lk->handler(lk->user_data, how)".
@@ -240,7 +226,7 @@ extern MT_LOCK MT_LOCK_Delete(MT_LOCK lk);
  * NOTE:  use MT_LOCK_Do() to avoid overhead!
  */
 #define MT_LOCK_Do(lk,how)  (lk ? MT_LOCK_DoInternal(lk, how) : -1)
-extern int/*bool*/ MT_LOCK_DoInternal
+extern NCBI_XCONNECT_EXPORT int/*bool*/ MT_LOCK_DoInternal
 (MT_LOCK  lk,
  EMT_Lock how
  );
@@ -271,7 +257,7 @@ typedef enum {
 
 /* Return verbal description of the log level
  */
-extern const char* LOG_LevelStr(ELOG_Level level);
+extern NCBI_XCONNECT_EXPORT const char* LOG_LevelStr(ELOG_Level level);
 
 
 /* Message and miscellaneous data to pass to the log post callback FLOG_Handler
@@ -307,7 +293,7 @@ typedef void (*FLOG_Cleanup)
  * ATTENTION:  if non-NULL "lk" is specified then MT_LOCK_Delete() will be
  *             called when this LOG is destroyed -- be aware of it!
  */
-extern LOG LOG_Create
+extern NCBI_XCONNECT_EXPORT LOG LOG_Create
 (void*        user_data, /* the data to call "handler" and "cleanup" with */
  FLOG_Handler handler,   /* handler */
  FLOG_Cleanup cleanup,   /* cleanup */
@@ -318,7 +304,7 @@ extern LOG LOG_Create
 /* Reset the "lg" to use the new "user_data", "handler" and "cleanup".
  * NOTE:  it does not change ref.counter.
  */
-extern void LOG_Reset
+extern NCBI_XCONNECT_EXPORT void LOG_Reset
 (LOG          lg,         /* created by LOG_Create() */
  void*        user_data,  /* new user data */
  FLOG_Handler handler,    /* new handler */
@@ -328,7 +314,7 @@ extern void LOG_Reset
 
 /* Increment ref.counter by 1,  then return "lg"
  */
-extern LOG LOG_AddRef(LOG lg);
+extern NCBI_XCONNECT_EXPORT LOG LOG_AddRef(LOG lg);
 
 
 /* Decrement ref.counter by 1.
@@ -336,7 +322,7 @@ extern LOG LOG_AddRef(LOG lg);
  * call "lg->cleanup(lg->user_data)", destroy the handle, and return NULL.
  * Otherwise (if ref.counter is still > 0), return "lg".
  */
-extern LOG LOG_Delete(LOG lg);
+extern NCBI_XCONNECT_EXPORT LOG LOG_Delete(LOG lg);
 
 
 /* Write message (maybe, with raw data attached) to the log -- e.g. call:
@@ -344,7 +330,7 @@ extern LOG LOG_Delete(LOG lg);
  * NOTE:  Do not call this function directly, if possible. Instead, use
  *        LOG_WRITE() and LOG_DATA() macros from <ncbi_util.h>!
  */
-extern void LOG_WriteInternal
+extern NCBI_XCONNECT_EXPORT void LOG_WriteInternal
 (LOG         lg,        /* created by LOG_Create() */
  ELOG_Level  level,     /* severity */
  const char* module,    /* module name */
@@ -416,7 +402,7 @@ typedef void (*FREG_Cleanup)
  * ATTENTION:  if non-NULL "mt_lock" is specified then MT_LOCK_Delete() will be
  *             called when this REG is destroyed -- be aware of it!
  */
-extern REG REG_Create
+extern NCBI_XCONNECT_EXPORT REG REG_Create
 (void*        user_data, /* the data to call "set", "get" and "cleanup" with */
  FREG_Get     get,       /* the get method */
  FREG_Set     set,       /* the set method */
@@ -428,7 +414,7 @@ extern REG REG_Create
 /* Reset the "rg" to use the new "user_data", "set", "get" and "cleanup".
  * NOTE:  it does not change ref.counter.
  */
-extern void REG_Reset
+extern NCBI_XCONNECT_EXPORT void REG_Reset
 (REG          rg,         /* created by REG_Create() */
  void*        user_data,  /* new user data */
  FREG_Get     get,        /* the get method */
@@ -440,7 +426,7 @@ extern void REG_Reset
 
 /* Increment ref.counter by 1,  then return "rg"
  */
-extern REG REG_AddRef(REG rg);
+extern NCBI_XCONNECT_EXPORT REG REG_AddRef(REG rg);
 
 
 /* Decrement ref.counter by 1.
@@ -448,7 +434,7 @@ extern REG REG_AddRef(REG rg);
  * call "rg->cleanup(rg->user_data)", destroy the handle, and return NULL.
  * Otherwise (if ref.counter is still > 0), return "rg".
  */
-extern REG REG_Delete(REG rg);
+extern NCBI_XCONNECT_EXPORT REG REG_Delete(REG rg);
 
 
 /* Copy the registry value stored in "section" under name "name"
@@ -460,7 +446,7 @@ extern REG REG_Delete(REG rg);
  * Return "value" (however, if "value_size" is zero, then return NULL).
  * If non-NULL, the returned "value" will be terminated by '\0'.
  */
-extern char* REG_Get
+extern NCBI_XCONNECT_EXPORT char* REG_Get
 (REG         rg,         /* created by REG_Create() */
  const char* section,    /* registry section name */
  const char* name,       /* registry entry name  */
@@ -473,7 +459,7 @@ extern char* REG_Get
 /* Store the "value" to  the registry section "section" under name "name",
  * in storage "storage".
  */
-extern void REG_Set
+extern NCBI_XCONNECT_EXPORT void REG_Set
 (REG          rg,        /* created by REG_Create() */
  const char*  section,
  const char*  name,
@@ -486,4 +472,93 @@ extern void REG_Set
 }  /* extern "C" */
 #endif
 
-#endif /* NCBI_CORE__H */
+
+/* @} */
+
+
+/*
+ * ===========================================================================
+ *
+ * $Log: ncbi_core.h,v $
+ * Revision 6.25  2004/03/12 23:26:22  gorelenk
+ * Added export prefixes.
+ *
+ * Revision 6.24  2003/09/02 20:45:45  lavr
+ * -<connect/connect_export.h> -- now included from <connect/ncbi_types.h>
+ *
+ * Revision 6.23  2003/04/09 17:58:48  siyan
+ * Added doxygen support
+ *
+ * Revision 6.22  2003/03/06 19:42:47  lavr
+ * Isolate special inclusion of <stdio.h>, <stdlib.h> in DEBUG branch only
+ *
+ * Revision 6.21  2003/03/06 01:42:14  lavr
+ * Include <stdio.h> and <stdlib.h> required by <assert.h> under Codewarrior
+ *
+ * Revision 6.20  2003/01/08 01:59:32  lavr
+ * DLL-ize CONNECT library for MSVC (add NCBI_XCONNECT_EXPORT)
+ *
+ * Revision 6.19  2002/09/19 18:00:33  lavr
+ * Header file guard macro changed
+ *
+ * Revision 6.18  2002/08/13 19:26:39  lavr
+ * Add eIO_Interrupt to EIO_Status
+ *
+ * Revision 6.17  2002/08/08 14:21:11  ucko
+ * Fix C++-style comment; really move CVS log to end.
+ *
+ * Revision 6.16  2002/08/07 16:28:25  lavr
+ * Added EIO_WriteMethod; EIO_ReadMethod enums changed; log moved to end
+ *
+ * Revision 6.15  2002/05/06 19:07:48  lavr
+ * Added notes to EIO_Event enum type
+ *
+ * Revision 6.14  2002/03/28 13:28:43  kans
+ * undef verify if already defined, such as on Mac OS X Darwin (EN)
+ *
+ * Revision 6.13  2001/09/27 22:29:35  vakatov
+ * Always define "NDEBUG" if "_DEBUG" is not defined (mostly for the C Toolkit)
+ *
+ * Revision 6.12  2001/08/09 16:22:51  lavr
+ * Remove last (unneeded) parameter from LOG_Reset()
+ *
+ * Revision 6.11  2001/06/19 20:16:18  lavr
+ * Added #include <connect/ncbi_types.h>
+ *
+ * Revision 6.10  2001/06/19 19:08:28  lavr
+ * STimeout and ESwitch moved to <connect/ncbi_types.h>
+ *
+ * Revision 6.9  2001/05/17 18:10:22  vakatov
+ * Moved the logging macros from <ncbi_core.h> to <ncbi_util.h>.
+ * Logging::  always call the logger if severity is eLOG_Fatal.
+ *
+ * Revision 6.8  2001/03/02 20:06:54  lavr
+ * Typos fixed
+ *
+ * Revision 6.7  2001/01/23 23:07:30  lavr
+ * A make-up change
+ *
+ * Revision 6.6  2001/01/11 16:41:18  lavr
+ * Registry Get/Set methods got the 'user_data' argument, forgotten earlier
+ *
+ * Revision 6.5  2000/10/18 20:29:41  vakatov
+ * REG_Get::  pass in the default value (rather than '\0')
+ *
+ * Revision 6.4  2000/06/23 19:34:41  vakatov
+ * Added means to log binary data
+ *
+ * Revision 6.3  2000/04/07 19:55:14  vakatov
+ * Standard indentation
+ *
+ * Revision 6.2  2000/03/24 23:12:03  vakatov
+ * Starting the development quasi-branch to implement CONN API.
+ * All development is performed in the NCBI C++ tree only, while
+ * the NCBI C tree still contains "frozen" (see the last revision) code.
+ *
+ * Revision 6.1  2000/02/23 22:30:40  vakatov
+ * Initial revision
+ *
+ * ===========================================================================
+ */
+
+#endif /* CONNECT___NCBI_CORE__H */
