@@ -1,4 +1,4 @@
-/*  $Id: ncbi_file_connector.c,v 6.6 2001/12/04 15:56:35 lavr Exp $
+/*  $Id: ncbi_file_connector.c,v 6.12 2003/05/31 05:14:56 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,35 +31,12 @@
  *   See in "connectr.h" for the detailed specification of the underlying
  *   connector("CONNECTOR", "SConnectorTag") methods and structures.
  *
- * --------------------------------------------------------------------------
- * $Log: ncbi_file_connector.c,v $
- * Revision 6.6  2001/12/04 15:56:35  lavr
- * Use strdup() instead of explicit strcpy(malloc(...), ...)
- *
- * Revision 6.5  2001/01/25 17:04:43  lavr
- * Reversed:: DESTROY method calls free() to delete connector structure
- *
- * Revision 6.4  2001/01/23 23:11:20  lavr
- * Status virtual method implemented
- *
- * Revision 6.3  2001/01/11 16:38:16  lavr
- * free(connector) removed from s_Destroy function
- * (now always called from outside, in METACONN_Remove)
- *
- * Revision 6.2  2000/12/29 17:55:53  lavr
- * Adapted for use of new connector structure.
- *
- * Revision 6.1  2000/04/12 15:18:13  vakatov
- * Initial revision
- *
- * ==========================================================================
  */
 
-#include <connect/ncbi_ansi_ext.h>
+#include "ncbi_ansi_ext.h"
 #include <connect/ncbi_file_connector.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 
 
 /***********************************************************************
@@ -84,31 +61,31 @@ typedef struct {
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
-    static const char* s_VT_GetType(CONNECTOR       connector);
-    static EIO_Status  s_VT_Open   (CONNECTOR       connector,
-                                    const STimeout* timeout);
-    static EIO_Status  s_VT_Wait   (CONNECTOR       connector,
-                                    EIO_Event       event,
-                                    const STimeout* timeout);
-    static EIO_Status  s_VT_Write  (CONNECTOR       connector,
-                                    const void*     buf,
-                                    size_t          size,
-                                    size_t*         n_written,
-                                    const STimeout* timeout);
-    static EIO_Status  s_VT_Flush  (CONNECTOR       connector,
-                                    const STimeout* timeout);
-    static EIO_Status  s_VT_Read   (CONNECTOR       connector,
-                                    void*           buf,
-                                    size_t          size,
-                                    size_t*         n_read,
-                                    const STimeout* timeout);
-    static EIO_Status  s_VT_Status (CONNECTOR       connector,
-                                    EIO_Event       dir);
-    static EIO_Status  s_VT_Close  (CONNECTOR       connector,
-                                    const STimeout* timeout);
-    static void        s_Setup     (SMetaConnector* meta,
-                                    CONNECTOR       connector);
-    static void        s_Destroy   (CONNECTOR       connector);
+    static const char* s_VT_GetType (CONNECTOR       connector);
+    static EIO_Status  s_VT_Open    (CONNECTOR       connector,
+                                     const STimeout* timeout);
+    static EIO_Status  s_VT_Wait    (CONNECTOR       connector,
+                                     EIO_Event       event,
+                                     const STimeout* timeout);
+    static EIO_Status  s_VT_Write   (CONNECTOR       connector,
+                                     const void*     buf,
+                                     size_t          size,
+                                     size_t*         n_written,
+                                     const STimeout* timeout);
+    static EIO_Status  s_VT_Flush   (CONNECTOR       connector,
+                                     const STimeout* timeout);
+    static EIO_Status  s_VT_Read    (CONNECTOR       connector,
+                                     void*           buf,
+                                     size_t          size,
+                                     size_t*         n_read,
+                                     const STimeout* timeout);
+    static EIO_Status  s_VT_Status  (CONNECTOR       connector,
+                                     EIO_Event       dir);
+    static EIO_Status  s_VT_Close   (CONNECTOR       connector,
+                                     const STimeout* timeout);
+    static void        s_Setup      (SMetaConnector* meta,
+                                     CONNECTOR       connector);
+    static void        s_Destroy    (CONNECTOR       connector);
 #  ifdef IMPLEMENTED__CONN_WaitAsync
     static EIO_Status s_VT_WaitAsync(void*                   connector,
                                      FConnectorAsyncHandler  func,
@@ -119,6 +96,7 @@ extern "C" {
 #endif /* __cplusplus */
 
 
+/*ARGSUSED*/
 static const char* s_VT_GetType
 (CONNECTOR connector)
 {
@@ -126,6 +104,7 @@ static const char* s_VT_GetType
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Open
 (CONNECTOR       connector,
  const STimeout* timeout)
@@ -167,6 +146,7 @@ static EIO_Status s_VT_Open
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Write
 (CONNECTOR       connector,
  const void*     buf,
@@ -190,6 +170,7 @@ static EIO_Status s_VT_Write
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Read
 (CONNECTOR       connector,
  void*           buf,
@@ -213,6 +194,7 @@ static EIO_Status s_VT_Read
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Wait
 (CONNECTOR       connector,
  EIO_Event       event,
@@ -222,6 +204,7 @@ static EIO_Status s_VT_Wait
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Flush
 (CONNECTOR       connector,
  const STimeout* timeout)
@@ -249,6 +232,7 @@ static EIO_Status s_VT_Status
 }
 
 
+/*ARGSUSED*/
 static EIO_Status s_VT_Close
 (CONNECTOR       connector,
  const STimeout* timeout)
@@ -279,6 +263,7 @@ static void s_Setup
 #ifdef IMPLEMENTED__CONN_WaitAsync
     CONN_SET_METHOD(meta, wait_async, s_VT_WaitAsync, connector);
 #endif
+    meta->default_timeout = 0/*infinite*/;
 }
 
 
@@ -290,6 +275,7 @@ static void s_Destroy
     free(xxx->inp_file_name);
     free(xxx->out_file_name);
     free(xxx);
+    connector->handle = 0;
     free(connector);
 }
 
@@ -333,3 +319,47 @@ extern CONNECTOR FILE_CreateConnectorEx
 
     return ccc;
 }
+
+
+/*
+ * --------------------------------------------------------------------------
+ * $Log: ncbi_file_connector.c,v $
+ * Revision 6.12  2003/05/31 05:14:56  lavr
+ * Add ARGSUSED where args are meant to be unused
+ *
+ * Revision 6.11  2003/05/14 03:55:53  lavr
+ * Slight VT table reformatting
+ *
+ * Revision 6.10  2002/10/28 15:46:20  lavr
+ * Use "ncbi_ansi_ext.h" privately
+ *
+ * Revision 6.9  2002/10/22 15:11:24  lavr
+ * Zero connector's handle to crash if revisited
+ *
+ * Revision 6.8  2002/09/24 15:06:00  lavr
+ * Log moved to end
+ *
+ * Revision 6.7  2002/04/26 16:32:36  lavr
+ * Added setting of default timeout in meta-connector's setup routine
+ *
+ * Revision 6.6  2001/12/04 15:56:35  lavr
+ * Use strdup() instead of explicit strcpy(malloc(...), ...)
+ *
+ * Revision 6.5  2001/01/25 17:04:43  lavr
+ * Reversed:: DESTROY method calls free() to delete connector structure
+ *
+ * Revision 6.4  2001/01/23 23:11:20  lavr
+ * Status virtual method implemented
+ *
+ * Revision 6.3  2001/01/11 16:38:16  lavr
+ * free(connector) removed from s_Destroy function
+ * (now always called from outside, in METACONN_Remove)
+ *
+ * Revision 6.2  2000/12/29 17:55:53  lavr
+ * Adapted for use of new connector structure.
+ *
+ * Revision 6.1  2000/04/12 15:18:13  vakatov
+ * Initial revision
+ *
+ * ==========================================================================
+ */
