@@ -1,7 +1,7 @@
-#ifndef NCBI_PRIV__H
-#define NCBI_PRIV__H
+#ifndef CONNECT___NCBI_PRIV__H
+#define CONNECT___NCBI_PRIV__H
 
-/*  $Id: ncbi_priv.h,v 6.4 2001/05/17 18:07:15 vakatov Exp $
+/*  $Id: ncbi_priv.h,v 6.10 2004/03/12 23:25:37 gorelenk Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -33,29 +33,12 @@
  *
  *********************************
  * Critical section (basic multi-thread synchronization):
- *    private:    g_CORE_MT_Lock
- *    macro:      CORE_LOCK_WRITE, CORE_LOCK_READ, CORE_UNLOCK
+ *    private global:  g_CORE_MT_Lock
+ *    macros:          CORE_LOCK_WRITE, CORE_LOCK_READ, CORE_UNLOCK
  * Tracing and logging:
- *    private:    g_CORE_Log
- *    macro:      CORE_LOG(), CORE_DATA(),
- *                CORE_LOG_SYS_ERRNO(), CORE_LOG_ERRNO()
+ *    private global:  g_CORE_Log
+ *    macros:          CORE_LOG[F](), CORE_DATA[F](), CORE_LOG[F]_ERRNO()
  *
- * ---------------------------------------------------------------------------
- * $Log: ncbi_priv.h,v $
- * Revision 6.4  2001/05/17 18:07:15  vakatov
- * Logging::  always call the logger if severity is eLOG_Fatal
- *
- * Revision 6.3  2001/01/08 22:37:36  lavr
- * Added GNU attribute to g_CORE_sprintf for compiler to trace
- * format specifier/parameter correspondence
- *
- * Revision 6.2  2000/06/23 19:34:44  vakatov
- * Added means to log binary data
- *
- * Revision 6.1  2000/03/24 22:53:35  vakatov
- * Initial revision
- *
- * ===========================================================================
  */
 
 #include <connect/ncbi_util.h>
@@ -73,7 +56,7 @@ extern "C" {
 /* Always use the following macros and functions to access "g_CORE_MT_Lock",
  * dont access/change it directly!
  */
-extern MT_LOCK g_CORE_MT_Lock;
+extern NCBI_XCONNECT_EXPORT MT_LOCK g_CORE_MT_Lock;
 
 #define CORE_LOCK_WRITE  verify( MT_LOCK_Do(g_CORE_MT_Lock, eMT_Lock    ) )
 #define CORE_LOCK_READ   verify( MT_LOCK_Do(g_CORE_MT_Lock, eMT_LockRead) )
@@ -88,7 +71,13 @@ extern MT_LOCK g_CORE_MT_Lock;
 /* Always use the following macros and functions to access "g_CORE_Log",
  * dont access/change it directly!
  */
-extern LOG g_CORE_Log;
+extern NCBI_XCONNECT_EXPORT LOG g_CORE_Log;
+
+extern NCBI_XCONNECT_EXPORT const char* g_CORE_Sprintf(const char* fmt, ...)
+#ifdef __GNUC__
+         __attribute__((format(printf, 1, 2)))
+#endif
+;
 
 #define CORE_LOG(level, message)  do { \
     if (g_CORE_Log  ||  level == eLOG_Fatal) { \
@@ -98,6 +87,14 @@ extern LOG g_CORE_Log;
     } \
 } while (0)
 
+#define CORE_LOGF(level, fmt_args)  do { \
+    if (g_CORE_Log  ||  level == eLOG_Fatal) { \
+        CORE_LOCK_READ; \
+        LOG_WRITE(g_CORE_Log, level, g_CORE_Sprintf fmt_args); \
+        CORE_UNLOCK; \
+    } \
+} while (0)
+
 #define CORE_DATA(data, size, message)  do { \
     if ( g_CORE_Log ) { \
         CORE_LOCK_READ; \
@@ -106,20 +103,6 @@ extern LOG g_CORE_Log;
     } \
 } while (0)
 
-extern const char* g_CORE_Sprintf(const char* fmt, ...)
-#ifdef __GNUC__
-         __attribute__((format(printf, 1, 2)))
-#endif
-;
-
-#define CORE_LOGF(level, fmt_args)  do { \
-    if (g_CORE_Log  ||  level == eLOG_Fatal) { \
-        CORE_LOCK_READ; \
-        LOG_WRITE(g_CORE_Log, level, g_CORE_Sprintf fmt_args); \
-        CORE_UNLOCK; \
-    } \
-} while (0)
-
 #define CORE_DATAF(data, size, fmt_args)  do { \
     if ( g_CORE_Log ) { \
         CORE_LOCK_READ; \
@@ -128,21 +111,36 @@ extern const char* g_CORE_Sprintf(const char* fmt, ...)
     } \
 } while (0)
 
-#define CORE_LOG_SYS_ERRNO(level, message)  do { \
+#define CORE_LOG_ERRNO(level, x_errno, message)  do { \
     if (g_CORE_Log  ||  level == eLOG_Fatal) { \
         CORE_LOCK_READ; \
-        LOG_WRITE_ERRNO(g_CORE_Log, level, message); \
+        LOG_WRITE_ERRNO_EX(g_CORE_Log, level, message, x_errno, 0); \
         CORE_UNLOCK; \
     } \
 } while (0)
 
-#define CORE_LOG_ERRNO(x_errno, level, message)  do { \
+#define CORE_LOGF_ERRNO(level, x_errno, fmt_args)  do { \
     if (g_CORE_Log  ||  level == eLOG_Fatal) { \
-        char buf[2048]; \
-        int xx_errno = x_errno; \
         CORE_LOCK_READ; \
-        LOG_WRITE(g_CORE_Log, level, \
-                  MessagePlusErrno(message, xx_errno, "", buf, sizeof(buf))); \
+        LOG_WRITE_ERRNO_EX(g_CORE_Log, level, g_CORE_Sprintf fmt_args, \
+                           x_errno, 0); \
+        CORE_UNLOCK; \
+    } \
+} while (0)
+
+#define CORE_LOG_ERRNO_EX(level, x_errno, x_descr, message)  do { \
+    if (g_CORE_Log  ||  level == eLOG_Fatal) { \
+        CORE_LOCK_READ; \
+        LOG_WRITE_ERRNO_EX(g_CORE_Log, level, message, x_errno, x_descr); \
+        CORE_UNLOCK; \
+    } \
+} while (0)
+
+#define CORE_LOGF_ERRNO_EX(level, x_errno, x_descr, fmt_args)  do { \
+    if (g_CORE_Log  ||  level == eLOG_Fatal) { \
+        CORE_LOCK_READ; \
+        LOG_WRITE_ERRNO_EX(g_CORE_Log, level, g_CORE_Sprintf fmt_args, \
+                           x_errno, x_descr); \
         CORE_UNLOCK; \
     } \
 } while (0)
@@ -156,7 +154,7 @@ extern const char* g_CORE_Sprintf(const char* fmt, ...)
 /* Always use the following macros and functions to access "g_CORE_Registry",
  * dont access/change it directly!
  */
-extern REG g_CORE_Registry;
+extern NCBI_XCONNECT_EXPORT REG g_CORE_Registry;
 
 #define CORE_REG_GET(section, name, value, value_size, def_value) \
     g_CORE_RegistryGET(section, name, value, value_size, def_value)
@@ -169,7 +167,7 @@ extern REG g_CORE_Registry;
 
 
 /* (private, to be used exclusively by the above macro CORE_REG_GET) */
-extern char* g_CORE_RegistryGET
+extern NCBI_XCONNECT_EXPORT char* g_CORE_RegistryGET
 (const char* section,
  const char* name,
  char*       value,
@@ -181,4 +179,42 @@ extern char* g_CORE_RegistryGET
 }  /* extern "C" */
 #endif
 
-#endif /* NCBI_PRIV__H */
+
+/*
+ * ---------------------------------------------------------------------------
+ * $Log: ncbi_priv.h,v $
+ * Revision 6.10  2004/03/12 23:25:37  gorelenk
+ * Added export prefixes.
+ *
+ * Revision 6.9  2002/12/05 21:43:00  lavr
+ * Swap level and errno in CORE_LOG[F]_ERRNO(); add CORE_LOG[F]_ERRNO_EX()
+ *
+ * Revision 6.8  2002/12/04 21:00:43  lavr
+ * -CORE_LOG[F]_SYS_ERRNO()
+ *
+ * Revision 6.7  2002/12/04 19:51:40  lavr
+ * Enable MessageWithErrno() to print error description
+ *
+ * Revision 6.6  2002/09/19 18:08:31  lavr
+ * Header file guard macro changed; log moved to end
+ *
+ * Revision 6.5  2002/04/13 06:38:45  lavr
+ * More *_LOGF_* macros added
+ *
+ * Revision 6.4  2001/05/17 18:07:15  vakatov
+ * Logging::  always call the logger if severity is eLOG_Fatal
+ *
+ * Revision 6.3  2001/01/08 22:37:36  lavr
+ * Added GNU attribute to g_CORE_sprintf for compiler to trace
+ * format specifier/parameter correspondence
+ *
+ * Revision 6.2  2000/06/23 19:34:44  vakatov
+ * Added means to log binary data
+ *
+ * Revision 6.1  2000/03/24 22:53:35  vakatov
+ * Initial revision
+ *
+ * ===========================================================================
+ */
+
+#endif /* CONNECT___NCBI_PRIV__H */
