@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVICEP__H
 #define CONNECT___NCBI_SERVICEP__H
 
-/*  $Id: ncbi_servicep.h,v 6.23 2003/06/26 15:19:56 lavr Exp $
+/*  $Id: ncbi_servicep.h,v 6.25 2004/08/19 15:48:04 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,7 +58,7 @@ typedef struct {
  */
 struct SSERV_IterTag {
     const char*  service;        /* requested service name                 */
-    TSERV_Type   type;           /* requested server type(s)               */
+    TSERV_Type   types;          /* requested server type(s)               */
     unsigned int preferred_host; /* preferred host to select, network b.o. */
     double       preference;     /* range [0..100] %%                      */
     SSERV_Info** skip;           /* servers to skip                        */
@@ -93,7 +93,7 @@ SSERV_Info* SERV_GetInfoP
  int/*bool*/         external       /* whether mapping is not local to NCBI  */
  );
 
-/* same as above but creates an iterator to get services one by one */
+/* same as the above but creates an iterator to get services one by one */
 SERV_ITER SERV_OpenP
 (const char*         service,
  TSERV_Type          type,
@@ -114,7 +114,9 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text);
  * contained in the iterator; to be used in mapping requests to DISPD.
  * Return value must be 'free'd.
  */
-char* SERV_Print(SERV_ITER iter);
+char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer);
+
+#define SERV_Print(iter) SERV_PrintEx(iter, 0)
 
 
 /* Get name of underlying service mapper.
@@ -153,6 +155,12 @@ double SERV_Preference(double pref, double gap, unsigned int n);
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_servicep.h,v $
+ * Revision 6.25  2004/08/19 15:48:04  lavr
+ * SERV_ITER::type renamed into SERV_ITER::types to reflect its bitmask nature
+ *
+ * Revision 6.24  2004/07/01 16:27:55  lavr
+ * +SERV_PrintEx()
+ *
  * Revision 6.23  2003/06/26 15:19:56  lavr
  * Additional parameter "external" for SERV_{Open|GetInfo}P()
  *
