@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_connutil_misc.c,v 6.4 2000/11/07 23:24:43 vakatov Exp $
+/*  $Id: test_ncbi_connutil_misc.c,v 6.14 2004/04/01 14:14:02 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -28,30 +28,13 @@
  * File Description:
  *   Tests for "ncbi_connutil.c"
  *
- * ---------------------------------------------------------------------------
- * $Log: test_ncbi_connutil_misc.c,v $
- * Revision 6.4  2000/11/07 23:24:43  vakatov
- * [MIME]  In-sync with the C Toolkit "connutil.c:R6.15"
- *
- * Revision 6.3  2000/04/12 15:22:07  vakatov
- * Always #undef NDEBUG
- *
- * Revision 6.2  2000/03/29 17:21:48  vakatov
- * + CORE_SetLOG(0) at the program end.
- *
- * Revision 6.1  2000/03/24 22:53:38  vakatov
- * Initial revision
- *
- * ===========================================================================
  */
 
-#if defined(NDEBUG)
-#  undef NDEBUG
-#endif 
-
-#include <connect/ncbi_util.h>
 #include <connect/ncbi_connutil.h>
+#include <connect/ncbi_util.h>
 #include <string.h>
+/* This header must go last */
+#include "test_assert.h"
 
 
 /***********************************************************************
@@ -227,7 +210,8 @@ static void TEST_MIME(void)
         for (i = 0, subtype = (EMIME_SubType) i;
              i <= (int) eMIME_Unknown;  i++, subtype = (EMIME_SubType) i) {
             for (j = 0, encoding = (EMIME_Encoding) j; 
-                 j <= (int) eENCOD_None;  j++, encoding = (EMIME_Encoding) j) {
+                 j < (int) eENCOD_Unknown;
+                 j++, encoding = (EMIME_Encoding) j) {
                 assert(!s_Try_MIME(str, type, subtype, encoding));
                 MIME_ComposeContentTypeEx(type, subtype, encoding,
                                           str, sizeof(str));
@@ -244,13 +228,55 @@ static void TEST_MIME(void)
                       eMIME_T_NcbiData, eMIME_AsnText, eENCOD_Url));
     assert(s_Try_MIME("x-ncbi-data/x-eeee",
                       eMIME_T_NcbiData, eMIME_Unknown, eENCOD_None));
+    assert(s_Try_MIME("x-ncbi-data/plain-",
+                      eMIME_T_NcbiData, eMIME_Unknown, eENCOD_None));
 
     assert(!s_Try_MIME("content-TYPE : x-ncbi-data/x-unknown\r",
                        eMIME_T_NcbiData, eMIME_Unknown, eENCOD_None));
     assert(s_Try_MIME("text/html",
                       eMIME_T_Text, eMIME_Html, eENCOD_None));
-    assert(!s_Try_MIME("", eMIME_T_NcbiData, eMIME_Unknown, eENCOD_None));
-    assert(!s_Try_MIME(0, eMIME_T_NcbiData, eMIME_Unknown, eENCOD_None));
+    assert(s_Try_MIME("application/xml+soap",
+                      eMIME_T_Application, eMIME_XmlSoap, eENCOD_None));
+    assert(!s_Try_MIME("", eMIME_T_NcbiData, eMIME_Unknown, eENCOD_Unknown));
+    assert(!s_Try_MIME(0, eMIME_T_NcbiData, eMIME_Unknown, eENCOD_Unknown));
+}
+
+
+static void TEST_ConnNetInfo(void)
+{
+    SConnNetInfo* net_info = ConnNetInfo_Create(0);
+
+    assert(net_info);
+    printf("HTTP User Header:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_AppendUserHeader(net_info,
+                                 "My-Tag1: Value1\r\n"
+                                 "My-Tag2: Value2\r\n"
+                                 "My-Tag3: Value3\r\n");
+    printf("HTTP User Header after append:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_OverrideUserHeader(net_info,
+                                   "My-TAG1:    \t  \r\n"
+                                   "My-TaG2: Value 2.1\r\n"
+                                   "My-Tag4: Value 4\r\n");
+    printf("HTTP User Header after override:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_ExtendUserHeader(net_info,
+                                 "My-Tag3: \t \r\n"
+                                 "My-Tag4: Value 4.1\r\n"
+                                 "My-Tag5: \t \r\n"
+                                 "My-Tag6: Value 6\r\n");
+    printf("HTTP User Header after extend:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_SetUserHeader(net_info, 0);
+    printf("HTTP User Header after set:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_ExtendUserHeader(net_info,
+                                 "My-Tag7: Value7\r\n"
+                                 "My-Tag8: \t \r\n");
+    printf("HTTP User Header after second extend:\n\"%s\"\n",
+           net_info->http_user_header ? net_info->http_user_header : "<NONE>");
+    ConnNetInfo_Destroy(net_info);
 }
 
 
@@ -266,7 +292,57 @@ int main(void)
 
     TEST_URL_Encoding();
     TEST_MIME();
+    TEST_ConnNetInfo();
 
     CORE_SetLOG(0);
     return 0;
 }
+
+
+/*
+ * ---------------------------------------------------------------------------
+ * $Log: test_ncbi_connutil_misc.c,v $
+ * Revision 6.14  2004/04/01 14:14:02  lavr
+ * Spell "occurred", "occurrence", and "occurring"
+ *
+ * Revision 6.13  2004/01/14 18:53:09  lavr
+ * Use "application/xml+soap" in the test case
+ *
+ * Revision 6.12  2004/01/07 19:24:03  lavr
+ * Added test for MIME content-type "application/xml"
+ *
+ * Revision 6.11  2002/12/13 21:20:55  lavr
+ * Move log to end
+ *
+ * Revision 6.10  2002/11/22 15:09:40  lavr
+ * Replace all occurrences of "ray" with "yar"
+ *
+ * Revision 6.9  2002/10/11 19:57:17  lavr
+ * Add tests for ConnNetInfo_*UserHeader() routines
+ *
+ * Revision 6.8  2002/03/22 19:46:51  lavr
+ * Test_assert.h made last among the include files
+ *
+ * Revision 6.7  2002/02/20 19:12:39  lavr
+ * Swapped eENCOD_Url and eENCOD_None; eENCOD_Unknown introduced; test cleaned
+ *
+ * Revision 6.6  2002/02/05 21:45:55  lavr
+ * Included header files rearranged
+ *
+ * Revision 6.5  2002/01/16 21:23:15  vakatov
+ * Utilize header "test_assert.h" to switch on ASSERTs in the Release mode too
+ *
+ * Revision 6.4  2000/11/07 23:24:43  vakatov
+ * [MIME]  In-sync with the C Toolkit "connutil.c:R6.15"
+ *
+ * Revision 6.3  2000/04/12 15:22:07  vakatov
+ * Always #undef NDEBUG
+ *
+ * Revision 6.2  2000/03/29 17:21:48  vakatov
+ * + CORE_SetLOG(0) at the program end.
+ *
+ * Revision 6.1  2000/03/24 22:53:38  vakatov
+ * Initial revision
+ *
+ * ===========================================================================
+ */
