@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_sendmail.c,v 6.5 2001/03/07 20:49:29 lavr Exp $
+/*  $Id: test_ncbi_sendmail.c,v 6.11 2003/12/09 15:39:30 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -28,32 +28,15 @@
  * File Description:
  *   Simple test for CORE_Sendmail
  *
- * --------------------------------------------------------------------------
- * $Log: test_ncbi_sendmail.c,v $
- * Revision 6.5  2001/03/07 20:49:29  lavr
- * Forgotten #include <string.h> added
- *
- * Revision 6.4  2001/03/06 04:32:31  lavr
- * Custom header test added
- *
- * Revision 6.3  2001/03/02 20:01:53  lavr
- * "../ncbi_priv.h" explained
- *
- * Revision 6.2  2001/02/28 17:48:07  lavr
- * Huge body test added
- *
- * Revision 6.1  2001/02/28 00:53:45  lavr
- * Initial revision
- *
- * ==========================================================================
  */
 
 #include "../ncbi_priv.h"               /* CORE logging facilities */
 #include <connect/ncbi_sendmail.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+/* This header must go last */
+#include "test_assert.h"
 
 
 #define TEST_HUGE_BODY_SIZE     (1024*100)
@@ -61,6 +44,11 @@
 
 int main(void)
 {
+    const char custom_body[] =
+        "Subject: Custom sized body\n"
+        "\n"
+        "Custom sized body\n"
+        "0123456789\n"; /* these 11 chars to ignore */
     const char* body[] = {
         "This is a simple test",
         "This is a test with\n.",
@@ -79,7 +67,8 @@ int main(void)
     };
     const char* to[] = {
         "lavr",
-        "lavr@pavo"
+        "lavr@pavo",
+        " \"Anton Lavrentiev\"   <lavr@pavo>  , lavr, <lavr>   ",
     };
     size_t i, j, k, n, m;
     const char* mx_host;
@@ -90,6 +79,8 @@ int main(void)
     short mx_port;
     FILE *fp;
 
+    CORE_SetLOGFormatFlags(fLOG_None          | fLOG_Level   |
+                           fLOG_OmitNoteLevel | fLOG_DateTime);
     CORE_SetLOGFILE(stderr, 0/*false*/);
     srand(time(0));
 
@@ -99,7 +90,7 @@ int main(void)
         (sizeof(subject)/sizeof(subject[0]))*
         (sizeof(body)/sizeof(body[0]));
     m = 0;
-    for (i = 0; i < sizeof(to)/sizeof(to[0]); i++)
+    for (i = 0; i < sizeof(to)/sizeof(to[0]); i++) {
         for (j = 0; j < sizeof(subject)/sizeof(subject[0]); j++)
             for (k = 0; k < sizeof(body)/sizeof(body[0]); k++) {
                 CORE_LOGF(eLOG_Note, ("Test %u of %u",
@@ -108,14 +99,17 @@ int main(void)
                 if (retval != 0)
                     CORE_LOGF(eLOG_Fatal, ("Test failed: %s", retval));
             }
+    }
 
     CORE_LOG(eLOG_Note, "Phase 2 of 2: Testing CORE_SendMailEx");
 
     SendMailInfo_Init(&info);
-    mx_port = info.mx_port;
     mx_host = info.mx_host;
+    mx_port = info.mx_port;
     mx_tmo  = info.mx_timeout;
 
+    info.mx_host = "localhost";
+
     CORE_LOG(eLOG_Note, "Testing bad port");
     info.mx_port = 10;
     retval = CORE_SendMailEx("lavr", "CORE_SendMailEx Test", "Test", &info);
@@ -131,6 +125,7 @@ int main(void)
     CORE_LOGF(eLOG_Note, ("Test passed: %s", retval));
 
     CORE_LOG(eLOG_Note, "Testing timeout");
+    info.mx_host = "www.ncbi.nlm.nih.gov";
     info.mx_timeout.sec = 5;
     info.mx_port = 80;
     retval = CORE_SendMailEx("lavr", "CORE_SendMailEx Test", "Test", &info);
@@ -149,7 +144,7 @@ int main(void)
         CORE_LOG(eLOG_Fatal, "Test failed");
     CORE_LOGF(eLOG_Note, ("Test passed: %s", retval));
 
-    info.mx_host = mx_host;
+    info.mx_host = "localhost";
 
     CORE_LOG(eLOG_Note, "Testing cc");
     info.cc = "vakatov";
@@ -178,12 +173,12 @@ int main(void)
     if (retval)
         CORE_LOGF(eLOG_Fatal, ("Test failed: %s", retval));
     if (!(fp = fopen("test_ncbi_sendmail.out", "w")) ||
-        fwrite(huge_body, TEST_HUGE_BODY_SIZE, 1, fp) != 1) {
+        fwrite(huge_body, TEST_HUGE_BODY_SIZE - 1, 1, fp) != 1) {
         CORE_LOG(eLOG_Error, "Test failed: Cannot store huge body to file");
     } else {
         fclose(fp);
         CORE_LOG(eLOG_Note, "Success: Check test_ncbi_sendmail.out");
-    }       
+    }
     free(huge_body);
 
     CORE_LOG(eLOG_Note, "Testing custom headers");
@@ -200,21 +195,89 @@ int main(void)
         CORE_LOG(eLOG_Fatal, "Test failed");
     CORE_LOGF(eLOG_Note, ("Test passed: %s", retval));
 
+    CORE_LOG(eLOG_Note, "Testing AS-IS message");
+    info.mx_no_header = 1/*true*/;
+    retval = CORE_SendMailEx("lavr",
+                             "BAD SUBJECT SHOULD NOT APPEAR BUT IGNORED",
+                             "From: yourself\n"
+                             "To: yourself\n"
+                             "Subject: AS-IS message\n"
+                             "\n"
+                             "AS-IS",
+                             &info);
+    if (retval)
+        CORE_LOGF(eLOG_Fatal, ("Test failed: %s", retval));
+    CORE_LOG(eLOG_Note, "Test passed");
+
+    CORE_LOG(eLOG_Note, "Testing AS-IS custom sized message");
+    info.body_size = strlen(custom_body) - 11/*to ignore*/;
+    retval = CORE_SendMailEx("<lavr@pavo>",
+                             "BAD SUBJECT SHOULD NOT APPEAR BUT IGNORED",
+                             custom_body,
+                             &info);
+    if (retval)
+        CORE_LOGF(eLOG_Fatal, ("Test failed: %s", retval));
+    CORE_LOG(eLOG_Note, "Test passed");
+
+    info.body_size = 0;
+    info.mx_no_header = 0;
+    info.mx_host = mx_host;
+
     CORE_LOG(eLOG_Note, "Testing bad from");
     strcpy(info.from, "blahblah@blahblah");
     retval = CORE_SendMailEx("lavr", "CORE_SendMailEx Test", "Test", &info);
     if (!retval)
         CORE_LOG(eLOG_Error, "Test failed");
     else
-        CORE_LOGF(eLOG_Error, ("Test passed: %s", retval));
-    
+        CORE_LOGF(eLOG_Note, ("Test passed: %s", retval));
+
     CORE_LOG(eLOG_Note, "Testing bad magic");
     info.magic_number = 0;
     retval = CORE_SendMailEx("lavr", "CORE_SendMailEx Test", "Test", &info);
     if (!retval)
         CORE_LOG(eLOG_Fatal, "Test failed");
     CORE_LOGF(eLOG_Note, ("Test passed: %s", retval));
-    
+
     CORE_LOG(eLOG_Note, "Test completed");
     return 0;
 }
+
+
+/*
+ * --------------------------------------------------------------------------
+ * $Log: test_ncbi_sendmail.c,v $
+ * Revision 6.11  2003/12/09 15:39:30  lavr
+ * Added new test of custom-sized message body
+ *
+ * Revision 6.10  2003/12/05 18:39:35  lavr
+ * Test multiple recipients and as-is message
+ *
+ * Revision 6.9  2003/05/14 03:58:43  lavr
+ * Match changes in respective APIs of the tests
+ *
+ * Revision 6.8  2002/09/12 16:53:50  lavr
+ * Do not write '\0' into test file; log moved to end
+ *
+ * Revision 6.7  2002/04/15 19:21:45  lavr
+ * +#include "../test/test_assert.h"
+ *
+ * Revision 6.6  2002/03/22 19:48:58  lavr
+ * Removed <stdio.h>: included from ncbi_util.h or ncbi_priv.h
+ *
+ * Revision 6.5  2001/03/07 20:49:29  lavr
+ * Forgotten #include <string.h> added
+ *
+ * Revision 6.4  2001/03/06 04:32:31  lavr
+ * Custom header test added
+ *
+ * Revision 6.3  2001/03/02 20:01:53  lavr
+ * "../ncbi_priv.h" explained
+ *
+ * Revision 6.2  2001/02/28 17:48:07  lavr
+ * Huge body test added
+ *
+ * Revision 6.1  2001/02/28 00:53:45  lavr
+ * Initial revision
+ *
+ * ==========================================================================
+ */
