@@ -46,6 +46,39 @@
  *       GIF drawing functions
  *
  * $Log: gifgen.c,v $
+ * Revision 6.15  2001/03/21 18:54:57  juran
+ * Fix "for (...);" warnings by adding a space before the semicolon.
+ *
+ * Revision 6.14  2001/01/05 20:10:29  vakatov
+ * s_ComposeEffClip() -- fixed a typo (".lt" -> ".rb")
+ *
+ * Revision 6.13  2000/04/06 17:12:54  chetvern
+ * gdImageFilledPolygon
+ * zero section width case fixed
+ *
+ * Revision 6.12  1999/10/20 19:48:39  vakatov
+ * + gdImageGetDimensions()
+ *
+ * Revision 6.11  1999/08/13 21:23:08  vakatov
+ * Renamed "gd[Set|Get]ImageColor()" to "gdImage[Set|Get]Color()"
+ *
+ * Revision 6.10  1999/08/13 20:57:40  vakatov
+ * + gdSetImageColor()
+ * + gdImageGetAutoCropRectangle(), gdImageCrop(), gdImageAutoCrop()  <in
+ * a close collab. with S.Resenchuk & R.Tatusov>
+ * + static s_*Rect() to handle "gdRect"
+ *
+ * Revision 6.9  1999/05/20 21:32:45  vakatov
+ * s_DrawEllipse():  "ad hoc" patches for the case of zero radius.
+ * [_DEBUG] s_DrawConic():  just do nothing(and dont crash) if obtained
+ * a wrong octant
+ *
+ * Revision 6.8  1998/06/26 19:57:15  vakatov
+ * gdImageRoundRectangle():  check and adjust the passed args
+ *
+ * Revision 6.7  1998/06/15 22:02:53  vakatov
+ * Added gdImageSetClip() to allow clipping(rectangular area only)
+ *
  * Revision 6.6  1998/05/08 21:25:29  vakatov
  * Fixed a typo in the IMAGE_BOUNDS_SAFE macro(Roman Tatusov)
  *
@@ -108,8 +141,9 @@
 #define costScale 1024
 #define sintScale 1024
 
-#define IMAGE_BOUNDS_SAFE(im, x, y) \
-((x) >= 0  &&  (y) >= 0  &&  (x) < im->sx  &&  (y) < im->sy)
+#define IMAGE_BOUNDS_SAFE(im, X, Y) \
+((X) >= im->eff_clip.lt.x  &&  (Y) >= im->eff_clip.lt.y  && \
+ (X) <  im->eff_clip.rb.x  &&  (Y) <  im->eff_clip.rb.y)
 
 
 #define directSet(im, x, y, color) \
@@ -138,7 +172,7 @@
 /**************************************************************************/
 /* STATIC VARIABLE */
 /**************************************************************************/
-static int cost[] = {
+static const int cost[] = {
   1024, 1023, 1023, 1022, 1021, 1020, 1018,
   1016, 1014, 1011, 1008, 1005, 1001, 997,
   993, 989, 984, 979, 973, 968, 962,
@@ -193,7 +227,7 @@ static int cost[] = {
   1022, 1023, 1023 
 };
 
-static int sint[] = { 
+static const int sint[] = { 
   0, 17, 35, 53, 71, 89, 107,
   124, 142, 160, 177, 195, 212, 230,
   247, 265, 282, 299, 316, 333, 350,
@@ -251,6 +285,74 @@ static int sint[] = {
 /**************************************************************************/
 /* STATIC FUNCTION */
 /**************************************************************************/
+
+
+static void s_NormalizeRect(gdRect* rect)
+{
+  if (rect->lt.x > rect->rb.x) {
+    int x = rect->lt.x;
+    rect->lt.x = rect->rb.x;
+    rect->rb.x = x;
+  }
+
+  if (rect->lt.y > rect->rb.y) {
+    int y = rect->lt.y;
+    rect->lt.y = rect->rb.y;
+    rect->rb.y = y;
+  }
+}
+
+
+static void s_ExpandRect(gdRect* rect, int dX, int dY)
+{
+  s_NormalizeRect(rect);
+  rect->lt.x -= dX;
+  rect->rb.x += dX;
+  rect->lt.y -= dY;
+  rect->rb.y += dY;
+
+  if (rect->lt.x > rect->rb.x)
+    rect->lt.x = rect->rb.x = (rect->lt.x + rect->rb.x) / 2;
+  if (rect->lt.y > rect->rb.y)
+    rect->lt.y = rect->rb.y = (rect->lt.y + rect->rb.y) / 2;
+}
+
+
+static void s_CropRect(gdRect* rect, const gdRect* crop)
+{
+  s_NormalizeRect(rect);
+
+  if (rect->lt.x < crop->lt.x)
+    rect->lt.x = crop->lt.x;
+  if (rect->rb.x > crop->rb.x)
+    rect->rb.x = crop->rb.x;
+
+  if (rect->lt.y < crop->lt.y)
+    rect->lt.y = crop->lt.y;
+  if (rect->rb.y > crop->rb.y)
+    rect->rb.y = crop->rb.y;
+  
+}
+
+
+static void s_GetRectImage(gdRect* rect, const gdImage* im)
+{
+  rect->lt.x = 0;
+  rect->lt.y = 0;
+  rect->rb.x = (im ? im->sx : INT2_MAX) - 1;
+  rect->rb.y = (im ? im->sy : INT2_MAX) - 1;
+}
+
+
+static void s_CropRectImage(gdRect* rect, const gdImage* im)
+{
+  gdRect crop;
+  s_GetRectImage(&crop, im);
+  s_CropRect(rect, &crop);
+}
+
+
+
 static void 
 /*FCN*/gdImageBrushApply(gdImagePtr im, int x, int y)
 {
@@ -305,6 +407,12 @@ static void
    }
 }      
 
+#ifdef __cplusplus
+extern "C" {
+  static int LIBCALLBACK gdCompareInt(Nlm_VoidPtr a, Nlm_VoidPtr b);
+}
+#endif
+
 static 
 /*FCN*/int LIBCALLBACK gdCompareInt(Nlm_VoidPtr a, Nlm_VoidPtr b)
 {
@@ -327,7 +435,7 @@ static
  * Version 2: 6-Oct-95
  *      Bugfixes from Arne Steinarson <arst@ludd.luth.se>
  *      (ftp://ftp.dai.ed.ac.uk/pub/vision/src/conic-patch1.[ch])
- * Adopted and "beautified" for NCBI toolkit by D.Vakatov, Feb-98;
+ * Adopted and "beautified" for the NCBI toolkit by D.Vakatov, Feb-98;
  * and, s_GetOctant() rewritten to fix at the octant boundary points
  */
 
@@ -349,8 +457,8 @@ static int s_GetOctant(int gx, int gy)
   if (gy < 0)
     gy = -gy;
 
-  ASSERT ( 1 <= OCTANT[Xvs0][Yvs0][A_VS_B(gx, gy)] );
-  ASSERT ( 8 >= OCTANT[Xvs0][Yvs0][A_VS_B(gx, gy)] );
+  /* ASSERT ( 1 <= OCTANT[Xvs0][Yvs0][A_VS_B(gx, gy)] ); */
+  /* ASSERT ( 8 >= OCTANT[Xvs0][Yvs0][A_VS_B(gx, gy)] ); */
   return OCTANT[Xvs0][Yvs0][A_VS_B(gx, gy)];
 }
 
@@ -365,10 +473,10 @@ static void s_DrawConic
  /* call "draw_func" to draw a point */
  Nlm_DrawPoint draw_func, void *draw_data)
 {
-static const int DIAGx[] = {999, 1,  1, -1, -1, -1, -1,  1,  1};
-static const int DIAGy[] = {999, 1,  1,  1,  1, -1, -1, -1, -1};
-static const int SIDEx[] = {999, 1,  0,  0, -1, -1,  0,  0,  1};
-static const int SIDEy[] = {999, 0,  1,  1,  0,  0, -1, -1,  0};
+  static const int DIAGx[] = {999, 1,  1, -1, -1, -1, -1,  1,  1};
+  static const int DIAGy[] = {999, 1,  1,  1,  1, -1, -1, -1, -1};
+  static const int SIDEx[] = {999, 1,  0,  0, -1, -1,  0,  0,  1};
+  static const int SIDEy[] = {999, 0,  1,  1,  0,  0, -1, -1,  0};
 
   int octant;
   int dxS, dyS, dxD, dyD;
@@ -438,13 +546,13 @@ static const int SIDEy[] = {999, 0,  1,  1,  0,  0, -1, -1,  0};
     v = u - E;
     break;
   default:
-    ASSERT ( FALSE );
+    /* ASSERT ( FALSE );  cannot be drawn by this algorithm */
     return;
   }
   
   k1sign = dyS * dyD;
   k1 = 2 * (A + k1sign * (C - A));
-  Bsign = dxD*dyD;
+  Bsign = dxD * dyD;
   k2 = k1 + Bsign * B;
   k3 = 2 * (A + C + Bsign * B);
 
@@ -709,6 +817,22 @@ static void s_DrawPointFilledEllipse(int x, int y, void *data)
 }
 
 
+static void s_ComposeEffClip(gdImagePtr im)
+{
+  if ( !im->use_clip ) {
+    im->eff_clip.lt.x = im->eff_clip.lt.y = 0;
+    im->eff_clip.rb.x = im->sx;
+    im->eff_clip.rb.y = im->sy;
+    return;
+  }
+
+  im->eff_clip.lt.x = MAX(im->clip.lt.x, 0);
+  im->eff_clip.lt.y = MAX(im->clip.lt.y, 0);
+  im->eff_clip.rb.x = MIN(im->clip.rb.x + 1, im->sx);
+  im->eff_clip.rb.y = MIN(im->clip.rb.y + 1, im->sy);
+}
+
+
 /**************************************************************************/
 /* GLOBAL FUNCTIONS */
 /**************************************************************************/
@@ -731,6 +855,8 @@ NLM_EXTERN gdImagePtr
    im->transparent = (-1);
    im->interlace = 0;
    im->use_pattern = 0;
+   im->use_clip = 0;
+   s_ComposeEffClip(im);
    return im;
 }
 
@@ -823,10 +949,10 @@ NLM_EXTERN void
 }
 
 NLM_EXTERN int 
-/*FCN*/gdGetImageColor(gdImagePtr im, int color, int *r, int *g, int *b)
+/*FCN*/gdImageGetColor(gdImagePtr im, int color, int *r, int *g, int *b)
 {
-  ASSERT ( color >= 0 ); 
-  if (color >= im->colorsTotal  ||  im->open[color])
+  ASSERT ( color >= 0 );
+  if (color < 0  ||  color >= im->colorsTotal  ||  im->open[color])
     return FALSE;
 
   if ( r )
@@ -839,6 +965,20 @@ NLM_EXTERN int
 }
 
 
+NLM_EXTERN int 
+/*FCN*/gdImageSetColor(gdImagePtr im, int color, int r, int g, int b)
+{
+  ASSERT ( color >= 0 );
+  if (color < 0  ||  color >= im->colorsTotal  ||  im->open[color])
+    return FALSE;
+
+  im->red  [color] = r;
+  im->green[color] = g;
+  im->blue [color] = b;
+  return TRUE;
+}
+
+
 NLM_EXTERN void 
 /*FCN*/gdImageColorTransparent(gdImagePtr im, int color)
 {
@@ -1218,10 +1358,9 @@ NLM_EXTERN void
     {
       /* filled arc */
       SStorePoint sp_arc;
-
-      sp_arc.xdim = im->sx;
-      sp_arc.ydim = im->sy;
-      sp_arc.n = 0;
+      sp_arc.xdim   = im->sx;
+      sp_arc.ydim   = im->sy;
+      sp_arc.n      = 0;
       sp_arc.points = (SPoint *)Nlm_Malloc(2 * (sp_arc.xdim + sp_arc.ydim));
       s_DrawConic(A, B, C, D, E, F, xs, ys, xe, ye, s_StorePoint, &sp_arc);
       if (!sp_arc.n  &&  (A + B + C + D + E + F > 0))
@@ -1238,6 +1377,31 @@ static void s_DrawEllipse(gdImagePtr im,
   int A, B, C, D, E, F;
   int xs, ys, xe, ye;
 
+  if (!rx  &&  !ry) {
+    gdImageSetPixel(im, cx, cy, color);  
+    return;
+  }
+
+  if ( !rx ) {
+    ys = (quadrant == 0  ||  quadrant == 3  ||  quadrant == 4) ?
+      cy + ry : cy;
+    ye = (quadrant == 0  ||  quadrant == 1  ||  quadrant == 2) ?
+      cy - ry : cy;
+    gdImageLine(im, cx, ys, cx, ye, color);
+    return;
+  }
+
+  if ( !ry ) {
+    xs = (quadrant == 0  ||  quadrant == 1  ||  quadrant == 4) ?
+      cx + rx : cx;
+    xe = (quadrant == 0  ||  quadrant == 2  ||  quadrant == 3) ?
+      cx - rx : cx;
+    gdImageLine(im, xs, cy, xe, cy, color);
+    return;
+  }
+
+  ASSERT( rx > 0  &&  ry > 0 );
+
   A = ry * ry;
   B = 0;
   C = rx * rx;
@@ -1306,10 +1470,23 @@ NLM_EXTERN void
                              int oval_w, int oval_h,
                              int color, Nlm_Boolean fill)
 {
-  int X1 = x1 + oval_w;
-  int Y1 = y1 + oval_h;
-  int X2 = x2 - oval_w;
-  int Y2 = y2 - oval_h;
+  int X1 = MIN(x1, x2);
+  int Y1 = MIN(y1, y2);
+  int X2 = MAX(x1, x2);
+  int Y2 = MAX(y1, y2);
+
+  int w = x2 - x1 + 1;
+  int h = y2 - y1 + 1;
+
+  if (oval_w > w/2)
+    oval_w = w/2;
+  if (oval_h > h/2)
+    oval_h = h/2;
+
+  x1 = X1;  y1 = Y1;  x2 = X2;  y2 = Y2;  
+
+  X1 = X1 + oval_w;  Y1 = Y1 + oval_h;
+  X2 = X2 - oval_w;  Y2 = Y2 - oval_h;
 
   if ( fill ) {
     gdImageFilledRectangle(im, x1, Y1, x2, Y2, color);
@@ -1455,13 +1632,13 @@ NLM_EXTERN void
          }
       }
 
-      HeapSort((Nlm_VoidPtr)im->polyInts, ints, sizeof(int), &gdCompareInt);
-      {{
+      if (ints > 1) {
         size_t j;
+        HeapSort((Nlm_VoidPtr)im->polyInts, ints, sizeof(int), &gdCompareInt);
         for (j = 0;  j < (ints-1);  j += 2) {
-           gdImageLine(im, im->polyInts[j], y, im->polyInts[j+1], y, c);
+          gdImageLine(im, im->polyInts[j], y, im->polyInts[j+1], y, c);
         }
-      }}
+      }
    }
 }
    
@@ -1654,4 +1831,142 @@ NLM_EXTERN void
 	free(sty);
 }
 
+
+NLM_EXTERN int gdImageSetClip(gdImagePtr im,
+                              const gdRect *clip, gdRect *old_clip)
+{
+  static const gdRect empty_clip = { {0, 0}, {0, 0} };
+
+  int was_clip = im->use_clip;
+  if ( old_clip ) {
+    if ( im->use_clip )
+      MemCpy(old_clip, &im->clip, sizeof(gdRect));
+    else
+      MemSet(old_clip, '\0', sizeof(gdRect));
+  }
+
+  if (clip  &&  MemCmp(&clip, &empty_clip, sizeof(gdRect))) {
+    im->clip.lt.x = MIN(clip->lt.x, clip->rb.x);
+    im->clip.lt.y = MIN(clip->lt.y, clip->rb.y);
+    im->clip.rb.x = MAX(clip->lt.x, clip->rb.x);
+    im->clip.rb.y = MAX(clip->lt.y, clip->rb.y);
+    im->use_clip = 1;
+  } else {
+    im->use_clip = 0;
+  }
+
+  s_ComposeEffClip(im);
+
+  return was_clip;
+}
+
+
+NLM_EXTERN void gdImageGetDimensions(gdImagePtr im, int* width, int* height)
+{
+  if ( width )
+    *width = im->sx;
+  if ( height )
+    *height = im->sy;
+}
+
+
+NLM_EXTERN void gdImageGetAutoCropRectangle(gdImagePtr im, gdRect* rect)
+{
+  int width  = im->sx;
+  int height = im->sy;
+  int x, y, xb, yb, ye;
+  Uint1 *base_ptr, *ptr;
+
+  /* find upper colored pixel (CP);  background is zero */
+  for (y = 0, base_ptr = im->pixels;  y < height;  y++, base_ptr += width) {
+    for (x = 0, ptr = base_ptr;  x < width  &&  !*ptr;  x++, ptr++) ;
+    if (x < width)
+      break;
+  }
+  rect->lt.y = yb = y;
+
+  /* Found no CPs at all? */
+  if (y == height) {
+    rect->lt.x = rect->lt.y = 0;
+    rect->rb.x = MAX(im->sx-1, 0);
+    rect->rb.y = MAX(im->sy-1, 0);
+    return;
+  }
+
+  /* find lower CP */
+  base_ptr = im->pixels + width * (height - 1);
+  for (y = height-1;  y > yb;  y--, base_ptr -= width) {
+    for (x = 0, ptr = base_ptr;  x < width  &&  !*ptr;  x++, ptr++) ;
+    if (x < width)
+      break;
+  }
+  rect->rb.y = ye = y;
+
+  /* find left CP */
+  base_ptr = im->pixels + width * yb;
+  for (x = 0;  x < width;  x++, base_ptr++) {
+    for (y = yb, ptr = base_ptr;  y <= ye  &&  !*ptr;  y++, ptr += width) ;
+    if (y <= ye)
+      break;
+  }
+  rect->lt.x = xb = x;
+
+  /* find right CP */
+  base_ptr = im->pixels + width * (ye + 1) - 1;
+  for (x = width-1;  x > xb;  x--, base_ptr--) {
+    for (y = ye, ptr = base_ptr;  y >= yb  &&  !*ptr;  y--, ptr -= width) ;
+    if (y >= yb)
+      break;
+  }
+  rect->rb.x = x;
+}
+
+
+
+NLM_EXTERN void gdImageCrop(gdImagePtr im, const gdRect* rect)
+{
+  Uint1* base_ptr = im->pixels;
+  gdRect crop     = *rect;
+  int    width    = im->sx;
+
+  int    new_width;
+  int    y;
+  Uint1* ptr;
+   
+  /* adjust crop rectangle to fit image boundaries */
+  s_CropRectImage(&crop, im);
+
+  /* nothing to crop;  or crop without memcpy? */
+  if (crop.lt.x == 0  &&  crop.lt.y == 0  &&  crop.rb.x == im->sx - 1) {
+    im->sy = crop.rb.y + 1;
+    return;
+  }
+
+  /* "crop" -- move the content of crop rectangle to (0, 0) */
+  new_width = crop.rb.x - crop.lt.x + 1;
+  ptr = base_ptr + crop.lt.y * width + crop.lt.x;
+  for (y = crop.lt.y;  y <= crop.rb.y;  y++) {
+    memcpy(base_ptr, ptr, new_width);
+    base_ptr += new_width;
+    ptr  += width;
+  }
+
+  /* alternate image width/height */
+  im->sx = new_width;
+  im->sy = crop.rb.y - crop.lt.y + 1;
+}
+
+
+NLM_EXTERN void gdImageAutoCrop(gdImagePtr im, int border)
+{
+  gdRect crop;
+  if (border < 0)
+    border = -border;
+
+  gdImageGetAutoCropRectangle(im, &crop);
+  s_ExpandRect(&crop, border, border);
+  gdImageCrop(im, &crop);
+}
+
+
 /*EOF*/
