@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/7/91
 *
-* $Revision: 6.25 $
+* $Revision: 6.37 $
 *
 * File Description:
 *       portable environment functions, companions for ncbimain.c
@@ -37,6 +37,45 @@
 * Modifications:
 * --------------------------------------------------------------------------
 * $Log: ncbienv.c,v $
+* Revision 6.37  2004/08/06 20:56:27  kans
+* G/SetAppParam on PC no longer uses PrivateProfileString functions - instead it checks USERPROFILE, SYSTEMROOT, and then NCBI environment variables
+*
+* Revision 6.36  2004/02/11 18:40:00  kans
+* enhanced GetOpSysString to report specific version of MS Windows
+*
+* Revision 6.35  2003/09/19 15:49:20  coulouri
+* NetBSD fixes
+*
+* Revision 6.34  2003/06/24 19:45:39  ucko
+* GCC/Darwin: Restore Nlm_IsApplicationPackage and the skeleton of
+* Nlm_SetupArguments_ST_Mac.
+*
+* Revision 6.33  2003/06/24 15:39:42  ucko
+* Switch back to the generic Unix ProgramPath code when building Darwin
+* binaries with GCC until we find a framework that defines the relevant
+* symbols without breaking support for remote execution.
+*
+* Revision 6.32  2003/06/06 15:01:03  rsmith
+* fixed ProgramPath for OS Darwin & Mac, Added IsApplicationPackage for Darwin. ProgramPath on Darwin now returns the seen application not the executable buried in the package.
+*
+* Revision 6.31  2003/01/29 20:57:36  kans
+* added linux, sgi, and solaris intel to GetOpSysString
+*
+* Revision 6.30  2003/01/29 19:37:14  kans
+* added GetOpSysString to return allocated string describing operating system running the program
+*
+* Revision 6.29  2003/01/27 14:54:24  kans
+* for darwin, also check /Contents/Resources/ of Mac OS X package for initial prepackaged config file
+*
+* Revision 6.28  2003/01/17 20:59:02  kans
+* for Mach-O, try home/Library/Preferences/xxx.cnf first
+*
+* Revision 6.27  2002/10/03 16:22:03  kans
+* changed fgets to Nlm_FileGets
+*
+* Revision 6.26  2002/03/28 13:29:08  kans
+* checks for OS_UNIX_DARWIN (EN)
+*
 * Revision 6.25  2001/08/02 14:44:10  vakatov
 * [OSF1]  NLM_XOPEN_SOURCE_500:: Kludge-fix for the weak-minded native
 * preprocessor
@@ -267,7 +306,6 @@ static void Nlm_TransientLogSetApp(const Nlm_Char* file, const Nlm_Char* section
 static void Nlm_FreeEnvData(Nlm_env_sectPtr esp);
 static void Nlm_FreeTransientData(void);
 
-#ifndef OS_MSWIN
 static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm_Boolean create);
 static Nlm_Char* Nlm_TrimString(Nlm_Char* str);
 static Nlm_Boolean Nlm_ReadConfigFile(FILE* fp);
@@ -521,7 +559,7 @@ static Nlm_Boolean s_GetHomeByUID(Nlm_Char* buf, size_t buf_size)
     struct passwd* pwd_ptr = 0;
 
     /* Get the info using user ID */
-#if  (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN)
+#if  (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN) && !defined(OS_UNIX_NETBSD)
     struct passwd pwd;
     Nlm_Char      pwd_buffer[LOGNAME_MAX + PATH_MAX + 1024 + 1];
 
@@ -551,7 +589,7 @@ static Nlm_Boolean s_GetHomeByLOGIN(Nlm_Char* buf, Nlm_Int2 buf_size)
     struct passwd* pwd_ptr = 0;
 
     /* Get the user login name */
-#if (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN)
+#if (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN) && !defined(OS_UNIX_NETBSD)
     struct passwd pwd;
     Nlm_Char      login_name[LOGNAME_MAX + 1];
     Nlm_Char      pwd_buffer[LOGNAME_MAX + PATH_MAX + 1024 + 1];
@@ -571,7 +609,7 @@ static Nlm_Boolean s_GetHomeByLOGIN(Nlm_Char* buf, Nlm_Int2 buf_size)
     if ( !ok )
         return FALSE;
 
-#if (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN)
+#if (defined(SOLARIS_THREADS_AVAIL) || defined(POSIX_THREADS_AVAIL)) && !defined(OS_UNIX_FREEBSD) && !defined(OS_UNIX_DARWIN) && !defined(OS_UNIX_NETBSD)
     pwd_ptr = &pwd;
 #  if NLM_POSIX1B
     if (getpwnam_r(login_name, &pwd, pwd_buffer, sizeof(pwd_buffer),
@@ -645,6 +683,7 @@ static Nlm_Boolean Nlm_GetHome(Nlm_Char* buf, Nlm_Int2 buflen)
 static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm_Boolean create)
 
 {
+  Nlm_Char  ch;
   FILE      *fp;
   Nlm_Int2  i;
   Nlm_Int2  len;
@@ -656,6 +695,50 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
 
   fp = NULL;
   if (file != NULL) {
+#ifdef OS_UNIX_DARWIN
+    /* For Mach-O executables, check username/Library/Preferences/xxx.cnf first */
+    Nlm_StringNCpy_0(str, file, sizeof(str) - 4);
+    if ( ! Nlm_Qualified (str) ) {
+        /* if the user has already supplied a name with .xxx use that name
+         * otherwise add the .cnf here */
+        Nlm_StringCat(str, ".cnf");
+    }
+    /* if the name isn't all lowercase, make it so now */
+    len = (Nlm_Int2) Nlm_StringLen (str);
+    for (i = 0; i < len; i++) {
+      ch = str [i];
+      str [i] = TO_LOWER (ch);
+    }
+    if (Nlm_GetHome (path, sizeof (path))) {
+      Nlm_FileBuildPath(path, "Library", NULL);
+      Nlm_FileBuildPath(path, "Preferences", NULL);
+      Nlm_FileBuildPath(path, NULL, str);
+      fp = Nlm_FileOpen (path, "r");
+      if (fp == NULL && create) {
+        fp = Nlm_FileOpen (path, "w");
+        Nlm_FileClose (fp);
+        fp = Nlm_FileOpen (path, "r");
+      }
+      if (writeMode && fp != NULL) {
+        Nlm_FileClose (fp);
+        fp = Nlm_FileOpen (path, "w");
+      }
+      if (fp != NULL) {
+        return fp;
+      }
+    }
+    /* also check within Contents/Resources of Mac OS X package */
+    ProgramPath (path, sizeof (path));
+    if (IsApplicationPackage(path)) {
+      FileBuildPath(path, "Contents", NULL);
+      FileBuildPath (path, "Resources", NULL);
+      Nlm_FileBuildPath (path, NULL, str);
+      fp = Nlm_FileOpen (path, "r");
+      if (fp != NULL) {
+        return fp;
+      }
+    }
+#endif
     dontUseLocalConfig = getenv("NCBI_DONT_USE_LOCAL_CONFIG") != NULL;
     newfp = NULL;
     Nlm_StringMove(str, ".");
@@ -666,7 +749,8 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     }
     len = (Nlm_Int2) Nlm_StringLen (str);
     for (i = 0; i < len; i++) {
-      str [i] = TO_LOWER (str [i]);
+      ch = str [i];
+      str [i] = TO_LOWER (ch);
     }
     Nlm_StringNCpy_0(path, str, sizeof(path));
 
@@ -717,8 +801,111 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
 
 #endif /* OS_UNIX */
 
+#ifdef OS_MSWIN
+/*****************************************************************************
+*
+*   Nlm_OpenConfigFile (file, writeMode, create)
+*      returns a file pointer to the specified configuration file.
+*      1)  looks in the current directory for "file.ini", but will not
+*          create a new file in this directory.
+*      2)  then looks in the home directory for ".filerc".
+*      3)  then looks for an environment variable "NCBI" and takes its
+*          value as a complete path to a directory containing the
+*          configuration file "filerc" or ".filerc".
+*
+*      Steps (1) and (2) above are omitted if the NCBI_DONT_USE_LOCAL_CONFIG
+*      environment variable is set.  This can be used to allow specific
+*      production applications to avoid stray .ncbirc files which may have
+*      been erroneously generated.
+*
+*****************************************************************************/
 
-#ifdef OS_MAC
+static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm_Boolean create)
+
+{
+  Nlm_Char  ch;
+  FILE      *fp;
+  Nlm_Int2  i;
+  Nlm_Int2  len;
+  FILE      *newfp;
+  Nlm_Char  path [PATH_MAX+1];
+  Nlm_Char  str [FILENAME_MAX+1];
+  Nlm_CharPtr  tmp;
+
+  fp = NULL;
+  newfp = NULL;
+
+  if (file != NULL) {
+
+    /* normalize file name */
+    Nlm_StringNCpy_0(str, file, sizeof(str) - 4);
+    if ( ! Nlm_Qualified (str) ) {
+        /* if the user has already supplied a name with .xxx use that name
+         * otherwise add the .ini here */
+        Nlm_StringCat(str, ".INI");
+    }
+    /* if the name isn't all uppercase, make it so now */
+    len = (Nlm_Int2) Nlm_StringLen (str);
+    for (i = 0; i < len; i++) {
+      ch = str [i];
+      str [i] = TO_UPPER (ch);
+    }
+
+    /* first try user directory */
+
+    tmp = getenv ("USERPROFILE");
+    if (tmp != NULL && *tmp != '\0') {
+      StringNCpy_0 (path, tmp, sizeof (path));
+      Nlm_FileBuildPath(path, NULL, str);
+      fp = Nlm_FileOpen (path, "r");
+      if (fp == NULL && create) {
+        newfp = Nlm_FileOpen (path, "w");
+        Nlm_FileClose (newfp);
+        newfp = Nlm_FileOpen (path, "r");
+      }
+    }
+
+    /* next try c:\winnt for backward compatibility - read only */
+
+    if (fp == NULL) {
+      tmp = getenv ("SYSTEMROOT");
+      if (tmp != NULL && *tmp != '\0') {
+        StringNCpy_0 (path, tmp, sizeof (path));
+        Nlm_FileBuildPath(path, NULL, str);
+        fp = Nlm_FileOpen (path, "r");
+      }
+    }
+
+    /* last try environment variable path - read only */
+
+    if (fp == NULL) {
+      tmp = getenv ("NCBI");
+      if (tmp != NULL && *tmp != '\0') {
+        StringNCpy_0 (path, tmp, sizeof (path));
+        Nlm_FileBuildPath(path, NULL, str);
+        fp = Nlm_FileOpen (path, "r");
+      }
+    }
+
+    if (newfp != NULL) {
+      if (fp != NULL) {
+        Nlm_FileClose (newfp);
+        newfp = NULL;
+      } else {
+        fp = newfp;
+      }
+    }
+    if (writeMode && fp != NULL) {
+      Nlm_FileClose (fp);
+      fp = Nlm_FileOpen (path, "w");
+    }
+  }
+  return fp;
+}
+#endif /* OS_MSWIN */
+
+
+#if defined(OS_MAC) && !defined(OS_UNIX_DARWIN)
 /*****************************************************************************
  *
  *   Nlm_OpenConfigFile (file, writeMode, create)
@@ -753,6 +940,7 @@ Nlm_OpenConfigFile(const Nlm_Char* file,
                    Nlm_Boolean writeMode,
                    Nlm_Boolean create )
 {
+    Nlm_Char    ch;
     Nlm_Char    str [FILENAME_MAX+1];
     Nlm_Int2    len;
     long        gesResponse;
@@ -778,7 +966,8 @@ Nlm_OpenConfigFile(const Nlm_Char* file,
     /* if the name isn't all lowercase, make it so now */
     len = (Nlm_Int2) Nlm_StringLen (str);
     for (i = 0; i < len; i++) {
-      str [i] = TO_LOWER (str [i]);
+      ch = str [i];
+      str [i] = TO_LOWER (ch);
     }
     
     /* convert to pascal string for Mac toolbox */
@@ -845,7 +1034,7 @@ Nlm_OpenConfigFile(const Nlm_Char* file,
     }
   return fp;
 }
-#endif /* OS_MAC */
+#endif /* defined(OS_MAC) && !defined(OS_UNIX_DARWIN) */
 
 #ifdef OS_VMS
 /*****************************************************************************
@@ -877,6 +1066,7 @@ static Nlm_Boolean Nlm_GetHome(Nlm_Char* buf, Nlm_Int2 buflen)
 static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm_Boolean create)
 
 {
+  Nlm_Char  ch;
   FILE      *fp;
   Nlm_Int2  i;
   Nlm_Int2  len;
@@ -895,7 +1085,8 @@ static FILE* Nlm_OpenConfigFile(const Nlm_Char* file, Nlm_Boolean writeMode, Nlm
     }
     len = (Nlm_Int2) Nlm_StringLen (str);
     for (i = 0; i < len; i++) {
-      str [i] = TO_LOWER (str [i]);
+      ch = str [i];
+      str [i] = TO_LOWER (ch);
     }
     Nlm_StringNCpy_0(path, str, sizeof(path));
 
@@ -1033,7 +1224,7 @@ static Nlm_Boolean Nlm_ReadConfigFile(FILE* fp)
     eip = NULL;
     lastEip = NULL;
     comment = NULL;
-    while (fgets (str, sizeof (str), fp)) {
+    while (Nlm_FileGets (str, sizeof (str), fp)) {
       ch = *str;
       if (ch != '\n' && ch != '\r') {
         if (ch == ';') { /* comment */
@@ -1328,89 +1519,6 @@ static void Nlm_PutComment(const Nlm_Char* s, FILE* fp)
     fputs(s, fp);
 }
 
-#else /* ndef OS_MSWIN */
-
-static void        Nlm_FlushAppParam_ST(void) {}
-static Nlm_Boolean Nlm_CacheAppParam_ST(Nlm_Boolean value)  { return TRUE; }
-
-/*****************************************************************************
-*
-* The "guts" of:
-*   Nlm_GetAppParam (file, section, type, buf, buflen)
-*      finds parameters from configuration files
-*      if configuration file is found, trys to read the parameter from it.
-*
-*****************************************************************************/
-static Nlm_Int2 Nlm_WorkGetAppParam(const Nlm_Char* file, const Nlm_Char* section, const Nlm_Char* type, const Nlm_Char* dflt, Nlm_Char* buf, Nlm_Int2 buflen, Nlm_Boolean searchTransient)
-{
-  static char _empty_string[] = "";
-  Nlm_Char path[PATH_MAX + 1];
-
-  if (buf == NULL  ||  buflen <= 0)
-    return 0;
-
-  *buf = '\0';
-  if (searchTransient  &&
-      Nlm_TransientLookup(file, section, type, dflt, buf, buflen))
-    {
-      return (Nlm_Int2)Nlm_StringLen(buf);
-    }
-
-  if ( dflt )
-    Nlm_StringNCpy_0(buf, dflt, buflen);
-
-  if (file != NULL && *file != '\0' && section != NULL && *section != '\0')
-    {
-      Nlm_StringNCpy_0(path, file, sizeof(path) - 4);
-      if ( !Nlm_Qualified( path ) )
-        Nlm_StringCat(path, ".INI");
-      if (dflt == NULL)
-        dflt = _empty_string;  /* can't use NULL, must be empty string */
-      return (Nlm_Int2)GetPrivateProfileString(section,
-                                               type, dflt, buf, buflen, path);
-    }
-  else
-    return (Nlm_Int2)Nlm_StringLen( buf );
-}
-
-/*****************************************************************************
-*
-*   Nlm_SetAppParam (file, section, type, value)
-*      finds paths for types of data and fills in path in buf
-*      if configuration file is found, trys to write the parameter to it.
-*
-*****************************************************************************/
-
-static Nlm_Boolean Nlm_SetAppParam_ST(const Nlm_Char* file, const Nlm_Char* section, const Nlm_Char* type, const Nlm_Char* value)
-{
-  Nlm_Char     path [PATH_MAX+1];
-  Nlm_Boolean  rsult;
-
-  rsult = FALSE;
-  if (file != NULL && *file != '\0' && section != NULL && *section != '\0') {
-    Nlm_StringNCpy_0(path, file, sizeof(path) - 4);
-    if ( ! Nlm_Qualified (path) ) {
-      Nlm_StringCat (path, ".INI");
-    }
-    Nlm_TransientLogSetApp (file, section, type, value);
-    if (WritePrivateProfileString (section, type, value, path)) {
-      rsult = TRUE;
-    }
-  }
-  return rsult;
-}
-
-/*****************************************************************************
-*   Nlm_FreeConfigStruct ()
-*      frees parameter structure in memory
-*****************************************************************************/
-static void Nlm_FreeConfigStruct_ST(void)
-{
-  Nlm_FreeTransientData ();
-}
-
-#endif /* else !OS_MSWIN */
-
 
 /*****************************************************************************
 *   Nlm_Qualified ()
@@ -1785,7 +1893,7 @@ static int    targc = 0;
 static char **targv = NULL;
 
 
-#ifdef WIN_MAC
+#if defined(WIN_MAC)
 static FSSpec       apFileSpec;
 static Str255       apName;
 static Handle       apParam;
@@ -1793,21 +1901,67 @@ static short        apRefNum;
 
 static Nlm_Boolean Nlm_SetupArguments_ST_Mac(void)
 {
+#ifndef __GNUC__
+/* At least in 10.1, this seems to introduce an unwanted dep. on Carbon. */
   ProcessInfoRec       pirec;
   ProcessSerialNumber  psn;
 
   GetCurrentProcess (&psn);
   pirec.processInfoLength = sizeof (ProcessInfoRec);
-  pirec.processName = NULL;
+  pirec.processName = apName;
   pirec.processAppSpec = &apFileSpec;
   GetProcessInformation (&psn, &pirec);
   Nlm_PtoCstr ((Nlm_Char*) apFileSpec.name);
+  Nlm_PtoCstr ((Nlm_Char*) apName);
 
   SetAppProperty("ProgramName",(void*)apName);
+#endif
   return TRUE;
 }
 
+#if defined(OS_UNIX_DARWIN)
+#ifndef __GNUC__
+/* At least in 10.1, this seems to introduce an unwanted dep. on Carbon. */
+static void Nlm_ProgramPath_ST(Nlm_Char* appPath, size_t pathSize)
+{
+    OSErr               err;
+    ProcessSerialNumber psn;
+    
+    if (appPath != NULL) {
+        FSRef       fsRef;
+        appPath[0] = '\0';
+        GetCurrentProcess (&psn);
+        err = GetProcessBundleLocation (&psn, &fsRef);
+        if (err == noErr) {
+            FSRefMakePath (&fsRef, (UInt8 *) appPath, pathSize);
+        }
+    }
+}
+#endif
 
+/*
+  is the application at filePath actually a application bundle/package?
+  i.e. a folder containing subfolders, resource files and the actual executable.
+  Pass the value returned by ProgramPath for best results.
+*/
+NLM_EXTERN Nlm_Boolean Nlm_IsApplicationPackage(char *filePath)
+{
+    OSErr   err;
+    char    aPath[1024];
+    FSRef   contentsFRef;
+    Boolean isDirectory;
+    
+    StrCpy(aPath, filePath);
+    FileBuildPath(aPath, "Contents", NULL);
+    err = FSPathMakeRef ((unsigned char *) aPath, &contentsFRef, &isDirectory);
+    if (err == noErr  &&  isDirectory) {
+        return TRUE;   
+    }
+    /* else gets err == -120, no such directory. */
+    return FALSE;
+}
+
+#else
 static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
 {
   CInfoPBRec  block;
@@ -1840,7 +1994,8 @@ static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
     }
   }
 }
-#endif /* WIN_MAC */
+#endif /* defined(OS_UNIX_DARWIN) */
+#endif /* defined(WIN_MAC) */
 
 
 #if defined(OS_MSWIN) || defined(OS_VMS)
@@ -1856,7 +2011,7 @@ static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
 #endif  /* OS_MSWIN || OS_VMS */
 
 
-#ifdef OS_UNIX
+#if defined(OS_UNIX)  &&  (defined(__GNUC__)  ||  !defined(OS_UNIX_DARWIN))
 static void Nlm_ProgramPath_ST(Nlm_Char* buf, size_t maxsize)
 {
   Nlm_Char     path [PATH_MAX];
@@ -1963,8 +2118,8 @@ NLM_EXTERN Nlm_Boolean Nlm_ParseCmdLineArguments
     size_t size = strlen(prog_name) + strlen(cmd_line) + 2;
     str = (char*) Nlm_MemNew(size);
 
-    strcpy(str, prog_name);
-    strcpy(str + strlen(prog_name) + 1, cmd_line);
+    StrCpy(str, prog_name);
+    StrCpy(str + strlen(prog_name) + 1, cmd_line);
   }}
 
   /* Count command-line arguments and separate them by '\0' */
@@ -2033,7 +2188,9 @@ NLM_EXTERN void Nlm_SetupArguments(int argc, char *argv[])
 {
   NlmMutexLockEx( &corelibMutex );
   wasSetup = TRUE;
-#if defined(OS_UNIX)
+#if defined(WIN_MAC)
+  wasSetup = Nlm_SetupArguments_ST_Mac();
+#elif defined(OS_UNIX)
   {{
     char *p;
     if ((p = strrchr(argv[0],DIRDELIMCHR)) != NULL)  p++;
@@ -2041,8 +2198,6 @@ NLM_EXTERN void Nlm_SetupArguments(int argc, char *argv[])
       p = argv[0];
     SetAppProperty("ProgramName", (void*)p);  
   }}
-#elif defined(WIN_MAC)
-  wasSetup = Nlm_SetupArguments_ST_Mac();
 #endif
   targc = argc;
   targv = argv;
@@ -2197,3 +2352,78 @@ NLM_EXTERN Nlm_Boolean Nlm_FreeArgs(Nlm_Int2 numargs, Nlm_ArgPtr ap)
     }
   return TRUE;
 }
+
+NLM_EXTERN Nlm_CharPtr Nlm_GetOpSysString (void)
+
+{
+  Nlm_CharPtr  str = "unknown";
+
+#if defined(OS_MAC) && !defined(OS_UNIX_DARWIN)
+  long  sysVer;
+
+  if ( Gestalt (gestaltSystemVersion, &sysVer) == noErr) {
+    /* system version in low order word is hexadecimal */
+    if (sysVer >= 4096) {
+      str = "MAC Carbon on OS X";
+    } else if (sysVer >= 2304) {
+      str = "MAC Carbon on OS 9";
+    } else {
+      str = "MAC Carbon on OS 8";
+    }
+  }
+#endif
+
+#ifdef OS_UNIX
+  /* initial nonspecific UNIX string */
+  str = "UNIX";
+#ifdef OS_UNIX_DARWIN
+  str = "MAC UNIX on OS X";
+#endif
+#ifdef OS_UNIX_SYSV
+  str = "SYSV UNIX";
+#endif
+
+#ifdef OS_UNIX_SOL
+  str = "SOLARIS UNIX";
+#ifdef PROC_I80X86
+  str = "SOLARIS INTEL UNIX";
+#endif
+#endif
+
+#ifdef OS_UNIX_SUN
+  str = "SUN UNIX";
+#endif
+#ifdef OS_UNIX_IRIX
+  str = "SGI UNIX";
+#endif
+#ifdef OS_UNIX_LINUX
+  str = "LINUX UNIX";
+#endif
+#endif
+
+#ifdef OS_MSWIN
+  DWORD  version, lowbyte;
+
+  str = "MS WINDOWS";
+  version = GetVersion ();
+  lowbyte = (version & 0x0000FF);
+  if ((version & 0x80000000) == 0) {
+    if (lowbyte == 5) {
+      str = "MS WINDOWS 2000/XP";
+    } else if (lowbyte == 4) {
+      str = "MS WINDOWS NT 4.0";
+    } else if (lowbyte == 3) {
+      str = "MS WINDOWS NT 3.51";
+    }
+  } else {
+    if (lowbyte == 4) {
+      str = "MS WINDOWS 95/98/Me";
+    } else if (lowbyte == 3) {
+      str = "MS WINDOWS 3.1";
+    }
+  }
+#endif
+
+  return Nlm_StringSave (str);
+}
+
