@@ -1,4 +1,6 @@
-/*  $RCSfile: blastclust.c,v $  $Revision: 6.28 $  $Date: 2001/11/16 20:34:17 $
+static char const rcsid[] = "$Id: blastclust.c,v 6.45 2004/08/02 20:08:49 dondosha Exp $";
+
+/*  $RCSfile: blastclust.c,v $  $Revision: 6.45 $  $Date: 2004/08/02 20:08:49 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -26,10 +28,63 @@
 * Author: Ilya Dondoshansky 
 *
 * File Description:
-*   This is server which does neighboring searches
+*   Program to perform a single-linkage clustering of a set of protein or DNA 
+*   sequences. See file README.bcl for description
 *
 * ---------------------------------------------------------------------------
 * $Log: blastclust.c,v $
+* Revision 6.45  2004/08/02 20:08:49  dondosha
+* Write and read header integers separately instead of as a structure; changed first number from 1 byte to 4, to prevent padding with garbage
+*
+* Revision 6.44  2004/06/30 21:03:57  madden
+* Add include for blfmtutl.h
+*
+* Revision 6.43  2004/05/03 19:59:48  dondosha
+* Display version number when run with no arguments
+*
+* Revision 6.42  2004/02/24 15:18:15  dondosha
+* Fixed several memory leaks for nucleotide clustering
+*
+* Revision 6.41  2004/02/18 20:42:59  dondosha
+* Cleaned up how id strings are retrieved for output
+*
+* Revision 6.40  2004/02/18 15:18:46  dondosha
+* Minor fix when freeing title strings
+*
+* Revision 6.39  2003/07/22 17:59:48  dondosha
+* Added call to BlastErrorPrint to make warnings more informative
+*
+* Revision 6.38  2003/06/13 19:59:29  dondosha
+* Fixed 2 purify errors
+*
+* Revision 6.37  2003/05/30 17:31:09  coulouri
+* add rcsid
+*
+* Revision 6.36  2003/05/13 16:02:42  coulouri
+* make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+*
+* Revision 6.35  2003/05/06 20:21:13  dondosha
+* Typo correction
+*
+* Revision 6.34  2003/05/06 20:19:45  dondosha
+* Changed a confusing error message to a meaningful one
+*
+* Revision 6.33  2002/10/08 15:32:42  dondosha
+* Corrected file description comment in the NCBI header
+*
+* Revision 6.32  2002/09/03 16:01:11  dondosha
+* Introduced a restriction on the total length of concatenated sequences for nucleotide case
+*
+* Revision 6.31  2002/05/30 16:20:16  dondosha
+* 1. Correction in evaluation of hits for DNA case,
+* 2. Added word size parameter to make DNA clustering more flexible
+*
+* Revision 6.30  2002/03/14 21:07:11  dondosha
+* Slight improvement in finding the used id list from a gi list when reclustering
+*
+* Revision 6.29  2002/02/11 21:32:10  dondosha
+* Free Bioseqs after use to avoid accumulation
+*
 * Revision 6.28  2001/11/16 20:34:17  dondosha
 * Bug fix: BioseqUnlock was missing; also added ObjMgrFreeCache calls
 *
@@ -130,6 +185,7 @@
 #include <sqnutils.h>
 #include <mbalign.h>
 #include <mblast.h>
+#include <blfmtutl.h>
 
 #define DEBUG 0
 
@@ -159,12 +215,6 @@ typedef struct cluster_parameters
    FILE *logfp;
 } ClusterParameters, PNTR ClusterParametersPtr;
 
-typedef struct cluster_log_header
-{
-   Boolean numeric_id_type;
-   Int4 size;
-} ClusterLogHeader, PNTR ClusterLogHeaderPtr;
-
 static ClusterParametersPtr global_parameters;
 
 typedef struct cluster_log_info
@@ -352,19 +402,20 @@ static Int4 ReclusterFromFile(FILE *infofp, FILE *outfp, Int4Ptr PNTR gilp,
    CharPtr PNTR id_list = NULL;
    CharPtr ptr, id_string;
    FloatHi length_coverage, score_coverage; 
-   ClusterLogHeader header;
+   Uint4 header_size, numeric_id_type;
    Int4 last_seq = -1;
    CharPtr id = NULL;
    Boolean PNTR used_id_index = NULL;
 
-   FileRead(&header, sizeof(ClusterLogHeader), 1, infofp);
+   FileRead(&numeric_id_type, sizeof(Int4), 1, infofp);
+   FileRead(&header_size, sizeof(Int4), 1, infofp);
 
-   if (header.numeric_id_type) {
-      num_queries = header.size;
+   if (numeric_id_type) {
+      num_queries = header_size;
       gi_list = (Int4Ptr) MemNew(num_queries*sizeof(Int4));
       FileRead(gi_list, sizeof(Int4), num_queries, infofp);
    } else {
-      total_id_len = header.size;
+      total_id_len = header_size;
       num_queries = 0;
       id_string = (CharPtr) MemNew(total_id_len+1);
       FileRead(id_string, sizeof(Char), total_id_len, infofp);
@@ -397,7 +448,7 @@ static Int4 ReclusterFromFile(FILE *infofp, FILE *outfp, Int4Ptr PNTR gilp,
 
    /* Test for list of ids to use for reclustering */
    if (idfp) {
-      CharPtr id_str, used_id_string;
+      CharPtr used_id_string;
       CharPtr delimiter_list = " \t\n,;";
       Int4 length;
 
@@ -406,8 +457,6 @@ static Int4 ReclusterFromFile(FILE *infofp, FILE *outfp, Int4Ptr PNTR gilp,
       while((length = 
 	     FileRead(used_id_string, sizeof(Char), FILE_BUFFER_SIZE, idfp))) {
 	 ptr = used_id_string;
-	 if (gi_list)
-	    id_str = (CharPtr) Malloc(10);
 	 while ((id = StringTokMT(ptr, delimiter_list, &ptr)) 
 		!= NULL) {
 	    if (ptr==NULL && length==FILE_BUFFER_SIZE) {
@@ -415,19 +464,14 @@ static Int4 ReclusterFromFile(FILE *infofp, FILE *outfp, Int4Ptr PNTR gilp,
 	       break;
 	    }
 	    for (i=0; i<num_queries; i++) {
-	       if (gi_list) 
-		  sprintf(id_str, "%ld", gi_list[i]);
-	       else
-		  id_str = id_list[i];
-	       if (!StrCmp(id_str, id))
+	       if ((gi_list && atoi(id) == gi_list[i]) ||
+                   (!gi_list && !StrCmp(id_list[i], id)))
 		  break;
 	    }
 	    if (i < num_queries)
 	       used_id_index[i] = TRUE;
 	 }
       }
-      if (gi_list)
-	 MemFree(id_str);
    }
 
    while ((num_hits = FileRead(info, sizeof(ClusterLogInfo), INFO_LIST_SIZE,
@@ -553,7 +597,7 @@ PrintNeighbors(VoidPtr ptr)
     if (search->prog_number == blast_type_blastp)
         subject_length = readdb_get_sequence(search->rdfp, search->subject_id, &subject);
     else { /* Mega BLAST saves ncbi4na-encoded sequence */
-        subject = search->subject->sequence_start;
+        subject = search->subject->sequence_start + 1;
         subject_length = search->subject->length;
     }
     hspcnt = current_hitlist->hspcnt;
@@ -563,8 +607,8 @@ PrintNeighbors(VoidPtr ptr)
     else 
        id1 = -1;
     id2 = search->subject_id;
-    
-    if (id1 < id2) { /* Must be always true */
+
+    if (id1 < id2) { 
 #define BUF_CHUNK_SIZE 1024
         Int4 query_length, q_length, s_length;
         FloatHi length_coverage, bit_score, score_coverage, perc_identity; 
@@ -595,6 +639,8 @@ PrintNeighbors(VoidPtr ptr)
            }
            HeapSort(hsp_info_array, hspcnt, sizeof(BlastClustHspInfoPtr), 
                     hsp_info_id_score_compare);
+           /* Leave only one highest scoring hsp per "query" sequence.
+              Deallocate memory for all others. */
            for (index=1; index<hspcnt; index++) {
               if (hsp_info_array[index]->id < id2 && 
                   hsp_info_array[index]->id != 
@@ -671,7 +717,6 @@ PrintNeighbors(VoidPtr ptr)
                                         &start, &length, &strands, 
                                         &hsp->query.offset, 
                                         &hsp->subject.offset);
-              GapXEditBlockDelete(hsp->gap_info);
               perc_identity = 0;
               for (i=0; i<numseg; i++) {
                  align_length += length[i];
@@ -679,6 +724,9 @@ PrintNeighbors(VoidPtr ptr)
                     perc_identity += MegaBlastGetNumIdentical(query, subject, start[2*i], start[2*i+1], length[i], FALSE);
               }
               perc_identity = perc_identity / align_length * 100;
+              start = MemFree(start);
+              length = MemFree(length);
+              strands = MemFree(strands);
            }
            if (global_parameters->score_threshold < 3.0)
               score_coverage = bit_score / (MAX(q_length, s_length));
@@ -711,6 +759,9 @@ PrintNeighbors(VoidPtr ptr)
                  root[root1] = root2;
               NlmMutexUnlock(root_mutex);
            }
+
+           /* For blastn, get to the highest scoring hsp for the next 
+              "query" */
            if (search->prog_number == blast_type_blastn && 
                ++index < query_count)
               hsp =
@@ -719,21 +770,35 @@ PrintNeighbors(VoidPtr ptr)
               hsp = NULL;
         }
 
+        /* Deallocate memory for the auxiliary array of HSP info 
+           structures. Also free all gap information structures, 
+           because they are not freed when hit list is cleaned. */
         if (search->prog_number == blast_type_blastn) {
            for (index=0; index<query_count; index++)
               MemFree(hsp_info_array[index]);
            hsp_info_array = MemFree(hsp_info_array);
+           
+           for (index=0; index < current_hitlist->hspcnt; ++index) {
+              hsp = current_hitlist->hsp_array[index];
+              hsp->gap_info = GapXEditBlockDelete(hsp->gap_info);
+           }
         }
 
-	if (global_parameters->logfp) {
+	if (global_parameters->logfp && loginfo) {
 	   FileWrite(loginfo, sizeof(ClusterLogInfo), query_count, 
 		     global_parameters->logfp);
-           MemFree(loginfo);
-	   /*fflush(global_parameters->logfp);*/
+           loginfo = MemFree(loginfo);
+	   fflush(global_parameters->logfp);
 	}
-    } else
-       fprintf(stderr, "Error: this can't happen!\n");
-    return 1;
+    } else {
+       /* This can't happen in normal situation. If it does, the most likely
+          reason is presense of non-unique identifiers in the input file */
+       ErrPostEx(SEV_FATAL, 1, 0, "Blastclust cannot process input files with"
+                 " non-unique sequence identifiers\n");
+       return 1;
+    }
+       
+    return 0;
 }
 
 
@@ -774,13 +839,17 @@ static Args myargs [] = {
    { "Enable id parsing in database formatting?",                /* 13 */
       "TRUE", NULL, NULL, FALSE, 'e', ARG_BOOLEAN, 0.0, 0, NULL},
    { "Configuration file with advanced options",                 /* 14 */
-      NULL, NULL, NULL, TRUE, 'c', ARG_FILE_IN, 0.0, 0, NULL} 
+     NULL, NULL, NULL, TRUE, 'c', ARG_FILE_IN, 0.0, 0, NULL},
+   { "Word size to use (0 for default: proteins 3, nucleotides 32)",/* 15 */
+     "0", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL}       
+ 
 };
 
 #define MAX_DB_SIZE 100000
 #define MAX_GI_LENGTH 9
 #define PROGRESS_INTERVAL 1000
 #define MAX_NUM_QUERIES 16383 /* == 1/2 INT2_MAX */
+#define MAX_TOTAL_LENGTH 5000000
 
 Int2 Main (void)
 {
@@ -801,15 +870,20 @@ Int2 Main (void)
     Int8 total_length;
     Int4Ptr gi_list = NULL, seq_len = NULL;
     CharPtr PNTR id_list = NULL, id_string = NULL;
-    Boolean db_formatted = FALSE, numeric_id_type = TRUE;
+    Boolean db_formatted = FALSE;
+    Boolean numeric_id_type = TRUE;
     Char db_file[BUFFER_SIZE];
-    ClusterLogHeader header;
     Boolean print_progress, finish_incomplete, is_prot, parse_mode;
     FDB_optionsPtr fdb_options;
     Char timestr[24];
     CharPtr tmpdir;
+    Int4 total_query_length;
+    CharPtr title = NULL;
+    Char buf[256] = { '\0' };
 
-    if (! GetArgs ("blastclust", NUMARG, myargs))
+    StringCpy(buf, "blastclust ");
+    StringNCat(buf, BlastGetVersionNumber(), sizeof(buf)-StringLen(buf)-1);
+    if (! GetArgs (buf, NUMARG, myargs))
        return (1);
     
     UseLocalAsnloadDataAndErrMsg ();
@@ -832,7 +906,7 @@ Int2 Main (void)
 
     if (progress_file != NULL &&
 	(progressfp = FileOpen(progress_file, "w")) == NULL) {
-       ErrPostEx(SEV_FATAL, 0, 0, "blastclust: Unable to open progress file %s\n",
+       ErrPostEx(SEV_FATAL, 1, 0, "blastclust: Unable to open progress file %s\n",
 		 progress_file);
        return (1);
     }
@@ -841,7 +915,7 @@ Int2 Main (void)
     outfp = NULL;
     if (blast_outputfile != NULL) {
         if ((outfp = FileOpen(blast_outputfile, "w")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "blastclust: Unable to open output file %s\n", blast_outputfile);
+            ErrPostEx(SEV_FATAL, 1, 0, "blastclust: Unable to open output file %s\n", blast_outputfile);
             return (1);
         }
     }
@@ -850,12 +924,12 @@ Int2 Main (void)
     if (info_file) {
        /* Non-empty string means only retrieve neighbors for reclustering */
        if ((infofp = FileOpen(info_file, "rb")) == NULL) { 
-	  ErrPostEx(SEV_FATAL, 0, 0, "blastclust: Unable to open neighbors file %s for reading\n", info_file);
+	  ErrPostEx(SEV_FATAL, 1, 0, "blastclust: Unable to open neighbors file %s for reading\n", info_file);
 	  return (1);
        }
        if (myargs[11].strvalue) {
 	  if ((idfp = FileOpen(myargs[11].strvalue, "r")) == NULL) {
-	     ErrPostEx(SEV_FATAL, 0, 0, "blastclust: Unable to open id file %s for reading\n", myargs[11].strvalue);
+	     ErrPostEx(SEV_FATAL, 1, 0, "blastclust: Unable to open id file %s for reading\n", myargs[11].strvalue);
 	     return (1);
 	  }
        } else
@@ -896,28 +970,25 @@ Int2 Main (void)
        } else
           blast_database = blast_inputfile;
 
-       fdb_options = MemNew(sizeof(FDB_options));
-       fdb_options->db_file = StringSave(blast_inputfile);
-       fdb_options->is_protein = (Int4) is_prot; 
-       fdb_options->parse_mode = (Int4) parse_mode; 
-       fdb_options->base_name = StringSave(blast_database);
-       FastaToBlastDB(fdb_options, blast_database, 0);
+       fdb_options = FDBOptionsNew(blast_inputfile, is_prot, NULL, FALSE, 
+                                   FALSE, FALSE, FALSE, TRUE, parse_mode, 
+                                   blast_database, NULL, 0, 0, FORMATDB_VER,
+                                   FALSE, FALSE);
+       FastaToBlastDB(fdb_options, 0);
 
-       MemFree(fdb_options->db_file);
-       MemFree(fdb_options->base_name);
-       MemFree(fdb_options);
+       fdb_options = FDBOptionsFree(fdb_options);
     }
 
     logfile = myargs[6].strvalue;
     if (logfile) { /* Empty string means do not write log information */
        if (finish_incomplete) {
 	  if ((global_parameters->logfp = FileOpen(logfile, "ab+")) == NULL) { 
-	     ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open log file %s for appending\n", logfile);
+	     ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open log file %s for appending\n", logfile);
 	     return (1);
 	  }
        } else {
 	  if ((global_parameters->logfp = FileOpen(logfile, "wb+")) == NULL) { 
-	     ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open log file %s for writing\n", logfile);
+	     ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open log file %s for writing\n", logfile);
 	     return (1);
 	  }
        }
@@ -946,12 +1017,15 @@ Int2 Main (void)
     options->number_of_cpus = myargs[1].intvalue;
     if (!is_prot) {
        options->is_megablast_search = TRUE;
-       options->wordsize = 32;
        options->gap_open = options->gap_extend = 0;
+       options->wordsize = 32;
        options->block_width = 0;
        options->window_size = 0;
     }    
 
+    if (myargs[15].intvalue != 0)
+       options->wordsize = myargs[15].intvalue;
+
     if (myargs[14].strvalue) {
        CharPtr string_options = NULL;
        Int2 string_options_len = 0;
@@ -960,7 +1034,7 @@ Int2 Main (void)
        Char opt_line[BUFFER_SIZE+1];
        
        if ((config_file = FileOpen(myargs[14].strvalue, "r")) == NULL) {
-          ErrPostEx(SEV_FATAL, 0, 0, 
+          ErrPostEx(SEV_FATAL, 1, 0, 
                     "Cannot open advanced options configuration file %s\n",
                     myargs[14].strvalue);
        }
@@ -998,12 +1072,12 @@ Int2 Main (void)
 
     root = (Int4Ptr) Malloc(num_queries*sizeof(Int4));
 
-    ReadDBBioseqFetchEnable ("blastclust", blast_database, db_is_na, TRUE);
-       
     if (is_prot && finish_incomplete) {
        first_seq = ReclusterFromFile(global_parameters->logfp, NULL, &gi_list,
 				    &id_list, &seq_len, NULL);
     } else {
+       Uint4 header_size = 0;
+       Uint4 header_numeric_id_type = 0;
        first_seq = 0;
        for (index=0; index<num_queries; index++)
 	  root[index] = index;
@@ -1013,46 +1087,34 @@ Int2 Main (void)
        seq_len = (Int4Ptr) MemNew(num_queries*sizeof(Int4));
        
        for (index=0; index<num_queries; index++) {
-	  readdb_get_descriptor(rdfp, index, &sip, NULL);
+	  readdb_get_descriptor(rdfp, index, &sip, &title);
 	  seq_len[index] = readdb_get_sequence_length(rdfp, index);
 	  
-          if (sip->choice == SEQID_LOCAL) {
-             BioseqPtr bsp = BioseqLockById(sip);
-             CharPtr title = StringSave(BioseqGetTitle(bsp));
-             if (title) {
-                numeric_id_type = FALSE;
-                id_list[index] = StringTokMT(title, " ", &title);
-             } else {
-                numeric_id_type &=
-                   GetAccessionFromSeqId(bsp->id, &gi_list[index], 
-                                         &id_list[index]);
-             }
-             BioseqUnlock(bsp);
-          } else if (sip->choice != SEQID_GENERAL)
+          if (sip->choice != SEQID_GENERAL ||
+              StringCmp(((DbtagPtr)sip->data.ptrvalue)->db, "BL_ORD_ID")) {
              numeric_id_type &= 
                 GetAccessionFromSeqId(sip, &gi_list[index], 
                                       &id_list[index]); 
-          else {
-             BioseqPtr bsp = BioseqLockById(sip);
-             CharPtr title = StringSave(BioseqGetTitle(bsp));
-             if (title) {
-                numeric_id_type = FALSE;
-                id_list[index] = StringTokMT(title, " ", &title);
+          } else {
+             CharPtr dummy_ptr = NULL;
+             numeric_id_type = FALSE;
+             if (title && *title != NULLB) {
+                id_list[index] = 
+                   StringSave(StringTokMT(title, " ", &dummy_ptr));
              } else {
-                numeric_id_type = FALSE;
                 id_list[index] = (CharPtr) Malloc(BUFFER_SIZE+1);
                 SeqIdWrite(sip, id_list[index],
                            PRINTID_FASTA_SHORT, BUFFER_SIZE);
              }
-             BioseqUnlock(bsp);
           }
-	  sip = SeqIdSetFree(sip);
+          title = MemFree(title);
+          sip = SeqIdSetFree(sip);
        }
-       header.numeric_id_type = numeric_id_type;
-       
+
        if (numeric_id_type) {
 	  id_list = MemFree(id_list);
-	  header.size = num_queries;
+	  header_size = num_queries;
+          header_numeric_id_type = 1;
        } else {
 	  total_id_len = 0;
 	  /* Check if some ids were gis and convert them to strings */
@@ -1069,12 +1131,13 @@ Int2 Main (void)
 	     StringCat(id_string, id_list[i]);
 	     StringCat(id_string, " ");
 	  }
-	  header.size = total_id_len;
+	  header_size = total_id_len;
        }
        
-       FileWrite(&header, sizeof(ClusterLogHeader), 1, 
-		 global_parameters->logfp);
-       
+       FileWrite(&header_numeric_id_type, sizeof(Uint4), 1, 
+                 global_parameters->logfp);
+       FileWrite(&header_size, sizeof(Uint4), 1, global_parameters->logfp);
+
        if (numeric_id_type) 
 	  FileWrite(gi_list, sizeof(Int4), num_queries, 
 		    global_parameters->logfp);
@@ -1085,7 +1148,7 @@ Int2 Main (void)
        }
        
        FileWrite(seq_len, sizeof(Int4), num_queries, global_parameters->logfp);
-       /*fflush(global_parameters->logfp);*/
+       fflush(global_parameters->logfp);
     }
 
     if (print_progress) {
@@ -1107,8 +1170,14 @@ Int2 Main (void)
        num_left = num_queries;
        while (num_left>0) {
           num_bsps = MIN(num_left, MAX_NUM_QUERIES);
-          for (index=0; index<num_bsps; index++)
+          total_query_length = 0;
+          for (index=0; index<num_bsps; index++) {
              query_bsp_array[index] = readdb_get_bioseq(rdfp, index);
+             total_query_length += query_bsp_array[index]->length;
+             if (total_query_length > MAX_TOTAL_LENGTH)
+                break;
+          }
+          num_bsps = index;
           query_bsp_array[num_bsps] = NULL;
           
           head = BioseqMegaBlastEngine(query_bsp_array, blast_program, 
@@ -1126,7 +1195,9 @@ Int2 Main (void)
           /* Set up the search */
           options->first_db_seq = index + 1;
           
-          search = BLASTSetUpSearchWithReadDbInternal(NULL, query_bsp, blast_program, seq_len[index], blast_database, options, NULL, NULL, NULL, 0, rdfp);
+          search = BLASTSetUpSearchWithReadDbInternal(NULL, query_bsp,
+                      blast_program, seq_len[index], blast_database, 
+                      options, NULL, NULL, NULL, 0, rdfp);
           if (search != NULL && !search->query_invalid) {
              search->handle_results = PrintNeighbors;
              
@@ -1135,10 +1206,11 @@ Int2 Main (void)
              search->thr_info->tick_callback = NULL;
              
              do_the_blast_run(search);
+          } else if (search) {
+             BlastErrorPrint(search->error_return);
           }
           search = BlastSearchBlkDestruct(search);
           query_bsp = BioseqFree(query_bsp);
-          ObjMgrFreeCache(0);
           
           if (print_progress && (index + 1)%PROGRESS_INTERVAL == 0) {
              DayTimeStr(timestr, TRUE, TRUE);
@@ -1147,6 +1219,7 @@ Int2 Main (void)
           }
        } /* End of loop on queries */
     }
+    /*ReadDBBioseqFetchDisable();*/
     rdfp = readdb_destruct(rdfp);
     BlastClusterNeighbours(num_queries, seq_len, id_list, gi_list, NULL);
 
