@@ -1,4 +1,6 @@
-/* $Id: blastpgp.c,v 6.104 2001/10/12 14:55:41 dondosha Exp $ */
+static char const rcsid[] = "$Id: blastpgp.c,v 6.126 2004/10/12 15:14:39 papadopo Exp $";
+
+/* $Id: blastpgp.c,v 6.126 2004/10/12 15:14:39 papadopo Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -24,8 +26,76 @@
 * appreciated.                                                            *
 *                                                                         *
 **************************************************************************
- * $Revision: 6.104 $ 
+ * $Revision: 6.126 $ 
  * $Log: blastpgp.c,v $
+ * Revision 6.126  2004/10/12 15:14:39  papadopo
+ * add gap open and extend penalties to [BC]posComputation calls
+ *
+ * Revision 6.125  2004/07/19 17:16:19  papadopo
+ * add capability to perform input and output of residue frequencies in scoremat form
+ *
+ * Revision 6.124  2004/06/30 12:33:30  madden
+ * Add include for blfmtutl.h
+ *
+ * Revision 6.123  2004/06/25 20:58:49  dondosha
+ * Ungapped option not supported for multi-iterational search, but OK otherwise
+ *
+ * Revision 6.122  2004/06/24 21:48:16  dondosha
+ * Made ungapped search option deprecated
+ *
+ * Revision 6.121  2004/06/24 19:02:23  dondosha
+ * Exit with error status if PGPReadBlastOptions returns NULL
+ *
+ * Revision 6.120  2004/04/29 19:56:00  dondosha
+ * Mask filtered locations in query sequence lines in XML output
+ *
+ * Revision 6.119  2004/03/31 17:59:32  dondosha
+ * Fix for XML output for multiple queries: append full XML outputs in one file
+ *
+ * Revision 6.118  2003/09/26 16:01:34  madden
+ * Change threshold to 0.002
+ *
+ * Revision 6.117  2003/05/30 17:31:09  coulouri
+ * add rcsid
+ *
+ * Revision 6.116  2003/05/13 16:02:42  coulouri
+ * make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+ *
+ * Revision 6.115  2003/03/20 14:47:16  madden
+ * StringSave on asn1_mode
+ *
+ * Revision 6.114  2003/03/20 13:44:23  madden
+ * Fix -m 10/11 output to make them SeqAnnots
+ *
+ * Revision 6.113  2002/12/31 22:47:16  boemker
+ * Added support for printing output as ASN (text, with -m 10, or binary, with
+ * -m 11).
+ *
+ * Revision 6.112  2002/11/12 16:04:24  dondosha
+ * Do not print the no hits found message for tabular output
+ *
+ * Revision 6.111  2002/09/23 21:00:00  madden
+ * Fix to correctly output SeqAlign for every query and iteration
+ *
+ * Revision 6.110  2002/09/19 13:25:00  camacho
+ * Fix incorrect index into myargs array
+ *
+ * Revision 6.109  2002/09/18 20:34:30  camacho
+ * Restored -P option
+ *
+ * Revision 6.108  2002/08/09 19:41:25  camacho
+ * 1) Added blast version number to command-line options
+ * 2) Added explanations for some default parameters
+ *
+ * Revision 6.107  2002/06/19 22:50:17  dondosha
+ * Added all queries information for tabular output with multiple queries
+ *
+ * Revision 6.106  2002/04/29 19:55:25  madden
+ * Use ARG_FLOAT for db length
+ *
+ * Revision 6.105  2002/03/13 22:48:06  dondosha
+ * Avoid freeing masking locations after each iteration with XML output
+ *
  * Revision 6.104  2001/10/12 14:55:41  dondosha
  * Changed description of the -t option
  *
@@ -423,6 +493,7 @@
 #include <sqnutils.h>
 #include <xmlblast.h>
 #include <ddvcreate.h>
+#include <blfmtutl.h>
 
 /* Used by the callback function. */
 FILE *global_fp=NULL;
@@ -457,25 +528,25 @@ star_callback(Int4 sequence_number, Int4 number_of_positive_hits)
 
 #define NUMARG (sizeof(myargs)/sizeof(myargs[0]))
 
-static Args myargs [] = {
+static Args myargs[] = {
     { "Database",               /* 0 */
       "nr", NULL, NULL, FALSE, 'd', ARG_STRING, 0.0, 0, NULL},
     { "Query File",             /* 1 */
       "stdin", NULL, NULL, FALSE, 'i', ARG_FILE_IN, 0.0, 0, NULL},
-    { "Multiple Hits window size (zero for single hit algorithm)", /* 2 */
+    { "Multiple Hits window size", /* 2 */
       "40", NULL, NULL, FALSE, 'A', ARG_INT, 0.0, 0, NULL},
     { "Threshold for extending hits", /* 3 */
-      "0", NULL, NULL, FALSE, 'f', ARG_INT, 0.0, 0, NULL},
+      "11", NULL, NULL, FALSE, 'f', ARG_INT, 0.0, 0, NULL},
     { "Expectation value (E)",  /* 4 */
       "10.0", NULL, NULL, FALSE, 'e', ARG_FLOAT, 0.0, 0, NULL},
-    { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output,\n8 = Tabular output, \n9 = Tabular output with comments", /* 5 */
+    { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output,\n8 = Tabular output, \n9 = Tabular output with comments\n10 = ASN, text\n11 = ASN, binary", /* 5 */
       "0", NULL, NULL, FALSE, 'm', ARG_INT, 0.0, 0, NULL},
     { "Output File for Alignment", /* 6 */
       "stdout", NULL, NULL, TRUE, 'o', ARG_FILE_OUT, 0.0, 0, NULL},
     { "Dropoff (X) for blast extensions in bits (default if zero)", /* 7 */
       "7.0", NULL, NULL, FALSE, 'y', ARG_FLOAT, 0.0, 0, NULL},
-    { "0 for multiple hits 1-pass, 1 for single hit 1-pass, 2 for 2-pass", /* 8 */
-      "0", NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},
+    { "0 for multiple hit, 1 for single hit",/* 8 */
+       "0",  NULL, NULL, FALSE, 'P', ARG_INT, 0.0, 0, NULL},
     { "Filter query sequence with SEG", /* 9 */
       "F", NULL, NULL, FALSE, 'F', ARG_STRING, 0.0, 0, NULL},
     { "Cost to open a gap",     /* 10 */
@@ -497,7 +568,7 @@ static Args myargs [] = {
     { "Show GI's in deflines",  /* 18 */
       "F", NULL, NULL, FALSE, 'I', ARG_BOOLEAN, 0.0, 0, NULL},
     { "e-value threshold for inclusion in multipass model", /* 19 */
-      "0.005", NULL, NULL, FALSE, 'h', ARG_FLOAT, 0.0, 0, NULL},
+      "0.002", NULL, NULL, FALSE, 'h', ARG_FLOAT, 0.0, 0, NULL},
     { "Constant in pseudocounts for multipass version", /* 20 */
       "9", NULL, NULL, FALSE, 'c', ARG_INT, 0.0, 0, NULL},
     { "Maximum number of passes to use in  multipass version", /* 21 */
@@ -518,13 +589,13 @@ static Args myargs [] = {
       NULL, NULL, NULL, TRUE, 'C', ARG_FILE_OUT, 0.0, 0, NULL},
     { "Input File for PSI-BLAST Restart", /* 29 */
       NULL, NULL, NULL, TRUE, 'R', ARG_FILE_IN, 0.0, 0, NULL},
-    { "Word size, default if zero", /* 30 */
-      "0", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL},
+    { "Word size", /* 30 */
+      "3", NULL, NULL, FALSE, 'W', ARG_INT, 0.0, 0, NULL},
     { "Effective length of the database (use zero for the real size)", /* 31 */
-      "0", NULL, NULL, FALSE, 'z', ARG_INT, 0.0, 0, NULL},
+      "0", NULL, NULL, FALSE, 'z', ARG_FLOAT, 0.0, 0, NULL},
     { "Number of best hits from a region to keep", /* 32 */
       "0", NULL, NULL, FALSE, 'K', ARG_INT, 0.0, 0, NULL},
-    { "Compute locally optimal Smith-Waterman alignments", /*33*/
+    { "Compute locally optimal Smith-Waterman alignments", /* 33 */
       "F", NULL, NULL, FALSE, 's', ARG_BOOLEAN, 0.0, 0, NULL},
     { "Effective length of the search space (use zero for the real size)", /* 34 */
       "0", NULL, NULL, FALSE, 'Y', ARG_FLOAT, 0.0, 0, NULL},
@@ -542,10 +613,20 @@ static Args myargs [] = {
       NULL, NULL, NULL, TRUE, 'l', ARG_STRING, 0.0, 0, NULL},
     {"Use lower case filtering of FASTA sequence",    /* 41 */
      "F", NULL,NULL,TRUE,'U',ARG_BOOLEAN, 0.0,0,NULL},
-    { "Use composition based statistics", /*42*/
+    { "Use composition based statistics", /* 42 */
       "T", NULL, NULL, FALSE, 't', ARG_BOOLEAN, 0.0, 0, NULL},
+    { "ASN.1 Scoremat input of checkpoint data:\n"
+      "0: no scoremat input\n"
+      "1: Restart is from ASCII scoremat checkpoint file,\n"
+      "2: Restart is from binary scoremat checkpoint file", /* 43 */
+      "0", NULL, NULL, TRUE, 'q', ARG_INT, 0.0, 0, NULL},
+    { "ASN.1 Scoremat output of checkpoint data:\n"
+      "0: no scoremat output\n"
+      "1: Output is ASCII scoremat checkpoint file (requires -J),\n"
+      "2: Output is binary scoremat checkpoint file (requires -J)", /* 44 */
+      "0", NULL, NULL, TRUE, 'u', ARG_INT, 0.0, 0, NULL},
 #ifdef YES_TO_DECLINE_TO_ALIGN
-    { "Cost to decline alignment (disabled when 0)", /* 43 */
+    { "Cost to decline alignment (disabled when 0)", /* 45 */
       "0", NULL, NULL, FALSE, 'L', ARG_INT, 0.0, 0, NULL},
 #endif
 };
@@ -568,6 +649,8 @@ typedef struct _pgp_blast_options {
     FILE *patfp; 
     seedSearchItems *seedSearch;
     Boolean is_xml_output;
+    Boolean is_asn1_output;
+    char* asn1_mode;  /* "w" or "wb" */
 } PGPBlastOptions, PNTR PGPBlastOptionsPtr;
 
 void PGPGetPrintOptions(Boolean gapped, Uint4Ptr align_options_out, 
@@ -618,6 +701,7 @@ void PGPFreeBlastOptions(PGPBlastOptionsPtr bop)
     
     FileClose(bop->infp);
     FileClose(bop->outfp);
+    bop->aip_out = AsnIoClose(bop->aip_out);
     
     MemFree(bop->blast_database);
     MemFree(bop->patfile);
@@ -635,21 +719,24 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
     SeqEntryPtr sep;
     Boolean is_dna;
     ObjectIdPtr obidp;
+    Int4 align_view;
+
+    align_view = myargs[5].intvalue;
 
     bop = MemNew(sizeof(PGPBlastOptions));
     
-    bop->blast_database   = StringSave(myargs [0].strvalue);
+    bop->blast_database   = StringSave(myargs[0].strvalue);
 
-    if ((bop->infp = FileOpen(myargs [1].strvalue, "r")) == NULL) {
-        ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open input file %s\n", 
-                  myargs [1].strvalue);
+    if ((bop->infp = FileOpen(myargs[1].strvalue, "r")) == NULL) {
+        ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", 
+                  myargs[1].strvalue);
         return NULL;
     }
     
-    if (myargs [6].strvalue != NULL) {
-        if ((bop->outfp = FileOpen(myargs [6].strvalue, "w")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output "
-                      "file %s\n", myargs [6].strvalue);
+    if (align_view != 7 && align_view != 10 && align_view != 11 && myargs[6].strvalue != NULL) {
+        if ((bop->outfp = FileOpen(myargs[6].strvalue, "w")) == NULL) {
+            ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output "
+                      "file %s\n", myargs[6].strvalue);
             return NULL;
         }
     }
@@ -660,32 +747,76 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
     if (myargs[22].intvalue != 0)
         bop->believe_query = TRUE;
     
-    if (myargs[24].strvalue != NULL) {
-        
+    if (myargs[43].intvalue != NO_SCOREMAT_IO && 
+        myargs[43].intvalue != ASCII_SCOREMAT &&
+        myargs[43].intvalue != BINARY_SCOREMAT) {
+        ErrPostEx(SEV_FATAL, 1, 0,"Invalid choice for scoremat input\n");
+        return NULL;
+    }
+    if (myargs[44].intvalue != NO_SCOREMAT_IO && 
+        myargs[44].intvalue != ASCII_SCOREMAT &&
+        myargs[44].intvalue != BINARY_SCOREMAT) {
+        ErrPostEx(SEV_FATAL, 1, 0,"Invalid choice for scoremat output\n");
+        return NULL;
+    }
+    if (myargs[44].intvalue != NO_SCOREMAT_IO) {
         if (bop->believe_query == FALSE) {
-            ErrPostEx(SEV_FATAL, 0, 0, 
-                      "-J option must be TRUE to use this option");
-            return NULL;
-        }
-        
-        if ((bop->aip_out = AsnIoOpen (myargs[24].strvalue,"w")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open output "
-                      "file %s\n", myargs[24].strvalue);
+            ErrPostEx(SEV_FATAL, 1, 0, 
+                      "-J option must be TRUE for scoremat output");
             return NULL;
         }
     }
 
-    options = BLASTOptionNew("blastp", (Boolean)myargs[14].intvalue);
+    if (myargs[24].strvalue != NULL) {
+        
+        if (bop->believe_query == FALSE) {
+            ErrPostEx(SEV_FATAL, 1, 0, 
+                      "-J option must be TRUE for ASN.1 output");
+            return NULL;
+        }
+        
+        if ((bop->aip_out = AsnIoOpen (myargs[24].strvalue,"w")) == NULL) {
+            ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output "
+                      "file %s\n", myargs[24].strvalue);
+            return NULL;
+        }
+    }
+    else if (align_view == 10 || align_view == 11)
+    {
+        const char* mode = (align_view == 10) ? "w" : "wb";
+        if (bop->believe_query == FALSE) {
+            ErrPostEx(SEV_FATAL, 1, 0, 
+                      "-J option must be TRUE for ASN.1 output");
+            return NULL;
+        }
+        if ((bop->aip_out = AsnIoOpen(myargs[6].strvalue, (char*) mode)) == NULL) {
+                ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", myargs[6].strvalue);
+                return NULL;
+        }
+	bop->is_asn1_output = TRUE;
+        if(align_view == 10)
+            bop->asn1_mode = StringSave("w");
+        else 
+            bop->asn1_mode = StringSave("wb");
+    }
+
+    if (!myargs[14].intvalue && myargs[21].intvalue > 1) {
+       ErrPostEx(SEV_ERROR, 1, 0, "Ungapped search option is not supported"
+                 " with more than one iteration\n");
+       return NULL;
+    }
+
+    options = BLASTOptionNew("blastp", TRUE);
     bop->options = options;
 
     if(myargs[41].intvalue) {
         if((sep = FastaToSeqEntryForDb (bop->infp, FALSE, NULL, bop->believe_query, NULL, NULL, &options->query_lcase_mask)) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "Unable to read input FASTA file\n");
+            ErrPostEx(SEV_FATAL, 1, 0, "Unable to read input FASTA file\n");
             return NULL;
         }
     } else {
         if((sep = FastaToSeqEntryEx(bop->infp, FALSE, NULL, bop->believe_query)) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "Unable to read input FASTA file\n");
+            ErrPostEx(SEV_FATAL, 1, 0, "Unable to read input FASTA file\n");
             return NULL;
         }
     }
@@ -695,7 +826,7 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
           SeqEntryFree(sep); */
     
     if (bop->query_bsp == NULL) {
-        ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+        ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
         return NULL;
     }    
     
@@ -716,44 +847,40 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
         options->required_end--;
     }
     
-    options->window_size = myargs [2].intvalue;
+    options->window_size = myargs[2].intvalue;
 
-    if(myargs [3].intvalue)    
-        options->threshold_second = (Int4) myargs [3].intvalue;
+    if(myargs[3].intvalue)    
+        options->threshold_second = (Int4) myargs[3].intvalue;
     
-    options->dropoff_2nd_pass  = myargs [7].floatvalue;
-    options->expect_value  = (Nlm_FloatHi) myargs [4].floatvalue;
+    options->dropoff_2nd_pass  = myargs[7].floatvalue;
+    options->expect_value  = (Nlm_FloatHi) myargs[4].floatvalue;
     options->hitlist_size = MAX(bop->number_of_descriptions, 
                                 bop->number_of_alignments);
     
-    if (myargs[14].intvalue != 0) {
-        if (myargs[8].intvalue == 0) {
-            options->two_pass_method  = FALSE;
-            options->multiple_hits_only  = TRUE;
-        } else if (myargs[8].intvalue == 1) {
-            options->two_pass_method  = FALSE;
-            options->multiple_hits_only  = FALSE;
-        } else {
-            options->two_pass_method  = TRUE;
-            options->multiple_hits_only  = FALSE;
-        }
-        options->gap_open = myargs[10].intvalue;
-        options->gap_extend = myargs[11].intvalue;
-        
-#ifdef YES_TO_DECLINE_TO_ALIGN
-        if(myargs[43].intvalue != 0) {
-            options->discontinuous = TRUE;
-            options->decline_align = myargs[43].intvalue;
-        } else {
-            options->discontinuous = FALSE;
-            options->decline_align = INT2_MAX;
-        }
-#endif
-
-        options->gap_x_dropoff = myargs[12].intvalue;
-        options->gap_x_dropoff_final = myargs[23].intvalue;
-        options->gap_trigger = myargs[13].floatvalue;
+    if (myargs[8].intvalue == 1) {
+       options->two_pass_method  = FALSE;
+       options->multiple_hits_only  = FALSE;
+    } else { 
+       /* all other inputs, including the default 0 use 2-hit method */
+       options->two_pass_method  = FALSE;
+       options->multiple_hits_only  = TRUE;
     }
+    options->gap_open = myargs[10].intvalue;
+    options->gap_extend = myargs[11].intvalue;
+    
+#ifdef YES_TO_DECLINE_TO_ALIGN
+    if(myargs[45].intvalue != 0) {
+       options->discontinuous = TRUE;
+       options->decline_align = myargs[45].intvalue;
+    } else {
+       options->discontinuous = FALSE;
+       options->decline_align = INT2_MAX;
+    }
+#endif
+    
+    options->gap_x_dropoff = myargs[12].intvalue;
+    options->gap_x_dropoff_final = myargs[23].intvalue;
+    options->gap_trigger = myargs[13].floatvalue;
     
     if (StringICmp(myargs[9].strvalue, "T") == 0) {
         options->filter_string = StringSave("S");
@@ -777,7 +904,7 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
     if (options->isPatternSearch) {
         bop->patfile = StringSave(myargs[36].strvalue);
         if ((bop->patfp = FileOpen(bop->patfile, "r")) == NULL) {
-            ErrPostEx(SEV_FATAL, 0, 0, "blast: Unable to open pattern "
+            ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open pattern "
                       "file %s\n", bop->patfile);
             return NULL;
         }
@@ -797,8 +924,8 @@ PGPBlastOptionsPtr PGPReadBlastOptions(void)
         options->ethresh = 0.0;
     if (myargs[30].intvalue)
         options->wordsize = myargs[30].intvalue;
-    if (myargs[31].intvalue)
-        options->db_length = (Int8) myargs[31].intvalue;
+    if (myargs[31].floatvalue)
+        options->db_length = (Int8) myargs[31].floatvalue;
     
     options->hsp_range_max  = myargs[32].intvalue;
     if (options->hsp_range_max != 0)
@@ -872,7 +999,7 @@ Boolean PGPReadNextQuerySequence(PGPBlastOptionsPtr bop)
     SeqEntryExplore(sep, &bop->query_bsp, FindProt);    
     
     if (bop->query_bsp == NULL) {
-        ErrPostEx(SEV_FATAL, 0, 0, "Unable to obtain bioseq\n");
+        ErrPostEx(SEV_FATAL, 1, 0, "Unable to obtain bioseq\n");
         return 0;
     }    
     
@@ -892,6 +1019,9 @@ Boolean PGPFormatHeader(PGPBlastOptionsPtr bop)
 {
     Boolean html = myargs[37].intvalue;
 
+    if (bop->outfp == NULL)
+	return FALSE;
+
     if (html)
         fprintf(bop->outfp, "<PRE>\n");
     
@@ -918,6 +1048,9 @@ Boolean  PGPFormatFooter(PGPBlastOptionsPtr bop, BlastSearchBlkPtr search)
     BLAST_MatrixPtr blast_matrix;
     Boolean html = myargs[37].intvalue;
 
+    if (bop->outfp == NULL)
+	return FALSE;
+
     if (html)
         fprintf(bop->outfp, "<PRE>\n");
 
@@ -992,7 +1125,7 @@ Boolean PGPrintPosMatrix(CharPtr filename, posSearchItems *posSearch,
     FILE *fp;
     
     if ((fp = FileOpen(filename, "w")) == NULL) {
-        ErrPostEx(SEV_FATAL, 0, 0, "Unable to open matrix output file %s\n", 
+        ErrPostEx(SEV_FATAL, 1, 0, "Unable to open matrix output file %s\n", 
                   filename);
         return FALSE;
     }
@@ -1042,8 +1175,8 @@ SeqAlignPtr PGPSeedSearch(PGPBlastOptionsPtr bop, BlastSearchBlkPtr search,
     search->gap_align->decline_align = INT2_MAX;
 
 #ifdef YES_TO_DECLINE_TO_ALIGN
-    if(myargs[43].intvalue != 0) {
-        search->gap_align->decline_align = myargs[43].intvalue;
+    if(myargs[45].intvalue != 0) {
+        search->gap_align->decline_align = myargs[45].intvalue;
     } else {
         search->gap_align->decline_align = INT2_MAX;
     }
@@ -1131,7 +1264,7 @@ void PGPFormatMainOutput(SeqAlignPtr head, PGPBlastOptionsPtr bop,
     Int4Ptr PNTR txmatrix;
     BioseqPtr query_bsp;
 
-    if(head == NULL) {
+    if(head == NULL && myargs[5].intvalue < 7) {
         fprintf(bop->outfp, "\n\n ***** No hits found ******\n\n");
         return;
     }
@@ -1145,7 +1278,7 @@ void PGPFormatMainOutput(SeqAlignPtr head, PGPBlastOptionsPtr bop,
        BlastPrintTabulatedResults(head, query_bsp, NULL, 
                                   bop->number_of_alignments,
                                   "blastp", FALSE,
-                                  bop->believe_query, 0, 0, bop->outfp);
+                                  bop->believe_query, 0, 0, bop->outfp, FALSE);
        return;
     }
 
@@ -1154,7 +1287,7 @@ void PGPFormatMainOutput(SeqAlignPtr head, PGPBlastOptionsPtr bop,
     AddAlignInfoToSeqAnnot(seqannot, 2);
     seqannot->data = head;
 
-    if (search->pbp->maxNumPasses != 1) {
+    if (search->pbp->maxNumPasses != 1 && myargs[5].intvalue < 7) {
         fprintf(bop->outfp, "\nResults from round %d\n", 
                 thisPassNum);
     }
@@ -1198,7 +1331,7 @@ void PGPFormatMainOutput(SeqAlignPtr head, PGPBlastOptionsPtr bop,
 
     /* ------- --------------------------------------- ------- */
 
-    if (ALL_ROUNDS && search->posConverged) {
+    if (ALL_ROUNDS && search->posConverged && myargs[5].intvalue < 7) {
         fprintf(bop->outfp, "\nCONVERGED!\n");
     }
 
@@ -1298,7 +1431,9 @@ void PGPSeqAlignOut(PGPBlastOptionsPtr bop, SeqAlignPtr head)
     seqannot->data = head;
     SeqAnnotAsnWrite((SeqAnnotPtr) seqannot, bop->aip_out, NULL);
     AsnIoReset(bop->aip_out);
+/*
     bop->aip_out = AsnIoClose(bop->aip_out);
+*/
     seqannot->data = NULL;
     seqannot = SeqAnnotFree(seqannot);
 }
@@ -1310,6 +1445,7 @@ Int2 Main (void)
     BlastSearchBlkPtr search;
     SeqAlignPtr  head = NULL;
     SeqLocPtr seed_seqloc = NULL;
+    Char buf[256] = { '\0' };
     
     /* used for psi-blast */
     Int4 thisPassNum;       
@@ -1323,6 +1459,7 @@ Int2 Main (void)
     Boolean  checkReturn = FALSE;
     Boolean  next_query = FALSE;
     Boolean tabular_output;
+    AsnIoPtr aip = NULL;
 
     SeqAlignPtr PNTR lastSeqAligns = NULL; 
                                 /*keeps track of the last SeqAlign in
@@ -1336,7 +1473,9 @@ Int2 Main (void)
 
     /* ----- End of definitions ----- */
     
-    if (! GetArgs ("blastpgp", NUMARG, myargs))
+    StringCpy(buf, "blastpgp ");
+    StringNCat(buf, BlastGetVersionNumber(), sizeof(buf)-StringLen(buf)-1);
+    if (! GetArgs (buf, NUMARG, myargs))
         return (1);
     ErrSetMessageLevel(SEV_WARNING);
     
@@ -1345,8 +1484,14 @@ Int2 Main (void)
     if (! SeqEntryLoad())
         return 1;    
     
-    bop = PGPReadBlastOptions();
+    if ((bop = PGPReadBlastOptions()) == NULL)
+       return 1;
 
+    if(bop->is_xml_output) {
+       if((aip = AsnIoOpen(myargs[6].strvalue, "wx")) == NULL)
+          return 1;
+    }
+    
     /* Here we will start main do/while loop over many query entries in 
        the input file. If there is an option for checkpoint recover or
        Output File for PSI-BLAST Checkpointing all but first entries in
@@ -1360,17 +1505,14 @@ Int2 Main (void)
     
         if (search == NULL)
             return 1;
+
     
         if(bop->is_xml_output) {
-            AsnIoPtr aip;
-            if((aip = AsnIoOpen(myargs[6].strvalue, "wx")) == NULL)
-                return 1;
-        
-            psixp = PSIXmlInit(aip, "blastp", bop->blast_database, 
-                               bop->options, 
-                               bop->fake_bsp, 0);
+           psixp = PSIXmlInit(aip, "blastp", bop->blast_database, 
+                              bop->options, 
+                              bop->fake_bsp, 0);
         }
-    
+
         /*AAS*/
         if ((NULL != myargs[29].strvalue) || (NULL != myargs[39].strvalue)) {
             recoverCheckpoint = TRUE;
@@ -1409,13 +1551,13 @@ Int2 Main (void)
             posInitializeInformation(posSearch,search);
             /*AAS*/
             if (freqCheckpoint) {
-                checkReturn = posReadCheckpoint(posSearch, compactSearch, myargs[29].strvalue, &(search->error_return));
+                checkReturn = posReadCheckpoint(posSearch, compactSearch, myargs[29].strvalue, myargs[43].intvalue, &(search->error_return));
                 search->sbp->posMatrix = posSearch->posMatrix;
                 if (NULL == search->sbp->posFreqs)
                     search->sbp->posFreqs =  allocatePosFreqs(compactSearch->qlength, compactSearch->alphabetSize);
                 copyPosFreqs(posSearch->posFreqs,search->sbp->posFreqs, compactSearch->qlength, compactSearch->alphabetSize);
             } else {
-                search->sbp->posMatrix = BposComputation(posSearch, search, compactSearch, myargs[39].strvalue, myargs[28].strvalue, &(search->error_return)); 
+                search->sbp->posMatrix = BposComputation(posSearch, search, compactSearch, myargs[39].strvalue, myargs[28].strvalue, myargs[44].intvalue, bop->query_bsp, myargs[10].intvalue, myargs[11].intvalue, &(search->error_return)); 
 		posComputationCalled = TRUE;
                 if (NULL == search->sbp->posMatrix)
                     checkReturn = FALSE;
@@ -1432,7 +1574,7 @@ Int2 Main (void)
                search->error_return = ValNodeFree(search->error_return);
             }
             if (!checkReturn) {
-                ErrPostEx(SEV_FATAL, 0, 0, "blast: Error recovering from checkpoint");
+                ErrPostEx(SEV_FATAL, 1, 0, "blast: Error recovering from checkpoint");
                 return 1;
             }
         
@@ -1450,7 +1592,7 @@ Int2 Main (void)
                 head = SeqAlignSetFree(head);
             
 #ifdef OS_UNIX
-            if(!bop->is_xml_output && !tabular_output) {
+            if(!bop->is_xml_output && !tabular_output && !bop->is_asn1_output) {
                 search->thr_info->tick_callback =  tick_callback;
                 fprintf(global_fp, "%s", "Searching");
                 fflush(global_fp);
@@ -1513,7 +1655,7 @@ Int2 Main (void)
             }
             
 #ifdef OS_UNIX
-            if(!bop->is_xml_output && !tabular_output) {
+            if(!bop->is_xml_output && !tabular_output && !bop->is_asn1_output) {
                 fprintf(global_fp, "%s", "done\n\n");
             }
 #endif
@@ -1536,16 +1678,26 @@ Int2 Main (void)
             /* Here is all BLAST formating of the main output done */
             
             if(bop->is_xml_output) {
-                
                 ValNodePtr other_returns;
                 IterationPtr iterp;
-
+                ValNodePtr vnp = search->mask;
+                /* Avoid linking masking locations into other_returns, 
+                   lest they will be freed */
+                search->mask = NULL;
                 other_returns = BlastOtherReturnsPrepare(search);
+                search->mask = vnp;
 
                 if (head == NULL) {                
-                    iterp = BXMLBuildOneIteration(head, other_returns, bop->options->is_ooframe, !bop->options->gapped_calculation, thisPassNum, "No hits found");
+                    iterp = BXMLBuildOneIteration(head, other_returns,
+                               bop->options->is_ooframe,
+                               !bop->options->gapped_calculation, thisPassNum,
+                               "No hits found", search->mask);
                 } else {
-                    iterp = BXMLBuildOneIteration(head, other_returns, bop->options->is_ooframe, !bop->options->gapped_calculation, thisPassNum, search->posConverged ? "CONVERGED" : NULL);
+                    iterp = BXMLBuildOneIteration(head, other_returns,
+                               bop->options->is_ooframe,
+                               !bop->options->gapped_calculation, thisPassNum,
+                               (search->posConverged ? "CONVERGED" : NULL), 
+                               search->mask);
                 }
                 
                 IterationAsnWrite(iterp, psixp->aip, psixp->atp);
@@ -1553,7 +1705,7 @@ Int2 Main (void)
                 
                 IterationFree(iterp);
                 BlastOtherReturnsFree(other_returns);
-            } else {
+            } else if(!bop->is_asn1_output){
                 PGPFormatMainOutput(head, bop, search, thisPassNum,
                                     lastSeqAligns, numLastSeqAligns, 
                                     seed_seqloc, posSearch->posRepeatSequences);
@@ -1575,7 +1727,11 @@ Int2 Main (void)
                         CposComputation(posSearch, search, compactSearch, 
                                         head, myargs[28].strvalue, 
                                         (bop->options->isPatternSearch && 
-                                         (1== thisPassNum)), 
+                                         (1== thisPassNum)),
+                                        myargs[44].intvalue, 
+                                        bop->query_bsp,  
+                                        myargs[10].intvalue,
+                                        myargs[11].intvalue,
                                         &(search->error_return), 1.0); /*AAS*/
 		    posComputationCalled = TRUE;
                     if (search->error_return) {
@@ -1622,11 +1778,11 @@ Int2 Main (void)
                     PGPrintPosMatrix(myargs[38].strvalue, posSearch, 
                                      compactSearch, posComputationCalled);
             }
+
+        	if ((bop->is_asn1_output || bop->aip_out != NULL) && head != NULL)
+            		PGPSeqAlignOut(bop, head);
             
         } while (( 0 == search->pbp->maxNumPasses || thisPassNum < (search->pbp->maxNumPasses)) && (! (search->posConverged)));
-
-        if (bop->aip_out != NULL && head != NULL)
-            PGPSeqAlignOut(bop, head);
         
         head = SeqAlignSetFree(head);
         
@@ -1660,6 +1816,10 @@ Int2 Main (void)
         next_query = PGPReadNextQuerySequence(bop);
         if (bop->options->tweak_parameters)
            bop->options->hitlist_size *= 2;   
+
+        if (psixp) {
+           PSIXmlReset(psixp);
+        }
     } while (next_query);       /* End of main do {} while (); loop */
 
     ReadDBBioseqFetchDisable();
