@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   06/16/00
 *
-* $Revision: 6.7 $
+* $Revision: 6.25 $
 *
 * File Description: 
 *
@@ -50,6 +50,7 @@
 #include <document.h>
 #include <asn.h>
 #include <ent2api.h>
+#include <pmfapi.h>
 #include <urlquery.h>
 #include <dlogutil.h>
 #include <medview.h>
@@ -60,7 +61,7 @@
 
 #include <entrez2.h>
 
-#define ENTREZ_APP_VERSION "9.00"
+#define ENTREZ_APP_VERSION "9.30"
 
 #define MAX_QUERY_FORMS 256
 
@@ -223,19 +224,26 @@ static void SetupAppProperties (void)
 
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &mapPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &sumPageData);
-  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gphPageData);
-  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &alnPageData);
-  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &seqPageData);
+  /* AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gphPageData); */
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &asn2gphGphPageData);
+  /* AddBioseqPageToList (&(seqviewprocs.pageSpecs), &alnPageData); */
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &seqpnlPageData);
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &seqAlnPnlPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gbgnPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gnbkPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &emblPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gnptPageData);
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &ftblPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &fstaPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &qualPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &asnPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &xmlPageData);
+  AddBioseqPageToList (&(seqviewprocs.pageSpecs), &gbseqPageData);
   AddBioseqPageToList (&(seqviewprocs.pageSpecs), &dskPageData);
 
+  SetAppProperty ("NewSequinGraphicalViewer", (void *) 1024);
+  SetAppProperty ("NewSequinLayoutOverride", (void *) 1024);
+
   SetAppProperty ("SeqDisplayForm", &seqviewprocs);
 
   if (GetEntrezAppParam ("PREFERENCES", "TEXTEDITDISPLAYS", "", str, sizeof (str))) {
@@ -292,6 +300,13 @@ static void SetupAppProperties (void)
     seqviewprocs.initGenomeLabel = StringSaveNoNull (str);
   }
 
+  seqviewprocs.lockFarComponents = TRUE;
+  if (GetEntrezAppParam ("SETTINGS", "LOCKFAR", NULL, str, sizeof (str))) {
+    if (StringICmp (str, "FALSE") == 0) {
+      seqviewprocs.lockFarComponents = FALSE;
+    }
+  }
+
   if (GetEntrezAppParam ("SETTINGS", "XMLPREFIX", NULL, str, sizeof (str))) {
     AsnSetXMLmodulePrefix (StringSave (str));
   }
@@ -501,6 +516,11 @@ static void RetrieveDocs_Callback (ButtoN buttonPressed)
   /*-------------------------------------*/
 
   e2BooleanPtr = Query_FetchUIDs (s_mainForm [s_currFormNum]);
+  if (e2BooleanPtr == NULL) {
+    ArrowCursor ();
+    return;
+  }
+
   e2UidList = e2BooleanPtr->uids;
   uidList = (Int4Ptr) BSMerge (e2UidList->uids, NULL);
 
@@ -524,8 +544,6 @@ static void RetrieveDocs_Callback (ButtoN buttonPressed)
   Entrez2BooleanReplyFree (e2BooleanPtr);
   ArrowCursor ();
   Update ();
-
-  return;
 }
 
 /*==================================================================*/
@@ -719,13 +737,19 @@ static void AboutProc (IteM i)
   Select (w);
 }
 
-#ifdef WIN_MAC
 /*==================================================================*/
 /*                                                                  */
 /*  SetupMacMenus ()                                                */
 /*                                                                  */
 /*==================================================================*/
 
+#ifdef WIN_MAC
+static void DoQuit (IteM i)
+
+{
+  QuitProc ();
+}
+
 static void SetupMacMenus (void)
 
 {
@@ -735,6 +759,9 @@ static void SetupMacMenus (void)
   CommandItem (m, "About Entrez...", AboutProc);
   SeparatorItem (m);
   DeskAccGroup (m);
+
+  m = PulldownMenu (NULL, "File");
+  CommandItem (m, "Quit/Q", DoQuit);
 }
 
 static void MacDeactProc (WindoW w)
@@ -744,24 +771,99 @@ static void MacDeactProc (WindoW w)
 }
 #endif
 
+/*==================================================================*/
+/*                                                                  */
+/*  ConfigForm... ()                                                */
+/*                                                                  */
+/*==================================================================*/
+
+static void ConfigFormMessage (ForM f, Int2 mssg)
+
+{
+  BaseFormPtr  bfp;
+
+  bfp = (BaseFormPtr) GetObjectExtra (f);
+  if (bfp != NULL) {
+    switch (mssg) {
+      case VIB_MSG_CUT :
+        StdCutTextProc (NULL);
+        break;
+      case VIB_MSG_COPY :
+        StdCopyTextProc (NULL);
+        break;
+      case VIB_MSG_PASTE :
+        StdPasteTextProc (NULL);
+        break;
+      case VIB_MSG_DELETE :
+        StdDeleteTextProc (NULL);
+        break;
+      default :
+        break;
+    }
+  }
+}
+
+static void ConfigAccepted (void)
+
+{
+  Message (MSG_OK, "Please rerun Entrez now that it is configured");
+  QuitProgram ();
+}
+
+static void ConfigCancelled (void)
+
+{
+  Message (MSG_OK, "Entrez cannot run without ncbi configuration file");
+  QuitProgram ();
+}
+
 /*==================================================================*/
 /*                                                                  */
 /*  Main () - The main controlling logic for this program.          */
 /*                                                                  */
 /*==================================================================*/
 
+static CharPtr getInfoFailed =
+  "Unable to connect to Entrez2 server, use\n\
+'setenv CONN_DEBUG_PRINTOUT TRUE' to assist\n\
+debugging, or use -c command line argument\n\
+for firewall configuration dialog";
+
+static QUEUE bouncequeue = NULL;
+
+static Boolean LIBCALLBACK BounceProc (
+  CONN conn, VoidPtr userdata, EIO_Status status
+)
+
+{
+  BoolPtr  bp;
+
+  if (NetTestReadReply (conn, status)) {
+    bp = (BoolPtr) userdata;
+    *bp = TRUE;
+  }
+  return TRUE;
+}
+
 Int2 Main (void)
 
 {
   Boolean  advancedQueryToggle;
+  Boolean  bouncefound = FALSE;
+  time_t   currsecs = 0;
   Boolean  delayedNeighbor;
   Boolean  explodeToggle;
   Int2     i;
+  ErrSev   oldsev;
   PaneL    p;
   CharPtr  path = "/entrez/utils/entrez2server.fcgi";
   Uint2    port = 2441;
-  CharPtr  server = "neptune.nlm.nih.gov";
-  Char     str [32];
+  time_t   prevsecs = 0;
+  CharPtr  server = "www.ncbi.nlm.nih.gov";
+  Boolean  showConfigForm = FALSE;
+  time_t   starttime = 0;
+  time_t   stoptime = 0;
+  Char     str [64];
   Boolean  useNormalServ = FALSE;
   Boolean  useTestServ = FALSE;
   Boolean  useURL = FALSE;
@@ -810,6 +912,7 @@ Int2 Main (void)
   /*----------------------------------------------*/
 
   SetTitle (w, "Loading parse tables");
+  Update ();
 
   if (! AllObjLoad ()) {
     ArrowCursor ();
@@ -896,11 +999,23 @@ Int2 Main (void)
           useTestServ = TRUE;
         } else if (StringCmp (argv [j], "-u") == 0) {
           useURL = TRUE;
+        } else if (StringCmp (argv [j], "-c") == 0) {
+          showConfigForm = TRUE;
         }
       }
   } }
 #endif
 
+  if (showConfigForm) {
+    Remove (w);
+    ArrowCursor ();
+    ShowNewNetConfigForm (NULL, ConfigFormMessage,
+                          ConfigAccepted, ConfigCancelled,
+                          NULL, TRUE);
+    ProcessEvents ();
+    return 0;
+  }
+
   /*--------------------------------------*/
   /* Optionally choose the Entrez2 server */
   /*--------------------------------------*/
@@ -913,24 +1028,67 @@ Int2 Main (void)
     EntrezSetServer (server, port, path);
   }
 
+  /*---------------------------------------*/
+  /* Check for connection to NCBI services */
+  /*---------------------------------------*/
+
+  SetTitle (w, "Checking Internet connection to NCBI");
+  Update ();
+
+  bouncefound = FALSE;
+  starttime = GetSecs ();
+  oldsev = ErrSetMessageLevel (SEV_FATAL);
+  if (! NetTestAsynchronousQuery (&bouncequeue, BounceProc, (Pointer) &bouncefound)) {
+    ArrowCursor ();
+    ErrSetMessageLevel (oldsev);
+    Message (MSG_FATAL, "NetTestAsynchronousQuery failed");
+    return 1;
+  }
+
+  /* busy wait here, would normally call NetTestCheckQueue from event loop timer */
+  while (! bouncefound) {
+    stoptime = GetSecs ();
+    currsecs = stoptime - starttime;
+    if (currsecs != prevsecs && currsecs > 5) {
+      sprintf (str, "Checking connection - %ld seconds", (long) currsecs);
+      SetTitle (w, str);
+      Update ();
+      prevsecs = currsecs;
+    }
+    if (stoptime - starttime >= 30) {
+      ArrowCursor ();
+      Message (MSG_OK, "Internet connection attempt timed out, exiting");
+      return 1;
+    }
+    NetTestCheckQueue (&bouncequeue);
+  }
+  QUERY_CloseQueue (&bouncequeue);
+  ErrSetMessageLevel (oldsev);
+
   /*---------------------------------*/
   /* Get info about the DB server(s) */
   /*---------------------------------*/
 
   SetTitle (w, "Finding Entrez2 Server");
+  Update ();
 
   if (Query_GetInfo () == NULL) {
     ArrowCursor ();
-    Message (MSG_OK, "Unable to connect to Entrez2");
+    Message (MSG_OK, "%s", getInfoFailed);
     QuitProgram ();
     return 1;
   }
 
+  if (! PubSeqFetchEnable ()) {
+    Message (MSG_POSTERR, "PubSeqFetchEnable failed");
+  }
+
   /*--------------------------------*/
   /* Register viewers, NCBI Desktop */
   /*--------------------------------*/
 
   SetTitle (w, "Registering Viewers");
+  Update ();
 
   REGISTER_MEDLINE_VIEW;
   REGISTER_NEW_SEQENTRY_VIEW;
@@ -975,6 +1133,7 @@ Int2 Main (void)
   s_currFormNum = GetNewMainFormKey ();
 
   SetTitle (w, "Creating Query Window");
+  Update ();
 
   s_mainForm [s_currFormNum] = CreateTermlistForm (-50, -33,
                                                    "Query",
@@ -988,6 +1147,7 @@ Int2 Main (void)
   /*------------------------------------*/
 
   SetTitle (w, "Creating Document Window");
+  Update ();
 
   s_docSumForm [s_currFormNum] = CreateDocsumForm (-10, -90, "Document",
                                                    DocumentSummaryActivate_Callback,
@@ -1008,6 +1168,8 @@ Int2 Main (void)
   /* Exit */
   /*------*/
 
+  PubSeqFetchDisable ();
+
   return 0;
 }
 
