@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.29 $
+* $Revision: 6.49 $
 *
 * File Description: 
 *
@@ -916,17 +916,26 @@ typedef struct fieldpage {
   ButtoN             PNTR boxes;
 } FieldPage, PNTR FieldPagePtr;
 
-static Boolean ShouldBeAGBQual (Int2 qual, Boolean allowProductGBQual)
+static Boolean ShouldBeAGBQual (SeqFeatPtr sfp, Int2 qual, Boolean allowProductGBQual)
 
 {
   if (qual < 0) return FALSE;
   if (allowProductGBQual && qual == GBQUAL_product) return TRUE;
-  if (qual == GBQUAL_citation || qual == GBQUAL_db_xref || qual == GBQUAL_evidence ||
-      qual == GBQUAL_exception || qual == GBQUAL_gene || qual == GBQUAL_label ||
-      qual == GBQUAL_map || qual == GBQUAL_note || qual == GBQUAL_partial ||
-      qual == GBQUAL_product || qual == GBQUAL_pseudo) {
+  if (qual == GBQUAL_citation ||
+      qual == GBQUAL_db_xref ||
+      qual == GBQUAL_evidence ||
+      qual == GBQUAL_exception ||
+      qual == GBQUAL_gene ||
+      qual == GBQUAL_label ||
+      qual == GBQUAL_locus_tag ||
+      qual == GBQUAL_note ||
+      qual == GBQUAL_partial ||
+      qual == GBQUAL_product ||
+      qual == GBQUAL_pseudo) {
     return FALSE;
   }
+  if (qual == GBQUAL_map && (sfp == NULL || sfp->idx.subtype != FEATDEF_repeat_region)) return FALSE;
+  if (qual == GBQUAL_operon && (sfp == NULL || sfp->idx.subtype != FEATDEF_operon)) return FALSE;
   if (Nlm_GetAppProperty ("SequinUseEMBLFeatures") == NULL) {
     if (qual == GBQUAL_usedin) {
       return FALSE;
@@ -1115,13 +1124,19 @@ extern DialoG CreateImportFields (GrouP h, CharPtr name, SeqFeatPtr sfp, Boolean
 
         for (i = 0; i < sefp->mand_num; i++) {
           qual = sefp->mand_qual [i];
-          if (qual > -1 && ShouldBeAGBQual (qual, allowProductGBQual)) {
+          if (qual > -1 && ShouldBeAGBQual (sfp, qual, allowProductGBQual)) {
             seen [qual] = LEGAL_FEATURE;
           }
         }
+        if (StringCmp (name, "repeat_region") == 0) {
+          seen [GBQUAL_map] = TRUE;
+        }
+        if (StringCmp (name, "operon") == 0) {
+          seen [GBQUAL_operon] = TRUE;
+        }
         for (i = 0; i < sefp->opt_num; i++) {
           qual = sefp->opt_qual [i];
-          if (qual > -1 && ShouldBeAGBQual (qual, allowProductGBQual)) {
+          if (qual > -1 && ShouldBeAGBQual (sfp, qual, allowProductGBQual)) {
             seen [qual] = LEGAL_FEATURE;
           }
         }
@@ -1249,12 +1264,25 @@ static void ChangeCannedMessage (PopuP p)
       SetStatus (ffp->exception, TRUE);
       break;
     case 5 :
-      SetTitle (ffp->exceptText, "trans splicing");
+      SetTitle (ffp->exceptText, "trans-splicing");
       SetStatus (ffp->exception, TRUE);
       break;
     case 6 :
       SetTitle (ffp->exceptText, "artificial frameshift");
       SetStatus (ffp->exception, TRUE);
+      break;
+    case 7 :
+      SetTitle (ffp->exceptText, "nonconsensus splice site");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 8 :
+      SetTitle (ffp->exceptText, "rearrangement required for product");
+      SetStatus (ffp->exception, TRUE);
+      break;
+    case 9 :
+      SetTitle (ffp->exceptText, "alternative start codon");
+      SetStatus (ffp->exception, TRUE);
+      break;
     default :
       break;
   }
@@ -1273,6 +1301,10 @@ static void GeneXrefWarn (GrouP g)
 
 {
   Int2  val;
+  Boolean indexerVersion;
+
+  indexerVersion = (Boolean) (GetAppProperty ("InternalNcbiSequin") != NULL);
+  if (indexerVersion) return;
 
   val = GetValue (g);
   if (val == 2) {
@@ -1387,24 +1419,32 @@ extern GrouP CreateCommonFeatureGroupEx (GrouP h, FeatureFormPtr ffp,
       PopupItem (canned, "RNA editing");
       PopupItem (canned, "reasons given in citation");
       PopupItem (canned, "ribosomal slippage");
-      PopupItem (canned, "trans splicing");
+      PopupItem (canned, "trans-splicing");
       PopupItem (canned, "artificial frameshift");
+      PopupItem (canned, "nonconsensus splice site");
+      PopupItem (canned, "rearrangement required");
+      PopupItem (canned, "alternative start codon");
       if (sfp != NULL && sfp->excpt) {
         if (StringICmp (sfp->except_text, "RNA editing") == 0) {
           SetValue (canned, 2);
         } else if (StringICmp (sfp->except_text, "reasons given in citation") == 0 ||
                    StringICmp (sfp->except_text, "reasons cited in publication") == 0) {
           SetValue (canned, 3);
-        } else if (StringICmp (sfp->except_text, "ribosomal slippage") == 0) {
+        } else if (StringICmp (sfp->except_text, "ribosomal slippage") == 0 ||
+                   StringICmp (sfp->except_text, "ribosome slippage") == 0) {
           SetValue (canned, 4);
-        } else if (StringICmp (sfp->except_text, "ribosome slippage") == 0) {
-          SetValue (canned, 4);
-        } else if (StringICmp (sfp->except_text, "trans splicing") == 0) {
-          SetValue (canned, 5);
-        } else if (StringICmp (sfp->except_text, "trans-splicing") == 0) {
+        } else if (StringICmp (sfp->except_text, "trans-splicing") == 0 ||
+                   StringICmp (sfp->except_text, "trans splicing") == 0) {
           SetValue (canned, 5);
         } else if (StringICmp (sfp->except_text, "artificial frameshift") == 0) {
           SetValue (canned, 6);
+        } else if (StringICmp (sfp->except_text, "non-consensus splice site") == 0 ||
+                   StringICmp (sfp->except_text, "nonconsensus splice site") == 0) {
+          SetValue (canned, 7);
+        } else if (StringICmp (sfp->except_text, "rearrangement required for product") == 0) {
+          SetValue (canned, 8);
+        } else if (StringICmp (sfp->except_text, "alternative start codon") == 0) {
+          SetValue (canned, 9);
         }
       } else {
         SetValue (canned, 1);
@@ -1426,7 +1466,9 @@ extern GrouP CreateCommonFeatureGroupEx (GrouP h, FeatureFormPtr ffp,
     ffp->useGeneXref = NULL;
     ffp->newGeneGrp = NULL;
     ffp->geneSymbol = NULL;
+    ffp->geneAllele = NULL;
     ffp->geneDesc = NULL;
+    ffp->locusTag = NULL;
     for (page = 0; page < 5; page++) {
       ffp->commonSubGrp [page] = NULL;
     }
@@ -1460,8 +1502,12 @@ extern GrouP CreateCommonFeatureGroupEx (GrouP h, FeatureFormPtr ffp,
       ffp->newGeneGrp = HiddenGroup (y, 2, 0, NULL);
       StaticPrompt (ffp->newGeneGrp, "Gene Symbol", 0, dialogTextHeight, programFont, 'l');
       ffp->geneSymbol = DialogText (ffp->newGeneGrp, "", 20, NULL);
+      StaticPrompt (ffp->newGeneGrp, "Allele", 0, dialogTextHeight, programFont, 'l');
+      ffp->geneAllele = DialogText (ffp->newGeneGrp, "", 20, NULL);
       StaticPrompt (ffp->newGeneGrp, "Description", 0, dialogTextHeight, programFont, 'l');
       ffp->geneDesc = DialogText (ffp->newGeneGrp, "", 20, NULL);
+      StaticPrompt (ffp->newGeneGrp, "Locus Tag", 0, dialogTextHeight, programFont, 'l');
+      ffp->locusTag = DialogText (ffp->newGeneGrp, "", 20, NULL);
       Hide (ffp->newGeneGrp);
       ffp->editGeneBtn = PushButton (y, "Edit Gene Feature", Nlm_LaunchGeneFeatEd);
       SetObjectExtra (ffp->editGeneBtn, ffp, NULL);
@@ -1623,6 +1669,7 @@ extern Boolean FileToScrollText (TexT t, CharPtr path)
 {
   FILE     *fp;
   Int4     len;
+  Int4     read_len;
   Int4     max;
   CharPtr  str;
 #ifdef WIN_MAC
@@ -1635,17 +1682,25 @@ extern Boolean FileToScrollText (TexT t, CharPtr path)
 
   if (t != NULL && path != NULL && *path != '\0') {
     len = FileLength (path);
+    max = (Int4) INT2_MAX;
 #ifdef WIN_MOTIF
     max = INT4_MAX;
-#else
-    max = (Int4) INT2_MAX;
+#endif
+#ifdef WIN_MSWIN
+    max = INT4_MAX;
+#endif
+#ifdef WIN_MAC
+#ifdef OS_UNIX_DARWIN
+    max = INT4_MAX;
+#endif
 #endif
     if (len > 0 && len < max - 4) {
       str = MemNew (sizeof (char) * (len + 3));
       if (str != NULL) {
         fp = FileOpen (path, "r");
         if (fp != NULL) {
-          FileRead (str, sizeof (char), (size_t) len, fp);
+          read_len = FileRead (str, sizeof (char), (size_t) len, fp);
+          str [ read_len ] = 0;
 #if (defined(OS_DOS) || defined (OS_NT))
           p = str;
           q = str;
@@ -1685,7 +1740,6 @@ extern void ScrollTextToFile (TexT t, CharPtr path)
 {
   FILE     *fp;
   size_t   len;
-  Int4     max;
   CharPtr  str;
 #ifdef WIN_MAC
   CharPtr  p;
@@ -1693,12 +1747,7 @@ extern void ScrollTextToFile (TexT t, CharPtr path)
 
   if (t != NULL && path != NULL && *path != '\0') {
     len = TextLength (t);
-#ifdef WIN_MOTIF
-    max = INT4_MAX;
-#else
-    max = (Int4) INT2_MAX;
-#endif
-    if (len > 0 && (Int4) len < max - 4) {
+    if (len > 0) {
 #ifdef WIN_MAC
       fp = FileOpen (path, "r");
       if (fp != NULL) {
@@ -1795,6 +1844,8 @@ typedef struct textviewform {
 
   DoC              doc;
   TexT             text;
+
+  TexT             find;
 } TextViewForm, PNTR TextViewFormPtr;
 
 static ParData txtParFmt = {FALSE, FALSE, FALSE, FALSE, TRUE, 0, 0};
@@ -1924,10 +1975,126 @@ static void TextViewFormMessage (ForM f, Int2 mssg)
   }
 }
 
-extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title)
+
+static void FindInGeneralText (ButtoN b)
 
 {
+  Int2             actual;
+  Char             buf [1030];
+  Char             ch;
+  Int2             cnt;
+  Int4             cntr;
+  Int2             first;
+  FILE             *fp;
+  Char             lastch;
+  Int4             line;
+  ValNodePtr       matches;
+  Int2             next;
+  Int2             offset = 0;
+  Char             path [PATH_MAX];
+  CharPtr          ptr;
+  Int2             state;
+  CharPtr          str;
+  TextFsaPtr       tbl;
+  TextViewFormPtr  tfp;
+  Int4             max;
+
+  tfp = (TextViewFormPtr) GetObjectExtra (b);
+  if (tfp == NULL) return;
+  if (tfp->doc != NULL) {
+    GetOffset (tfp->doc, NULL, &offset);
+  } else if (tfp->text != NULL) {
+    GetOffset (tfp->text, NULL, &offset);
+  }
+  first = -1;
+  next = -1;
+  max = INT2_MAX;
+
+  str = SaveStringFromText (tfp->find);
+
+  if (StringDoesHaveText) {
+    TmpNam (path);
+    if (ExportForm (tfp->form, path)) {
+      tbl = TextFsaNew ();
+      if (tbl != NULL) {
+        TextFsaAdd (tbl, str);
+        fp = FileOpen (path, "r");
+        if (fp != NULL) {
+          line = 0;
+          state = 0;
+          cntr = FileLength (path);
+          cnt = (Int2) MIN (cntr, 1024L);
+          lastch = '\0';
+          while (cnt > 0 && cntr > 0 && line <= max && 
+                 ((next == -1 && offset > first) || first == -1)) {
+            actual = (Int2) FileRead (buf, 1, cnt, fp);
+            if (actual > 0) {
+              cnt = actual;
+              buf [cnt] = '\0';
+              ptr = buf;
+              ch = *ptr;
+              while (ch != '\0') {
+                if (ch == '\n' || ch == '\r') {
+                  if (ch == '\n' && lastch == '\r') {
+                    /* do not increment line */
+                  } else if (ch == '\r' && lastch == '\n') {
+                    /* do not increment line */
+                  } else {
+                    line++;
+                  }
+                }
+                state = TextFsaNext (tbl, state, ch, &matches);
+                if (matches != NULL) {
+                  if (first == -1) {
+                    first = line;
+                  }
+                  if (next == -1 && line > offset) {
+                    next = line;
+                  }
+                }
+                lastch = ch;
+                ptr++;
+                ch = *ptr;
+              }
+              cntr -= cnt;
+              cnt = (Int2) MIN (cntr, 1024L);
+            } else {
+              cnt = 0;
+              cntr = 0;
+            }
+          }
+        }
+        FileClose (fp);
+      }
+      TextFsaFree (tbl);
+    }
+    FileRemove (path);
+  }
+  MemFree (str);
+  if (line > max) {
+    Message (MSG_ERROR, "Too many lines for search");
+  }
+
+  if (next >= 0) {
+    offset = next;
+  } else if (first >= 0) {
+    offset = first;
+  } else return;
+  if (tfp->doc != NULL) {
+    SetOffset (tfp->doc, 0, offset);
+    Update ();
+  } else if (tfp->text != NULL) {
+    SetOffset (tfp->text, 0, offset);
+    Update ();
+  }
+}
+
+static void LaunchGeneralTextViewerEx (CharPtr path, CharPtr title, Boolean useScrollText)
+
+{
+  ButtoN             b;
   FonT               fnt;
+  GrouP              g;
   Int2               pixheight;
   Int2               pixwidth;
   StdEditorProcsPtr  sepp;
@@ -1939,71 +2106,95 @@ extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title)
 #endif
 
   tfp = (TextViewFormPtr) MemNew (sizeof (TextViewForm));
-  if (tfp != NULL) {
-    w = DocumentWindow (-50, -33, -10, -10, title, StdCloseWindowProc, ResizeTextViewer);
-    SetObjectExtra (w, tfp, StdCleanupFormProc);
-    tfp->form = (ForM) w;
-    tfp->exportform = ExportTextViewForm;
-    tfp->formmessage = TextViewFormMessage;
+  if (tfp == NULL) return;
 
-    sepp = (StdEditorProcsPtr) GetAppProperty ("StdEditorForm");
-    if (sepp != NULL) {
-      SetActivate (w, sepp->activateForm);
-      tfp->appmessage = sepp->handleMessages;
+  w = DocumentWindow (-50, -33, -10, -10, title, StdCloseWindowProc, ResizeTextViewer);
+  SetObjectExtra (w, tfp, StdCleanupFormProc);
+  tfp->form = (ForM) w;
+  tfp->exportform = ExportTextViewForm;
+  tfp->formmessage = TextViewFormMessage;
+
+  sepp = (StdEditorProcsPtr) GetAppProperty ("StdEditorForm");
+  if (sepp != NULL) {
+    SetActivate (w, sepp->activateForm);
+    tfp->appmessage = sepp->handleMessages;
+  }
+
+  fnt = programFont;
+  pixwidth = 35 * stdCharWidth + 17;
+  pixheight = 20 * stdLineHeight;
+
+  tvpp = (TextViewProcsPtr) GetAppProperty ("TextDisplayForm");
+  if (tvpp != NULL) {
+    pixwidth = MAX (pixwidth, tvpp->minPixelWidth);
+    pixheight = MAX (pixheight, tvpp->minPixelHeight);
+    if (tvpp->displayFont != NULL) {
+      fnt = tvpp->displayFont;
     }
-
-    fnt = programFont;
-    pixwidth = 35 * stdCharWidth + 17;
-    pixheight = 20 * stdLineHeight;
-
-    tvpp = (TextViewProcsPtr) GetAppProperty ("TextDisplayForm");
-    if (tvpp != NULL) {
-      pixwidth = MAX (pixwidth, tvpp->minPixelWidth);
-      pixheight = MAX (pixheight, tvpp->minPixelHeight);
-      if (tvpp->displayFont != NULL) {
-        fnt = tvpp->displayFont;
-      }
-      if (tvpp->activateForm != NULL) {
-        SetActivate (w, tvpp->activateForm);
-      }
+    if (tvpp->activateForm != NULL) {
+      SetActivate (w, tvpp->activateForm);
     }
+  }
 
 #ifndef WIN_MAC
-    m = PulldownMenu (w, "File");
-    FormCommandItem (m, "Export...", (BaseFormPtr) tfp, VIB_MSG_EXPORT);
-    SeparatorItem (m);
-    FormCommandItem (m, "Close", (BaseFormPtr) tfp, VIB_MSG_CLOSE);
-    if (tvpp != NULL && tvpp->useScrollText) {
-      m = PulldownMenu (w, "Edit");
-      FormCommandItem (m, CUT_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_CUT);
-      FormCommandItem (m, COPY_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_COPY);
-      FormCommandItem (m, PASTE_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_PASTE);
-      FormCommandItem (m, CLEAR_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_DELETE);
-    }
+  m = PulldownMenu (w, "File");
+  FormCommandItem (m, "Export...", (BaseFormPtr) tfp, VIB_MSG_EXPORT);
+  SeparatorItem (m);
+  FormCommandItem (m, "Close", (BaseFormPtr) tfp, VIB_MSG_CLOSE);
+  if (tvpp != NULL && tvpp->useScrollText) {
+    m = PulldownMenu (w, "Edit");
+    FormCommandItem (m, CUT_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_CUT);
+    FormCommandItem (m, COPY_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_COPY);
+    FormCommandItem (m, PASTE_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_PASTE);
+    FormCommandItem (m, CLEAR_MENU_ITEM, (BaseFormPtr) tfp, VIB_MSG_DELETE);
+  }
 #endif
 
-    if (tvpp != NULL && tvpp->useScrollText) {
-      tfp->text = ScrollText (w, (pixwidth + stdCharWidth - 1) / stdCharWidth,
-                              (pixheight + stdLineHeight - 1) / stdLineHeight,
-                              fnt, FALSE, NULL);
-      SetObjectExtra (tfp->text, tfp, NULL);
-      RealizeWindow (w);
-      if (! FileToScrollText (tfp->text, path)) {
-        SetTitle (tfp->text, "(Text is too large to be displayed in this control.)");
-      }
-    } else {
-      tfp->doc = DocumentPanel (w, pixwidth, pixheight);
-      SetObjectExtra (tfp->doc, tfp, NULL);
-      RealizeWindow (w);
-      txtColFmt.pixWidth = screenRect.right - screenRect.left;
-      txtColFmt.pixInset = 8;
-      DisplayFancy (tfp->doc, path, &txtParFmt, &txtColFmt, fnt, 0);
-      /* document.c: SaveTableItem does not strip preceeding tabs if tabCount is 0 */
+  /* right now Find button is only in indexer Sequin */
+  if (useScrollText) {
+    g = HiddenGroup (w, 5, 0, NULL);
+    b = PushButton (g, "Find", FindInGeneralText);
+    SetObjectExtra (b, tfp, NULL);
+    tfp->find = DialogText (g, "", 10, NULL);
+  }
+
+  if (useScrollText) {
+    tfp->text = ScrollText (w, (pixwidth + stdCharWidth - 1) / stdCharWidth,
+                            (pixheight + stdLineHeight - 1) / stdLineHeight,
+                            fnt, FALSE, NULL);
+    SetObjectExtra (tfp->text, tfp, NULL);
+    RealizeWindow (w);
+    if (! FileToScrollText (tfp->text, path)) {
+      /* SetTitle (tfp->text, "(Text is too large to be displayed in this control.)"); */
+      Remove (w);
+      LaunchGeneralTextViewerEx (path, title, FALSE);
+      return;
     }
-    Show (w);
-    Select (w);
-    Update ();
-  } 
+  } else {
+    tfp->doc = DocumentPanel (w, pixwidth, pixheight);
+    SetObjectExtra (tfp->doc, tfp, NULL);
+    RealizeWindow (w);
+    txtColFmt.pixWidth = screenRect.right - screenRect.left;
+    txtColFmt.pixInset = 8;
+    DisplayFancy (tfp->doc, path, &txtParFmt, &txtColFmt, fnt, 0);
+    /* document.c: SaveTableItem does not strip preceeding tabs if tabCount is 0 */
+  }
+  Show (w);
+  Select (w);
+  Update ();
+}
+
+extern void LaunchGeneralTextViewer (CharPtr path, CharPtr title)
+
+{
+  TextViewProcsPtr tvpp;
+
+  tvpp = (TextViewProcsPtr) GetAppProperty ("TextDisplayForm");
+  if (tvpp != NULL && tvpp->useScrollText) {
+    LaunchGeneralTextViewerEx (path, title, TRUE);
+  } else {
+    LaunchGeneralTextViewerEx (path, title, FALSE);
+  }
 }
 
 #ifndef WIN_MAC
