@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   2/19/97
 *
-* $Revision: 6.26 $
+* $Revision: 6.28 $
 *
 * File Description: 
 *
@@ -85,7 +85,7 @@ typedef struct seqeditview {
                    viewmodeitem;
   IteM             prefitem;
   IteM             undoitem;
-  IteM             cutitem, delitem, copyitem, pasteitem;
+  IteM             cutitem, delitem, copyitem, pasteitem, insitem;
   IteM             rfitem[10];
   IteM             conscolor;
   IteM             importseq,
@@ -199,6 +199,7 @@ extern Int2 LIBCALLBACK SeqEditFunc PROTO((Pointer data));
 extern Int2 LIBCALLBACK AlgEditFunc PROTO((Pointer data));
 extern Int2 LIBCALLBACK AnnotAlgEditFunc PROTO((Pointer data));
 extern Int2 LIBCALLBACK AlgViewFunc (Pointer data);
+extern void OldSequenceEditor (IteM i);
 
 extern void LaunchAnnotAlignEditor (SeqAnnotPtr sap);
 extern void LaunchAlignEditor (SeqAlignPtr salp);
