@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/27/96
 *
-* $Revision: 6.7 $
+* $Revision: 6.8 $
 *
 * File Description: 
 *
@@ -130,6 +130,10 @@ extern void ColorIdentityDialogItem (IteM i);
 
 NLM_EXTERN void PropagateFeatDialog (IteM i);
 
+extern ValNodePtr JK_NTPattern (CharPtr pattern, ValNodePtr sqloc_list, Boolean flagInvert, Uint1 mol_type);
+extern ValNodePtr JK_NTPattern2 (CharPtr pattern, ValNodePtr sqloc_list, Boolean flagInvert, Uint2 entityID);
+
+
 #ifdef __cplusplus
 }
 #endif
