@@ -1,4 +1,4 @@
-/*  $RCSfile: ncbicli.c,v $  $Revision: 4.9 $  $Date: 1998/05/08 15:22:33 $
+/*  $RCSfile: ncbicli.c,v $  $Revision: 4.20 $  $Date: 2000/12/01 23:06:07 $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,6 +31,42 @@
 *
 * --------------------------------------------------------------------------
 * $Log: ncbicli.c,v $
+* Revision 4.20  2000/12/01 23:06:07  vakatov
+* s_SendHeaderWWW() -- added missing \r before \n (HTTP requirement)
+*
+* Revision 4.19  2000/02/09 15:54:31  vakatov
+* NIC_GetService() -- Changed error message format
+*
+* Revision 4.18  1999/06/24 17:59:35  vakatov
+* Untie the HTTP and regular proxy servers: SRV_HTTP_PROXY_***, SRV_PROXY_HOST
+*
+* Revision 4.17  1999/03/12 22:38:23  vakatov
+* More detailed debug printout in "s_ProcessReply_WWWClient()"
+*
+* Revision 4.16  1999/03/11 19:26:30  vakatov
+* s_ProcessReply_WWWClient():  Use nic->server_host(Int4) rather then
+* nic->server_host(unsigned long) in the SOCK_Address()
+*
+* Revision 4.15  1999/03/11 15:20:20  vakatov
+* Added "timeout" arg to SOCK_Create() and SOCK_Reconnect()
+*
+* Revision 4.14  1999/01/22 22:04:58  vakatov
+* Uint4toInaddr() to take address in the network byte order
+*
+* Revision 4.13  1998/12/15 17:26:53  vakatov
+* + Handle the case of connecting through a CERN-like non-transparent proxy
+*
+* Revision 4.12  1998/10/13 20:51:14  vakatov
+* Added to HTTP header:  &platform=<Nlm_PlatformName()> -- to report
+* the client's platform
+*
+* Revision 4.11  1998/09/08 17:59:03  vakatov
+* Added WWW/Firewall network interface
+*
+* Revision 4.10  1998/08/05 20:19:17  vakatov
+* [eNIC_WWWClient]  NIC_GetService():  send the bytestore contents to
+* server rather than to dispatcher
+*
 * Revision 4.9  1998/05/08 15:22:33  vakatov
 * NIC_GetService():  added more detailed error diagnostics
 *
@@ -66,9 +102,9 @@ typedef struct Nlm_NICtag
 {
   SOCK    sock;
   CharPtr client_host;
-  Uint4   server_host;
-  Uint2   server_port;
-  Uint4   ticket;
+  Uint4   server_host;  /* local   byte-order */
+  Uint2   server_port;  /* local   byte-order */
+  Uint4   ticket;       /* network byte-order */
 } Nlm_NICstruct;
 
 
@@ -85,7 +121,8 @@ static Boolean s_SendHeaderWWW
   static char X_POST_1[] = "POST ";
   static char X_POST_2[] = "?service=";
   static char X_POST_3[] = "&address=";
-  static char X_POST_4[] = " HTTP/1.0\n";
+  static char X_POST_4[] = "&platform=";
+  static char X_POST_E[] = " HTTP/1.0\r\n";
   static char X_AGENT_1[] = "User-Agent: ";
   static char X_AGENT_2[] = " from ";
 
@@ -104,6 +141,10 @@ static Boolean s_SendHeaderWWW
                  StrLen(client_host), 0) != eSOCK_ESuccess  ||
       SOCK_Write(sock, (const void *)X_POST_4,  StrLen(X_POST_4 ), 0)
       != eSOCK_ESuccess  ||
+      SOCK_Write(sock, (const void *)Nlm_PlatformName(),
+                 StrLen(Nlm_PlatformName()), 0) != eSOCK_ESuccess  ||
+      SOCK_Write(sock, (const void *)X_POST_E,  StrLen(X_POST_E ), 0)
+      != eSOCK_ESuccess  ||
       SOCK_Write(sock, (const void *)X_AGENT_1, StrLen(X_AGENT_1), 0)
       != eSOCK_ESuccess  ||
       SOCK_Write(sock, (const void *)client_agent,
@@ -114,8 +155,8 @@ static Boolean s_SendHeaderWWW
       SOCK_Write(sock, (const void *)client_host,
                  StrLen(client_host), 0)
       != eSOCK_ESuccess  ||
-      sprintf(buffer, "\nContent-type: application/x-www-form-urlencoded"
-              "\nContent-Length: %lu\n\n", (unsigned long)content_length)
+      sprintf(buffer, "\r\nContent-type: application/x-www-form-urlencoded"
+              "\r\nContent-Length: %lu\r\n\r\n", (unsigned long)content_length)
       <= 0  ||
       SOCK_Write(sock, (const void *)buffer,    StrLen(buffer   ), 0)
       != eSOCK_ESuccess) {
@@ -173,6 +214,7 @@ static Uint4 s_NIC_SockSkip(SOCK sock, Uint4 n_skip)
 /* Get and process reply from dispatcher(WWW Special-Client agent)
  */
 static Boolean s_ProcessReply_WWWClient(NIC nic, const STimeout *timeout,
+                                        const Char *proxy_host,
                                         Boolean debug_printout)
 {
   Uint4         buf_read;
@@ -199,8 +241,8 @@ static Boolean s_ProcessReply_WWWClient(NIC nic, const STimeout *timeout,
     buffer[buf_read] = '\0';
 
     if ( debug_printout ) {
-      fprintf(stderr, "[WWW Special Client, reply %lu:%lu]: \"%s\"",
-              (unsigned long)buf_read, (unsigned long)(buf_read + n_read),
+      fprintf(stderr, "[WWW Special Client, reply %lu:%lu]: \"%s\"\n",
+              (unsigned long)(buf_read - n_read), (unsigned long)buf_read,
               buffer + buf_read - n_read);
     }
 
@@ -222,7 +264,7 @@ static Boolean s_ProcessReply_WWWClient(NIC nic, const STimeout *timeout,
       break;
     }
 
-    /* avoid the read-buffer overflow */
+    /* avoid overflow in the read buffer */
 #define SHIFT_SIZE (sizeof(buffer) / 2)
     if (buf_read > SHIFT_SIZE + sizeof(X_TAG)) {
       buf_read -= SHIFT_SIZE;
@@ -238,13 +280,29 @@ static Boolean s_ProcessReply_WWWClient(NIC nic, const STimeout *timeout,
     return FALSE; /* i.e. the reply is bad or missing */
 
   {{ /* Connect to the "real" server and send back the ticket */
-    VERIFY( Uint4toInaddr(nic->server_host, buffer, sizeof(buffer)) );
-    if (SOCK_Create(buffer, nic->server_port, &nic->sock) != eSOCK_ESuccess) {
+    if (proxy_host  &&  *proxy_host) {
+      StringNCpy_0(buffer, proxy_host, sizeof(buffer));
+    } else {
+      VERIFY( Uint4toInaddr(Nlm_htonl(nic->server_host),
+                            buffer, sizeof(buffer)) );
+    }
+
+    if (SOCK_Create(buffer, nic->server_port, timeout, &nic->sock)
+        != eSOCK_ESuccess) {
       ErrPostEx(SEV_ERROR, NIC_ERROR, 7,
                 "[WWW Special] Cannot connect to server \"%s\", port %u",
                 buffer, (unsigned int)nic->server_port);
       return FALSE;
     }
+    if ( debug_printout ) {
+      fprintf(stderr,
+              "[WWW Special Client] Connected to server \"%s\", port %u\n",
+              buffer, (unsigned int)nic->server_port);
+    }
+
+    /* store the server host */
+    SOCK_Address(nic->sock, &nic->server_host, 0, FALSE);
+
 
     VERIFY( SOCK_SetTimeout(nic->sock, eSOCK_OnReadWrite, timeout, 0, 0)
             == eSOCK_ESuccess );
@@ -255,6 +313,9 @@ static Boolean s_ProcessReply_WWWClient(NIC nic, const STimeout *timeout,
                 "[WWW Special] Cannot send the ticket back to server");
       return FALSE;
     }
+    if ( debug_printout ) {
+      fprintf(stderr, "[WWW Special Client] Sent ticket back to the server\n");
+    }
   }}
 
   return TRUE;
@@ -316,6 +377,7 @@ static Boolean s_ProcessReply_WWWDirect(NIC nic, Boolean debug_printout)
  *  EXTERNAL
  ***********************************************************************/
 
+
 NLM_EXTERN NIC NIC_GetService
 (const Char      *service_name,
  const Char      *disp_host,
@@ -324,8 +386,9 @@ NLM_EXTERN NIC NIC_GetService
  const STimeout  *timeout,
  ENIC_Agent       client_agent,
  const Char      *client_host,
+ const Char      *proxy_host,
  const ByteStore *service_query,
- Uint4            flags)
+ TNIC_Flags       flags)
 {
   NIC  nic = (NIC)MemNew(sizeof(Nlm_NICstruct));
 
@@ -335,7 +398,8 @@ NLM_EXTERN NIC NIC_GetService
 
   for (;;) {{ /* TRY to establish a connection */
     /* connect to dispatcher(or server) */
-    if (SOCK_Create(disp_host, disp_port, &nic->sock) != eSOCK_ESuccess) {
+    if (SOCK_Create(disp_host, disp_port, timeout, &nic->sock)
+        != eSOCK_ESuccess) {
       ErrPostEx(SEV_ERROR, NIC_ERROR, 1,
                 "[NIC_GetService]  Cannot connect to host \"%s\", port %d;",
                 disp_host, (int)disp_port);
@@ -365,24 +429,31 @@ NLM_EXTERN NIC NIC_GetService
       break;
     }
 
-    /* send HTTP header-request */
-    if ( !s_SendHeaderWWW(nic->sock, service_name, disp_path,
-                          (client_agent == eNIC_WWWClient) ?
-                          "NCBISpecialClient" : "NCBIDirectClient",
-                          nic->client_host,
-                          (Uint4)BSLen((ByteStorePtr)service_query)) )
-      break;
-
-    /* send the bytestore content(query), if any */
-    if (service_query  &&  !s_SendBS(nic->sock, service_query))
-      break;
-
-    /* get reply from dispatcher and process it */
+    /* send HTTP header, get reply from dispatcher and process it */
     if (client_agent == eNIC_WWWClient) {
-      if (!s_ProcessReply_WWWClient
-          (nic, timeout, (Boolean)(flags & NIC_DEBUG_PRINTOUT)) )
+      /* send HTTP header */
+      if ( !s_SendHeaderWWW(nic->sock, service_name, disp_path,
+                            (flags & NIC_FIREWALL) ?
+                            "NCBISpecialFWClient" : "NCBISpecialClient",
+                            nic->client_host, 0) )
+        break;
+
+      /* process reply from the dispatcher */
+      if ( !s_ProcessReply_WWWClient
+           (nic, timeout, proxy_host,
+            (Boolean)(flags & NIC_DEBUG_PRINTOUT)) )
         break;
     } else { /* i.e. eNIC_WWWDirect */
+      /* send HTTP header(along with the bytestore content, if any) */
+      if ( !s_SendHeaderWWW(nic->sock, service_name, disp_path,
+                            "NCBIDirectClient", nic->client_host,
+                            (Uint4)BSLen((ByteStorePtr)service_query)) )
+        break;
+
+      if (service_query  &&  !s_SendBS(nic->sock, service_query))
+        break;
+
+      /* process reply from the dispatcher */
       if (!s_ProcessReply_WWWDirect
           (nic, (Boolean)(flags & NIC_DEBUG_PRINTOUT)) )
         break;
@@ -397,14 +468,19 @@ NLM_EXTERN NIC NIC_GetService
       break;
     }
 
+    /* for "eNIC_WWWClient", send the bytestore content(query) now */
+    if (client_agent == eNIC_WWWClient  &&  service_query  &&
+        !s_SendBS(nic->sock, service_query))
+      break;
+
     /* success */
     return nic;
   }} /* end-of-TRY */
 
   /* error */
   ErrPostEx(SEV_ERROR, NIC_ERROR, 10,
-            "[NIC_GetService]  Failed to establish connection to dispatcher "
-            " %s:%d/%s, for service\"%s\"",
+            "[NIC_GetService]  Failed to establish connection to dispatcher"
+            " at \"%s:%d\", path \"%s\", for service \"%s\"",
             disp_host, (int)disp_port, disp_path, service_name);
   NIC_CloseService(nic);
   return 0;
@@ -462,7 +538,7 @@ static Nlm_Int2 TEST__ncbicli_1(SOCK sock)
 
 static Nlm_Int2 TEST__ncbicli(ENIC_Agent agent)
 {
-  static Char *service_name  = "Entrez";
+  static Char service_name[]  = "Entrez";
   Char        *service_query = service_name;
 
   NIC          nic;
@@ -478,7 +554,7 @@ static Nlm_Int2 TEST__ncbicli(ENIC_Agent agent)
   BSWrite(service_query_bsp, service_query, StrLen(service_query)+1);
   nic = NIC_GetService(service_name,
                        TEST_ENGINE_HOST, TEST_ENGINE_PORT, TEST_ENGINE_URL,
-                       &timeout, agent, 0, service_query_bsp,
+                       &timeout, agent, 0, 0, service_query_bsp,
                        NIC_DEBUG_PRINTOUT);
   ASS_RET(nic, 1);
   BSFree(service_query_bsp);
