@@ -29,7 +29,7 @@
 *
 * Version Creation Date:        2/14/94
 *
-* $Revision: 6.1 $
+* $Revision: 6.2 $
 *
 * File Description:
 *   Supports RSAREF-based encryption for NCBI Network Services client-server
@@ -38,8 +38,6 @@
 *
 * Modifications:
 * --------------------------------------------------------------------------
-* Date     Name        Description of modification
-* -------  ----------  -----------------------------------------------------
 * 02/22/94 Epstein     Fix reading of public keys from config. file, and
 *                      initialization of random data structure for RSA
 *                      encryption.
@@ -55,30 +53,19 @@
 *                      from trying to decrypt a buffer when we don't have
 *                      at least 8 bytes of input to work with.
 *
-*
-* RCS Modification History:
 * $Log: ni_encr.c,v $
+* Revision 6.2  1998/06/23 19:45:19  vakatov
+* [WIN32,MSVC++]  Made some functions be DLL-exportable
+*
 * Revision 6.1  1997/12/02 16:09:37  epstein
 * fix use of sprintf
 *
-* Revision 6.0  1997/08/25 18:38:32  madden
-* Revision changed to 6.0
+* Revision 4.2  1996/02/22  18:43:03  epstein
+* Correct boundary condition in DoDesRead()
 *
-* Revision 5.0  1996/05/28 14:11:55  ostell
-* Set to revision 5.0
-*
- * Revision 4.2  1996/02/22  18:43:03  epstein
- * Correct boundary condition in DoDesRead()
- *
- * Revision 4.1  1995/10/06  12:51:34  epstein
- * cleaned-up port to MS Windows
- *
- * Revision 4.0  1995/07/26  13:56:32  ostell
- * force revision to 4.0
- *
- * Revision 1.8  1995/05/17  17:52:00  epstein
- * add RCS log revision history
- *
+* Revision 4.1  1995/10/06  12:51:34  epstein
+* cleaned-up port to MS Windows
+* --------------------------------------------------------------------------
 */
 
 #include <ncbi.h>
@@ -100,7 +87,7 @@
 
 
 
-/*
+/*
  * Purpose:     Encrypts a buffer using DES
  *
  * Parameters:
@@ -152,7 +139,7 @@ DoDesWrite(NI_EncrDataPtr encr, CharPtr buf, int len, CharPtr tmpbuf)
 }
 
 
-/*
+/*
  * Purpose:     Encryption write-filter for DES
  *
  * Parameters:
@@ -181,7 +168,7 @@ DesWriteFilt(VoidPtr mhvoid, CharPtr buf, int len, CharPtr tmpbuf)
 }
 
 
-/*
+/*
  * Purpose:     Decrypts a buffer using DES
  *
  * Parameters:
@@ -328,7 +315,7 @@ DoDesRead(NI_EncrDataPtr encr, CharPtr buf, int bytesRead, int len)
 }
 
 
-/*
+/*
  * Purpose:     DES read filter
  *
  * Parameters:
@@ -358,7 +345,7 @@ DesReadFilt(VoidPtr mhvoid, CharPtr buf, int bytesRead, int len, CharPtr PNTR ex
 }
 
 
-/*
+/*
  * Purpose:     Setup DES encryption for this message handle
  *
  * Parameters:
@@ -381,7 +368,7 @@ DesReadFilt(VoidPtr mhvoid, CharPtr buf, int bytesRead, int len, CharPtr PNTR ex
  *              handle.
  */
 
-Boolean LIBCALL
+NLM_EXTERN Boolean LIBCALL
 NI_SetupDESEncryption(NI_HandPtr mh, UcharPtr desKey)
 {
     Uchar iv[8];
@@ -407,7 +394,7 @@ NI_SetupDESEncryption(NI_HandPtr mh, UcharPtr desKey)
 }
 
 
-/*
+/*
  * Purpose:     Seed the random number generator if necessary
  *
  * Description:
@@ -454,7 +441,7 @@ SetRandomSeed(void)
 
 
 
-/*
+/*
  * Purpose:     Initialize an RSAREF random data structure used for RSA encryption
  *
  * Parameters:
@@ -497,7 +484,7 @@ R_RANDOM_STRUCT *randomStruct;
 
 
 
-/*
+/*
  * Purpose:     Convert pub encryption key from RSAREF format to internal NCBI fmt
  *
  * Parameters:
@@ -528,7 +515,7 @@ PubKeyToInternalFormat(R_RSA_PUBLIC_KEY PNTR publicKey)
 
 
 
-/*
+/*
  * Purpose:     Convert pub encryption key from internal NCBI format to RSAREF fmt
  *
  * Parameters:
@@ -560,7 +547,7 @@ InternalToPubKeyFormat(NI_PubKeyPtr internal)
 }
 
 
-/*
+/*
  * Purpose:     Compare two public encryption keys for equality
  *
  * Parameters:
@@ -575,7 +562,7 @@ InternalToPubKeyFormat(NI_PubKeyPtr internal)
  *              structures and hence easy to compare. 
  */
 
-Boolean LIBCALL
+NLM_EXTERN Boolean LIBCALL
 NI_PubKeysEqual(NI_PubKeyPtr x, NI_PubKeyPtr y)
 {
     R_RSA_PUBLIC_KEY PNTR xRsa = InternalToPubKeyFormat(x);
@@ -597,7 +584,7 @@ NI_PubKeysEqual(NI_PubKeyPtr x, NI_PubKeyPtr y)
 
 
 
-/*
+/*
  * Purpose:     Pseudo-randomly generate an 8-byte DES key
  *
  * Parameters:
@@ -614,7 +601,7 @@ NI_PubKeysEqual(NI_PubKeyPtr x, NI_PubKeyPtr y)
  *              use, but this is a reasonable implementation.
  */
 
-void
+NLM_EXTERN void
 NI_GenerateDESKey(UcharPtr desKey)
 {
     Int2 i;
@@ -631,7 +618,7 @@ NI_GenerateDESKey(UcharPtr desKey)
 
 
 
-/*
+/*
  * Purpose:     Generate pub encryption public + private keys, and write to files
  *
  * Parameters:
@@ -656,7 +643,7 @@ NI_GenerateDESKey(UcharPtr desKey)
  *              encryption to NCBI Network Services.
  */
 
-Boolean LIBCALL
+NLM_EXTERN Boolean LIBCALL
 NI_GenAndWritePEMKeys(Int2 bits, AsnIoPtr pubAip, FILE *privFp)
 {
     R_RSA_PUBLIC_KEY publicKey;
@@ -698,7 +685,7 @@ NI_GenAndWritePEMKeys(Int2 bits, AsnIoPtr pubAip, FILE *privFp)
 
 
 
-/*
+/*
  * Purpose:     Write a public key to a standard location on client machines
  *
  * Parameters:
@@ -714,7 +701,7 @@ NI_GenAndWritePEMKeys(Int2 bits, AsnIoPtr pubAip, FILE *privFp)
  *              originally it was stored in the NCBI configuration file. 
  */
 
-Boolean LIBCALL
+NLM_EXTERN Boolean LIBCALL
 NI_WritePubKeyToConfig (NI_PubKeyPtr pub)
 {
     Char fname[PATH_MAX];
@@ -753,7 +740,7 @@ NI_WritePubKeyToConfig (NI_PubKeyPtr pub)
     return TRUE;
 }
 
-/*
+/*
  * Purpose:     Read a public key from a standard configuration location
  *
  * Returns:
@@ -765,7 +752,7 @@ NI_WritePubKeyToConfig (NI_PubKeyPtr pub)
  *              originally it was stored in the NCBI configuration file. 
  */
 
-NI_PubKeyPtr LIBCALL
+NLM_EXTERN NI_PubKeyPtr LIBCALL
 NI_ReadPubKeyFromConfig (void)
 {
     Char fname[PATH_MAX];
@@ -806,7 +793,7 @@ NI_ReadPubKeyFromConfig (void)
     return pub;
 }
 
-/*
+/*
  * Purpose:     Make a copy of a public key
  *
  * Parameters:
@@ -819,7 +806,7 @@ NI_ReadPubKeyFromConfig (void)
  *              Makes a copy of a public key
  */
 
-NI_PubKeyPtr LIBCALL
+NLM_EXTERN NI_PubKeyPtr LIBCALL
 NI_PubKeyDup (NI_PubKeyPtr orig)
 {
     NI_PubKeyPtr dup;
@@ -834,7 +821,7 @@ NI_PubKeyDup (NI_PubKeyPtr orig)
 }
     
 
-/*
+/*
  * Purpose:     Load a private-key from the specified file pointer
  *
  * Parameters:
@@ -848,7 +835,7 @@ NI_PubKeyDup (NI_PubKeyPtr orig)
  *              Reads private key from data file.
  */
 
-VoidPtr LIBCALL
+NLM_EXTERN VoidPtr LIBCALL
 NI_LoadPrivKey(FILE *fp, Int2Ptr privKeyLenPtr)
 {
     R_RSA_PRIVATE_KEY PNTR privKey;
@@ -865,7 +852,7 @@ NI_LoadPrivKey(FILE *fp, Int2Ptr privKeyLenPtr)
     return privKey;
 }
 
-/*
+/*
  * Purpose:     Perform public-key decryption
  *
  * Parameters:
@@ -886,7 +873,7 @@ NI_LoadPrivKey(FILE *fp, Int2Ptr privKeyLenPtr)
  *              The caller must free the pointer to the resulting plaintext.
  */
 
-Int2 LIBCALL
+NLM_EXTERN Int2 LIBCALL
 NI_PubKeyDecrypt(VoidPtr pKey, UcharPtr PNTR plainText, UcharPtr cipherText, Int2 cipherTextLen)
 {
     R_RSA_PRIVATE_KEY PNTR privKey = (R_RSA_PRIVATE_KEY PNTR) pKey;
@@ -912,7 +899,7 @@ NI_PubKeyDecrypt(VoidPtr pKey, UcharPtr PNTR plainText, UcharPtr cipherText, Int
 }
 
 
-/*
+/*
  * Purpose:     Perform public-key encryption
  *
  * Parameters:
@@ -933,7 +920,7 @@ NI_PubKeyDecrypt(VoidPtr pKey, UcharPtr PNTR plainText, UcharPtr cipherText, Int
  *              The caller must free the pointer to the resulting ciphertext.
  */
 
-Int2 LIBCALL
+NLM_EXTERN Int2 LIBCALL
 NI_PubKeyEncrypt(NI_PubKeyPtr pub, UcharPtr plainText, Int2 plainTextLen, UcharPtr PNTR cipherText)
 {
     R_RSA_PUBLIC_KEY PNTR pubKeyPtr;
@@ -966,7 +953,7 @@ NI_PubKeyEncrypt(NI_PubKeyPtr pub, UcharPtr plainText, Int2 plainTextLen, UcharP
     return ((Int2) cipherTextLen);
 }
 
-/*
+/*
  * Purpose:     Free an encryption data structure, and erase secret data
  *
  * Parameters:
@@ -979,7 +966,7 @@ NI_PubKeyEncrypt(NI_PubKeyPtr pub, UcharPtr plainText, Int2 plainTextLen, UcharP
  *              The caller must free the pointer to the resulting ciphertext.
  */
 
-void
+NLM_EXTERN void
 NI_DestroyEncrStruct (NI_EncrDataPtr encr)
 {
     if (encr == NULL)
@@ -999,7 +986,7 @@ NI_DestroyEncrStruct (NI_EncrDataPtr encr)
     MemFree(encr);
 }
 
-/*
+/*
  * Purpose:     Indicates whether encryption is available
  *
  * Returns:     Always TRUE for this file, always FALSE for stub file
@@ -1011,7 +998,7 @@ NI_DestroyEncrStruct (NI_EncrDataPtr encr)
  *              functions.
  */
 
-Boolean LIBCALL
+NLM_EXTERN Boolean LIBCALL
 NI_EncrAvailable(void)
 {
     return TRUE;
