@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   07/15/97
 *
-* $Revision: 1.12 $
+* $Revision: 1.16 $
 *
 * File Description: 
 *       API for Taxonomy service
@@ -44,6 +44,18 @@
 *
 * RCS Modification History:
 * $Log: txcdproc.c,v $
+* Revision 1.16  2003/11/04 17:13:58  soussov
+* adds NetFini() call if service can not start properly
+*
+* Revision 1.15  2003/06/12 16:46:25  soussov
+* changes severity for AsnRead failures
+*
+* Revision 1.14  2003/03/06 16:30:32  kans
+* tdp->org needed a cast to (OrgRefPtr) due to Mac compiler complaint
+*
+* Revision 1.13  2003/03/05 21:32:00  soussov
+* new lookup procedure
+*
 * Revision 1.12  2001/09/28 15:53:15  soussov
 * tax1e_maxTaxId() added
 *
@@ -127,7 +139,7 @@ static clock_t tax_time1;
 static void report_service_error(CharPtr proc_name, Taxon1RespPtr taxbp)
 {
     if(taxbp == NULL) {
-	ErrPostEx(SEV_ERROR, 0, 0, "%s got NULL responce from service");
+	ErrPostEx(SEV_ERROR, 0, 0, "%s got NULL responce from service", proc_name);
     }
     else if(taxbp->choice == Taxon1Resp_error) {
 	Taxon1ErrorPtr tep= taxbp->data.ptrvalue;
@@ -157,7 +169,7 @@ static Taxon1RespPtr NetTaxArchReadAsn(void)
 
     if ((taxbp == NULL) || ErrFetch(&err))
     {
-        ErrPost (CTX_UNKNOWN, 1, "Null message read from server");
+        ErrPostEx (SEV_WARNING, 1, 1, "Null message read from server");
     }
     ErrSetOpts(erract, 0);
 
@@ -206,6 +218,7 @@ static Boolean TaxServInit(void)
     Taxon1ReqFree (taxrp);
 
     if((taxbp = NetTaxArchReadAsn()) == NULL) {
+        NetFini();
         return FALSE;
     }
     else  {
@@ -1195,7 +1208,7 @@ static Boolean GenericReestablishNet(CharPtr svcName, Boolean showErrs)
 	    retval = myNetInit();
 	}
         else {
-            ErrPost(CTX_UNKNOWN, 1, "Unable to re-contact dispatcher");
+            ErrPostEx(SEV_ERROR, 1,2, "Unable to re-contact dispatcher");
             if (showErrs) {
                 ErrShow();
             }
@@ -1206,8 +1219,8 @@ static Boolean GenericReestablishNet(CharPtr svcName, Boolean showErrs)
 
     if (! retval )
     {
-        sprintf (buf, "Unable to re-establish %s service", svcName);
-        ErrPost(CTX_UNKNOWN, 1, buf);
+        /*sprintf (buf, "Unable to re-establish %s service", svcName);*/
+        ErrPostEx(SEV_ERROR, 1, 3, "Unable to re-establish %s service", svcName);
         if (showErrs) {
             ErrShow();
         }
@@ -1453,3 +1466,81 @@ Int4 tax1e_maxTaxId()
 
     return tax_id;
 }
+
+static OrgRefPtr s_findbyorg(OrgRefPtr orgRef)
+{
+    Taxon1ReqPtr taxrp;
+    Taxon1RespPtr taxbp;
+    Taxon1DataPtr tdp;
+
+    if((taxrp= ValNodeNew(NULL)) == NULL) return 0;
+    taxrp->choice= Taxon1Req_lookup;
+    taxrp->data.ptrvalue= orgRef;
+    Taxon1ReqAsnWrite(taxrp, asnout, NULL);
+    AsnIoReset(asnout);
+    taxrp->data.ptrvalue= NULL;
+    Taxon1ReqFree(taxrp);
+
+    if((taxbp= NetTaxArchReadAsn()) == NULL) return 0;
+    
+    if(taxbp->choice != Taxon1Resp_lookup) {
+        report_service_error("txc_getTaxIdByOrgRef", taxbp);
+        Taxon1RespFree(taxbp);
+        return NULL;
+    }
+    
+    tdp= taxbp->data.ptrvalue;
+    orgRef= (OrgRefPtr) tdp->org;
+    tdp->org= NULL;
+    Taxon1RespFree(taxbp);
+    
+    return orgRef;
+}
+
+Int4 txc_findByOrg(OrgRefPtr inp_orgRef, OrgModPtr* hitName)
+{
+    Int4 i;
+    short erract;
+    ErrDesc err;
+    OrgRefPtr orp= NULL;
+    Int4 tax_id= 0;
+
+    if(tax_track) tax_time1= clock();
+    for (i= TAXARCH_SERV_RETRIES; i >= 0; --i) {
+        ErrGetOpts(&erract, NULL);
+        ErrSetOpts(ERR_IGNORE, 0);
+        ErrFetch(&err);
+
+        orp = s_findbyorg(inp_orgRef);
+
+        ErrSetOpts(erract, 0);
+        if (!ErrFetch(&err))  break; /* success */
+        
+        if(!ReestablishNetTaxArch()) break;
+    }
+    
+    if(tax_track) {
+        double diff= (double)(clock() - tax_time1)/CLOCKS_PER_SEC;
+        printf("tax1_getTaxIdByOrgRef() %f\n", diff);
+    }
+    
+    if(orp && orp->db) {
+        ValNodePtr vnp= orp->db;
+        DbtagPtr dbtag= vnp->data.ptrvalue;
+        ObjectIdPtr object_id= dbtag->tag;
+        tax_id= object_id->id;
+        
+        if(hitName) {
+            if(orp->orgname) {
+                *hitName= orp->orgname->mod;
+                orp->orgname->mod= NULL;
+            }
+            else *hitName= NULL;
+        }
+    }
+
+    if(orp) OrgRefFree(orp);
+
+    return tax_id;
+}
+
