@@ -31,6 +31,15 @@
 * -------  ----------  -----------------------------------------------------
 *
 * $Log: objmime.c,v $
+* Revision 6.14  2002/08/23 20:35:34  thiessen
+* fix wrong user_annotations symbols
+*
+* Revision 6.13  2002/02/27 15:53:37  thiessen
+* fix typo
+*
+* Revision 6.12  2002/02/26 13:21:21  thiessen
+* add structure type to new general mime
+*
 * Revision 6.11  2001/11/21 16:38:23  thiessen
 * move cn3d stuff into bundle
 *
@@ -1060,7 +1069,7 @@ BiostrucAlignSeqAsnRead(AsnIoPtr aip, AsnTypePtr orig)
       }
       atp = AsnReadId(aip,amp, atp);
    }
-   if (atp == BIOSTRUC_ALIGN_user_annotations) {
+   if (atp == ALIGN_SEQ_user_annotations) {
       ptr -> user_annotations = Cn3dUserAnnotationsAsnRead(aip, atp);
       if (aip -> io_failure) {
          goto erret;
@@ -1129,7 +1138,7 @@ BiostrucAlignSeqAsnWrite(BiostrucAlignSeqPtr ptr, AsnIoPtr aip, AsnTypePtr orig)
       }
    }
    if (ptr -> user_annotations != NULL) {
-      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, BIOSTRUC_ALIGN_user_annotations)) {
+      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, ALIGN_SEQ_user_annotations)) {
          goto erret;
       }
    }
@@ -1252,7 +1261,7 @@ BiostrucSeqAsnRead(AsnIoPtr aip, AsnTypePtr orig)
       }
       atp = AsnReadId(aip,amp, atp);
    }
-   if (atp == BIOSTRUC_ALIGN_user_annotations) {
+   if (atp == BIOSTRUC_SEQ_user_annotations) {
       ptr -> user_annotations = Cn3dUserAnnotationsAsnRead(aip, atp);
       if (aip -> io_failure) {
          goto erret;
@@ -1323,7 +1332,7 @@ BiostrucSeqAsnWrite(BiostrucSeqPtr ptr, AsnIoPtr aip, AsnTypePtr orig)
       }
    }
    if (ptr -> user_annotations != NULL) {
-      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, BIOSTRUC_ALIGN_user_annotations)) {
+      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, BIOSTRUC_SEQ_user_annotations)) {
          goto erret;
       }
    }
@@ -1455,7 +1464,7 @@ BiostrucSeqsAsnRead(AsnIoPtr aip, AsnTypePtr orig)
       }
       atp = AsnReadId(aip,amp, atp);
    }
-   if (atp == BIOSTRUC_ALIGN_user_annotations) {
+   if (atp == BIOSTRUC_SEQS_user_annotations) {
       ptr -> user_annotations = Cn3dUserAnnotationsAsnRead(aip, atp);
       if (aip -> io_failure) {
          goto erret;
@@ -1529,7 +1538,7 @@ BiostrucSeqsAsnWrite(BiostrucSeqsPtr ptr, AsnIoPtr aip, AsnTypePtr orig)
       }
    }
    if (ptr -> user_annotations != NULL) {
-      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, BIOSTRUC_ALIGN_user_annotations)) {
+      if ( ! Cn3dUserAnnotationsAsnWrite(ptr -> user_annotations, aip, BIOSTRUC_SEQS_user_annotations)) {
          goto erret;
       }
    }
@@ -1671,6 +1680,13 @@ BiostrucSeqsAlignsCddAsnRead(AsnIoPtr aip, AsnTypePtr orig)
       }
       atp = AsnReadId(aip,amp, atp);
    }
+   if (atp == SEQS_ALIGNS_CDD_structure_type) {
+      if ( AsnReadVal(aip, atp, &av) <= 0) {
+         goto erret;
+      }
+      ptr -> structure_type = av.intvalue;
+      atp = AsnReadId(aip,amp, atp);
+   }
 
    if (AsnReadVal(aip, atp, &av) <= 0) {
       goto erret;
@@ -1809,6 +1825,8 @@ BiostrucSeqsAlignsCddAsnWrite(BiostrucSeqsAlignsCddPtr ptr, AsnIoPtr aip, AsnTyp
       }
    }
    AsnGenericUserSeqOfAsnWrite(ptr -> structures, (AsnWriteFunc) BiostrucAsnWrite, aip, SEQS_ALIGNS_CDD_structures, SEQS_ALIGNS_CDD_structures_E);
+   av.intvalue = ptr -> structure_type;
+   retval = AsnWrite(aip, SEQS_ALIGNS_CDD_structure_type,  &av);
    if (! AsnCloseStruct(aip, atp, (Pointer)ptr)) {
       goto erret;
    }
