@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.4 $
+* $Revision: 6.6 $
 *
 * File Description:  Object manager for module NCBI-Pub
 *
@@ -41,6 +41,12 @@
 *
 *
 * $Log: objpub.c,v $
+* Revision 6.6  2004/04/01 13:43:08  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.5  2002/01/08 20:48:22  kans
+* PubLabelUnique internal call to PubLabelUnique (..., unique), not PubLabel
+*
 * Revision 6.4  2001/11/20 14:50:54  kans
 * PubLabelUnique return immediately if PMID or MUID, already unique
 *
@@ -517,9 +523,9 @@ erret:
 *
 *   PubAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN ValNodePtr LIBCALL PubAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -767,9 +773,9 @@ erret:
 *
 *   PubSetAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN ValNodePtr LIBCALL PubSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -959,9 +965,9 @@ erret:
 *
 *   PubEquivAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN ValNodePtr LIBCALL PubEquivAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -1397,7 +1403,7 @@ NLM_EXTERN Int2 LIBCALL PubLabelUnique (ValNodePtr pub, CharPtr buf, Int2 buflen
 					else
 						first = FALSE;
 
-					diff = PubLabel(eq[i], buf, buflen, OM_LABEL_CONTENT);
+					diff = PubLabelUnique (eq[i], buf, buflen, OM_LABEL_CONTENT, unique);
 					buflen -= diff; buf += diff;
 				}
 			}
