@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.5 $
 *
 * File Description:  Object manager interface for module NCBI-Seq
 *
@@ -40,6 +40,21 @@
 *
 *
 * $Log: objseq.h,v $
+* Revision 6.5  2001/10/02 21:47:54  kans
+* added hooks for writing extra descriptors or features (JO)
+*
+* Revision 6.4  1999/09/27 17:48:38  kans
+* using GatherIndex structure
+*
+* Revision 6.3  1999/09/24 23:09:23  kans
+* adds EXTRA_OBJMGR_FIELDS to several objects
+*
+* Revision 6.2  1998/10/13 20:59:05  kans
+* Bioseq has new slot for omdp, to be used by SeqMgr indexing functions
+*
+* Revision 6.1  1998/06/29 20:31:32  kans
+* explicit defines for FEATDEF_MAX, SEQFEAT_MAX, and SEQDESCR_MAX Boolean array sizes
+*
 * Revision 6.0  1997/08/25 18:50:52  madden
 * Revision changed to 6.0
 *
@@ -123,6 +138,8 @@
 extern "C" {
 #endif
 
+typedef ValNode SeqEntry, FAR *SeqEntryPtr;
+
 /*****************************************************************************
 *
 *   loader
@@ -253,6 +270,7 @@ typedef struct seqannot {
     Uint1 type;             /* 1=ftable, 2=align, 3=graph 4=ids 5=locs */
     Pointer data;
     struct seqannot PNTR next;
+	GatherIndex idx;        /* internal gather/objmgr tracking fields */
 } SeqAnnot, PNTR SeqAnnotPtr;
 
 NLM_EXTERN SeqAnnotPtr LIBCALL SeqAnnotNew PROTO((void));
@@ -269,6 +287,11 @@ NLM_EXTERN Int2 LIBCALL SeqAnnotLabel PROTO((SeqAnnotPtr sap, CharPtr buffer, In
 NLM_EXTERN Boolean     LIBCALL SeqAnnotSetAsnWrite PROTO((SeqAnnotPtr sap, AsnIoPtr aip, AsnTypePtr set, AsnTypePtr element));
 NLM_EXTERN SeqAnnotPtr LIBCALL SeqAnnotSetAsnRead PROTO((AsnIoPtr aip, AsnTypePtr set, AsnTypePtr element));
 
+/** SeqAnnotSetExtraCheck looks for extra Features from SeqEntryAsnOut
+    and adds them to the stream when the annot pointer is NULL
+*********/
+NLM_EXTERN Boolean LIBCALL SeqAnnotSetExtraCheck (AsnIoPtr aip, AsnTypePtr set, AsnTypePtr element);
+
 
 /*****************************************************************************
 *
@@ -402,6 +425,9 @@ typedef struct bioseq {
     Pointer seq_ext;
     SeqAnnotPtr annot;
 	SeqHistPtr hist;
+	Pointer omdp;           /* internal objmgrdataptr to speed up indexed functions */
+	GatherIndex idx;        /* internal gather/objmgr tracking fields */
+	SeqEntryPtr seqentry;   /* internal seqentry that points to this bioseq */
 } Bioseq, PNTR BioseqPtr;
 
 NLM_EXTERN BioseqPtr LIBCALL BioseqNew PROTO((void));
@@ -449,6 +475,8 @@ typedef struct op_objseq {
 
 /* types for AsnIoOption OP_NCBIOBJSEQ */
 #define BIOSEQ_CHECK_ID 1    /* match Op_objseq.sip */
+#define CHECK_EXTRA_DESC 2   /* used by SeqEntryAsnOut */
+#define CHECK_EXTRA_FEAT 3   /* used by SeqEntryAsnOut */
 
 /*****************************************************************************
 *
@@ -510,6 +538,11 @@ typedef struct op_objseq {
 #define Seq_descr_source 23
 #define Seq_descr_molinfo 24
 
+#define SEQDESCR_MAX 25 /* size of array needed for seqdescr filter parameters */
+
+/* SeqDescr/SeqDesc will start to be allocated as ObjValNodes with vnp->extended
+   set to 1 and EXTRA_OBJMGR_FIELDS fields available with a cast pointer */
+
 typedef ValNode SeqDescr, FAR *SeqDescrPtr;
 
 NLM_EXTERN Boolean    LIBCALL SeqDescrAsnWrite PROTO((SeqDescrPtr anp, AsnIoPtr aip, AsnTypePtr atp));
@@ -525,6 +558,11 @@ NLM_EXTERN SeqDescPtr LIBCALL SeqDescAsnRead PROTO((AsnIoPtr aip, AsnTypePtr atp
 NLM_EXTERN SeqDescPtr LIBCALL SeqDescFree PROTO((SeqDescPtr anp));
 NLM_EXTERN Int2 LIBCALL SeqDescLabel PROTO((SeqDescPtr vnp, CharPtr buffer, Int2 buflen, Boolean content));
 
+/** SeqDescrExtraCheck looks for extra Descriptors from SeqEntryAsnOut
+    and adds them to the stream when the descr pointer is NULL
+*********/
+NLM_EXTERN Boolean LIBCALL SeqDescrExtraCheck (AsnIoPtr aip, AsnTypePtr atp);
+
 /*****************************************************************************
 *
 *   Pubdesc, Numbering, MolInfo, BioSource types defined in objpubd.h
@@ -544,3 +582,4 @@ NLM_EXTERN Int2 LIBCALL SeqDescLabel PROTO((SeqDescPtr vnp, CharPtr buffer, Int2
 
 #endif
 
+
