@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 4/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.8 $
 *
 * File Description:  Object manager for module NCBI-Seqset
 *
@@ -41,6 +41,30 @@
 *
 *
 * $Log: objsset.c,v $
+* Revision 6.8  2004/05/12 20:41:57  kans
+* set aip->io_failure in several erret blocks for compatibility of old object loaders with new ones
+*
+* Revision 6.7  2004/04/01 13:43:08  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.6  2002/08/19 20:05:54  kans
+* fixes for SeqEntryAsnGet when asking for nuc-prot retcode but Bioseq is not inside a nuc-prot set
+*
+* Revision 6.5  2002/03/11 22:22:58  kans
+* removed retcode >= 3 test from SeqEntryAsnGet
+*
+* Revision 6.4  2001/11/30 12:20:18  kans
+* ObjMgrDeleteAllInRecord called when freeing top bsp or bssp
+*
+* Revision 6.3  2001/11/16 19:31:09  kans
+* BioseqFree and BioseqSetFree on top-level object call SeqMgrDeleteIndexesInRecord
+*
+* Revision 6.2  2001/08/07 17:22:27  kans
+* added third party annotation SeqIDs to FindBestBioseqLabel
+*
+* Revision 6.1  1998/08/24 18:28:11  kans
+* removed solaris -v -fd warnings
+*
 * Revision 6.0  1997/08/25 18:50:55  madden
 * Revision changed to 6.0
 *
@@ -140,6 +164,10 @@ static void FindBestBioseqLabel (SeqEntryPtr sep, Pointer data, Int4 index, Int2
 			case SEQID_GENBANK:
 			case SEQID_EMBL:
 			case SEQID_DDBJ:
+			case SEQID_OTHER:
+			case SEQID_TPG:
+			case SEQID_TPE:
+			case SEQID_TPD:
 				has_gb = TRUE;
 			case SEQID_PIR:
 			case SEQID_SWISSPROT:
@@ -311,7 +339,7 @@ static Uint2 LIBCALLBACK SeqEntrySubTypeFunc (Pointer ptr)
 *   SeqSetAsnLoad()
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL SeqSetAsnLoad ()
+NLM_EXTERN Boolean LIBCALL SeqSetAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -356,7 +384,7 @@ NLM_EXTERN Boolean LIBCALL SeqSetAsnLoad ()
 *   BioseqSetNew()
 *
 *****************************************************************************/
-NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetNew ()
+NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetNew (void)
 {
     BioseqSetPtr bsp;
 
@@ -375,14 +403,27 @@ NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetNew ()
 *****************************************************************************/
 NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetFree (BioseqSetPtr bsp)
 {
+	Boolean top = FALSE;
+
     if (bsp == NULL)
         return bsp;
 
+	if (bsp->idx.parentptr == NULL || bsp->idx.parenttype == OBJ_SEQSUB) {
+		if (bsp->seqentry != NULL) {
+			SeqMgrDeleteIndexesInRecord (bsp->seqentry);
+			top = TRUE;
+		}
+	}
+
 	if (! SeqMgrDelete(SM_BIOSEQSET, (Pointer)bsp))
 	    ErrPostEx(SEV_ERROR, 0,0, "BioseqSetFree: pointer not registered");
 
 	BioseqSetFreeComponents(bsp, TRUE);
 
+	if (top) {
+		ObjMgrDeleteAllInRecord ();
+	}
+
 	return (BioseqSetPtr)MemFree(bsp);
 }
 
@@ -570,9 +611,9 @@ static void SeqDescrPack (ValNodePtr PNTR to, ValNodePtr PNTR from)
 *
 *   BioseqSetAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -675,7 +716,7 @@ NLM_EXTERN BioseqSetPtr LIBCALL BioseqSetAsnRead (AsnIoPtr aip, AsnTypePtr orig)
 					{
 						if (oop->working_on_set == 2)  /* found set in lower level */
 							got_it = TRUE;
-						else if ((oop->retcode == 2) && (! oop->in_right_set))
+						else if ((oop->retcode == 2 || oop->retcode == 3) && (! oop->in_right_set))
 						{
 							if (osp->found_it)   /* found the component Bioseq */
 							{
@@ -810,6 +851,7 @@ ret:
     AsnUnlinkType(orig);     /*  unlink local tree */
 	return bsp;
 erret:
+    aip->io_failure = TRUE;
 	bsp = BioseqSetFree(bsp);
 	goto ret;
 }
@@ -917,9 +959,9 @@ erret:
 *
 *   SeqEntryAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN SeqEntryPtr LIBCALL SeqEntryAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -970,6 +1012,7 @@ ret:
     AsnUnlinkType(orig);      /*  unlink local tree */
 	return sep;
 erret:
+    aip->io_failure = TRUE;
 	sep = SeqEntryFree(sep);
 	goto ret;
 }
@@ -993,9 +1036,9 @@ static void SeqEntryClearFeatures PROTO((SeqEntryPtr sep));
 *
 *   SeqEntryAsnGet(aip, atp, sip, retcode)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *       looks for entry with sip=SeqId
 *       returns a collection of maximum complexity = retcode
 *         where retcode =
@@ -1021,7 +1064,6 @@ NLM_EXTERN SeqEntryPtr LIBCALL SeqEntryAsnGet (AsnIoPtr aip, AsnTypePtr orig, Se
 	if ((aip == NULL) || (sip == NULL) || (retcode < 0) || (retcode > 4))
 		return sep;
 
-	if (retcode >= 3) retcode = 0;   /* only nuc-prots for now */
 	av.realvalue = 0.0;    /* just zeros it out to prevent debugger fuss */
 	oop = (Op_objssetPtr)MemNew(sizeof(Op_objsset));
 	if (oop == NULL) return sep;
@@ -1055,6 +1097,12 @@ NLM_EXTERN SeqEntryPtr LIBCALL SeqEntryAsnGet (AsnIoPtr aip, AsnTypePtr orig, Se
 		if (bssp->_class != 2)   /* not a seg-set */
 			pack_it = TRUE;
 	}
+	else if ((retcode == 3) && (IS_Bioseq_set(sep)))
+	{							/* got Bioseq, but not part of nuc-prot set */
+		bssp = (BioseqSetPtr)sep->data.ptrvalue;
+		if (bssp->_class != 1)   /* not a nuc-prot set */
+			pack_it = TRUE;
+	}
 
 
 	if (! osp->found_it)
