@@ -29,7 +29,7 @@
 *   
 * Version Creation Date: 1/1/91
 *
-* $Revision: 6.0 $
+* $Revision: 6.3 $
 *
 * File Description:  Object manager for module NCBI-Submit
 *
@@ -41,6 +41,15 @@
 *
 *
 * $Log: objsub.c,v $
+* Revision 6.3  2004/05/12 20:41:57  kans
+* set aip->io_failure in several erret blocks for compatibility of old object loaders with new ones
+*
+* Revision 6.2  2004/04/01 13:43:08  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.1  1998/08/24 18:28:12  kans
+* removed solaris -v -fd warnings
+*
 * Revision 6.0  1997/08/25 18:51:01  madden
 * Revision changed to 6.0
 *
@@ -318,7 +327,7 @@ NLM_EXTERN Int2 LIBCALL ContactInfoLabel (ContactInfoPtr cip, CharPtr buffer, In
 *   SubmitAsnLoad()
 *
 *****************************************************************************/
-NLM_EXTERN Boolean LIBCALL SubmitAsnLoad ()
+NLM_EXTERN Boolean LIBCALL SubmitAsnLoad (void)
 {
     if (loaded)
         return TRUE;
@@ -356,7 +365,7 @@ NLM_EXTERN Boolean LIBCALL SubmitAsnLoad ()
 *   SeqSubmitNew()
 *
 *****************************************************************************/
-NLM_EXTERN SeqSubmitPtr LIBCALL SeqSubmitNew ()
+NLM_EXTERN SeqSubmitPtr LIBCALL SeqSubmitNew (void)
 {
 	SeqSubmitPtr ssp;
 	
@@ -489,9 +498,9 @@ erret:
 *
 *   SeqSubmitAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN SeqSubmitPtr LIBCALL SeqSubmitAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -600,6 +609,7 @@ ret:
 	AsnUnlinkType(orig);       /* unlink local tree */
 	return ssp;
 erret:
+    aip->io_failure = TRUE;
     ssp = SeqSubmitFree(ssp);
     goto ret;
 }
@@ -720,9 +730,9 @@ erret:
 *
 *   SubmitBlockAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN SubmitBlockPtr LIBCALL SubmitBlockAsnRead (AsnIoPtr aip, AsnTypePtr orig)
@@ -795,6 +805,7 @@ ret:
 	AsnUnlinkType(orig);       /* unlink local tree */
 	return sbp;
 erret:
+    aip->io_failure = TRUE;
     sbp = SubmitBlockFree(sbp);
     goto ret;
 }
@@ -959,9 +970,9 @@ erret:
 *
 *   ContactInfoAsnRead(aip, atp)
 *   	atp is the current type (if identifier of a parent struct)
-*            assumption is readIdent has occured
+*            assumption is readIdent has occurred
 *       if atp == NULL, then assumes it stands alone and read ident
-*            has not occured.
+*            has not occurred.
 *
 *****************************************************************************/
 NLM_EXTERN ContactInfoPtr LIBCALL ContactInfoAsnRead (AsnIoPtr aip, AsnTypePtr orig)
