@@ -30,8 +30,16 @@ Author: Tom Madden
 Contents: #defines and definitions for structures used by BLAST.
 
 ******************************************************************************/
-/* $Revision: 6.157 $ 
+/* $Revision: 6.159 $ 
 * $Log: blastdef.h,v $
+* Revision 6.159  2004/09/28 16:02:29  papadopo
+* From Michael Gertz: Changed the "sumscore" field of an HSP to
+* "xsum" to represent a normalized sum score of linked HSPs;
+* the normalized score is more appropriate/useful in doing linking.
+*
+* Revision 6.158  2004/06/30 12:29:00  madden
+* Removed typedef for BlastPruneSapStruct and some defines, moved to blfmtutl.h
+*
 * Revision 6.157  2004/04/30 12:45:45  coulouri
 * bump version to 2.2.9
 *
@@ -879,10 +887,6 @@ Contents: #defines and definitions for structures used by BLAST.
 extern "C" {
 #endif
 
-/* the version of BLAST. */
-#define BLAST_ENGINE_VERSION "2.2.9"
-#define BLAST_RELEASE_DATE "May-01-2004"
-
 /* Defines for program numbers. (Translated in BlastGetProgramNumber). */
 #define blast_type_undefined 0
 #define blast_type_blastn 1
@@ -979,19 +983,6 @@ typedef struct _blast_time_keeper {
         } BlastTimeKeeper, PNTR BlastTimeKeeperPtr;
 
 
-/*****************************************************************
-*
-*	Used for pruing SeqALigns that are too big.
-*
-********************************************************************/
-
-typedef struct _blast_prune_hits_from_sap {
-		SeqAlignPtr sap;
-		Int4 	original_number,	/* how may unique hits were there originally. */
-			number;		/* How many hits on SeqALignPtr above. */
-		Boolean allocated; /* If FALSE, SeqAlignPtr above does NOT belong to this struc.*/
-        } BlastPruneSapStruct, PNTR BlastPruneSapStructPtr;
-
 /***************************************************************************
   Macros added by Andy Neuwald in order to allow easy modification of matrices.
 ***************************************************************************/
@@ -1498,8 +1489,8 @@ typedef struct _blast_hsp {
 		Int2		ordering_method; 
 /* how many HSP's make up this (sum) segment */
 		Int4		num;
-/* sumscore of a set of "linked" HSP's. */
-		BLAST_Score	sumscore;
+/* normalized score of a set of "linked" HSP's */
+        Nlm_FloatHi xsum;
 		/* If TRUE this HSP starts a chain along the "link" pointer. */
 		Boolean 	start_of_chain;
 		BLAST_Score	score;
