@@ -1,3 +1,5 @@
+static char const rcsid[] = "$Id: dotseq.c,v 6.13 2004/03/17 20:33:48 bollin Exp $";
+
 /* dotSeq.c */
 /* ===========================================================================
 *
@@ -29,7 +31,7 @@
 *
 * Version Creation Date:   8/9/01
 *
-* $Revision: 6.8 $
+* $Revision: 6.13 $
 *
 * File Description: computes local alignments for dot matrix
 *
@@ -38,8 +40,24 @@
 * Date     Name        Description of modification
 * -------  ----------  -----------------------------------------------------
 
-$Revision: 6.8 $
+$Revision: 6.13 $
 $Log: dotseq.c,v $
+Revision 6.13  2004/03/17 20:33:48  bollin
+In DOT_InitMainInfobyLoc, exit function and return NULL if unable to determine
+ID or Bioseq from SeqLoc provided.  This change averts a core dump.
+
+Revision 6.12  2003/09/17 20:55:12  kskatz
+Resetting to the way it was in revision 6.10; accidently checked in a kludge meant to be used only locally in DOT_GetNResidues
+
+Revision 6.11  2003/09/17 20:39:01  kskatz
+Commented out a temporary fix in SPI_AlignInWindows() [line 3880]
+
+Revision 6.10  2003/05/30 17:25:36  coulouri
+add rcsid
+
+Revision 6.9  2003/02/07 21:21:50  kans
+DOT_BuildPLookup was erroneously calling MemFree on elements of array
+
 Revision 6.8  2001/08/09 17:21:08  kans
 include alignmgr.h to avoid Mac compile error
 
@@ -918,10 +936,12 @@ static LookupTablePtr DOT_BuildPLookup(Uint1Ptr queryseq, Int4 qlen, Int4 word_s
         return NULL;
 
   /* deallocate */
+  /*
   for (index=0; index<num_cols; index++)
     {
       if (array[index]) MemFree(array[index]);
     }
+  */
   if (array) MemFree(array);
   if (all_words) MemFree(all_words);
 
@@ -1443,7 +1463,7 @@ static Boolean DOT_GetNResidues (DOTMainDataPtr mip, Boolean is_byLoc)
   MemSet((Pointer)qbuffer, '\0', sizeof(Char)*101); 
   MemSet((Pointer)sbuffer, '\0', sizeof(Char)*101);
 
-
+  
   mip->matrix = DOT_DNAScoringMatrix(-3, 1, 4);
   qlen=mip->qlen;
   slen=mip->slen;
@@ -1454,13 +1474,12 @@ static Boolean DOT_GetNResidues (DOTMainDataPtr mip, Boolean is_byLoc)
   if (!(qseq = (Uint1Ptr) MemNew (sizeof(Uint1)*(qlen)))) return FALSE;
   if (!(sseq = (Uint1Ptr) MemNew (sizeof(Uint1)*(slen)))) return FALSE;
 
-  if (is_byLoc)
-    {
+  if (is_byLoc){
       qspp = SeqPortNewByLoc(mip->qslp, Seq_code_ncbi2na);
       sspp = SeqPortNewByLoc(mip->sslp, Seq_code_ncbi2na);
-    }
+  }
   else
-    {
+      {
       qspp = SeqPortNew (mip->qbsp, MIN(mip->q_start, mip->q_stop), MAX(mip->q_start, mip->q_stop), 0, Seq_code_ncbi2na); 
       sspp = SeqPortNew (mip->sbsp, mip->s_start, mip->s_stop, 0, Seq_code_ncbi2na);
     }
@@ -1912,8 +1931,9 @@ static DOTMainDataPtr DOT_InitMainInfobyLoc (DOTMainDataPtr mip, SeqLocPtr slp1,
   Int4   qlen, temp;
   SeqLocPtr qslp, sslp;
   BioseqPtr qbsp, sbsp;
+  SeqIdPtr qsip, ssip;
 
-
+  if (mip == NULL || slp1 == NULL || slp2 == NULL) return MemFree (mip);
   qlen=SeqLocLen(slp1);
   slen=SeqLocLen(slp2);
 
@@ -1923,9 +1943,23 @@ static DOTMainDataPtr DOT_InitMainInfobyLoc (DOTMainDataPtr mip, SeqLocPtr slp1,
   mip->qstrand = SeqLocStrand(qslp);
   mip->sstrand = SeqLocStrand(sslp);
 
+  qsip = SeqLocId (qslp);
+  if (qsip == NULL) {
+    return MemFree (mip);
+  }
+  ssip = SeqLocId (sslp);
+  if (ssip == NULL) {
+    return MemFree (mip);
+  }
 
   mip->qbsp = BioseqFind(SeqLocId(qslp));
+  if (mip->qbsp == NULL) {
+    return MemFree (mip);
+  }    
   mip->sbsp = BioseqFind(SeqLocId(sslp));
+  if (mip->sbsp == NULL) {
+    return MemFree (mip);
+  }    
 
    if (!((ISA_aa (mip->qbsp->mol) && ISA_aa (mip->sbsp->mol))||(ISA_na(mip->qbsp->mol) && ISA_na (mip->sbsp->mol))))
     {
