@@ -1,4 +1,6 @@
-/* $Id: mbalign.c,v 6.35 2001/12/28 20:38:41 dondosha Exp $
+static char const rcsid[] = "$Id: mbalign.c,v 6.41 2003/10/29 17:46:59 dondosha Exp $";
+
+/* $Id: mbalign.c,v 6.41 2003/10/29 17:46:59 dondosha Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -30,12 +32,30 @@
 *
 * Initial Creation Date: 10/27/1999
 *
-* $Revision: 6.35 $
+* $Revision: 6.41 $
 *
 * File Description:
 *        Alignment functions for Mega Blast program
 *
 * $Log: mbalign.c,v $
+* Revision 6.41  2003/10/29 17:46:59  dondosha
+* Allow 2-stage greedy extension in megablast
+*
+* Revision 6.40  2003/05/30 17:25:36  coulouri
+* add rcsid
+*
+* Revision 6.39  2003/05/13 16:02:53  coulouri
+* make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+*
+* Revision 6.38  2003/01/30 19:40:13  dondosha
+* No need to add gap extension penalty to gap open in greedy alignment functions
+*
+* Revision 6.37  2002/08/01 21:02:12  dondosha
+* Added a sanity check in GreedyAlignMemFree
+*
+* Revision 6.36  2002/03/20 19:56:34  dondosha
+* Check entire list of readdb structures for maximal length when allocating memory for greedy algorithm
+*
 * Revision 6.35  2001/12/28 20:38:41  dondosha
 * Moved Mega BLAST related parameters into a separate structure
 *
@@ -241,7 +261,7 @@ static edit_op_t *edit_script_next(edit_script_t *es, edit_op_t *op)
 static Int4 edit_script_more(edit_script_t *data, Uint4 op, Uint4 k)
 {
     if (op == EDIT_OP_ERR) {
-        ErrPostEx(SEV_FATAL, 0, 0, 
+        ErrPostEx(SEV_FATAL, 1, 0, 
                   "edit_script_more: bad opcode %d:%d", op, k);
         return -1;
     }
@@ -502,12 +522,12 @@ BlastSearchBlkPtr GreedyAlignMemAlloc(BlastSearchBlkPtr search)
 
    if (gap_open == 0 && gap_extend == 0)
       gap_extend = reward / 2 + penalty;
-   else
-      gap_open += gap_extend;
 
-   if (search->rdfp)
-      max_len = readdb_get_maxlen(search->rdfp);
-   else
+   if (search->rdfp) {
+      ReadDBFILEPtr rdfp;
+      for (rdfp = search->rdfp, max_len = 0; rdfp; rdfp = rdfp->next)
+         max_len = MAX(max_len, readdb_get_maxlen(rdfp));
+   } else
       max_len = search->subject->length;
 
    max_len = MIN(max_len, MAX_DBSEQ_LEN);
@@ -575,8 +595,10 @@ GreedyAlignMemPtr GreedyAlignMemFree(GreedyAlignMemPtr abmp)
       MemFree(abmp->flast_d[0]);
       MemFree(abmp->flast_d);
    } else {
-      MemFree(abmp->flast_d_affine[0]);
-      MemFree(abmp->flast_d_affine);
+      if (abmp->flast_d_affine) {
+         MemFree(abmp->flast_d_affine[0]);
+         MemFree(abmp->flast_d_affine);
+      }
       MemFree(abmp->uplow_free);
    }
    MemFree(abmp->max_row_free);
@@ -672,6 +694,8 @@ Int4 MegaBlastGreedyAlign(const UcharPtr s1, Int4 len1,
     }
     if (S==NULL) 
        space = 0;
+    else if (!space)
+       abmp->space = space = new_mb_space();
     else 
        refresh_mb_space(space);
     
@@ -867,8 +891,6 @@ Int4 MegaBlastAffineGreedyAlign (const UcharPtr s1, Int4 len1,
                                    mismatch_score, e1, e2, abmp, S, rem);
     }
     
-    gap_open += gap_extend;
-    
     Mis_cost = mismatch_score + match_score;
     GO_cost = gap_open;
     GE_cost = gap_extend+M_half;
@@ -918,7 +940,9 @@ Int4 MegaBlastAffineGreedyAlign (const UcharPtr s1, Int4 len1,
     flast_d = abmp->flast_d_affine;
     if (S==NULL) {
         space = 0;
-    } else { 
+    } else if (!space) {
+       abmp->space = space = new_mb_space();
+    } else {
         refresh_mb_space(space);
     }
     max_row = max_row_free + D_diff;
