@@ -1,4 +1,6 @@
-/* $Id: ncbisort.c,v 6.3 1998/04/09 20:40:45 kans Exp $
+static char const rcsid[] = "$Id: ncbisort.c,v 6.7 2003/07/15 20:17:39 coulouri Exp $";
+
+/* $Id: ncbisort.c,v 6.7 2003/07/15 20:17:39 coulouri Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -29,12 +31,24 @@
 *
 * Initial Version Creation Date: 03/24/1997
 *
-* $Revision: 6.3 $
+* $Revision: 6.7 $
 *
 * File Description:
 *         Main file for SORTing library
 *
 * $Log: ncbisort.c,v $
+* Revision 6.7  2003/07/15 20:17:39  coulouri
+* remove signal() and setrlimit() calls
+*
+* Revision 6.6  2003/05/30 17:25:37  coulouri
+* add rcsid
+*
+* Revision 6.5  2003/02/26 17:48:09  kimelman
+* bugfix: format/argument mismatch in error message
+*
+* Revision 6.4  1998/06/01 14:52:37  madden
+* Change to using TmpNam
+*
 * Revision 6.3  1998/04/09 20:40:45  kans
 * use TmpNam for Mac sorting
 *
@@ -117,12 +131,6 @@ SORTObjectPtr SORTObjectNew(CharPtr prefix, Uchar tab, Int4 linelength,
   sdp->reverse  = reverse;
   sdp->unique = unique;
 
-#ifdef OS_UNIX
-  signal(SIGINT,  inthandler);
-  signal(SIGTERM, inthandler);
-  /*  signal(SIGSEGV, inthandler); */
-#endif
-
   return (SORTObjectPtr) sdp;
 }
 
@@ -424,11 +432,7 @@ static void SORTCleanup(SORTempNodePtr temphead)
 /* Return a name for a temporary file. */
 CharPtr SORTAddTempName(SORTObjectPtr sop)
 {
-#ifdef WIN_MAC
   Char path [PATH_MAX];
-#else
-  Int4 len;
-#endif
   CharPtr name;
   SORTempNodePtr node;
   SORTDataPtr sdp;
@@ -436,7 +440,6 @@ CharPtr SORTAddTempName(SORTObjectPtr sop)
   if((sdp = (SORTDataPtr) sop) == NULL)
     return NULL;
 
-#ifdef WIN_MAC
 
   TmpNam (path);
   name = StringSave (path);
@@ -446,24 +449,6 @@ CharPtr SORTAddTempName(SORTObjectPtr sop)
     return NULL;
   }
 
-#else
-
-  len = StringLen(sdp->prefix);
-  name = (CharPtr) MemNew(len + 16);
-  
-  if((node = (SORTempNodePtr)MemNew(sizeof(SORTempNode))) == NULL) {
-    SORTCleanup(sdp->temphead);
-    return NULL;
-  }
-
-  if (len && sdp->prefix[len - 1] != '/')
-    sprintf(name, DIRPREFIX, sdp->prefix, 
-            (long) Nlm_GetAppProcessID(), (long) ++sdp->temp_seq);
-  else
-      sprintf(name, PATHPREFIX, sdp->prefix, 
-            (long) Nlm_GetAppProcessID(), (long) ++sdp->temp_seq);
-#endif
-  
   node->name = name;
   node->next = sdp->temphead->next;
   sdp->temphead->next = node;
@@ -524,7 +509,7 @@ static Int4 SORTFillBuf(SORTBufferPtr buf, FILE *fp)
 
     if((cc = FileRead(buf->buf + buf->used, 
                       1, buf->alloc - buf->used, fp)) == 0 && ferror(fp)) {
-      ErrLogPrintf("sort: read error (%s)\n", errno);
+      ErrLogPrintf("sort: read error (%s)\n",  strerror(errno));
       return (Int4) SORTReadError;
     }
 
@@ -1261,17 +1246,3 @@ SORTErrorCode SORTInsertKey(SORTKeyFieldPtr key, SORTKeyFieldPtr keyhead)
 
   return SORTNoError;
 }
-
-#ifdef OS_UNIX
-#ifdef OS_UNIX_IRIX
-static void inthandler(void)
-#else
-static void inthandler(int i)
-#endif
-{
-  signal(SIGINT, SIG_DFL);
-  SORTCleanup(global_temphead);
-  ErrLogPrintf("ERROR: Program terminated by signal");
-  kill(Nlm_GetAppProcessID(), SIGINT);
-}
-#endif
