@@ -1,4 +1,6 @@
-/* $Id: pattern1.c,v 6.11 1999/09/22 17:51:02 shavirin Exp $
+static char const rcsid[] = "$Id: pattern1.c,v 6.18 2004/04/01 13:43:08 lavr Exp $";
+
+/* $Id: pattern1.c,v 6.18 2004/04/01 13:43:08 lavr Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -31,9 +33,30 @@ Original Author: Zheng Zhang
  
 Contents: central pattern matching routines for PHI-BLAST and pseed3
 
-$Revision: 6.11 $ 
+$Revision: 6.18 $ 
 
 $Log: pattern1.c,v $
+Revision 6.18  2004/04/01 13:43:08  lavr
+Spell "occurred", "occurrence", and "occurring"
+
+Revision 6.17  2003/08/06 15:11:17  dondosha
+Fixed search for pattern remainder when it starts from a base non-divisible by 4
+
+Revision 6.16  2003/05/30 17:25:37  coulouri
+add rcsid
+
+Revision 6.15  2003/05/13 16:02:53  coulouri
+make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+
+Revision 6.14  2003/03/06 21:33:13  madden
+Move big arrays from stack to heap
+
+Revision 6.13  2002/08/28 13:37:29  madden
+Fix memory leaks
+
+Revision 6.12  2002/08/09 17:32:30  madden
+Add warning if MAX_HIT is reached
+
 Revision 6.11  1999/09/22 17:51:02  shavirin
 Now functions will collect messages in ValNodePtr before printing out.
 
@@ -156,7 +179,7 @@ Int4 LIBCALL init_pattern(Uint1 *pattern, Boolean is_dna, patternSearchItems * p
     Int4  tj; /*temporary copy of j*/
     Int4 tempInputPatternMasked[MaxP]; /*local copy of parts
             of inputPatternMasked*/
-    Uint1 c;  /*character occuring in pattern*/
+    Uint1 c;  /*character occurring in pattern*/
     Uint1 localPattern[MaxP]; /*local variable to hold
                                for each position whether it is
                                last in pattern (1) or not (0) */
@@ -211,7 +234,7 @@ Int4 LIBCALL init_pattern(Uint1 *pattern, Boolean is_dna, patternSearchItems * p
 		patternSearch->inputPatternMasked[j++] = allone; 
 		if (j >= MaxP) {
                   BlastConstructErrorMessage("init_pattern", "pattern too long", 1, error_return);
-		  /*ErrPostEx(SEV_FATAL, 0, 0, "pattern too long\n");*/
+		  /*ErrPostEx(SEV_FATAL, 1, 0, "pattern too long\n");*/
 		  return(-1);
 		}
 	      }
@@ -257,7 +280,7 @@ Int4 LIBCALL init_pattern(Uint1 *pattern, Boolean is_dna, patternSearchItems * p
             add its probability to positionProbability*/
 	  while ((c=pattern[++i]) != ']') { /*end of set*/
             if ((c < 'A') || (c > 'Z') || (c == '\0')) {
-	      /* ErrPostEx(SEV_FATAL, 0, 0, "your pattern description has a non-alphabetic character inside a bracket\n"); */
+	      /* ErrPostEx(SEV_FATAL, 1, 0, "your pattern description has a non-alphabetic character inside a bracket\n"); */
               BlastConstructErrorMessage("init_pattern", "your pattern description has a non-alphabetic character inside a bracket", 1, error_return);
               return(-1);
 	    }
@@ -367,7 +390,7 @@ static Int4 lenof(Int4 s, Int4 mask)
 	if ((mask >> rightOne) %2  == 1) 
 	  rightMaskOnly = rightOne;
     }
-    ErrPostEx(SEV_FATAL, 0, 0, "wrong\n");
+    ErrPostEx(SEV_FATAL, 1, 0, "wrong\n");
     return(-1);
 }
 
@@ -405,6 +428,11 @@ static Int4 find_hitsS(Int4 *hitArray, Uint1Ptr seq, Int4 len1,
           ends; second part is where match starts*/
 	hitArray[numMatches++] = i;
 	hitArray[numMatches++] = i - lenof(prefixMatchedBitPattern, mask)+1;
+	if (numMatches == MAX_HIT)
+	{
+    		ErrPostEx(SEV_WARNING, 0, 0, "%ld matches saved, discarding others", numMatches);
+		break;
+	}
       }
     }
     return numMatches;
@@ -472,25 +500,28 @@ static Int4 find_hitsS_DNA(Int4Ptr hitArray, Uint1Ptr seq, Char pos, Int4 len,
   Int4 j; /*index on suffixRemnant*/
   Int4 twiceNumHits = 0; /*twice the number of hits*/
 
-
   mask2 = patternSearch->match_mask*BITS_PACKED_PER_WORD+15; 
   maskShiftPlus1 = (patternSearch->match_mask << 1)+1;
+
   if (pos != 0) {
+    pos = 4 - pos;
     prefixMatchedBitPattern = ((patternSearch->match_mask * ((1 << (pos+1))-1)*2) +
 	 (1 << (pos+1))-1)& patternSearch->DNAwhichSuffixPosPtr[seq[0]];
     seq++;
+    end = (len-pos)/4; 
+    remain = (len-pos) % 4;
   } 
   else {
     prefixMatchedBitPattern = maskShiftPlus1;
+    end = len/4; 
+    remain = len % 4;
   }
-  end = (len-pos)/4; 
-  remain = (len-pos) % 4;
   for (i = 0; i < end; i++) {
     if (tmp = (prefixMatchedBitPattern & patternSearch->DNAwhichPrefixPosPtr[seq[i]])) {
       for (j = 0; j < 4; j++) {
 	if (tmp & patternSearch->match_mask) {
-	  hitArray[twiceNumHits++] = pos+i*4+j;
-	  hitArray[twiceNumHits++] = pos+i*4+j-lenof(tmp & patternSearch->match_mask, 
+	  hitArray[twiceNumHits++] = i*4+j + pos;
+	  hitArray[twiceNumHits++] = i*4+j + pos -lenof(tmp & patternSearch->match_mask, 
 					  patternSearch->match_mask)+1;
 	}
 	tmp = (tmp << 1);
@@ -498,16 +529,15 @@ static Int4 find_hitsS_DNA(Int4Ptr hitArray, Uint1Ptr seq, Char pos, Int4 len,
     }
     prefixMatchedBitPattern = (((prefixMatchedBitPattern << 4) | mask2) & patternSearch->DNAwhichSuffixPosPtr[seq[i]]);
   }
-  suffixRemnant = seq[i]; 
-  j = 0; 
-  while (j < remain) {
-    prefixMatchedBitPattern =  ((prefixMatchedBitPattern << 1) | maskShiftPlus1) & patternSearch->whichPositionPtr[seq[i]];
-    if (prefixMatchedBitPattern & patternSearch->match_mask) { 
-      hitArray[twiceNumHits++] = pos+i*4+j;
-      hitArray[twiceNumHits++] = pos+i*4+j-lenof(prefixMatchedBitPattern, patternSearch->match_mask)+1;
-    }
-    j++;
-    suffixRemnant <<= 2;
+  /* In the last byte check bits only up to 'remain' */
+  if (tmp = (prefixMatchedBitPattern & patternSearch->DNAwhichPrefixPosPtr[seq[i]])) {
+     for (j = 0; j < remain; j++) {
+        if (tmp & patternSearch->match_mask) {
+           hitArray[twiceNumHits++] = i*4+j + pos;
+           hitArray[twiceNumHits++] = i*4+j + pos - lenof(tmp & patternSearch->match_mask, patternSearch->match_mask)+1;
+        }
+        tmp = (tmp << 1);
+     }
   }
   return twiceNumHits;
 }
@@ -664,7 +694,7 @@ static Int4 lenofL(Int4 *s, Int4 *mask, patternSearchItems *patternSearch)
 	  firstOneInMask = wordIndex*BITS_PACKED_PER_WORD+bitIndex;
       }
     }
-    ErrPostEx(SEV_FATAL, 0, 0, "wrong\n");
+    ErrPostEx(SEV_FATAL, 1, 0, "wrong\n");
     return(-1);
 }
 
@@ -1298,6 +1328,8 @@ static void get_patL(Uint1 *seq, Int4 len, Int4 *start, Int4 *end,
       wordIndex--;
       j --;
     }
+    prefixMatchedBitPattern = MemFree(prefixMatchedBitPattern);
+    mask = MemFree(mask);
     *start = rightMaskOnly+1; 
     *end = wordIndex*BITS_PACKED_PER_WORD+j;
 }
@@ -1356,7 +1388,7 @@ static void get_patLL(Uint1 *seq, Int4 len, Int4 *hitArray,
 	hitArray[i] = hitArray[i+j];
       return;
     }
-    ErrPostEx(SEV_FATAL, 0, 0, "getLL wrong\n");
+    ErrPostEx(SEV_FATAL, 1, 0, "getLL wrong\n");
     exit(1);
 }
 
@@ -1390,7 +1422,7 @@ Int4 LIBCALL align_of_pattern(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
     Int4 wordIndex; /*index over words*/
 
     Int4 placeIndex1, placeIndex2; /*indices over places in pattern*/
-    Int4  hitArray1[MAX_HIT], hitArray2[MAX_HIT];
+    Int4  *hitArray1=NULL, *hitArray2=NULL;
     Int4 numPlacesInWord[MAX_WORDS_IN_PATTERN];
     Int4 **matrix;  /*score matrix*/
     Int4 gap_open; /*gap opening penalty*/
@@ -1415,6 +1447,8 @@ Int4 LIBCALL align_of_pattern(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
                     patternSearch);
       } 
       else {
+	hitArray1 = Nlm_Malloc(MAX_HIT*sizeof(Int4));
+	hitArray2 = Nlm_Malloc(MAX_HIT*sizeof(Int4));
 	if (alignScript) {
 	    data->sapp = alignScript;
 	    data->last = 0;
@@ -1467,6 +1501,10 @@ Int4 LIBCALL align_of_pattern(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
 	}
 	else 
 	  *tback = data->sapp;
+
+	hitArray1 = MemFree(hitArray1);
+	hitArray2 = MemFree(hitArray2);
+
 	return local_score;     
       }
 
