@@ -1,6 +1,9 @@
+static char const rcsid[] = "$Id: pobutil.c,v 6.9 2003/05/30 17:25:37 coulouri Exp $";
+
 #include <pobutil.h>
 #include <gather.h>
 #include <edutil.h>
+#include <salign.h>
 #include <salutil.h>
 #include <salsap.h>
 
@@ -619,6 +622,7 @@ void restore_blast_id(SeqAnnotPtr sap, SeqIdPtr sip)
 {
 	SeqAlignPtr align;
 	DenseDiagPtr ddp;
+	DenseSegPtr dsp;
 	SeqIdPtr next_id;
 	StdSegPtr ssp;
 	SeqLocPtr slp;
@@ -644,6 +648,18 @@ void restore_blast_id(SeqAnnotPtr sap, SeqIdPtr sip)
 				}
 				break;
 
+			case 2:
+				dsp = align->segs;
+				if(dsp)
+				{
+					next_id = dsp->ids->next;
+					dsp->ids->next = NULL;
+					SeqIdFree(dsp->ids);
+					dsp->ids = SeqIdDup(sip);
+					dsp->ids->next = next_id;
+				}
+				break;
+
 			case 3:
 				ssp = align->segs;
 				while(ssp)
@@ -792,28 +808,38 @@ void filter_blast_query(SeqAnnotPtr blast_sap, BioseqPtr query)
 	while(align)
 	{
 		next = align->next;
-		ddp = align->segs;
 		free_it = FALSE;
-		if(ddp && ddp->id != NULL)
-		{
-			if(BioseqMatch(query, ddp->id->next))
-			{
-				filter_self_diagnol(&ddp);
-				if(ddp == NULL)
+		switch(align->segtype) {
+			case 1:
+				ddp = align->segs;
+				if(ddp && ddp->id != NULL)
 				{
-					if(prev == NULL)
-						blast_sap->data = next;
-					else
-						prev->next = next;
-					align->next = NULL;
-					align->segs = NULL;
-					SeqAlignFree(align);
-					free_it = TRUE;
-				}
-				else
-					align->segs = ddp;
+					if(BioseqMatch(query, ddp->id->next))
+					{
+						filter_self_diagnol(&ddp);
+						if(ddp == NULL)
+						{
+							if(prev == NULL)
+								blast_sap->data = next;
+							else
+								prev->next = next;
+							align->next = NULL;
+							align->segs = NULL;
+							SeqAlignFree(align);
+							free_it = TRUE;
+						}
+						else
+							align->segs = ddp;
 					
-			}
+					}
+				}
+				break;
+			case 2:
+				break;
+			case 3:
+				break;
+			default:
+				break;
 		}
 		if(!free_it)
 			prev = align;
@@ -1161,20 +1187,6 @@ static SeqAlignPtr find_previous_align_in_list(SeqAlignPtr list, SeqAlignPtr new
 }
 
 
-static void get_ddp_num(DenseDiagPtr ddp)
-{
-	Int2 i;
-
-	i =0;
-	while(ddp)
-	{
-		++i;
-		ddp = ddp->next;
-	}
-	/*printf("num of ddp is %d\n", i);*/
-}
-
-
 Int4 get_score_value(ScorePtr sp)
 {
 	ObjectIdPtr oip;
@@ -1811,7 +1823,7 @@ static Boolean switch_Dense_diag(DenseDiagPtr ddp_1, DenseDiagPtr ddp_2)
 static DenseDiagPtr sort_dense_diag(DenseDiagPtr head)
 {
 	DenseDiagPtr PNTR list;
-	DenseDiagPtr curr, last;
+	DenseDiagPtr curr, last = NULL;
 	Int4 num =0, i, j;
 	Boolean s_witch = TRUE;
 
@@ -1878,7 +1890,7 @@ static Boolean switch_std_seg(StdSegPtr ssp_1, StdSegPtr ssp_2)
 static StdSegPtr sort_std_seg(StdSegPtr ssp)
 {
 	StdSegPtr PNTR list;
-	StdSegPtr curr, last;
+	StdSegPtr curr, last = NULL;
 	Int4 num =0, i, j;
 	Boolean s_witch = TRUE;
 
@@ -2216,7 +2228,7 @@ SeqAnnotPtr  sim_for_blast(SeqAnnotPtr blast_sap, SeqIdPtr query_id, Int4 which_
 
         if (which_sim == RUN_BANDALGN) 
 	{
-	   vnp = WholeSeqLocListFromSeqAlign (align);
+	   vnp = SeqLocListOfBioseqsFromSeqAlign (align);
 	   msp = MashNew (is_aa);
            align = SeqLocListToSeqAlign (vnp, PRGALIGNDEFAULT, msp); 
 	   annot = SeqAnnotForSeqAlign (align);
@@ -2224,13 +2236,25 @@ SeqAnnotPtr  sim_for_blast(SeqAnnotPtr blast_sap, SeqIdPtr query_id, Int4 which_
 	}
 	while(align)
    	   {
-		ddp = align->segs;
-		while(ddp)
-		{
-			check_sim_list(ddp, &sim_list);
-			ddp = ddp->next;
-		}
-
+   	   switch (align->segtype) {
+   	   	case 1:
+			ddp = align->segs;
+			while(ddp)
+			{
+				check_sim_list(ddp, &sim_list);
+				ddp = ddp->next;
+			}
+   	   		break;
+   	   	case 2:
+   	   		dsp = align->segs;
+   	   		if (dsp) {
+   	   		}
+   	   		break;
+   	   	case 3:
+   	   		break;
+   	   	default:
+   	   		break;
+   	   }
 		align = align->next;
 	}
 	sim_align = NULL;
@@ -2327,6 +2351,9 @@ SeqAnnotPtr  sim_for_blast(SeqAnnotPtr blast_sap, SeqIdPtr query_id, Int4 which_
 	}
 
 	ValNodeFreeData(sim_list);
+
+	if (sim_align == NULL) return NULL;
+
 	annot = SeqAnnotNew();
 	annot->type = 2;
 	annot->data = sim_align;
@@ -2492,7 +2519,7 @@ static SeqFeatPtr convert_align_to_interval(SeqAlignPtr align, Int4 min_gap_len)
 
 Boolean load_align_to_interval(SeqAnnotPtr align_annot, Int4 min_gap_len, BioseqPtr bsp)
 {
-	SeqFeatPtr sfp_head, prev, sfp;
+	SeqFeatPtr sfp_head = NULL, prev, sfp;
 	SeqAlignPtr align;
 	ValNodePtr desc;
 	SeqAnnotPtr annot, p_annot;
