@@ -1,4 +1,6 @@
-/* $Id: pseed3.c,v 6.36 2001/05/04 14:14:52 madden Exp $ */
+static char const rcsid[] = "$Id: pseed3.c,v 6.42 2004/04/01 13:43:08 lavr Exp $";
+
+/* $Id: pseed3.c,v 6.42 2004/04/01 13:43:08 lavr Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -33,9 +35,27 @@ Maintainer: Alejandro Schaffer
  
 Contents: high-level routines for PHI-BLAST and pseed3
 
-$Revision: 6.36 $
+$Revision: 6.42 $
 
 $Log: pseed3.c,v $
+Revision 6.42  2004/04/01 13:43:08  lavr
+Spell "occurred", "occurrence", and "occurring"
+
+Revision 6.41  2003/08/04 19:47:56  dondosha
+Correct alignment endpoints after traceback in PHI BLAST
+
+Revision 6.40  2003/07/15 20:17:39  coulouri
+remove signal() and setrlimit() calls
+
+Revision 6.39  2003/05/30 17:25:37  coulouri
+add rcsid
+
+Revision 6.38  2003/05/13 16:02:53  coulouri
+make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+
+Revision 6.37  2002/08/28 13:38:06  madden
+Do not double close patfp, fix memory leaks
+
 Revision 6.36  2001/05/04 14:14:52  madden
 Fixes for multiple patterns in phi-blast
 
@@ -130,11 +150,6 @@ Now functions will collect messages in ValNodePtr before printing out.
 #include <ncbithr.h>
 #include <seed.h> 
 
-#if defined(OS_UNIX) && !defined(OS_UNIX_SUN) && !defined(OS_UNIX_LINUX)
-#include <sys/resource.h>
-#include <signal.h>
-#endif
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -150,8 +165,6 @@ static seedParallelItems * seedParallelFill(ReadDBFILEPtr rdpt, qseq_ptr query_s
 
 static void do_the_seed_search PROTO((BlastSearchBlkPtr search, Int4 num_seq, qseq_ptr query_seq, Int4 lenPatMatch, Boolean is_dna, GapAlignBlkPtr gap_align, patternSearchItems * patternSearch, seedSearchItems * seedSearch, Int4 * matchIndex, Int4 * totalOccurrences, seedResultItems * seedResults));
 
-static Boolean BlastSetLimits PROTO((Int4 cpu_limit, Int2 num_cpu));
-
 void PGPOutTextMessages(ValNodePtr info_vnp, FILE *fd)
 {
     ValNodePtr vnp;
@@ -314,7 +327,9 @@ Char * LIBCALL get_a_pat(
 	  }
 	}
       }
+/*
       FileClose(fp);
+*/
       if (unfilterHitArray)
         MemFree(unfilterHitArray);
       return NULL;
@@ -390,7 +405,7 @@ ValNodePtr LIBCALL seedEngineCore(BlastSearchBlkPtr search,
     Int4  numPatOccur, maxNumPatterns; /*number of pattern occurrences in query string*/
     Int4  effectiveOccurrences; /*number of occurrences not overlapping
                                   in more than half the pattern*/
-    Int4  occurIndex;  /*index over pattern ocuurences*/
+    Int4  occurIndex;  /*index over pattern occurrences*/
     Int4  twiceNumMatches; /*stores return value from find_hits*/
     Int4  matchIndex;  /*index for matches to a single sequence*/
     Int4 totalOccurrences = 0; /*total occurrences of pattern in database*/
@@ -455,7 +470,7 @@ ValNodePtr LIBCALL seedEngineCore(BlastSearchBlkPtr search,
                     maxNumPatterns = numPatOccur;
                 } else {
 
-                    sprintf(buffer, "Number of times pattern occur in the sequence %d exceded limit %d. Processing only %d occurences.\n", numPatOccur, options->max_num_patterns, options->max_num_patterns);
+                    sprintf(buffer, "Number of times pattern occur in the sequence %d exceded limit %d. Processing only %d occurrences.\n", numPatOccur, options->max_num_patterns, options->max_num_patterns);
                     ValNodeCopyStr(info_vnp, 0, buffer);
                     
                     maxNumPatterns = options->max_num_patterns;
@@ -778,7 +793,7 @@ Int4 LIBCALL convertProgramToFlag(Char * program, Boolean * is_dna)
 		if (strsame(program, "patmatchp")) 
 		  program_flag = PAT_MATCH_FLAG;
 		else {
-		  ErrPostEx(SEV_FATAL, 0, 0, "name of program not recognized %s \n", program);
+		  ErrPostEx(SEV_FATAL, 1, 0, "name of program not recognized %s \n", program);
 		  return(1);
 		}
   return(program_flag);
@@ -953,20 +968,22 @@ hit_ptr LIBCALL get_hits(qseq_ptr qp, Int4 len_of_pat,
                                 pattern matches*/
     Int4 llen, rlen; /*lengths of lseq, rseq*/
     hit_ptr hit_list = NULL;
-    Int4 hitL[MAX_HIT]; /*array of hit pairs between seq_db and pattern
+    Int4 *hitL; /*array of hit pairs between seq_db and pattern
                          two entries are used per hit*/
     
+    hitL = (Int4Ptr) MemNew(MAX_HIT*sizeof(Int4));
+
     (*newOccurrences) = 0;
     lseq = qp->lseq; 
     llen = qp->llen;
     rseq = qp->rseq; 
     rlen = qp->rlen;
     sseq = qp->sseq;
-    leftPartseq_dbReversed = (Uint1Ptr) ckalloc(len_seq_db+1);
+    leftPartseq_dbReversed = (Uint1Ptr) MemNew((len_seq_db+1)*sizeof(Uint1));
     twiceNumMatches = find_hits(hitL, seq_db, len_seq_db, is_dna, patternSearch);
     if (twiceNumMatches > 0 && is_dna) {
       if (len_seq_db > MAXDNA) {
-	ErrPostEx(SEV_FATAL, 0, 0, "MAX DNA Sequence length exceeded %d \n",
+	ErrPostEx(SEV_FATAL, 1, 0, "MAX DNA Sequence length exceeded %d \n",
 MAXDNA);
         exit(1);
       }
@@ -997,7 +1014,8 @@ MAXDNA);
 		   llen+len_of_pat+endPosrseq, lenLeft+(matchEnd-matchStart)+1+endPosrseq_db, &hit_list, mul);
       }
     }
-    free(leftPartseq_dbReversed);
+    MemFree(hitL);
+    MemFree(leftPartseq_dbReversed);
     if (buffer)
       MemFree(buffer);
     return hit_list;
@@ -1084,7 +1102,7 @@ SeqAlignPtr LIBCALL output_hits(ReadDBFILEPtr rdpt,
             Boolean is_dna, Int4 effectiveOccurrences,
             seedSearchItems *seedSearch, seedResultItems *seedResults, 
             patternSearchItems * patternSearch, Boolean reverse, 
-            Int4 numOccurences, Nlm_FloatHi eThresh,
+            Int4 numOccurrences, Nlm_FloatHi eThresh,
             SeqIdPtr query_id, Nlm_FloatHi posEthresh, 
             posSearchItems *posSearch, Int4 numMatches,
             Int4 * totalBelowEThresh, Boolean showDiagnostics,
@@ -1136,7 +1154,7 @@ SeqAlignPtr LIBCALL output_hits(ReadDBFILEPtr rdpt,
 
     Char tmpbuf[512];
 
-    probability = ((Nlm_FloatHi) numOccurences /  dbLength);
+    probability = ((Nlm_FloatHi) numOccurrences /  dbLength);
     oldNumber = -1;
     if (score_only) {
       for (oneMatch = seedResults->listOfMatchingSequences; 
@@ -1234,10 +1252,16 @@ SeqAlignPtr LIBCALL output_hits(ReadDBFILEPtr rdpt,
 	      *revASptr = *ASptr; 
 	      *ASptr = temp;
 	    }
+       oneHit->bi = llen - endPosQuery;
+       oneHit->bj = lenPrefix - endPosDbSeq;
+
 	    align_of_pattern(sseq, &oneMatch->seq[oneHit->hit_pos-1], len,
 			     oneHit->hit_end-oneHit->hit_pos+2, alignScript, &alignScript, 
 			     gap_align, &patWildcardScore, &mul, patternSearch,
 			     seedSearch);
+       oneHit->ei = llen + len;
+       oneHit->ej = lenPrefix + len;
+
 	    SEMI_G_ALIGN((Uchar *) (rseq-1), (Uchar *) &oneMatch->seq[oneHit->hit_end], rlen, 
 			 lenDbSequence-oneHit->hit_end-1, 
 			 alignScript,  &endPosQuery, &endPosDbSeq, FALSE, &alignScript,gap_align,0, FALSE);
@@ -1245,7 +1269,8 @@ SeqAlignPtr LIBCALL output_hits(ReadDBFILEPtr rdpt,
 	       oneHit->ei-oneHit->bi,
 	       oneHit->ej-oneHit->bj, reverseAlignScript, oneHit->bi+1, 
 	       oneHit->bj+1, is_dna); */
-	    
+	    oneHit->ei += endPosQuery;
+       oneHit->ej += endPosDbSeq;
 	    nextEditBlock = TracebackToGapXEditBlock(seq1 - 1 + oneHit->bi,
 						     oneMatch->seq-1 + oneHit->bj, oneHit->ei - oneHit->bi,
 						     oneHit->ej - oneHit->bj, reverseAlignScript, oneHit->bi,
@@ -1581,8 +1606,6 @@ static void do_the_seed_search(BlastSearchBlkPtr search, Int4 num_seq,
     else
         search->thr_info->realdb_done = FALSE;
     
-    BlastSetLimits(search->pbp->cpu_limit, search->pbp->process_num);	
-    
     if (NlmThreadsAvailable() && search->pbp->process_num > 1) {
         number_of_entries = INT4_MAX;
         /* Look for smallest database. */
@@ -1678,52 +1701,9 @@ static void do_the_seed_search(BlastSearchBlkPtr search, Int4 num_seq,
     }
     MemFree(seedParallelArray); 
 
-#ifdef RLIMIT_CPU
-    signal(SIGXCPU, SIG_IGN);
-#endif
     return;
 }
 
-Boolean  my_time_out_boolean;
-
-/* Called by UNIX signal for timeouts. */
-#ifdef RLIMIT_CPU
-static void 
-timeout_shutdown(int flag)
-
-{
-	my_time_out_boolean = TRUE;
-	signal(SIGXCPU, SIG_IGN);
-}
-#endif
-
-static Boolean BlastSetLimits(Int4 cpu_limit, Int2 num_cpu)
-
-{
-
-#ifdef RLIMIT_CPU
-	struct rlimit   rl;
-
-	if (cpu_limit <= 0)
-		return TRUE;
-
-	if (getrlimit(RLIMIT_CPU, &rl) != -1 )
-	{
-		if (rl.rlim_cur == RLIM_INFINITY)
-			rl.rlim_cur = 0;
-		rl.rlim_cur += cpu_limit/num_cpu;
-		setrlimit(RLIMIT_CPU, &rl);
-#ifdef SIGXCPU
-		sigset(SIGXCPU, timeout_shutdown);
-#endif
-	}
-#endif
-	my_time_out_boolean = FALSE;
-
-	return TRUE;
-
-}
-
 #ifdef __cplusplus
 }
 #endif
