@@ -1,3 +1,5 @@
+static char const rcsid[] = "$Id: suggest.c,v 6.11 2003/05/30 17:25:38 coulouri Exp $";
+
 /*   suggest.c
 * ===========================================================================
 *
@@ -29,7 +31,7 @@
 *
 * Version Creation Date:   3/17/94
 *
-* $Revision: 6.8 $
+* $Revision: 6.11 $
 *
 * File Description: 
 *
@@ -66,6 +68,15 @@
 *     
 *     A detailed change log follows:
 *     $Log: suggest.c,v $
+*     Revision 6.11  2003/05/30 17:25:38  coulouri
+*     add rcsid
+*
+*     Revision 6.10  2003/05/13 16:02:54  coulouri
+*     make ErrPostEx(SEV_FATAL, ...) exit with nonzero status
+*
+*     Revision 6.9  2002/02/06 21:10:38  kans
+*     PlusStrand and MinusStrand prefixed with Suggest to avoid collision with asn2graphic viewer symbols
+*
 *     Revision 6.8  2001/05/25 19:43:53  vakatov
 *     Nested comment typo fixed
 *
@@ -487,9 +498,9 @@ static	Int4	aori, ahz, aasc;
 
 Int4 Total_len;
 
-#define fatal(m)	{ErrPostEx(SEV_FATAL, 0, 0, "%s:  %s", module, m); goto Error;}
+#define fatal(m)	{ErrPostEx(SEV_FATAL, 1, 0, "%s:  %s", module, m); goto Error;}
 
-static char * suggest_revision = "$Revision: 6.8 $";
+static char * suggest_revision = "$Revision: 6.11 $";
 
 static Int4	suggest_get_frames_init = 1;
 
@@ -1003,15 +1014,15 @@ static Int4 AddInterval (Int4 lonum, Int4 hinum, Int4 orient)
       return 1;
     }
   } else {
-    if (nt_orient [start_nt] == MinusStrand) {
+    if (nt_orient [start_nt] == SuggestMinusStrand) {
       lonum -= nt_len_offsets [start_nt];
       hinum -= nt_len_offsets [start_nt];
       lonum = nt_dna_len [start_nt] - lonum + 1;
       hinum = nt_dna_len [start_nt] - hinum + 1;
-      if (orient == MinusStrand){
-        orient = PlusStrand;
-      } else if (orient == PlusStrand) {
-        orient = MinusStrand;
+      if (orient == SuggestMinusStrand){
+        orient = SuggestPlusStrand;
+      } else if (orient == SuggestPlusStrand) {
+        orient = SuggestMinusStrand;
       }
       lonum += nt_len_offsets [start_nt];
       hinum += nt_len_offsets [start_nt];
@@ -1103,7 +1114,7 @@ report(void)
 			to = dna_len;
 		}
 
-		if (m_stk[i].orient == MinusStrand) {
+		if (m_stk[i].orient == SuggestMinusStrand) {
 			from = dna_len - from + 1;
 			to = dna_len - to + 1;
 		}
@@ -1318,7 +1329,7 @@ dna_len, pro_len);
 		}
 	} else if (get_frames(dna_seq) != 0)
 		goto Error;
-	orientation = PlusStrand;
+	orientation = SuggestPlusStrand;
 	if (search() != 0)
 		goto Error;
 
@@ -1340,7 +1351,7 @@ dna_len, pro_len);
 		}
 	} else if (get_frames(rev_seq) != 0)
 		goto Error;
-	orientation = MinusStrand;
+	orientation = SuggestMinusStrand;
 	if (search() != 0)
 		goto Error;
 
@@ -1641,7 +1652,7 @@ static void Suggest_Intervals (SuggestRecPtr sugrp,
         dna_len = 0;
         nt_len_offsets [0] = 0;
         nt_id_seq[0] = 0;
-        last_orient = PlusStrand;
+        last_orient = SuggestPlusStrand;
         nt_orient[0] = last_orient;
 
         m_top = 0;
