@@ -1,3 +1,5 @@
+static char const rcsid[] = "$Id: tfuns.c,v 6.12 2004/04/01 13:43:08 lavr Exp $";
+
 #include <stdio.h>
 #include <ncbi.h>
 #include <objpub.h>
@@ -237,7 +239,7 @@ Int2 AStringIgnoreSpaceCmp(CharPtr s1, CharPtr s2)
 
 /*----------------------------------------------------------------------------*/
 
-WholeFeatPtr WholeFeatNew()
+WholeFeatPtr WholeFeatNew(void)
 {
 	WholeFeatPtr wfp;
 	
@@ -277,11 +279,6 @@ SeqFeatPtr tie_feat(SeqFeatPtr head, SeqFeatPtr next)
    return head;
 }
 
-/****************************************************************************
-*  tie_next_mol:
-*  -- ties next MolFix to the end of the chain
-*                                                                    08-4-93
-****************************************************************************/
 MolFixPtr tie_next_mol(MolFixPtr head, MolFixPtr next)
 {
 	MolFixPtr v;
@@ -592,6 +589,9 @@ SubSourcePtr remove_subtype(SubSourcePtr head, SubSourcePtr x)
 	if (head == NULL) {
 		return NULL;
 	}
+	if (x == NULL) {
+		return SubSourceSetFree (head);
+	}
 	if (x == head) {
 		head = x->next;
 		x->next = NULL;
@@ -616,6 +616,9 @@ OrgModPtr remove_OrgMod(OrgModPtr head, OrgModPtr x)
 	if (head == NULL) {
 		return NULL;
 	}
+	if (x == NULL) {
+		return OrgModSetFree (head);
+	}
 	if (x == head) {
 		head = x->next;
 		x->next = NULL;
@@ -798,7 +801,7 @@ CharPtr get_qvalue(GBQualPtr curq, CharPtr qual)
 
 /*****************************************************************************
 *   laststr:
-*     -- strstr, last occurance
+*     -- strstr, last occurrence
 ******************************************************************************/
 CharPtr laststr(CharPtr str, CharPtr word)
 {
@@ -853,7 +856,6 @@ static void find_orf(SeqPortPtr spp, Int2 ir, CharPtr vals, CharPtr codes, ValNo
 	Int4 len;
 	Int4 start = -1, stop = -1, met_start = -1;
 	SeqLocPtr slp = NULL, m_slp = NULL;
-	SeqIntPtr sip;
 	ValNodePtr prev = NULL;
 	IntFuzzPtr infrom=NULL, into=NULL;
 
@@ -917,6 +919,23 @@ static void find_orf(SeqPortPtr spp, Int2 ir, CharPtr vals, CharPtr codes, ValNo
 		}
 	}
 /*  treat end of sequence as a stop */
+	if (met_start != -2) {
+		stop = pos-1;	
+		if (met_start == -1) {
+			met_start = 0;
+		}
+		if (stop-met_start > limit) {
+			if (strand == Seq_strand_plus) {
+				stop = stop+ir;
+				if (stop > len-1) {
+					stop = len-1;
+				}
+				m_slp = SeqLocIntNew(met_start+ir, stop, strand, id);
+			} else {
+				m_slp = SeqLocIntNew(len-1-(stop+ir), len-1-(met_start+ir), strand, id);
+			}
+		}
+	}
 	if (start != -2) {
 		stop = pos-1;	
 		if (start == -1) {
@@ -932,9 +951,18 @@ static void find_orf(SeqPortPtr spp, Int2 ir, CharPtr vals, CharPtr codes, ValNo
 			} else {
 				slp = SeqLocIntNew(len-1-(stop+ir), len-1-(start+ir), strand, id);
 			}
-		ValNodeAddPointer(list, ir, slp);
 		}
 	}
+	if (m_slp) {
+		m_slp->next = slp;
+		ValNodeAddPointer(list, ir, m_slp);
+		m_slp = slp = NULL;
+	} else if (slp) {
+		m_slp = ValNodeNew(NULL);
+		m_slp->next = slp;
+		ValNodeAddPointer(list, ir, m_slp);
+		m_slp = slp = NULL;
+	}
 	return;
 }
 
@@ -1177,7 +1205,7 @@ static int LIBCALLBACK CompareLoc (VoidPtr vp1, VoidPtr vp2)
 static ValNodePtr find_all_orf(BioseqPtr bsp, CharPtr vals, CharPtr codes, Int2 limit, Int4 from, Int4 to)
 {
 	Int2 ir;
-	ValNodePtr list = NULL, vnp;
+	ValNodePtr list = NULL;
 	SeqPortPtr spp;
 	SeqLocPtr slp = NULL;
 	
@@ -1225,7 +1253,7 @@ static Boolean get_src (GatherContextPtr gcp)
 			vnp = (ValNodePtr) (gcp->thisitem);
 			if (vnp->choice == Seq_descr_source) {
 				if (vnp->data.ptrvalue != NULL) {
-					new = ValNodeNew(NULL);
+					new = SeqDescrNew(NULL);
 					new = MemCopy(new, vnp, sizeof(ValNode));
 					new->next = NULL;
 					*vnpp = new;
