@@ -1,4 +1,6 @@
-/* $Id: xmlblast.c,v 6.18 2001/05/01 20:54:32 madden Exp $ */
+static char const rcsid[] = "$Id: xmlblast.c,v 6.35 2004/06/30 12:32:20 madden Exp $";
+
+/* $Id: xmlblast.c,v 6.35 2004/06/30 12:32:20 madden Exp $ */
 /**************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -30,12 +32,63 @@
 *   
 * Version Creation Date: 05/17/2000
 *
-* $Revision: 6.18 $
+* $Revision: 6.35 $
 *
 * File Description:  Functions to print simplified BLAST output (XML)
 *
 * 
 * $Log: xmlblast.c,v $
+* Revision 6.35  2004/06/30 12:32:20  madden
+* Added include for blfmtutl.h, removed unused variable
+*
+* Revision 6.34  2004/06/23 21:08:58  dondosha
+* Fixed masking of filtered locations for translated queries
+*
+* Revision 6.33  2004/04/29 19:55:35  dondosha
+* Mask filtered locations in query sequence lines
+*
+* Revision 6.32  2004/03/31 17:58:23  dondosha
+* Added PSIXmlReset function to allow keeping the AsnIoPtr between outputs for multiple queries in blastpgp
+*
+* Revision 6.31  2003/08/04 16:19:16  dondosha
+* Added effective HSP length (length adjustment) to other returns, so it can be reported in XML output
+*
+* Revision 6.30  2003/05/30 17:25:38  coulouri
+* add rcsid
+*
+* Revision 6.29  2003/03/21 21:01:16  camacho
+* Fixed inversion of subject and query sequences for tblastx
+*
+* Revision 6.28  2003/02/19 15:42:32  madden
+* Check glb_matrix before freeing
+*
+* Revision 6.27  2003/01/28 16:57:11  dondosha
+* For single query, call the old BXMLPrintOutput function from BXMLPrintMultiQueryOutput
+*
+* Revision 6.26  2003/01/23 20:02:53  dondosha
+* Distinguish between blastn and megablast programs in multi-query XML output
+*
+* Revision 6.25  2003/01/23 19:55:20  dondosha
+* Added the closing part for multi-query XML output.
+*
+* Revision 6.24  2003/01/06 23:01:40  dondosha
+* Added function to create a multi-query XML output for web megablast
+*
+* Revision 6.23  2002/11/14 15:37:18  dondosha
+* Added functions to extract all hit information from seqalign that can be extracted without loading sequences
+*
+* Revision 6.22  2002/07/17 22:28:13  dondosha
+* Added support for megablast XML output
+*
+* Revision 6.21  2002/04/23 20:48:24  madden
+* Fix hsp_count for ungapped case
+*
+* Revision 6.20  2002/03/08 21:34:57  madden
+* If no title for Bioseq use "No definition line found"
+*
+* Revision 6.19  2002/01/15 21:56:38  madden
+* Fixes from Jed Wing (Turbogenomics) for ungapped runs
+*
 * Revision 6.18  2001/05/01 20:54:32  madden
 * Set glb_matrix in BXMLSeqAlignToHits if not already set
 *
@@ -98,11 +151,14 @@
 */
 
 #include <xmlblast.h>
+#include <blfmtutl.h>
 
 static Int4Ptr PNTR glb_matrix;
 
-Boolean BXMLGetSeqLineForDenseDiag(DenseDiagPtr ddp, HspPtr hsp, Int4 length, Boolean is_aa, Int4Ptr PNTR matrix)
+Boolean BXMLGetSeqLineForDenseDiag(DenseDiagPtr ddp, HspPtr hsp, Int4 length,
+                                   Boolean is_aa, Int4Ptr PNTR matrix)
 {
+    SeqIdPtr m_id, t_id;
     SeqInt si;
     SeqLoc sl;
     Int4 i;
@@ -125,17 +181,18 @@ Boolean BXMLGetSeqLineForDenseDiag(DenseDiagPtr ddp, HspPtr hsp, Int4 length, Bo
     
     /* SeqLoc for query sequence */
     
-    si.id = SeqIdDup(ddp->id);
+    m_id = SeqIdDup(ddp->id);
+    si.id = m_id;
     si.from = hsp->query_from;
     si.to = hsp->query_to;
     si.strand = (ddp->strands == NULL) ? 0 : ddp->strands[0];
 
-    
     m_spp = SeqPortNewByLoc(&sl, is_aa ? Seq_code_ncbieaa : Seq_code_iupacna);
     
     /* SeqLoc for the subject */
     
-    si.id = ddp->id->next;
+    t_id = SeqIdDup(ddp->id->next);
+    si.id = t_id;
     si.from = hsp->hit_from;
     si.to = hsp->hit_to;
     si.strand = (ddp->strands == NULL) ? 0 : ddp->strands[1];
@@ -171,7 +228,8 @@ Boolean BXMLGetSeqLineForDenseDiag(DenseDiagPtr ddp, HspPtr hsp, Int4 length, Bo
         }
     }
 
-    SeqIdFree(si.id);
+    SeqIdFree(t_id);
+    SeqIdFree(m_id);
     SeqPortFree(m_spp);
     SeqPortFree(t_spp);
     
@@ -348,8 +406,8 @@ Boolean BXMLGetSeqLineForStdSeg(StdSegPtr ssp, HspPtr hsp, Int4 length,
                         codon[2] = SeqPortGetResidue(spp1);
                         residue2 = AAForCodon(codon, genetic_code2);
                         
-                        hsp->qseq[abs_index] = residue1;
-                        hsp->hseq[abs_index] = residue2;
+                        hsp->qseq[abs_index] = residue2;
+                        hsp->hseq[abs_index] = residue1;
                         
                         if (residue1 == residue2)
                             hsp->midline[abs_index] = residue1;
@@ -534,8 +592,54 @@ Boolean BXMLGetSeqLines(SeqAlignPtr align, HspPtr hsp, Int4 length,
     return TRUE;
 }
 
+static void 
+MaskFilteredLocInHsp(HspPtr hsp, ValNodePtr mask, Boolean is_prot,
+                     Boolean query_translated)
+{
+   Char mask_char;
+   SeqLocPtr mask_loc;
+   SeqLocPtr seqloc = NULL;
+   Int4 start, stop, index;
+
+   if (!mask)
+      return;
+
+   mask_char = (is_prot ? 'X' : 'N');
+
+   for ( ; mask; mask = mask->next) {
+      if (mask->choice != FrameToDefine(hsp->query_frame))
+         continue;
+      mask_loc = (SeqLocPtr) mask->data.ptrvalue;
+
+      while((seqloc = SeqLocFindNext(mask_loc, seqloc)) != NULL) {
+         if (hsp->query_from < hsp->query_to) {
+            start = MAX(hsp->query_from, SeqLocStart(seqloc))
+               - hsp->query_from;
+            stop = MIN(SeqLocStop(seqloc), hsp->query_to) - 
+               hsp->query_from;
+         } else {
+            start = hsp->query_from - 
+               MIN(SeqLocStop(seqloc), hsp->query_from);
+            stop = hsp->query_from - 
+               MAX(hsp->query_to, SeqLocStart(seqloc));
+         }
+
+         if (query_translated) {
+            start /= CODON_LENGTH;
+            stop /= CODON_LENGTH;
+         }
+         
+         for (index = start; index <= stop; ++index) {
+            hsp->qseq[index] = mask_char;
+            if (hsp->hseq[index] != mask_char)
+               hsp->midline[index] = ' ';
+         }
+      }
+   }
+}
+
 HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
-                              Boolean is_ooframe)
+                              Boolean is_ooframe, ValNodePtr mask_loc)
 {
     HspPtr hsp;
     AlignSum as;
@@ -544,13 +648,14 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     BioseqPtr bsp;
     Char tmp[256];
     static Boolean master_checked;
+    Boolean prot_alphabet;
 
     if((hsp = HspNew()) == NULL)
         return NULL;
 
     MemSet(&as, NULLB, sizeof(AlignSum));
     
-    as.master_sip = SeqIdDup(TxGetQueryIdFromSeqAlign(sap));
+    as.master_sip = TxGetQueryIdFromSeqAlign(sap);
     as.target_sip = TxGetSubjectIdFromSeqAlign(sap);
 
     /* Checkup for query Bioseq to be available in the index */
@@ -615,13 +720,18 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     hsp->density = 0;             /* ???? */
     
 
-    /* For sequence retrievel from should always be less than to */
+    /* For sequence retrieval from should always be less than to */
     hsp->query_from = as.master_from;
     hsp->query_to = as.master_to;
     hsp->hit_from = as.target_from;
     hsp->hit_to = as.target_to;
 
     BXMLGetSeqLines(sap, hsp, as.totlen, is_aa, chain, as.matrix);
+    /* Sequences are shown in protein alphabet in all cases except blastn,
+       that is if either subject is protein, or it is a translated search, 
+       in which case alignment segments have StdSeg form. */
+    prot_alphabet = (is_aa || sap->segtype == SAS_STD);
+    MaskFilteredLocInHsp(hsp, mask_loc, prot_alphabet, as.m_frame_set);
 
     /* For display it depends on strand */
     
@@ -645,13 +755,11 @@ HspPtr BXMLGetHspFromSeqAlign(SeqAlignPtr sap, Boolean is_aa, Int4 chain,
     if(matrix_allocated)
         free_default_matrix(as.matrix);
 
-    SeqIdFree(as.master_sip);
-    
     return hsp;
 }
 
 HitPtr BXMLSeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped, 
-                          Boolean is_ooframe)
+                          Boolean is_ooframe, ValNodePtr mask_loc)
 {
     HitPtr hitp, hitp_head;
     HspPtr hspp;
@@ -692,7 +800,10 @@ HitPtr BXMLSeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped,
         
         is_aa = (bsp->mol == Seq_mol_aa);
            
-        hitp->def = StringSave(BioseqGetTitle(bsp));
+        if (BioseqGetTitle(bsp))
+        	hitp->def = StringSave(BioseqGetTitle(bsp));
+        else
+        	hitp->def = StringSave("No definition line found");
           
         SeqIdWrite(bsp->id, buffer, PRINTID_FASTA_LONG, sizeof(buffer));
         hitp->id = StringSave(buffer);
@@ -708,15 +819,20 @@ HitPtr BXMLSeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped,
         hsp_count = 1;          /* Hsps starting with 1 */
         chain = 1;
 
+        BioseqUnlock(bsp);
+
         for(sap2 = sap; sap2 != NULL;) {
             
             /* Filling info about specific alignments */
 
             while(TRUE) {
                 if(hitp->hsps == NULL) {
-                    hspp = hitp->hsps = BXMLGetHspFromSeqAlign(sap2, is_aa, chain, is_ooframe);
+                    hspp = hitp->hsps = 
+                       BXMLGetHspFromSeqAlign(sap2, is_aa, chain, is_ooframe, 
+                                              mask_loc);
                 } else {
-                    if((hspp->next = BXMLGetHspFromSeqAlign(sap2, is_aa, chain, is_ooframe)) == NULL)
+                    if((hspp->next = BXMLGetHspFromSeqAlign(sap2, is_aa, chain,
+                                        is_ooframe, mask_loc)) == NULL)
                         break;
                     else
                         hspp = hspp->next;
@@ -725,9 +841,9 @@ HitPtr BXMLSeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped,
                 if(hspp == NULL)
                     break;
                 
+                hspp->num = hsp_count;
+                hsp_count++;
                 if(!ungapped) {  /* Only one chain for gapped */
-                    hspp->num = hsp_count;
-                    hsp_count++;
                     break;
                 }
                 chain++;
@@ -747,58 +863,114 @@ HitPtr BXMLSeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped,
 
     return hitp_head;
 }
+
 IterationPtr BXMLBuildOneIteration(SeqAlignPtr seqalign, 
                                    ValNodePtr other_returns,
                                    Boolean is_ooframe, Boolean ungapped,
-                                   Int4 iter_num, CharPtr message)
+                                   Int4 iter_num, CharPtr message,
+                                   ValNodePtr mask_loc)
+{
+   return BXMLBuildOneQueryIteration(seqalign, other_returns, is_ooframe,
+                                     ungapped, iter_num, message, NULL, 
+                                     mask_loc);
+}
+
+IterationPtr BXMLBuildOneQueryIteration(SeqAlignPtr seqalign, 
+                                   ValNodePtr other_returns,
+                                   Boolean is_ooframe, Boolean ungapped,
+                                   Int4 iter_num, CharPtr message,
+                                   BioseqPtr query, ValNodePtr mask_loc)
 {
     IterationPtr iterp;
-    TxDfDbInfoPtr dbinfo=NULL;
     BLAST_KarlinBlkPtr ka_params_gap=NULL;
+    BLAST_KarlinBlkPtr ka_params_ungap=NULL;
     ValNodePtr vnp;
+    Char buffer[1024];
     
     iterp = IterationNew();
     iterp->iter_num = iter_num;
-    iterp->stat = StatisticsNew();
-    
-    if(seqalign != NULL) {
-        iterp->hits = BXMLSeqAlignToHits(seqalign, ungapped, is_ooframe);
+
+    if (query) {
+       SeqIdWrite(query->id, buffer, PRINTID_FASTA_LONG, sizeof(buffer));
+       iterp->query_ID = StringSave(buffer);
+
+       if((iterp->query_def = StringSave(BioseqGetTitle(query))) == NULL) {
+          iterp->query_def = StringSave("No definition line found");
+       }
+
+       iterp->query_len = query->length;
     }
 
-    if(other_returns != NULL) {    
-        for (dbinfo = NULL, vnp=other_returns; vnp; vnp = vnp->next) {
-            switch (vnp->choice) {
-            case TXDBINFO:
-                dbinfo = vnp->data.ptrvalue;
-                break;
-            case TXKABLK_GAP:
-                ka_params_gap = vnp->data.ptrvalue;
-                break;
-            case EFF_SEARCH_SPACE:
-                iterp->stat->eff_space = vnp->data.realvalue;
-                break;
-            default:
-                break;
-            }
-        }
-        
-        if(dbinfo != NULL) {
-            iterp->stat->db_num= dbinfo->number_seqs;
-            iterp->stat->db_len = dbinfo->total_length;
-        }
-        
-        if(ka_params_gap != NULL) {
-            iterp->stat->lambda = ka_params_gap->Lambda;
-            iterp->stat->kappa = ka_params_gap->K;
-            iterp->stat->entropy = ka_params_gap->H;
-        }
+    if(seqalign != NULL) {
+       iterp->hits = 
+          BXMLSeqAlignToHits(seqalign, ungapped, is_ooframe, mask_loc);
     }
+
+    iterp->stat = BXMLBuildStatistics(other_returns, ungapped);
     
     iterp->message = StringSave(message);
     
     return iterp;
 }
 
+StatisticsPtr
+BXMLBuildStatistics(ValNodePtr other_returns, Boolean ungapped)
+{
+   TxDfDbInfoPtr dbinfo=NULL;
+   BLAST_KarlinBlkPtr ka_params_gap=NULL;
+   BLAST_KarlinBlkPtr ka_params_ungap=NULL;
+   ValNodePtr vnp;
+   StatisticsPtr stat;
+
+   if (!other_returns) 
+      return NULL;
+
+   stat = StatisticsNew();
+
+   for (dbinfo = NULL, vnp=other_returns; vnp; vnp = vnp->next) {
+      switch (vnp->choice) {
+      case TXDBINFO:
+         dbinfo = vnp->data.ptrvalue;
+         break;
+      case TXKABLK_GAP:
+         ka_params_gap = vnp->data.ptrvalue;
+         break;
+      case TXKABLK_NOGAP:
+         ka_params_ungap = vnp->data.ptrvalue;
+         break;
+      case EFF_SEARCH_SPACE:
+         stat->eff_space = vnp->data.realvalue;
+         break;
+      case EFF_HSP_LENGTH:
+         stat->hsp_len = vnp->data.intvalue;
+         break;
+      default:
+         break;
+      }
+   }
+   
+   if(dbinfo != NULL) {
+      stat->db_num= dbinfo->number_seqs;
+      stat->db_len = dbinfo->total_length;
+   }
+   
+   if(ungapped) {
+      if(ka_params_ungap != NULL) {
+         stat->lambda = ka_params_ungap->Lambda;
+         stat->kappa = ka_params_ungap->K;
+         stat->entropy = ka_params_ungap->H;
+      }
+   } else {
+      if(ka_params_gap != NULL) {
+         stat->lambda = ka_params_gap->Lambda;
+         stat->kappa = ka_params_gap->K;
+         stat->entropy = ka_params_gap->H;
+      }
+   }
+
+   return stat;
+}
+
 BlastOutputPtr BXMLCreateBlastOutputHead(CharPtr program, CharPtr database, 
                                          BLAST_OptionsBlkPtr options, 
                                          BioseqPtr query, Int4 flags)
@@ -816,29 +988,30 @@ BlastOutputPtr BXMLCreateBlastOutputHead(CharPtr program, CharPtr database,
     if(glb_matrix == NULL)
         glb_matrix = load_default_matrix ();
     
-    SeqIdWrite(query->id, buffer, PRINTID_FASTA_LONG, sizeof(buffer));
-    boutp->query_ID = StringSave(buffer);
+    if (query) {
+       SeqIdWrite(query->id, buffer, PRINTID_FASTA_LONG, sizeof(buffer));
+       boutp->query_ID = StringSave(buffer);
 
-    if((boutp->query_def = StringSave(BioseqGetTitle(query))) == NULL) {
-        boutp->query_def = StringSave("No definition line found");
+       if((boutp->query_def = StringSave(BioseqGetTitle(query))) == NULL) {
+          boutp->query_def = StringSave("No definition line found");
+       }
+
+       boutp->query_len = query->length;
+
+       if(flags & BXML_INCLUDE_QUERY) {
+          boutp->query_seq = MemNew(query->length+1);
+          is_aa = (query->mol == Seq_mol_aa);
+          spp = SeqPortNew(query, 0, -1, Seq_strand_plus, 
+                           (is_aa) ? Seq_code_ncbieaa : Seq_code_iupacna);
+          
+          for (i = 0; i < query->length; i++) {
+             boutp->query_seq[i] = SeqPortGetResidue(spp);
+          }
+          spp = SeqPortFree(spp);
+       } else {
+          boutp->query_seq = NULL;    /* Do we need sequence here??? */
+       }
     }
-
-    boutp->query_len = query->length;
-    
-    if(flags & BXML_INCLUDE_QUERY) {
-        boutp->query_seq = MemNew(query->length+1);
-        is_aa = (query->mol == Seq_mol_aa);
-        spp = SeqPortNew(query, 0, -1, Seq_strand_plus, 
-                         (is_aa) ? Seq_code_ncbieaa : Seq_code_iupacna);
-        
-        for (i = 0; i < query->length; i++) {
-            boutp->query_seq[i] = SeqPortGetResidue(spp);
-        }
-        spp = SeqPortFree(spp);
-    } else {
-        boutp->query_seq = NULL;    /* Do we need sequence here??? */
-    }
-
     /* Program name */
     boutp->program = StringSave(program);
 
@@ -879,7 +1052,7 @@ Boolean BXMLPrintOutput(AsnIoPtr aip, SeqAlignPtr seqalign,
                         BLAST_OptionsBlkPtr options,CharPtr program,
                         CharPtr database, BioseqPtr query, 
                         ValNodePtr other_returns, Int4 flags, 
-                        CharPtr message)
+                        CharPtr message, ValNodePtr mask_loc)
 {
     BlastOutputPtr boutp;
     Boolean ungapped = FALSE;
@@ -894,12 +1067,13 @@ Boolean BXMLPrintOutput(AsnIoPtr aip, SeqAlignPtr seqalign,
     /* Here is one-iterational Blast output */
     boutp->iterations = BXMLBuildOneIteration(seqalign, other_returns,
                                               options->is_ooframe, ungapped,
-                                              1, message);
+                                              1, message, mask_loc);
     
     if (aip != NULL)      
         BlastOutputAsnWrite(boutp, aip, NULL);
     
-    free_default_matrix(glb_matrix);
+    if (glb_matrix)
+    	free_default_matrix(glb_matrix);
     glb_matrix = NULL;
     
     BlastOutputFree(boutp);
@@ -909,6 +1083,125 @@ Boolean BXMLPrintOutput(AsnIoPtr aip, SeqAlignPtr seqalign,
     return TRUE;
 }
 
+Boolean BXMLPrintMultiQueryOutput(AsnIoPtr aip, SeqAlignPtr seqalign, 
+           BLAST_OptionsBlkPtr options, CharPtr program, CharPtr database, 
+           BioseqSetPtr query_set, ValNodePtr other_returns, Int4 flags,
+           CharPtr message, ValNodePtr mask_loc)
+{
+    Boolean ungapped = FALSE;
+    BioseqPtr query;
+    SeqEntryPtr sep;
+    Boolean q_is_na, d_is_na;
+    MBXmlPtr mbxp = NULL;
+    IterationPtr iterp;
+    SeqEntryFunc seqentry_callback;
+    Int4 index;
+    Boolean query_found;
+    SeqIdPtr seqid = NULL;
+    SeqAlignPtr sap = NULL, next_seqalign = NULL;
+    ValNodePtr next_mask_loc = NULL, current_mask_loc = NULL;
+
+    sep = (SeqEntryPtr) ((BioseqSetPtr)query_set)->seq_set;
+    BlastGetTypes(program, &q_is_na, &d_is_na);
+    if (q_is_na)
+       seqentry_callback = FindNuc;
+    else 
+       seqentry_callback = FindProt;
+
+    /* If no queries, there is nothing to report */
+    if (!sep)
+       return FALSE;
+
+    /* If only one query, call a one-query output function */
+    if (!sep->next) {
+       Boolean return_value;
+       SeqEntryExplore(sep, &query, seqentry_callback);
+       return_value = BXMLPrintOutput(aip, seqalign, options, program, 
+                         database, query, other_returns, flags, 
+                         message, mask_loc);
+       /* This function is presumed to close the AsnIoPtr inside */
+       AsnIoClose(aip);
+       return return_value;
+    }
+
+    next_mask_loc = mask_loc;
+
+    index = 0;
+    while (seqalign) {
+       /* Find the corresponding query */
+       query_found = FALSE;
+       for ( ; sep; ++index, sep = sep->next) {
+          SeqEntryExplore(sep, &query, seqentry_callback);
+          seqid = TxGetQueryIdFromSeqAlign(seqalign);
+          if (SeqIdComp(query->id, seqid) == SIC_YES) {
+             query_found = TRUE;
+             break;
+          }
+       }
+       if (query_found) {
+          /* Find where seqaligns for this query end */
+          for (sap = seqalign; sap && sap->next; sap = sap->next) {
+             if (SeqIdComp(seqid, TxGetQueryIdFromSeqAlign(sap->next)) 
+                 != SIC_YES) {
+                break;
+             }
+          }
+          next_seqalign = sap->next;
+          /* Unlink this query seqaligns from the rest */
+          sap->next = NULL;
+       } else {
+          break;
+       }
+
+       /* Find the masking locations for this query */
+       if (next_mask_loc && 
+           SeqIdComp(SeqLocId((SeqLocPtr)next_mask_loc->data.ptrvalue), 
+                     seqid) == SIC_YES) {
+          current_mask_loc = (SeqLocPtr) 
+             MemDup(next_mask_loc, sizeof(SeqLoc));
+          next_mask_loc = next_mask_loc->next;
+          current_mask_loc->next = NULL;
+       } else {
+          current_mask_loc = NULL;
+       }
+
+       if (!mbxp) {
+          if (options->is_megablast_search) {
+             mbxp = PSIXmlInit(aip, "megablast", database, options, query, 0);
+          } else {
+             mbxp = PSIXmlInit(aip, program, database, options, query, 0);
+          }
+       }
+       if(options->gapped_calculation == FALSE || 
+          !StringICmp(program, "tblastx"))
+          ungapped = TRUE;
+    
+       /* Here is one-iterational Blast output */
+       iterp = BXMLBuildOneQueryIteration(seqalign, NULL, options->is_ooframe,
+                                          ungapped, index, NULL, query, 
+                                          current_mask_loc);
+       current_mask_loc = (ValNodePtr) MemFree(current_mask_loc);
+    
+       IterationAsnWrite(iterp, mbxp->aip, mbxp->atp);
+       AsnIoFlush(mbxp->aip);
+       IterationFree(iterp);
+       /* Reconnect the SeqAlign chain */
+       if (sap)
+          sap->next = next_seqalign;
+       seqalign = next_seqalign;
+    }
+    
+    free_default_matrix(glb_matrix);
+    glb_matrix = NULL;
+    
+    MBXmlClose(mbxp, other_returns, !options->gapped_calculation);
+
+    ObjMgrFreeCache(0);
+    
+    return TRUE;
+}
+
+
 PSIXmlPtr PSIXmlInit(AsnIoPtr aip, CharPtr program, CharPtr database, 
                      BLAST_OptionsBlkPtr options, BioseqPtr query, Int4 flags)
 {
@@ -930,6 +1223,7 @@ PSIXmlPtr PSIXmlInit(AsnIoPtr aip, CharPtr program, CharPtr database,
     AsnTypePtr       BLASTOUTPUT_param;
     AsnTypePtr       BLASTOUTPUT_iterations;
     AsnTypePtr       BLASTOUTPUT_iterations_E;
+    AsnTypePtr       BLASTOUTPUT_mbstat;
 
     psixp = (PSIXmlPtr) MemNew(sizeof(PSIXml));
     
@@ -953,15 +1247,19 @@ PSIXmlPtr PSIXmlInit(AsnIoPtr aip, CharPtr program, CharPtr database,
     MACRO_atp_find(BLASTOUTPUT_param,BlastOutput.param);
     MACRO_atp_find(BLASTOUTPUT_iterations,BlastOutput.iterations);
     MACRO_atp_find(BLASTOUTPUT_iterations_E,BlastOutput.iterations.E);
+    MACRO_atp_find(BLASTOUTPUT_mbstat,BlastOutput.mbstat);
 
     /* Start of iterations structure */
     psixp->atp = BLASTOUTPUT_iterations_E;
-    
+
     /* Head of all BlastOutput structure */
     psixp->BlastOutput = BLASTOUTPUT;
     
     /* Head of iterations strucure */
     psixp->BlastOutput_iterations = BLASTOUTPUT_iterations;
+
+    /* Head of the final statistics for Mega BLAST */
+    psixp->BlastOutput_mbstat = BLASTOUTPUT_mbstat;
     
     psixp->boutp = BXMLCreateBlastOutputHead(program, database, options, 
                                              query, flags);
@@ -1029,10 +1327,23 @@ PSIXmlPtr PSIXmlInit(AsnIoPtr aip, CharPtr program, CharPtr database,
     return psixp;
 }
 
-void PSIXmlClose(PSIXmlPtr psixp)
+void PSIXmlReset(PSIXmlPtr psixp)
 {
     AsnCloseStruct(psixp->aip, psixp->BlastOutput_iterations, NULL);
     AsnCloseStruct(psixp->aip, psixp->BlastOutput, NULL);
+    psixp->BlastOutput_iterations = NULL;
+    psixp->BlastOutput = NULL;
+    AsnIoReset(psixp->aip);
+    
+    psixp->boutp = BlastOutputFree(psixp->boutp);
+}
+
+void PSIXmlClose(PSIXmlPtr psixp)
+{
+   if (psixp->BlastOutput_iterations)
+      AsnCloseStruct(psixp->aip, psixp->BlastOutput_iterations, NULL);
+   if (psixp->BlastOutput)
+      AsnCloseStruct(psixp->aip, psixp->BlastOutput, NULL);
     
     AsnIoClose(psixp->aip);
     
@@ -1041,3 +1352,285 @@ void PSIXmlClose(PSIXmlPtr psixp)
     
     return;
 }
+
+void MBXmlClose(PSIXmlPtr mbxp, ValNodePtr other_returns, Boolean ungapped)
+{
+   StatisticsPtr stat;
+   AsnTypePtr atp;
+
+    AsnCloseStruct(mbxp->aip, mbxp->BlastOutput_iterations, NULL);
+
+    if (other_returns) {
+       atp = AsnLinkType(NULL, mbxp->BlastOutput_mbstat);
+
+       /*AsnOpenStruct(mbxp->aip, mbxp->BlastOutput_mbstat, NULL);*/
+       stat = BXMLBuildStatistics(other_returns, ungapped);
+
+       StatisticsAsnWrite(stat, mbxp->aip, atp);
+       AsnIoFlush(mbxp->aip);
+       StatisticsFree(stat);                 
+       /*AsnCloseStruct(mbxp->aip, mbxp->BlastOutput_mbstat, NULL);*/
+    }
+
+    AsnCloseStruct(mbxp->aip, mbxp->BlastOutput, NULL);
+    
+    AsnIoClose(mbxp->aip);
+    
+    BlastOutputFree(mbxp->boutp);
+    MemFree(mbxp);
+    
+    return;
+}
+
+static void FillHspScoreInfo(HspPtr hsp, ScorePtr score)
+{
+   ScorePtr sp;
+   for(sp = score; sp != NULL; sp = sp->next) {
+      if(!(StringICmp(sp->id->str, "e_value")) || 
+         !(StringICmp(sp->id->str, "sum_e")))
+         hsp->evalue = sp->value.realvalue;
+      if(!StringICmp(sp->id->str, "bit_score")) {
+         hsp->bit_score = sp->value.realvalue;
+      }
+      if(!StringICmp(sp->id->str, "score")) {
+         hsp->score = sp->value.intvalue;
+      }
+      if(!StringICmp(sp->id->str, "num_ident")) {
+         hsp->identity = sp->value.intvalue;
+      }
+      if (!StringICmp(sp->id->str, "sum_n")) {
+         hsp->num = sp->value.intvalue;
+      }
+   }
+}
+
+static HspPtr GetHspFromSeqAlign(SeqAlignPtr align, Boolean ungapped, 
+                          Int4Ptr hspcnt_ptr)
+{
+    HspPtr hsp, head_hsp = NULL, last_hsp = NULL;
+    ScorePtr score, sp;
+    DenseDiagPtr ddp;
+    DenseSegPtr dsp;
+    StdSegPtr ssp;
+    Int4 hspcnt = 0, last_seg, i;
+
+    *hspcnt_ptr = 0;
+
+    switch (align->segtype) {
+    case 1: /*Dense-diag; blastn, blastp ungapped */
+       ddp = (DenseDiagPtr) align->segs;
+       while(ddp) {
+          if((hsp = HspNew()) == NULL)
+             return head_hsp;
+          if (!head_hsp)
+             head_hsp = last_hsp = hsp;
+          else {
+             last_hsp->next = hsp;
+             last_hsp = last_hsp->next;
+          }
+
+          ++hspcnt;
+          hsp->align_len = ddp->len;
+          if (ddp->strands[0] == Seq_strand_minus) {
+             hsp->query_from = ddp->starts[0] + ddp->len;
+             hsp->query_to = ddp->starts[0] + 1;
+          } else {
+             hsp->query_from = ddp->starts[0] + 1;
+             hsp->query_to = ddp->starts[0] + ddp->len;
+          }
+          if (ddp->strands[1] == Seq_strand_minus) {
+             hsp->hit_from = ddp->starts[1] + ddp->len;
+             hsp->hit_to = ddp->starts[1] + 1;
+          } else {
+             hsp->hit_from = ddp->starts[1];
+             hsp->hit_to = ddp->starts[1] + ddp->len - 1;
+          }
+          FillHspScoreInfo(hsp, ddp->scores);
+          ddp = ddp->next;
+       }
+       break;
+    case 2: /* Dense-seg; blastn, blastp gapped */
+       dsp = (DenseSegPtr) align->segs;
+       if((head_hsp = hsp = HspNew()) == NULL)
+          return head_hsp;
+       if (dsp->scores)
+          score = dsp->scores;
+       else
+          score = align->score;
+       hspcnt = 1;
+       last_seg = dsp->numseg - 1;
+       
+       if (dsp->strands[0] == Seq_strand_minus) {
+          hsp->query_from = dsp->starts[0] + dsp->lens[0];
+          hsp->query_to = dsp->starts[2*last_seg] + 1;
+       } else {
+          hsp->query_from = dsp->starts[0] + 1;
+          hsp->query_to = dsp->starts[2*last_seg] + dsp->lens[last_seg];
+       }
+       if (dsp->strands[1] == Seq_strand_minus) {
+          hsp->hit_from = dsp->starts[1] + dsp->lens[1];
+          hsp->hit_to = dsp->starts[2*last_seg+1] + 1;
+       } else {
+          hsp->hit_from = dsp->starts[1] + 1;
+          hsp->hit_to = dsp->starts[2*last_seg+1] + dsp->lens[last_seg];
+       }
+       
+       for (i=0; i<dsp->numseg; i++) {
+          hsp->align_len += dsp->lens[i];
+          if (dsp->starts[2*i] == -1 || dsp->starts[2*i+1] == -1) {
+             hsp->gaps++;
+          }
+       }
+       FillHspScoreInfo(hsp, score);
+       break;
+    case 3: /* Std-seg; translated gapped or ungapped */
+       ssp = (StdSegPtr) align->segs;
+
+       if (ungapped) {
+          while(ssp) {
+             if((hsp = HspNew()) == NULL)
+                return head_hsp;
+             if (!head_hsp)
+                head_hsp = last_hsp = hsp;
+             else {
+                last_hsp->next = hsp;
+                last_hsp = last_hsp->next;
+             }
+             
+             ++hspcnt;
+             if (ssp->scores)
+                score = ssp->scores;
+             else 
+                score = align->score;
+             
+             FillHspScoreInfo(hsp, score);
+
+             if (SeqLocStrand(ssp->loc) == Seq_strand_minus) {
+                hsp->query_to = SeqLocStart(ssp->loc) + 1;
+                hsp->query_from = SeqLocStop(ssp->loc);
+             } else {
+                hsp->query_from = SeqLocStart(ssp->loc) + 1;
+                hsp->query_to = SeqLocStop(ssp->loc);
+             }
+             if (SeqLocStrand(ssp->loc->next) == Seq_strand_minus) {
+                hsp->hit_to = SeqLocStart(ssp->loc->next) + 1;
+                hsp->hit_from = SeqLocStop(ssp->loc->next);
+             } else {
+                hsp->hit_from = SeqLocStart(ssp->loc->next) + 1;
+                hsp->hit_to = SeqLocStop(ssp->loc->next);
+             }
+             if (SeqLocStrand(ssp->loc) == Seq_strand_unknown) {
+                /* Protein location */
+                hsp->align_len = SeqLocLen(ssp->loc);
+             } else { /* Nucleotide location; need to divide length by 3 
+                        to get protein length */
+                hsp->align_len = SeqLocLen(ssp->loc) / 3;
+             }
+             ssp = ssp->next;
+          }
+       } else {
+          SeqLocPtr slp;
+          StdSegPtr last_ssp;
+
+          if((head_hsp = hsp = HspNew()) == NULL)
+             return head_hsp;
+
+          hspcnt = 1;
+          FillHspScoreInfo(hsp, align->score);
+          /* Get relevant endpoints from the first StdSeg in the list */
+          if (SeqLocStrand(ssp->loc) == Seq_strand_minus) {
+             hsp->query_from = SeqLocStop(ssp->loc);
+          } else {
+             hsp->query_from = SeqLocStart(ssp->loc) + 1;
+          }
+          if (SeqLocStrand(ssp->loc->next) == Seq_strand_minus) {
+             hsp->hit_from = SeqLocStop(ssp->loc->next);
+          } else {
+             hsp->hit_from = SeqLocStart(ssp->loc->next) + 1;
+          }
+          /* Advance StdSeg to the end of the list, and calculate the 
+             alignment length in the process */
+          for ( ; ssp; ssp = ssp->next) {
+             slp = ((ssp->loc->choice == SEQLOC_EMPTY) ? ssp->loc->next : 
+                    ssp->loc);
+             if (SeqLocStrand(slp) == Seq_strand_unknown) {
+                hsp->align_len += SeqLocLen(slp);
+             } else {
+                hsp->align_len += (SeqLocLen(slp) / 3);
+             }
+             last_ssp = ssp;
+          }
+          /* Get relevant endpoints from the last StdSeg in the list */
+          if (SeqLocStrand(last_ssp->loc) == Seq_strand_minus) {
+             hsp->query_to = SeqLocStart(last_ssp->loc) + 1;
+          } else {
+             hsp->query_to = SeqLocStop(last_ssp->loc);
+          }
+          if (SeqLocStrand(last_ssp->loc->next) == Seq_strand_minus) {
+             hsp->hit_to = SeqLocStart(last_ssp->loc->next) + 1;
+          } else {
+             hsp->hit_to = SeqLocStop(last_ssp->loc->next);
+          }
+       }
+       break;
+    default: break;
+    }    
+    
+    *hspcnt_ptr = hspcnt;
+    return head_hsp;
+}
+
+HitPtr SeqAlignToHits(SeqAlignPtr seqalign, Boolean ungapped)
+{
+   HitPtr hitp, hitp_head;
+   SeqAlignPtr sap, sap2;
+   SeqIdPtr subject_id, sip;
+   Char buffer[526];
+   HspPtr hspp;
+   Int4 hsp_count, chain;
+   
+   hitp_head = NULL;
+   
+   for(sap = seqalign; sap != NULL;) {
+      subject_id = TxGetSubjectIdFromSeqAlign(sap);
+      
+      if(hitp_head == NULL) { /* first element */
+         hitp_head = hitp = HitNew();
+      } else {
+         hitp->next = HitNew();
+         hitp = hitp->next;
+      }
+      
+      SeqIdWrite(subject_id, buffer, PRINTID_FASTA_LONG, sizeof(buffer));
+      hitp->id = StringSave(buffer);
+      SeqIdWrite(SeqIdFindBestAccession(subject_id), buffer, 
+                 PRINTID_TEXTID_ACCESSION, sizeof(buffer));
+      
+      hitp->accession = StringSave(buffer);
+      
+      for(sap2 = sap; sap2 != NULL; ) {
+         /* Filling info about specific alignments */
+         if (!hitp->hsps) {
+            hspp = hitp->hsps = 
+               GetHspFromSeqAlign(sap2, ungapped, &hsp_count);
+         } else {
+            hspp->next = GetHspFromSeqAlign(sap2, ungapped, &hsp_count);
+            hspp = hspp->next;
+         }
+
+         hitp->num += hsp_count;
+
+         sap2 = sap2->next;
+         sip = TxGetSubjectIdFromSeqAlign(sap2);
+         
+         if(SeqIdMatch(subject_id, sip)) {
+            continue;
+         } else {
+            sap = sap2;
+            break;
+         }
+      }
+   }
+   
+   return hitp_head;
+}
