@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.11 $
+* $Revision: 6.14 $
 *
 * File Description: 
 *
@@ -40,6 +40,15 @@
 *
 *
 * $Log: vibforms.c,v $
+* Revision 6.14  2004/03/06 20:00:38  kans
+* SetEnumPopupEx and SetEnumPopupByNameEx set popup to 0 if desired choice not found
+*
+* Revision 6.13  2004/01/16 22:35:37  kans
+* added WidestString and WidestAlist
+*
+* Revision 6.12  2002/01/09 15:11:23  kans
+* SetEnumPopupEx and SetEnumPopupByNameEx to suppress error if 0 or NULL initial value, called by dialog creators
+*
 * Revision 6.11  1999/10/21 16:50:40  kans
 * use SafeSetValue for enum lists and spreadsheets
 *
@@ -303,12 +312,19 @@ Boolean GetEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnumPtr pval)
   return FALSE;
 } 
 
-void SetEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnum val) 
+static void SetEnumPopupEx (PopuP lst, EnumFieldAssocPtr al, UIEnum val, Boolean zeroOkay) 
 {
   Int2 i;
   for (i = 1; al->name != NULL; i++, al++)
      if (al->value == val) { SafeSetValue (lst, i); return; }
+  if (zeroOkay && val == 0) return;
   Message(MSG_POSTERR, "SetEnumPopup: %ld", (long)val);
+  SafeSetValue (lst, 0);
+} 
+
+void SetEnumPopup (PopuP lst, EnumFieldAssocPtr al, UIEnum val) 
+{
+  SetEnumPopupEx (lst, al, val, FALSE);
 } 
 
 CharPtr GetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al)
@@ -319,13 +335,21 @@ CharPtr GetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al)
   return NULL;
 } 
 
-void SetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al, CharPtr name)
+static void SetEnumPopupByNameEx (PopuP lst, EnumFieldAssocPtr al, CharPtr name, Boolean zeroOkay)
 
 {
   Int2 i;
   for (i = 1; al->name != NULL; i++, al++)
      if (StringICmp (al->name, name) == 0) { SafeSetValue (lst, i); return; }
+  if (zeroOkay && StringHasNoText (name)) return;
   Message(MSG_POSTERR, "SetEnumPopupByName: %s", name);
+  SafeSetValue (lst, 0);
+}
+
+void SetEnumPopupByName (PopuP lst, EnumFieldAssocPtr al, CharPtr name)
+
+{
+  SetEnumPopupByNameEx (lst, al, name, FALSE);
 }
 
 Boolean WhereInEnumPopup (EnumFieldAssocPtr al, CharPtr name, UIEnumPtr pval)
@@ -554,7 +578,7 @@ PopuP CreateEnumPopupDialog (GrouP prnt, Boolean macLike, PupActnProc actn,
   adp->alist = DuplicateEnumFieldAlist (al);
   adp->userdata = userdata;
   InitEnumPopup (p, adp->alist, NULL);
-  SetEnumPopup (p, adp->alist, val);
+  SetEnumPopupEx (p, adp->alist, val, TRUE);
   return p;
 }
 
@@ -598,7 +622,7 @@ LisT CreateEnumListDialog (GrouP prnt, Int2 width, Int2 height, LstActnProc actn
   for (ap = adp->alist; ap->name != NULL; ap++) {
     ListItem (p, ap->name);
   }
-  SetEnumPopup ((PopuP) p, adp->alist, val);
+  SetEnumPopupEx ((PopuP) p, adp->alist, val, TRUE);
   return p;
 }
 
@@ -1202,6 +1226,30 @@ extern Int2 MaxStringWidths (CharPtr PNTR strs)
   return (max + 2);
 }
 
+extern CharPtr WidestString (CharPtr PNTR strs)
+
+{
+  Int2     i;
+  Int2     len;
+  Int2     max;
+  CharPtr  str;
+
+  str = NULL;
+  max = 0;
+  if (strs != NULL) {
+    i = 0;
+    while (strs [i] != NULL) {
+      len = StringWidth (strs [i]);
+      if (len > max) {
+        max = len;
+        str = strs [i];
+      }
+      i++;
+    }
+  }
+  return str;
+}
+
 extern Int2 MaxAlistWidths (EnumFieldAssocPtr al)
 
 {
@@ -1230,6 +1278,28 @@ extern Int2 MaxAlistWidths (EnumFieldAssocPtr al)
   return max;
 }
 
+extern CharPtr WidestAlist (EnumFieldAssocPtr al)
+
+{
+  Int2     i;
+  Int2     len;
+  Int2     max;
+  CharPtr  str;
+
+  str = NULL;
+  max = 0;
+  if (al != NULL) {
+    for (i = 1; al->name != NULL; i++, al++) {
+      len = StringWidth (al->name);
+      if (len > max) {
+        max = len;
+        str = al->name;
+      }
+    }
+  }
+  return str;
+}
+
 extern GrouP MultiLinePromptEx (GrouP prnt, CharPtr text, Int2 maxWidth, FonT font, Boolean stripSpaces)
 
 {
