@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   7/1/91
 *
-* $Revision: 6.5 $
+* $Revision: 6.28 $
 *
 * File Description: 
 *       Vibrant procedure definitions
@@ -37,6 +37,80 @@
 * Modifications:  
 * --------------------------------------------------------------------------
 * $Log: vibprocs.h,v $
+* Revision 6.28  2004/04/01 13:43:09  lavr
+* Spell "occurred", "occurrence", and "occurring"
+*
+* Revision 6.27  2004/02/23 16:36:51  sinyakov
+* Use Int4 instead of Int2 for cursor position and text selection in text boxes
+*
+* Revision 6.26  2004/02/12 20:05:26  bazhin
+* Added callback function "Nlm_SetKeepCrNlTextFieldCallback(Nlm_TexT t)",
+* which allows to copy-paste texts from multiple columns to single string.
+*
+* Revision 6.25  2004/02/05 16:25:06  kans
+* revert to Nlm_GetTextVScrollBar and Nlm_GetTextHScrollBar being static, implement setOffset and getOffset for scroll text, since Windows scroll text has its own scroll bar
+*
+* Revision 6.24  2004/02/04 15:21:02  kans
+* make GetTextVScrollBar and GetTextHScrollBar extern, to add search function to general text viewer
+*
+* Revision 6.23  2004/01/05 17:08:09  kans
+* added functions to control use of dual screens
+*
+* Revision 6.22  2003/10/29 19:19:34  kans
+* added prototype for Nlm_SetTextColor
+*
+* Revision 6.21  2002/04/30 18:25:12  bazhin
+* Added function "Nlm_SetWindowConfigureCallback(WindoW w)", which
+* allows to catch events, when window just moved without resizing.
+* Fixed some "gcc -Wall -ansi" warnings.
+*
+* Revision 6.20  2002/01/09 15:23:54  kans
+* added HasAquaMenuLayout
+*
+* Revision 6.19  2001/09/10 17:34:33  bazhin
+* Added function Nlm_SetTextCursorBlinkRate(Nlm_TexT t, Nlm_Int2 msec).
+*
+* Revision 6.18  2000/08/01 15:57:32  kans
+* separate lines for Nlm_HANDLE on all platforms, even if two are currently the same, and no space between # and define in case some compilers complain
+*
+* Revision 6.17  2000/08/01 15:23:29  vakatov
+* Addition to R6.13:   error if neither of "WIN_***" is #defined
+*
+* Revision 6.16  2000/07/31 13:28:46  lewisg
+* Nlm_GetExecPath
+*
+* Revision 6.15  2000/07/28 21:25:11  lewisg
+* *** empty log message ***
+*
+* Revision 6.14  2000/07/28 21:05:54  lewisg
+* more c++ fixes
+*
+* Revision 6.13  2000/07/28 15:36:34  kans
+* for OS_UNIX, give error message if WIN_MOTIF not defined
+*
+* Revision 6.12  2000/07/25 21:19:00  lewisg
+* eliminate redundant windows pathname code
+*
+* Revision 6.11  2000/07/25 19:18:36  lewisg
+* get pathname for executable on windows
+*
+* Revision 6.10  2000/07/14 20:30:40  kans
+* added Nlm_MSWin_OpenApplication (for launching Cn3D with a data file)
+*
+* Revision 6.9  1999/04/06 14:23:26  lewisg
+* add opengl replacement for viewer3d
+*
+* Revision 6.8  1999/02/19 20:48:36  vakatov
+* [WIN_MSWIN]  Added Nlm_MSWin_OpenDocument(also can be used to
+* auto-launch IE or Netscape if passed an URL)
+*
+* Revision 6.7  1999/01/07 22:32:36  kans
+* added Mac-specific Nlm_SendURLAppleEvent
+*
+* Revision 6.6  1998/07/07 23:03:34  vakatov
+* Added Nlm_Execv() to spawn applications with cmd-line parameters;
+* now implemented for [OS_MSWIN, OS_UNIX].
+*
 * Revision 6.5  1998/03/22 03:00:59  kans
 * changed names to RegisterServiceProc and RegisterResultProc
 *
@@ -281,6 +355,16 @@ void        Nlm_SetColorCell PROTO((Nlm_GraphiC w, Nlm_Uint1 pixel,
 void        Nlm_SetWindowExtra PROTO((Nlm_WindoW w, Nlm_VoidPtr data, Nlm_WndFreeProc cleanup));
 Nlm_VoidPtr Nlm_GetWindowExtra PROTO((Nlm_WindoW w));
 
+void        Nlm_SetWindowConfigureCallback PROTO((Nlm_WindoW w));
+
+/* Dual screen functions control centering where the window left parameter is negative */
+
+Nlm_Boolean Nlm_HasDualScreen (void);
+void Nlm_UseFullScreen (void);
+void Nlm_UseLeftScreen (void);
+void Nlm_UseRightScreen (void);
+
+
 /***  GROUPING OBJECT  ***/
 
 /*
@@ -374,6 +458,7 @@ void        Nlm_PopupItems PROTO((Nlm_PopuP p, Nlm_CharPtr PNTR titles));
 void        Nlm_DeskAccGroup PROTO((Nlm_MenU m));
 Nlm_ChoicE  Nlm_FontGroup PROTO((Nlm_MenU m));
 void        Nlm_SeparatorItem PROTO((Nlm_MenU m));
+Nlm_Boolean Nlm_HasAquaMenuLayout (void);
 
 /***  Bar Object  ***/
 
@@ -471,7 +556,7 @@ Nlm_TexT  Nlm_CurrentText PROTO((void));
 
 /* return TRUE and selection range [begin,end] -- if there is a selection
  */
-Nlm_Boolean Nlm_TextSelectionRange PROTO((Nlm_TexT t, Nlm_Int2Ptr begin, Nlm_Int2Ptr end));
+Nlm_Boolean Nlm_TextSelectionRange PROTO((Nlm_TexT t, Nlm_Int4Ptr begin, Nlm_Int4Ptr end));
 
 void      Nlm_CutText PROTO((Nlm_TexT t));
 void      Nlm_CopyText PROTO((Nlm_TexT t));
@@ -479,15 +564,19 @@ void      Nlm_PasteText PROTO((Nlm_TexT t));
 void      Nlm_ClearText PROTO((Nlm_TexT t));
 
 size_t    Nlm_TextLength PROTO((Nlm_TexT t));
-void      Nlm_SelectText PROTO((Nlm_TexT t, Nlm_Int2 begin, Nlm_Int2 end));
+void      Nlm_SelectText PROTO((Nlm_TexT t, Nlm_Int4 begin, Nlm_Int4 end));
 void      Nlm_SetTextEditable PROTO((Nlm_TexT t, Nlm_Boolean editable));
+void      Nlm_SetTextCursorBlinkRate PROTO((Nlm_TexT t, Nlm_Int2 msec));
+void      Nlm_SetTextColor PROTO((Nlm_TexT t, Nlm_Uint4 r, Nlm_Uint4 g, Nlm_Uint4 b));
+void      Nlm_SetKeepCrNlTextFieldCallback PROTO((Nlm_TexT t));
 
 /* return the new insertion position in the text(it may differ from "pos"
- * if an error occured or "pos" is out of limits)
+ * if an error occurred or "pos" is out of limits)
  */
-Nlm_Int2  Nlm_SetTextCursorPos PROTO((Nlm_TexT t, Nlm_Int2 pos));
+Nlm_Int4  Nlm_SetTextCursorPos PROTO((Nlm_TexT t, Nlm_Int4 pos));
+
+Nlm_Int4  Nlm_GetTextCursorPos PROTO((Nlm_TexT t));
 
-Nlm_Int2  Nlm_GetTextCursorPos PROTO((Nlm_TexT t));
 
 
 /***  Static Prompt Objects  ***/
@@ -547,6 +636,23 @@ Nlm_PaneL  Nlm_SimplePanel PROTO((Nlm_GrouP prnt, Nlm_Int2 pixwidth, Nlm_Int2 pi
 Nlm_PaneL  Nlm_AutonomousPanel4 PROTO((Nlm_GrouP prnt, Nlm_Int2 pixwidth, Nlm_Int2 pixheight, Nlm_PnlActnProc draw, Nlm_SltScrlProc4 vscrl, Nlm_SltScrlProc4 hscrl, Nlm_Int2 extra, Nlm_PnlActnProc reset, Nlm_GphPrcsPtr classPtr));
 Nlm_PaneL  Nlm_AutonomousPanel PROTO((Nlm_GrouP prnt, Nlm_Int2 pixwidth, Nlm_Int2 pixheight, Nlm_PnlActnProc draw, Nlm_SltScrlProc vscrl, Nlm_SltScrlProc hscrl, Nlm_Int2 extra, Nlm_PnlActnProc reset, Nlm_GphPrcsPtr classPtr));
 
+
+#ifdef _OPENGL
+void Nlm_Set3DColorMap PROTO((Nlm_PaneL w, Nlm_Uint2 totalColors,
+                             Nlm_Uint1Ptr red, Nlm_Uint1Ptr green,
+                             Nlm_Uint1Ptr blue, void **display));
+
+
+Nlm_PaneL  Nlm_Autonomous3DPanel PROTO((Nlm_GrouP prnt, Nlm_Int2 pixwidth,
+                                       Nlm_Int2 pixheight, Nlm_PnlActnProc draw,
+                                       Nlm_SltScrlProc vscrl, Nlm_SltScrlProc hscrl,
+                                       Nlm_Int2 extra, Nlm_PnlActnProc reset,
+                                       Nlm_GphPrcsPtr classPtr, Nlm_Boolean *IndexMode,
+                                       void **display, void **visinfo));
+#endif /* _OPENGL */
+
+
+
 void       Nlm_SetPanelClick PROTO((Nlm_PaneL p, Nlm_PnlClckProc click, Nlm_PnlClckProc drag, Nlm_PnlClckProc hold, Nlm_PnlClckProc release));
 
 void       Nlm_SetPanelExtra PROTO((Nlm_PaneL p, Nlm_VoidPtr sptr));
@@ -863,17 +969,25 @@ void        Nlm_ProcessUpdatesFirst  PROTO((Nlm_Boolean val));
  */
 void        Nlm_PopupParentWindow PROTO((Nlm_Handle a));
 
+/* Run the specified application(executable located in "path").
+ * The "argv" argument is an array of character pointers to
+ * null-terminated strings.  These strings constitute the argument
+ * list available to the new process image.
+ * The "argv" argument vector is terminated by a null pointer.
+ */
+extern Nlm_Boolean Nlm_Execv PROTO((const Nlm_Char* path, Nlm_Char *const * argv));
+#define Execv Nlm_Execv
 
 /***  OBJECT ALIGNMENT PROCEDURE FOR USE WITHOUT GROUP REPOSITIONING  ***/
 
-#ifdef WIN_MAC
+#if defined(WIN_MAC)
 #define Nlm_HANDLE Nlm_Handle
-#endif
-#ifdef WIN_MSWIN
+#elif defined(WIN_MOTIF)
+#define Nlm_HANDLE Nlm_Handle
+#elif defined(WIN_MSWIN)
 #define Nlm_HANDLE int
-#endif
-#ifdef WIN_MOTIF
-#define Nlm_HANDLE Nlm_Handle
+#elif !defined(WIN_GIF)
+#error "Vibrant applications must be compiled with WIN_*** defined!!!"
 #endif
 
 #define ALIGN_LEFT      1
@@ -1104,6 +1218,19 @@ Nlm_VoidPtr Nlm_GetObject PROTO((Nlm_GraphiC a));
 #ifdef WIN_MAC
 extern void Nlm_SendOpenDocAppleEvent PROTO((Nlm_CharPtr datafile, Nlm_CharPtr sig));
 extern void Nlm_SendOpenDocAppleEventEx PROTO((Nlm_CharPtr datafile, Nlm_CharPtr sig, Nlm_CharPtr prog, Nlm_Boolean wantReply));
+extern void Nlm_SendURLAppleEvent (Nlm_CharPtr urlString, Nlm_CharPtr sig, Nlm_CharPtr prog);
+#endif
+
+#ifdef WIN_MSWIN
+/* Call application registered to open the given type of document.
+ * The doc type is recognized automagically using the file extention;
+ * if the doc is URL then its name must explicitely start from "http://".
+ */
+extern Nlm_Boolean Nlm_MSWin_OpenDocument(const Nlm_Char* doc_name);
+
+/* Call program (e.g., Cn3D) with parameters (e.g., data file name) */
+extern Nlm_Boolean Nlm_MSWin_OpenApplication(const Nlm_Char* program, const Nlm_Char* parameters);
+extern void Nlm_GetExecPath(char *filetype, char *buf, int buflen);
 #endif
 
 #ifdef WIN_MOTIF
@@ -1139,7 +1266,7 @@ typedef enum
 {
   Nlm_IO_READ,    /* file descriptor has data available    */
   Nlm_IO_WRITE,   /* file descriptor available for writing */
-  Nlm_IO_ERROR,   /* I/O error(exception) occured          */
+  Nlm_IO_ERROR,   /* I/O error(exception) occurred         */
   Nlm_IO_CLEANUP  /* when called from UnregisterIO()       */
 } Nlm_IOtype;
 
